#!/bin/bash



#Script écrit par Nicolas Aldegheri pour la mise en place d'un serveur LTSP sur une debian serveur de base. Tout poste du domaine ou non pourra booter par IPXE sur un un OS Linux, avec identification et montages des partages se4.
#ce script ne doit pas être lancé sur le se4fs ou se4ad!!!!
#Ce script est encore expériemental et va subir de nombreuses modifications . Demander conseil avant sa mise en place à l'équipe SambaEdu (Salon TCHAP, ou liste de diffusion sambaedu@groupes.renater.fr )

# Varaible à renseigner
domaine="sambaedu.demo.org"
proxy=""						# sous la forme 10.0.0.1:3128
WORKGROUP="SAMBAEDU"
ENVIRONNEMENT="se4"
IP_LTSP="172.16.1.15"
IP_SE4AD="172.16.1.11"
IP_SE4FS="172.16.1.11"
mask="255.255.0.0"

# Variables à ne pas modifier en principe
DOMAINE=$(echo $domaine | tr 'a-z' 'A-Z')
LTSP_HOSTNAME=$(cat /etc/hostname)			# tous les clients lourds sont intégrés au domaine AD de façon "générique", avec le nom du serveur ltsp
PREF_FIREFOX='firefox-esr/firefox-esr.js'   # fichier des préférences firefox
REP_MONTAGE='~'								# Répertoire contenant le montage des partages Samba : le home utilisateur avec le bureau MATE
FF_MONTAGE='/home/$USER'					# Valeur pour le bureau mate									# Valeur par défaut	pour la création du profil FF persistant

echo ' Pré-requis : '
echo ' 1 - Serveur Debian Buster "de base" avec serveur ssh, serveur d impression et utilitaires usuelles de cocher.'
echo ' 2 - IP statique de défini dans /etc/network/interfaces'
     
echo ' 3 - Renseigner les 6 variables du début de ce script.'

echo 'Si tous les pré-requis précédents sont valides, répondre o pour lancer l installation et la configuration du serveur ltsp :'
read REPONSE
if [ "$REPONSE" != "o" ]
then
	exit 0
fi

echo "------------------------------------------------------------------------------------------------------------------------------"
echo " 1 - On autorise les connections ssh avec le serveur ltsp et avec le compte root												"
echo "------------------------------------------------------------------------------------------------------------------------------"
sleep 5

sed -i 's/#PermitRootLogin prohibit-password/PermitRootLogin yes/' /etc/ssh/sshd_config
#service ssh restart
systemctl restart ssh

echo "------------------------------------------------------------------------------------------------------------------------------"
echo " 2 - Installation du serveur ltsp :																							"
echo "------------------------------------------------------------------------------------------------------------------------------"
sleep 5
apt-get install -y ltsp-server

echo "------------------------------------------------------------------------------------------------------------------------------"
echo " 3 - Installation du module php pour distribuer aux clients lourds : kernel, initrd, ... via ipxe								"
echo "------------------------------------------------------------------------------------------------------------------------------"
sleep 5
apt-get install -y libapache2-mod-php
ln -s /var/lib/tftpboot/ltsp /var/www/html/ltsp   # lien vers le dossier tftpboot contenant kernet, initrd ...


echo "--------------------------------------------------------------------------------------------------------------------------------------------------"
echo " 4 - Construction du chroot des clients lourds ltsp : les paquets téléchargés sont mis en cache, seule la 1ère construction sera longue			"
echo "     Le mot de pass du compte root du chroot sera demandé à la fin de l'installation 																"
echo "--------------------------------------------------------------------------------------------------------------------------------------------------"
sleep 5

# Ecriture de certains paramètres de la commande ltsp-build-client dans un fichier de conf, pour alleger la commande
cat <<EOF > /etc/ltsp/ltsp-build-client-se4.conf
ARCH="i386"
CHROOT="$ENVIRONNEMENT"
DIST="buster"
MIRROR="http://deb.debian.org/debian/"
COMPONENTS="main contrib non-free"
HTTP_PROXY="$proxy"
LOCALE="fr_FR.UTF-8"
LATE_PACKAGES="
	less
	nano
"
EOF

#ltsp-build-client --arch i386 --chroot se4 --fat-client-desktop task-mate-desktop --dist buster --mirror http://ftp.fr.debian.org/debian/ --locale fr_FR.UTF-8 --prompt-rootpass --purge-chroot
ltsp-build-client --config /etc/ltsp/ltsp-build-client-se4.conf --fat-client-desktop task-mate-desktop --keep-packages --copy-package-cache --no-squashfs-image --prompt-rootpass --purge-chroot 

echo "--------------------------------------------------------------------------------------------------------------"
echo " 5 - Création du compte local enseignant :																	"			
echo "--------------------------------------------------------------------------------------------------------------"
sleep 5
ltsp-chroot --arch "$ENVIRONNEMENT" adduser enseignant
ltsp-chroot --arch "$ENVIRONNEMENT" chmod -R 700 /home/enseignant

# Utilisation du module pam_mkhomedir.so pour créer automatiquement le home directory d'un utilisateur qui se connecte sur un client lourd
sed -i '/@include common-session/i \session required pam_mkhomedir.so skel=/etc/skel umask=0077' "/opt/ltsp/$ENVIRONNEMENT/etc/pam.d/lightdm"

# A noter : à ce stade, il devrait être possible d'ouvrir une session avec le compte local enseignant ...

echo "--------------------------------------------------------------------------------------------------------------"
echo " 6 - Intégration du chroot ltsp au domaine AD du se4 : 														"			
echo "--------------------------------------------------------------------------------------------------------------"
sleep 5

echo "--------------------------------------------------------------------------------------------------------------"
echo " 6.1 - Création du fichier de configation du chroot lts.conf 													"			
echo "--------------------------------------------------------------------------------------------------------------"
sleep 5
# Fichier de configuration du chroot ltsp
cat <<EOF > "/opt/ltsp/$ENVIRONNEMENT/etc/lts.conf"
[default]
LTSP_CONFIG=true
KEEP_SYSTEM_SERVICES="display-manager"      # Indique à l'environnement du client lourd de lancer le dm par défaut lors du démarrage du client lourd
DEFAULT_DISPLAY_MANAGER=""          		# Lance le gestionnaire d'affichage présent dans l'environnement des clients lourds (lightdm) à la place de LDM
XKBLAYOUT=fr								# Peut être util sur Ubuntu, pour certains bureaux (lubuntu)
USE_LOCAL_SWAP=true							# Pour utiliser une éventuelle partition swap présente sur le disque dur local du client lourd
DNS_SERVER="$IP_SE4AD"   					# IP du se4 AD
SEARCH_DOMAIN="$domaine"
HOSTNAME="$LTSP_HOSTNAME"          			# Tous les clients lourds auront le même nom, celui du serveur ltsp
TIMESERVER="$IP_SE4AD"  					# IP du se4 AD
EOF

echo "--------------------------------------------------------------------------------------------------------------"
echo " 6.2 - Configuration du timezone 																				"			
echo "--------------------------------------------------------------------------------------------------------------"
sleep 5
ltsp-chroot --arch "$ENVIRONNEMENT" debconf-set-selections <<EOF
tzdata	tzdata/Zones/Etc	select	UTC
tzdata	tzdata/Zones/Europe	select	Paris
tzdata  tzdata/Areas	select  Europe
EOF
ltsp-chroot --arch "$ENVIRONNEMENT" echo 'Europe/Paris' > /etc/timezone
ltsp-chroot --arch "$ENVIRONNEMENT" dpkg-reconfigure tzdata --frontend=noninteractive

# On prépare le chroot pour l'intégration au domaine AD
cat <<EOF > "/opt/ltsp/$ENVIRONNEMENT/etc/resolv.conf"
domain $domaine
search $domaine
nameserver $IP_SE4AD
EOF

# Installation et configuration du protocole d'authentification Kerberos (basé sur des clés)
ltsp-chroot --arch "$ENVIRONNEMENT" apt-get update
ltsp-chroot --arch "$ENVIRONNEMENT" apt-get install -y krb5-user

cat <<EOF > "/opt/ltsp/$ENVIRONNEMENT/etc/krb5.conf"
[libdefaults]
        default_realm = $DOMAINE
        dns_lookup_realm = false
        dns_lookup_kdc = true
EOF

# Installation et configuration de samba et winbind
ltsp-chroot --arch "$ENVIRONNEMENT" debconf-set-selections <<EOF
samba-common	samba-common/dhcp	boolean false
EOF

ltsp-chroot --arch "$ENVIRONNEMENT" apt-get install -y samba winbind samba-dsdb-modules

ltsp-chroot --arch "$ENVIRONNEMENT" systemctl disable nmbd.service
ltsp-chroot --arch "$ENVIRONNEMENT" systemctl disable smbd.service
ltsp-chroot --arch "$ENVIRONNEMENT" systemctl enable winbind.service

ltsp-chroot --arch "$ENVIRONNEMENT" find /var/lib/samba/ '(' -name '*.tdb' -o -name '*.ldb' ')' -delete

cat <<EOF > "/opt/ltsp/$ENVIRONNEMENT/etc/samba/smb.conf"
[global]
       security = ADS
       workgroup = $WORKGROUP
       realm = $DOMAINE

       log file = /var/log/samba/%m.log
       log level = 1

# performances
       socket options = SO_KEEPALIVE TCP_KEEPIDLE=300 TCP_KEEPINTVL=10 TCP_KEEPCNT=5

# utilisateurs locaux

       idmap config * : backend = tdb
       idmap config * : range = 3000-7999

# domaine sambaedu

       idmap config $WORKGROUP : backend = rid
       idmap config $WORKGROUP : range = 10000-999999

       template homedir = /home/%U
       template shell = /bin/bash
#       username map = /etc/samba/user.map
       winbind use default domain = yes
       winbind offline logon = yes
       kerberos method = secrets and keytab
       dedicated keytab file = /etc/krb5.keytab

# imprimantes

        rpc_server:spoolss = external
        rpc_daemon:spoolssd = fork
        spoolss: architecture = Windows x64
        printing = CUPS


# Partages
EOF

ltsp-chroot --arch "$ENVIRONNEMENT" apt-get install -y libnss-winbind

cat <<'EOF' > "/opt/ltsp/$ENVIRONNEMENT/etc/nsswitch.conf"
# /etc/nsswitch.conf
#
# Example configuration of GNU Name Service Switch functionality.
# If you have the `glibc-doc-reference' and `info' packages installed, try:
# `info libc "Name Service Switch"' for information about this file.

passwd:         files systemd winbind
group:          files systemd winbind
shadow:         files winbind
gshadow:        files

hosts:          files mdns4_minimal [NOTFOUND=return] dns
networks:       files

protocols:      db files
services:       db files
ethers:         db files
rpc:            db files

netgroup:       nis
EOF

## Script à lancer sur un client lourd afin de réaliser l'intégration du client lourd

#cat <<'EOF' > "/opt/ltsp/$ENVIRONNEMENT/usr/local/bin/rejoint_SE4.sh"
##!/bin/bash
#echo " Intégration du client lourd au domaine ad SE4 "
#echo " Saisir le mot de passe Administrator du domaine AD (variable ldap_admin_passwd du fichier /etc/sambaedu/sambaedu.conf sur le serveur SE4FS) : "
#read ADMINISTRATOR_PWD
#net ads join -U administrator%"$ADMINISTRATOR_PWD"
#EOF

#cat <<EOF >> "/opt/ltsp/$ENVIRONNEMENT/usr/local/bin/rejoint_SE4.sh"
#echo " Copie des fichiers d'intégration dans le chroot, sur le serveur ltsp "
#echo " Saisir le mot de passe root du serveur ltsp : "
#scp /var/lib/samba/group_mapping.tdb /var/lib/samba/private/*.ldb /var/lib/samba/private/*.tdb root@$IP_LTSP:/opt/ltsp/$ENVIRONNEMENT/var/lib/samba/
#EOF
#ltsp-chroot --arch "$ENVIRONNEMENT" chmod +x /usr/local/bin/rejoint_SE4.sh

#ltsp-update-image "$ENVIRONNEMENT"

#echo " Démarrer un client lourd de votre réseau et ouvrir une session avec le compte enseignant :"
#echo " Ouvrir un terminal, se connecter en root et lancer le script /root/rejoint_SE4.sh"
#echo " Une fois le script d'intégration exécuté, saisir o pour finaliser l'installation"
#read REPONSE
#if [ "$REPONSE" != "o" ]
#then
#	exit 0
#fi

# On donne au chroot le même nom que le serveur ltsp afin de l'intégrer au domaine AD avec le nom du serveur ltsp
mv "/opt/ltsp/$ENVIRONNEMENT/etc/hosts" "/opt/ltsp/$ENVIRONNEMENT/etc/hosts.old"   # Sauvegarde du fichier hosts
cat <<EOF > "/opt/ltsp/$ENVIRONNEMENT/etc/hosts"
127.0.0.1       localhost
127.0.1.1       ${LTSP_HOSTNAME}.${domaine}       ${LTSP_HOSTNAME}

# The following lines are desirable for IPv6 capable hosts
::1     localhost ip6-localhost ip6-loopback
ff02::1 ip6-allnodes
ff02::2 ip6-allrouters
EOF

echo "------------------------------------------------------------------------------------------------------------------------------------------------------"
echo " 6.3 - Intégration au domain AD 																														"			
echo "       Saisir le mot de passe Administrator du domaine AD (variable ldap_admin_passwd du fichier /etc/sambaedu/sambaedu.conf sur le serveur SE4FS) :  "
echo "------------------------------------------------------------------------------------------------------------------------------------------------------"
read ADMINISTRATOR_PWD
ltsp-chroot --arch "$ENVIRONNEMENT" net ads join -U administrator%"$ADMINISTRATOR_PWD"
sleep 5
# On remet le hosts par défaut de ltsp
cp -f "/opt/ltsp/$ENVIRONNEMENT/etc/hosts.old" "/opt/ltsp/$ENVIRONNEMENT/etc/hosts"

echo "------------------------------------------------------------------------------------------------------------------"
echo " 6.4 - Configuration de pam pour kerberos et winbind 																"			
echo "------------------------------------------------------------------------------------------------------------------"
sleep 5
# Configuration de pam pour une authentification dans un domaine AD
ltsp-chroot --arch "$ENVIRONNEMENT" apt-get install -y libpam-krb5 libpam-winbind

echo "--------------------------------------------------------------------------------------------------------------------------"
echo " 6.5 - Configuration de pam_mount pour réaliser le montage automatique des partages samba  à l'ouverture de session 		"			
echo "--------------------------------------------------------------------------------------------------------------------------"
sleep 5
# Configuration de pam pour le montage automatique des partages Samba à l'ouverture de session
ltsp-chroot --arch "$ENVIRONNEMENT" apt-get install -y libpam-mount cifs-utils # cifs-utils contient /sbin/mount.cifs nécessaire au montage des partages cifs

cat <<'EOF' > "/opt/ltsp/$ENVIRONNEMENT/etc/security/pam_mount.conf.xml"
<?xml version="1.0" encoding="utf-8"?>

<!--
/etc/security/pam_mount.conf.xml
Configuration pour le serveur
-->

<pam_mount>
  <debug enable="1" />
  <logout wait="0" hup="0" term="0" kill="0" />
  <mkmountpoint enable="1" remove="true" />

  <umount>umount %(MNTPT)</umount>

 <volume
    pgrp="domain users"
    fstype="cifs"
    server="se4fs"
    path="users/%(DOMAIN_USER)/Docs"
	mountpoint="~/Docs (sur le reseau)"
    options="username=%(DOMAIN_USER),uid=%(USERUID),gid=%(USERGID),cruid=%(USERUID),file_mode=0700,dir_mode=0700,vers=3.0"
  />

  <volume
    pgrp="domain users"
    fstype="cifs"
    server="se4fs"
	path="classes"
	mountpoint="~/Classes (sur le reseau)"
    options="username=%(USER),uid=%(USERUID),gid=%(USERGID),file_mode=0700,dir_mode=0700,vers=3.0"
  />
  
  <volume
    pgrp="domain users"
    fstype="cifs"
    server="se4fs"
	path="docs"
	mountpoint="~/Partages"
    options="username=%(USER),uid=%(USERUID),gid=%(USERGID),file_mode=0700,dir_mode=0700,vers=3.0"
  />

<msg-authpw>Mot de passe :</msg-authpw>
<msg-sessionpw>Mot de passe :</msg-sessionpw>

</pam_mount>
EOF

echo "--------------------------------------------------------------------------------------------------------------------------"
echo " 6.6 - Configuration de lightdm																					 		"			
echo "--------------------------------------------------------------------------------------------------------------------------"
sleep 5
cat <<EOF > "/opt/ltsp/$ENVIRONNEMENT/etc/lightdm/lightdm.conf"
[SeatDefaults]
greeter-show-manual-login=true
greeter-hide-users=true
allow-guest=false
EOF

echo "--------------------------------------------------------------------------------------------------------------------------"
echo " Souhaitez-vous poursuivre l'installation en réalisant la personalisation du chroot mate ? 								"			
echo " Saisir o pour oui						 																				"			
echo "--------------------------------------------------------------------------------------------------------------------------" 
read REPONSE
if [ "$REPONSE" != "o" ]
then
	echo "--------------------------------------------------------------------------------------------------------------------------"
	echo " 7 - Finalisation de l'installation par la construction de l'image nbd du chroot			   								"
	echo "--------------------------------------------------------------------------------------------------------------------------"
	sleep 5
	ltsp-update-image "$ENVIRONNEMENT"
	echo "----------------------------------------------------------------------------------------------------------------------------------------------------------"
	echo " Installation finie : vous pouvez démarrer un client lourd ltsp et vérifier que l'ouverture de session et le montage des partages samba est fonctionnel !	"
	echo "----------------------------------------------------------------------------------------------------------------------------------------------------------"
	exit 0
fi

echo "--------------------------------------------------------------------------------------------------------------------------"
echo " 7 - Désactivation de l'hibernation, la mise en veille, ...																"
echo "--------------------------------------------------------------------------------------------------------------------------"
sleep 5
# Desactivation "complète" de l'hibernation
cat << 'EOF' > "/opt/ltsp/$ENVIRONNEMENT/etc/polkit-1/localauthority/90-mandatory.d/disable-hibernate.pkla"
[Disable hibernate (upower)]
Identity=unix-user:*
Action=org.freedesktop.upower.hibernate
ResultActive=no
ResultInactive=no
ResultAny=no

[Disable hibernate (logind)]
Identity=unix-user:*
Action=org.freedesktop.login1.hibernate
ResultActive=no

[Disable hibernate for all sessions (logind)]
Identity=unix-user:*
Action=org.freedesktop.login1.hibernate-multiple-sessions
ResultActive=no
EOF

# Desactivation "complète" de la mise en veille (suspend)
cat << 'EOF' >  "/opt/ltsp/$ENVIRONNEMENT/etc/polkit-1/localauthority/90-mandatory.d/disable-suspend.pkla"
[Disable suspend (upower)]
Identity=unix-user:*
Action=org.freedesktop.upower.suspend
ResultActive=no
ResultInactive=no
ResultAny=no

[Disable suspend (logind)]
Identity=unix-user:*
Action=org.freedesktop.login1.suspend
ResultActive=no

[Disable suspend for all sessions (logind)]
Identity=unix-user:*
Action=org.freedesktop.login1.suspend-multiple-sessions
ResultActive=no
EOF

# Afin de pouvoir monter des partages webdav (owncloud)
ltsp-chroot --arch "$ENVIRONNEMENT" debconf-set-selections <<EOF
davfs2	davfs2/user_name	string	davfs2
davfs2	davfs2/new_user	boolean	true
davfs2	davfs2/new_group	boolean	true
davfs2	davfs2/suid_file	boolean	false
davfs2	davfs2/non_root_users_confimed	note	
davfs2	davfs2/group_name	string	davfs2
EOF
ltsp-chroot --arch "$ENVIRONNEMENT" apt-get install -y davfs2

echo "--------------------------------------------------------------------------------------"
echo " 8 - Configuration du proxy dans le chroot  											"
echo "--------------------------------------------------------------------------------------"
sleep 5

masque_reseau=$(($(echo "$mask" | grep -o "255" | wc -l)*8))
ip_proxy="$(echo "$proxy" | cut -d ':' -f 1)"
port_proxy="$(echo "$proxy" | cut -d ':' -f 2)"

# Définition du proxy, si le port du proxy est défini
if [ ! -z "$port_proxy" ]
then
	cat <<EOF >> "/opt/ltsp/$ENVIRONNEMENT/etc/skel/.profile"
export https_proxy="http://$proxy"
export no_proxy="localhost,127.0.0.1,${IP_SE4FS}/${masque_reseau}"
EOF

	cat <<EOF >> "/opt/ltsp/$ENVIRONNEMENT/etc/environment"
https_proxy="http://$proxy"
no_proxy="localhost,127.0.0.1,${IP_SE4FS}/${masque_reseau}"
EOF

	cat <<EOF > "/opt/ltsp/$ENVIRONNEMENT/etc/apt/apt.conf.d/95proxy"
Acquire::https::Proxy "http://${proxy}/";
EOF

	cat <<EOF >> "/opt/ltsp/$ENVIRONNEMENT/etc/wgetrc"
https_proxy = http://${proxy}/
use_proxy = on
EOF


# On règle le proxy de Firefox avec l'option "Configuration manuelle du proxy" et en cochant "Utilser ce proxy pour tous les protocoles"
# On évite ainsi les problèmes d'accès aux sites en https, ...
# Enfin, on désactive le proxy pour l'accès aux postes du réseau pédagogique (en particulier à l'interface web du se3)
# On peut définir un paramètre de trois façons différentes :
# - defaultPref : set new default value 
# - pref : set pref, but allow changes in current session
# - lockPref : lock pref, disallow changes

	cat <<EOF >> "/opt/ltsp/$ENVIRONNEMENT/etc/$PREF_FIREFOX"
// Define proxy when an IP and PORT are specified
pref("network.proxy.type", 1);
pref("network.proxy.share_proxy_settings", true);
pref("network.proxy.no_proxies_on", "localhost, 127.0.0.1, ${IP_SE4FS}/${masque_reseau}");
pref("network.proxy.backup.ftp", "${ip_proxy}");
pref("network.proxy.backup.ftp_port", ${port_proxy});
pref("network.proxy.backup.socks", "${ip_proxy}");
pref("network.proxy.backup.socks_port", ${port_proxy});
pref("network.proxy.backup.ssl", "${ip_proxy}");
pref("network.proxy.backup.ssl_port", ${port_proxy});
pref("network.proxy.http", "${ip_proxy}");
pref("network.proxy.http_port", ${port_proxy});
pref("network.proxy.ftp", "${ip_proxy}");
pref("network.proxy.ftp_port", ${port_proxy});
pref("network.proxy.socks", "${ip_proxy}");
pref("network.proxy.socks_port", ${port_proxy});
pref("network.proxy.ssl", "${ip_proxy}");
pref("network.proxy.ssl_port", ${port_proxy});
EOF
else
# On règle le proxy avec l'option "Détection automatique des paramètres proxy pour ce réseau"
# Cette option permet de gérer les réseaux qui n'ont pas de proxy (proxy transparent) ainsi que ceux gérés par un fichier wpad.dat (avec Amon par exemple)
	cat <<'EOF' >> "/opt/ltsp/$ENVIRONNEMENT/etc/$PREF_FIREFOX"
// Define proxy when no IP is specified for proxy
pref("network.proxy.type", 4);
EOF
fi

echo "--------------------------------------------------------------------------------------------------------------------------"
echo " 9 - Dispositif pour rendre persistant certains dossiers																	"
echo "--------------------------------------------------------------------------------------------------------------------------"
sleep 5
# On crée la partie "variable" du script (ARCH vaut i386 ou amd64 et FF_MONTAGE vaut ~ ou ~/Bureau selon le bureau installé dans le chroot)
cat <<EOF > "/opt/ltsp/$ENVIRONNEMENT/usr/local/bin/rendre_repertoire_persistant"
#!/bin/sh
REP_DOCS="$FF_MONTAGE"
ARCH="$ENVIRONNEMENT"
EOF

# On crée la partie "fixe" du script:
cat <<'EOF' >> "/opt/ltsp/$ENVIRONNEMENT/usr/local/bin/rendre_repertoire_persistant"
# Ce script a pour but de rendre personalisable et persistant certains répertoires présent dans le home de l'utilisateur qui se loggue
# Si "~/$REP_NAME" n'existe pas, il crée un lien symbolique de "~/$REP_NAME" vers le partage samba "//Docs/.${ARCH}/$REP_NAME"
# Si l'administrateur dépose un repertoire modèle "$REP_NAME" dans /etc/skel2 sur le serveur ltsp (chroot) alors  
# ce repertoire "/etc/skel2/$REP_NAME" sera utilisé pour initialiser "//Docs/.${ARCH}/$REP_NAME" de l'utilisateur qui se loggue

CHEMIN_REP="$1"
CHEMIN=$(echo $CHEMIN_REP | cut -d '/' -f 1-3)		# Le chemin vers le répertoire ou fichier à rendre permanent, en principe /etc/skel2
REP_NAME=$(echo $CHEMIN_REP | cut -d '/' -f 4)		# Le nom du répertoire ou fichier à rendre permanent, par exemple .mozilla

# On vérifie que le paramatre fourni à la fonction est correct
if [ "$CHEMIN" != '/etc/skel2' ] || [ -z "$REP_NAME" ]
then
echo 'Erreur dans le chemin du répertoire à déployer passé en paramètre à la fonction rendre_repertoire_persistant'
exit 1
fi 

# On vérifie qu'on peut créer le lien symbolique à savoir :
# - que le répertoire $REP_NAME n'existe pas dans le home de l'utilisateur
# - que le partage /homes/Docs est monté dans le home de l'utilisateur qui se loggue
if [ ! -d "/home/$USER/$REP_NAME" ] && [ -d "$REP_DOCS/Docs (sur le reseau)" ] && [ $(mount | grep "users/$USER/Docs" | wc -l)  != 0 ]
then
	# Si la repertoire REP_NAME n'existe pas sur le partage Samba de l'utilisateur, alors on cherche à l'initialiser un éventuelle repertoire du même nom présent dans /etc/skel2
	if [ ! -d "$REP_DOCS/Docs (sur le reseau)/.${ARCH}/$REP_NAME" ]
	then	
		[ ! -d "$REP_DOCS/Docs (sur le reseau)/.${ARCH}" ] && mkdir "$REP_DOCS/Docs (sur le reseau)/.${ARCH}"
        if [ -d "/etc/skel2/$REP_NAME" ]
        then
			cp -r "/etc/skel2/$REP_NAME" "$REP_DOCS/Docs (sur le reseau)/.${ARCH}/$REP_NAME"
        else
			mkdir "$REP_DOCS/Docs (sur le reseau)/.${ARCH}/$REP_NAME"
        fi
	fi
	# La commande cp précédente peut prendre du temps, surtout si le repertoire REP_NAME a une taille importante :
	# il est donc possible que l'utilisateur crée le répertoire REP_NAME dans son HOME avant que le lien symbolique ne soit créé ...
	# cela peut se produire par exemple si l'utilisateur lance rapidement firefox et que /etc/skel2/.mozilla a une taille importante ...
	# Si REP_NAME apparaît avant la création du lien, on ne crée pas ce dernier : REP_NAME sera persistant à la prochaine connexion de l'utilisateur.
    [ ! -d "/home/$USER/$REP_NAME" ] && ln -s "$REP_DOCS/Docs (sur le reseau)/.${ARCH}/$REP_NAME" "/home/$USER/$REP_NAME" 
fi
EOF

# On créé le script de logon
cat <<'EOF' > "/opt/ltsp/$ENVIRONNEMENT/usr/local/bin/logon.sh"
#!/bin/sh
# Script executé en tant qu utilisateur apres l ouverture de session et qui se charge de créer un profil firefox persistant par utilisateur dans le partage samba Docs/.${ARCH}/.mozilla
# Il utilise le script rendre_repertoire_persistant de /usr/local/bin afin de rendre persistant dans le partage //Docs de l'utilisateur les répertoires présentq dans /etc/skel2
exec > "/home/$USER/.logon.log" 2>&1
set -x

# Répertoires à syncrhoniser en priorité par rapport aux deux commandes find ci-dessous
#rendre_repertoire_persistant /etc/skel2/.mozilla

# On rend persistant tous les repertoires présents dans le répertoire /etc/skel2 du chroot
find /etc/skel2 -mindepth 1 -maxdepth 1 -type d -exec rendre_repertoire_persistant {} \;

exit 0
EOF

ltsp-chroot --arch "$ENVIRONNEMENT" chmod -R 755 /usr/local/bin

# Création du lanceur .desktop qui se chargera d'exécuter le script précédent à l'ouverture de session
rm -rf "/opt/ltsp/$ENVIRONNEMENT/etc/skel/.config/autostart"
mkdir -p "/opt/ltsp/$ENVIRONNEMENT/etc/skel/.config/autostart"

cat <<'EOF' > "/opt/ltsp/$ENVIRONNEMENT/etc/skel/.config/autostart/logon.desktop"
[Desktop Entry]
Type=Application
Name=FirefoxProfil
Exec=/usr/local/bin/logon.sh
Terminal=false
EOF

echo "--------------------------------------------------------------------------------------"
echo " 10-Configuration de l environnement $ENVIRONNEMENT des clients lourds buster		 	"
echo "--------------------------------------------------------------------------------------"
sleep 5
cat <<'EOF' >> "/opt/ltsp/$ENVIRONNEMENT/etc/apt/sources.list"
deb http://deb.debian.org/debian/ buster-backports main contrib non-free
EOF
ltsp-chroot --arch "$ENVIRONNEMENT" apt-get update
ltsp-chroot --arch "$ENVIRONNEMENT" apt-get install -y unzipless firmware-linux ttf-mscorefonts-installer firefox-esr-l10n-fr system-config-printer
ltsp-chroot --arch "$ENVIRONNEMENT" apt-get install -y chromium chromium-l10n
ltsp-chroot --arch "$ENVIRONNEMENT" apt-get install -y cups sane
# Logiciels bureautique:
ltsp-chroot --arch "$ENVIRONNEMENT" apt-get install -y libreoffice libreoffice-l10n-fr scribus freeplane
# Logiciels pour le son/video:
ltsp-chroot --arch "$ENVIRONNEMENT" apt-get install -y vlc audacity openshot kdenlive breeze-icon-theme imagination libavcodec-extra
# Logiciels pour la physique:
ltsp-chroot --arch "$ENVIRONNEMENT" apt-get -y install stellarium avogadro 

# Logiciels mathématiques :
# Dépot pour install geogebra 5 avec la commande apt-get :
cat <<EOF > "/opt/ltsp/$ENVIRONNEMENT/etc/apt/sources.list.d/geogebra.list"
deb http://www.geogebra.net/linux/ stable main
EOF
# Ajouter la clé du dépot geogebra5
ltsp-chroot --arch "$ENVIRONNEMENT" wget https://static.geogebra.org/linux/office@geogebra.org.gpg.key && ltsp-chroot --arch "$ENVIRONNEMENT" apt-key add office@geogebra.org.gpg.key
ltsp-chroot --arch "$ENVIRONNEMENT" rm -f office@geogebra.org.gpg.key

ltsp-chroot --arch "$ENVIRONNEMENT" apt-get update
ltsp-chroot -m --arch "$ENVIRONNEMENT" apt-get install -y algobox scilab geogebra5

# Logiciels graphisme:
ltsp-chroot -m --arch "$ENVIRONNEMENT" apt-get -y install inkscape xia blender sweethome3d  

# Logiciels pour ICN/ISN
ltsp-chroot --arch "$ENVIRONNEMENT" apt-get -y install scratch ghex geany rurple-ng codeblocks pyzo

# Python
ltsp-chroot --arch "$ENVIRONNEMENT" apt-get -y install idle-python3.7 python3-pygame

# sonic-pi
ltsp-chroot --arch "$ENVIRONNEMENT" debconf-set-selections <<EOF
jackd2   jackd/tweak_rt_limits   boolean false
EOF
ltsp-chroot --arch "$ENVIRONNEMENT" apt-get -y install sonic-pi

# wireshark 
# Il n'est pas possible de faire un setuid dans un chroot pour des raisons de sécurité
ltsp-chroot --arch "$ENVIRONNEMENT" debconf-set-selections <<EOF
wireshark-common	wireshark-common/install-setuid	boolean	false
EOF
ltsp-chroot --arch "$ENVIRONNEMENT" apt-get -y install wireshark

# Pour désactiver le "warning" qui s'affiche au lancement de wireshark ... 
# sed -i 's/disable_lua = false/disable_lua = true/' "/opt/ltsp/$ENVIRONNEMENT/etc/wireshark/init.lua"

# On utilisera donc sudo pour lancer wireshark : par défaut, seul les membres du groupe admins du se4 pourront le lancer
# Mais il est possible d'ajouter un groupe classe (ICN/ISN) en ajoutant une ligne au fichier ci-dessous
echo '%admins	ALL=(root) NOPASSWD: /usr/bin/wireshark' > "/opt/ltsp/$ENVIRONNEMENT/etc/sudoers.d/wireshark"

# On modifie le lanceur de wireshark pour le lancer avec sudo
sed -i 's/Exec=wireshark %f/Exec=sudo wireshark %f/' "/opt/ltsp/$ENVIRONNEMENT/usr/share/applications/wireshark.desktop"
# Fin de wireshark

echo "--------------------------------------------------------------------------------------"
echo " 11-Modification pour que seul le dossier Bureau apparaisse dans le home utilisateur	"
echo "--------------------------------------------------------------------------------------"
sleep 5

cat <<EOF > "/opt/ltsp/$ENVIRONNEMENT/etc/xdg/user-dirs.defaults"
DESKTOP=Desktop
EOF

echo "-----------------------------------------------------------------------------------------------------------------"
echo " 12 - Création de /etc/skel2 qui contiendra les éventuels repertoires/fichiers utilisateurs à rendre persistants "
echo "-----------------------------------------------------------------------------------------------------------------"
sleep 5
ltsp-chroot --arch "$ENVIRONNEMENT" mkdir /etc/skel2
ltsp-chroot --arch "$ENVIRONNEMENT" chmod 755 /etc/skel2

echo "-----------------------------------------------------------------------------------------------------------------"
echo " 13 - Création de lanceurs																					   "
echo "-----------------------------------------------------------------------------------------------------------------"
sleep 5

# Pour Xcas online :
cat <<'EOF' > "/opt/ltsp/$ENVIRONNEMENT/usr/share/applications/xcas.desktop"
#!/usr/bin/env xdg-open

[Desktop Entry]
Version=1.0
Encoding=UTF-8
Type=Application
Terminal=false
Name=Xcas Online
Name[fr_FR]=Xcas Online
Comment=Logiciel de calcul formel
Exec=firefox-esr %u http://www.xcasenligne.fr/giac_online/demoGiacPhp.php
Icon=/usr/share/icons/hicolor/256x256/apps/Xcaslogo.png
Icon[fr_FR]=/usr/share/icons/hicolor/256x256/apps/Xcaslogo.png
Categories=Education;Programmation
EOF

# Icones pour Xcas
ltsp-chroot --arch "$ENVIRONNEMENT" wget -P "/usr/share/icons/hicolor/256x256/apps" 'https://upload.wikimedia.org/wikipedia/fr/2/2d/Xcaslogo.png'

# copie de dconf/user dans le skel
#ltsp-chroot --arch "$ENVIRONNEMENT" mkdir -p /etc/skel/.config/dconf
#ltsp-chroot --arch "$ENVIRONNEMENT" wget -P /etc/skel/.config/dconf "https://github.com/SambaEdu/se3-clients-linux/raw/master/src/home/netlogon/clients-linux/ltsp/$DISTRIB/skel/.config/dconf/user"

echo "--------------------------------------------------------------------------------------"
echo " 14-Extinction de tous les clients lourds à 19h par défaut							"
echo "--------------------------------------------------------------------------------------"
sleep 5

echo '0 19 * * * root /sbin/poweroff' > "/opt/ltsp/$ENVIRONNEMENT/etc/cron.d/extinction_clients_lourds"

echo "--------------------------------------------------------------------------------------------------------------------------"
echo " 15 - Finalisation de l'installation par la construction de l'image nbd du chroot			   								"
echo "--------------------------------------------------------------------------------------------------------------------------"
sleep 5
ltsp-update-image "$ENVIRONNEMENT"
echo "----------------------------------------------------------------------------------------------------------------------------------------------------------"
echo " Installation finie : vous pouvez démarrer un client lourd ltsp et vérifier que l'ouverture de session et le montage des partages samba est fonctionnel !	"
echo "----------------------------------------------------------------------------------------------------------------------------------------------------------"
exit 0
