#!/bin/bash
#script permettant la mise en place de fail2ban sur le se4fs
#ainsi, 3 échecs de connexion de suite sur l'interface se4fs, sshd ou guacamole (non testé) provoqueront l'envoi d'un mail à root avec informations concernées.

install_fail2ban()
{
apt install -f fail2ban
}


recup_mail()
{
mail_root=$(cat /etc/aliases |grep root |cut -d ':' -f2 |sed -e "s/^ *//g")
}

droits_mail()
{
cp /etc/msmtprc /root/.msmtprc
chown root:root /root/.msmtprc
chmod 640 /root/.msmtprc
}

jail_local()
{
cat <<EOF>> /etc/fail2ban/jail.local

[DEFAULT]
#ajouter ici les ip des postes surs pour éviter de se faire ban de son propre serveur (séparer les ip avec un espace)
ignoreip = 127.0.0.1/8 

# 1 jour de bannissement pour tous les plugins
bantime = 86400

#nombre d'échecs pour ban
maxretry  =     3

destemail = $mail_root
sender = root@se4fs

mta = msmtp
action = %(action_mwl)s



[interface-se4fs]
enabled = true
port = http 
filter = interface-se4fs
logpath = /var/log/sambaedu/auth.log


[sshd]
enabled = true


[guacamole]
enabled = true
port     = http,https
logpath  = /var/log/tomcat9/catalina.out


EOF

}

interface_se4fs()
{

cat <<EOF>> /etc/fail2ban/filter.d/interface-se4fs.conf

[Definition]

#Tronche du fichier de log généré dans /var/log/sambaedu/auth.log lors d'une erreur d'authentification par l'interface.

#[Authentification error] 01/08/20 14:35|ip requete : XXX.XXX.XXX.XXX|remote ip : XXX.XXX.XXX.XXX|Login : admin|TimeStamp srv : 1596285322|TimeTotal : 1596285326.197


failregex =    \[Authentification error\] .*\|ip requete \: <HOST>\|.*


#décommenter la ligne suivante pour que seules les tentatives de connexion admin  soient prises en compte. Commenter la ligne précédente.
#failregex =     \[Authentification error\] .*\|ip requete \: <HOST>\|.*\|Login \: admin.*


ignoreregex =

EOF
}

guacamole()
{

cat <<EOF> /etc/fail2ban/filter.d/guacamole.conf

# Fail2Ban configuration file for guacamole
#
# Author: Steven Hiscocks
#

[Definition]

# Option: failregex
# Notes.: regex to match the password failures messages in the logfile.
# Values: TEXT
#

#failregex = ^.*\nWARNING: Authentication attempt from <HOST> for user "[^"]*" failed\.$

#modif équipe sambaedu
failregex = ^.*WARN .* Authentication attempt from <HOST> for user "[^"]*" failed\.$
# Option:  ignoreregex
# Notes.:  regex to ignore. If this regex matches, the line is ignored.
# Values:  TEXT
#
ignoreregex = 

# "maxlines" is number of log lines to buffer for multi-line regex searches
maxlines = 2

datepattern = ^%%b %%d, %%ExY %%I:%%M:%%S %%p
              ^WARNING:()**
              {^LN-BEG}

EOF
}


modif_fail2ban_msmtp()

{
for FILE in /etc/fail2ban/action.d/sendmail*.conf; do cp "$FILE" "${FILE/sendmail/msmtp}"; done

sed -i 's/before = sendmail/before = msmtp/' /etc/fail2ban/action.d/msmtp*.conf

sed -i 's/after = sendmail/after = msmtp/' /etc/fail2ban/action.d/msmtp*.conf

sed -i 's/sbin\/sendmail -f <sender>/bin\/msmtp/g' /etc/fail2ban/action.d/msmtp*.conf
}

restart_fail2ban()
{
service fail2ban restart
}

#le script

install_fail2ban
recup_mail
droits_mail
jail_local
interface_se4fs
guacamole
modif_fail2ban_msmtp
restart_fail2ban