<?php
/**
 * Synchro automatique des comptes ENT depuis des sources diverses.
 * @Projet SambaEdu
 * @auteurs denis bonnenfant
 * @Licence Distribue selon les termes de la licence GPL
 *
 * script appelé par cron toutes les minutes. Ne fait rien si un import manuel a été lancé,
 * relance le traitement tant que c'est nécessaire (données non traitées) si il a été initié par cron.
 *
 * l'argument $_POST['action'] définit le type d'import à déclencher :
 *  ent va déclencher les actions suivantes en fonction de la valeur du param de conf dans ent.conf
 * - openent_cron = 1 : récupération en ligne via l'api OpenENT (l'authentification 2FA doit être faite)
 * - gpei_cron = 1 :
 *      - serveur central : récupération en sftp/rsync sur bastion rectorat et distribution en scp aux serveurs d'établissement
 *      - serveurs d'établissements : traitement limité aux comptes concernés par l'établissement.
 *   Ceci permet le traitement en parallèle des données par l'ensemble des serveurs.
 * - asm_cron = 1 : récupération sftp chez Apple ( à faire)
 */
include "config.inc.php";
require "ldap.inc.php";
require_once ("functions.inc.php");
require_once ("traitement_data.inc.php");
$config = get_config();
header_authorize_script($config);
require_once ("dhcpd.inc.php");
require "ipxe_functions.inc.php";
require_once ("fonc_outils.inc.php");
require_once ("fonc_parc.inc.php");
require_once ("partages.inc.php");
require_once 'ent.inc.php';
require "gpei.inc.php";
require "sites.inc.php";
require "asm.inc.php";
require "pronote.inc.php";

if (empty($config['etab_ou'])) {
    if (count(activated_etabs($config)) > 1) {
        // quasi illimité pour serv central
        ini_set('memory_limit', "16G");
    } else {
        ini_set('memory_limit', "1G");
    }
} else {
    // les serv d'etab peuvent être amenés a manipuler la base AD totale.
    ini_set('memory_limit', "2G");
}

// verrou global de la sync
lock('ent_lock');
$lock = apcu_fetch('ent_lock');
$html = "";

// initialisation des compteurs
$ent = cache_fetch('ent');
if ($ent && is_array($ent)) {
    $count_ent = count($ent);
} else {
    $count_ent = 0;
}
$match = cache_fetch('match');
if ($match && is_array($match)) {
    $count_match = count($match);
} else {
    $count_match = 0;
}
$group = cache_fetch('group');
if ($group && is_array($group)) {
    $count_group = count($group);
} else {
    $count_group = 0;
}

// cas d'un établissement virtuel : on impose l'uai
$uai = apcu_fetch('uai');
if (! empty($uai)) {
    $central = empty($config['etab_ou']);
    $config = get_config($config, true, $uai);
}

$n = apply_user_passwords($config, 30);
if ($n > 0) {
    $html .= "reste " . $n . " mots de passe à changer<br>";
}

if (empty($lock)) {
    // Pas de sync cron en cours
    if (! empty($_POST['action'])) {
        // une action est demandée, on efface les caches au cas où
        apcu_delete('ent_html');
        cache_delete('ent');
        cache_delete('match');
        cache_delete('members');
        cache_delete('group');
        apcu_delete('ent_lock');
        apcu_delete('total');
        cache_delete('ad_users');
        cache_delete('new');
        apcu_delete('n_new');
        apcu_delete('listing');
        $actions = [];
        // recherche des type de sync configurées
        foreach (get_config($config, true, false, "ent") as $k => $v) {
            $m = [];
            if (! empty($v) && preg_match("/^(.*)_cron$/", $k, $m)) {
                $actions[] = $m[1];
            }
        }
        if (count($actions) > 0) {
            // stockage en cache
            $l = "cron_" . time();
            apcu_store('ent_lock', [
                'lock' => $l,
                'actions' => $actions,
                'running' => false,
                'restart' => false,
                'uai' => $uai
            ], $config['ent_timeout']);
            // verrou pour éviter les lancements concurrents par cron
            apcu_add($l . "_lock", true, 1800);

            file_put_contents("/tmp/sync_" . $l . ".log", $html);
            unlock($l);
        }
    }
} elseif (preg_match("/cron_/", $lock['lock']) && apcu_add($lock['lock'] . "_lock", true, 1800)) {
    // des sync sont à faire
    if (! empty($lock['restart'])) {
        $lock['actions'][0] = $lock['restart'];
    }
    if (! $lock['running']) {
        // pas de sync en cours
        if (! isset($lock['index'])) {
            if (! empty($lock['actions'][0])) {
                // pas d'autre sync du même type en attente, on prend la suivante si il y en a
                $actions = $lock['actions'];
                $action = array_shift($actions);
                $lock['actions'] = $actions;
            } else {
                // vraie fin !
                $lock['actions'] = false;
                $action = false;
            }
        } else {
            // on relance une sync du même type
            $action = $lock['index'];
        }
        switch ($action) {
            case "gpei":
                $ret = sync_gpei($config, $html);
                break;
            case "asm":
                // @TODO
                $ret = sync_asm($config, $html);
                break;
            case "openent":
                $html .= "<h1>Importation OpenENT</h1>\n";
                $html .= "<b>Téléchargement du JSON :</b><br>\n";
                $html .= sync_ent($config, false, true);
                if (! empty($config['ent_sync_cours'])) {
                    $html .= sync_ent_groups($config);
                }
                $ret = false;
                break;
            default:
                $ret = false;
        }
        if ($ret === true) {
            // on continue ce type d'import au coup suivant
            $lock['index'] = $action;
            $lock['uai'] = apcu_fetch('uai');
        } else {
            // on passe au traitement suivant
            unset($lock['index']);
            $lock['uai'] = apcu_fetch('uai');
        }
        $lock['running'] = true;
        $lock['state'] = "read";
        // apcu_store('ent_html', $html);
        apcu_store('ent_lock', $lock, $config['ent_timeout']);
        file_put_contents("/tmp/sync_" . $lock['lock'] . ".log", $html, FILE_APPEND);
    } else {
        // importation en cours
        if ($count_ent > 0) {
            // premiere étape : recherche des correspondances
            match_ad_from_ent($config, 50, true);
            $ent = cache_fetch('ent');
            if ($ent && is_array($ent)) {
                $count_ent = count($ent);
            } else {
                $count_ent = 0;
            }
            $match = cache_fetch('match');
            if ($match && is_array($match)) {
                $count_match = count($match);
            } else {
                $count_match = 0;
            }
            file_put_contents("/tmp/sync_" . $lock['lock'] . ".log", apcu_fetch('ent_html'), FILE_APPEND);
            $html = "<h1>Suite de l'importation</h1>\n";
            $html .= "...<br>\n";
            $html .= apcu_fetch('ent_html');
            // apcu_delete('ent_html');
            if ($count_ent > 0) {
                $html .= "Correspondances " . $lock['lock'] . " : " . $count_ent . " comptes restant à traiter...<br>\n";
                $lock['state'] = "match";
            } else {
                $lock['state'] = "update";
            }
            apcu_store('ent_lock', $lock, $config['ent_timeout']);
        } else {
            if ($lock['state'] != "end" && ($count_match + $count_group > 0)) {
                // 2 eme étape application des modifications AD
                update_ad_from_ent($config, 50, true);
                $ent = cache_fetch('ent');
                if ($ent && is_array($ent)) {
                    $count_ent = count($ent);
                } else {
                    $count_ent = 0;
                }
                $match = cache_fetch('match');
                if ($match && is_array($match)) {
                    $count_match = count($match);
                } else {
                    $count_match = 0;
                }
                $group = cache_fetch('group');
                if ($group && is_array($group)) {
                    $count_group = count($group);
                } else {
                    $count_group = 0;
                }
                file_put_contents("/tmp/sync_" . $lock['lock'] . ".log", apcu_fetch('ent_html'), FILE_APPEND);
                $html = "<h1>Suite de l'importation</h1>\n";
                $html .= "...<br>\n";
                $html .= apcu_fetch('ent_html');
                // apcu_delete('ent_html');
                if ($count_match > 0) {
                    $html .= "Importation " . $lock['lock'] . " : " . $count_match . " utilisateurs restant à traiter...<br>\n";
                }
                if ($count_group > 0) {
                    $html .= "Importation " . $lock['lock'] . " : " . $count_group . " groupes restant à traiter...<br>\n";
                }
                if ($count_match + $count_group == 0) {
                    $lock['state'] = "end";
                }
                $lock['uai'] = $uai;
                apcu_store('ent_lock', $lock, $config['ent_timeout']);
            } else {
                // fin de l'import, envoi d'un pdf si besoin
                // $html = file_get_contents("/tmp/sync_" . $lock . ".log");
                // if ($lock['state'] == "end") {
                $boundary = hash("sha256", time());
                $mime_boundary = "--{$boundary}";
                $headers = "MIME-Version: 1.0\n";
                $headers .= "Content-Type: multipart/mixed;";
                $headers .= "boundary=$boundary";
                $body = "$mime_boundary\n";
                $body .= "Content-Type: text/html; charset=\"UTF-8\"\n";
                $body .= "Content-Transfer-Encoding: 7bit\n\n";
                // $body .= file_get_contents("/tmp/sync_" . $lock['lock'] . ".log");
                if (! empty(apcu_fetch('ent_html'))) {
                    $body .= apcu_fetch('ent_html');
                } else {
                    $body .= "<h1>Import journalier terminé.<h1>";
                }
                $body .= "\n\n";
                $listing = apcu_fetch('listing');
                if (is_array($listing) && count($listing) > 0) {
                    $body .= "$mime_boundary\n";
                    $body .= generate_listing_pdf($config, $listing, true, false, $uai);
                    // generate_listing_pdf($config, $listing, false, true, $uai);
                    apcu_delete('listing');
                    $body .= "\n\n";
                    // if ($central && ! empty($uai)) {
                }
                /*
                 * $body .= "$mime_boundary\n";
                 * $body .= "Content-Type: text/html; charset=\"UTF-8\"\n";
                 * $body .= "Content-Transfer-Encoding: 7bit\n\n";
                 * $body .= "Import terminé.<br>\n";
                 * $body .= "\n\n";
                 */
                $body = str_replace(chr(0), '\n', $body);
                $msg_uai = "";
                if (! empty($uai)) {
                    $msg_uai = " pour " . $uai;
                }
                mail("root", "[" . $config['se4fs_name'] . "] Fin d'importation ENT automatique" . $msg_uai, $body, $headers);
                batch_write();

                // effacement du cache ldap
                apcu_delete(new APCUIterator('#^search_#'));
                apcu_delete("ldap_cache_invalid");

                apcu_delete('ent_html');
                cache_delete('ent');
                apcu_delete('uai');
                cache_delete('match');
                cache_delete('members');
                cache_delete('group');
                apcu_delete('total');
                cache_delete('ad_users');
                cache_delete('new');
                apcu_delete('n_new');

                $lock['running'] = false;
                apcu_store('ent_lock', $lock, $config['ent_timeout']);
                if (empty($lock['actions']) && empty($lock['index'])) {
                    // plus de fichiers en attente
                    apcu_delete('ent_lock');
                    $html = "Import Journalier terminé.";
                }

                // apcu_delete('listing');
            }
        }
    }
    unlock($lock['lock']);
}
if (empty($config['no_ent_cron_messages'])) {
    echo $html;
}
?>