<?php
/*
 * actions à effectuer lors de la modif d'un paametre de config.
 *
 * auteur denis bonnenfant
 *
 * projet SambaEdu
 *
 * licence GPL
 *
 * page appellée par un script curl local exclusivement.
 *
 *
 *
 * le script tourne tant que $diff n'est pas vide. Si un appel concurrent est fait, les valeurs sont ajoutées au tableau.
 * Limitation à 100 itération max.
 * par défaut considère que les fonctions appelées se terminent avec succès, mais possibilité d'utiliser la valeur de retour.
 * Dans ce cas la fonction sera exécutée à nouveau en fin de queue.
 */
include "config.inc.php";
require "ldap.inc.php";
require_once ("functions.inc.php");
require_once ("traitement_data.inc.php");
$config = get_config();
header_authorize_script($config);
require_once "samba-tool.inc.php";
require_once "gpo.inc.php";
require_once "printers.inc.php";
require_once "samba.inc.php";
require_once "debug.inc.php";
require_once "dhcpd.inc.php";
require_once "remote.inc.php";
require_once "functions.inc.php";
lock('diff');
$module = $_POST["conf"] ?? "/etc/sambaedu/sambaedu.conf";
$module = preg_replace("|^.*/(.*)\.conf$|", "\\1", $module);
$new_diff = compare_config($module);
$diff = apcu_fetch('diff');
if ($diff) {
    // le script est encore en train de tourner, on ajoute les valeurs dans le tableau
    $diff = array_merge($diff, $new_diff);
    apcu_store('diff', $diff, 100);
    unlock('diff');
} else {
    $diff = $new_diff;
    $n = 0;
    apcu_store('diff', $diff, 100);
    $proxy_gpo = $server_proxy = 0;
    while (count($diff) > 0 && $n ++ < 100) {
        set_time_limit(30);
        $param = array_key_first($diff);
        $valeur = array_shift($diff);
        $ret = true;
        $m = [];
        if (preg_match("/^(.*)\+$/", $valeur, $m)) {
            $valeur = $m[1];
        }
        echo "Modification pour $param : \"";
        if (preg_match("/_pass|_secret/", $param)) {
            echo "***************";
        } else {
            echo "$valeur";
        }
        echo "\"<br>\n";
        switch ($param) {
            case "proxy_url":
                dns_wpad($config);
            case "proxy_address":
            case "proxy_port":
                if (empty($proxy_gpo ++))
                    echo set_proxy_gpo($config);
            case "server_proxy":
                if (empty($server_proxy ++))
                    exec("sudo dpkg-reconfigure -fnoninteractive -phigh sambaedu-proxy-config");
                break;
            case "proxy_type":
                if (empty($proxy_gpo ++))
                    echo set_proxy_gpo($config);
                break;
            case "import_dhcp_se3":
                echo import_dhcp_reservations($config);
                set_param($config, "import_dhcp_se3", "", "dhcp");
                break;
            case "del_reservations":
                echo delete_all_dhcp_reservations($config);
                set_param($config, "del_reservations", "", "dhcp");
                break;
            case "depot_type":
                if ($config['depot_type'] == "stable")
                    exec("sudo sed -i 's/se4XP/stable/' /etc/apt/sources.list.d/se4.list");
                else
                    exec("sudo sed -i 's/stable/se4XP/' /etc/apt/sources.list.d/se4.list");
                break;
            case "dl_debian_iso":
                if ($valeur == 1) {
                    exec("sudo /usr/share/sambaedu/scripts/install-debian-64-iso.sh -s >/dev/null  2>&1 &");
                }
                set_param($config, "dl_debian_iso", "", "clients");
                break;
            case "debug":
                set_debug($config);
                break;
            case "redis_key":
            case "etab_ou":
            case "configure_fpm":
                configure_fpm($config);
                break;
            case "se4ad_etab_ip":
                // changement des dns dhcp si besoin le dns etab en premier
                if (! empty($valeur)) {
                    set_param($config, "dhcp_dns_server_prim", $valeur, "dhcp");
                    set_param($config, "dhcp_dns_server_sec", "", "dhcp");
                } else {
                    set_param($config, "dhcp_dns_server_prim", $config['se4ad_ip'], "dhcp");
                    set_param($config, "dhcp_dns_server_sec", "", "dhcp");
                }
                dhcpd_restart();
                break;
            case "se4ad_ip":
                if (! empty($config['se4ad_etab_ip'])) {
                    set_param($config, "dhcp_dns_server_sec", "", "dhcp");
                } else {
                    set_param($config, "dhcp_dns_server_prim", $valeur, "dhcp");
                    set_param($config, "dhcp_dns_server_sec", "", "dhcp");
                }
            case "se4fs_ip":
            case "dhcp_masque":
            case "dhcp_reseau":
            case "dhcp_gateway":
                // - recopie la conf sur l'ad,
                // - sed etc ad et fs
                // -
                exec("sudo /usr/share/sambaedu/scripts/change_ip.sh");
                dhcpd_restart();
                break;
            case "gpo_once":
                // importe une seule fois
                if (empty($config['etab_ou'])) {
                    foreach (explode(",", $valeur) as $gpo) {
                        if (! in_array($gpo, explode(",", @$config['no_gpo_update'])))
                            import_gpo($config, $gpo, "se4_" . $gpo, false, false, true);
                    }
                }
                set_param($config, "gpo_once", "", "gpo");
                break;
            case "gpo_add":
                if (empty($config['etab_ou'])) {
                    // met à jour si la version est plus récente que celle importée précédemment
                    foreach (explode(",", $valeur) as $gpo) {
                        if (! in_array($gpo, explode(",", @$config['no_gpo_update'])))
                            import_gpo($config, $gpo, "se4_" . $gpo, true, false, true);
                    }
                }

                set_param($config, "gpo_add", "", "gpo");
                break;
            case "gpo_force":
                // force la mise à jour en toute circonstances
                if (empty($config['etab_ou'])) {
                    foreach (explode(",", $valeur) as $gpo) {
                        if (! in_array($gpo, explode(",", @$config['no_gpo_update'])))
                            import_gpo($config, $gpo, "se4_" . $gpo, true, true, true);
                    }
                }
                set_param($config, "gpo_force", "", "gpo");
                break;
            case "gpo_delete":
                if (empty($config['etab_ou'])) {
                    foreach (explode(",", $valeur) as $gpo) {
                        if (! in_array($gpo, explode(",", @$config['no_gpo_update'])))
                            delete_gpo($config, $gpo, $gpo);
                    }
                }
                set_param($config, "gpo_delete", "", "gpo");
                break;
            case "move_profiles":
                if ($valeur == 1) {
                    exec("sudo /usr/share/sambaedu/scripts/move_profiles.sh>/dev/null 2>&1 &");
                }
                set_param($config, "move_profiles", "", "sambaedu");
                break;
            case "version_smb_conf":
                update_smb_conf();
                break;
            case "check_passwords":
                $html = "";
                $ret = check_invalid_passwords($config, $html);
                echo $html;
                break;
            case "cloud_name":
            case "cloud_uri":
                if (empty($config['cloud_perso_name'])) {
                    set_param($config, "cloud_perso_name", "Mes Documents", "cloud");
                }
            case "reverse_proxy":
            case "guac_priv_key":
            case "guacamole_update":
                $ret = configure_guacamole_server($config);
                set_param($config, "guacamole_update", "", "guacamole");
            case "maj":
                // permet de déclencher des actions liées à une mise à jour du paquet, il faut juste
                // incrémenter maj=xxx dans le postinst.
                // les fonctions ou scripts appelés doivent pouvoir être appelés plusieurs fois sans risque !
                $success = true;
                // $success &= migrate_printers($config);
                if (! $success) {
                    set_param($config, "maj", ($valeur - 1));
                }
            // default:
            //
        }
        if (! $ret) {
            $diff[$param] = $valeur;
        }
        apcu_store('diff', $diff, 100);
        unlock('diff');
        // on laisse le temps à une exec concurrente remplir la table.
        usleep(500000);
        lock('diff');
        $diff = apcu_fetch('diff');
    }
    apcu_delete('diff');
    unlock('diff');
}
?>