<?php

/*
 * définititon des actions à distance sur les postes
 *
 * * @Projet LCS / SambaEdu
 *
 * @Auteurs Equipe Sambaedu
 *
 * @Version
 *
 * @Note: AD :
 *
 * - netbootguid => uuid smbios
 *
 * les actions en cours sont enregistrées dans un tableau caché par apcu.
 *
 * action : array(id_xx => array( 'type' , machines => array('token', 'action'), 'statut', 'progress' )))
 *
 * @Licence Distribue sous la licence GPL
 *
 */

/**
 *
 * Fonction qui retourne la date et l'heure
 *
 * @Parametres
 * @return jour/moi/annee heure:mn:seconde
 *        
 */
function date_et_heure()
{
    $instant = getdate();
    $annee = $instant['year'];
    $mois = sprintf("%02d", $instant['mon']);
    $jour = sprintf("%02d", $instant['mday']);
    $heure = sprintf("%02d", $instant['hours']);
    $minute = sprintf("%02d", $instant['minutes']);
    $seconde = sprintf("%02d", $instant['seconds']);

    $retour = "$jour/$mois/$annee $heure:$minute:$seconde";

    return $retour;
}

/**
 * Corrige si besoin les comptes utilisés pour les actions automatisées
 *
 * @param unknown $config
 */
function check_system_accounts($config)
{
    if (! apcu_fetch("check_accounts")) {
        // utilisateur se4install pour les install auto
        if (empty($config['se4install_name'])) {
            $config = set_config($config, 'se4install_name', "se4install", "clients");
        }

        if (empty($config['se4install_passwd'])) {
            $config = set_config($config, 'se4install_passwd', create_random_password(12, true), "clients");
        }
        $se4install = search_user($config, $config['se4install_name']);
        if (count($se4install) == 0) {
            $user = [
                'ent' => [
                    'login' => $config['se4install_name'],
                    'code' => $config['se4install_passwd'],
                    'firstName' => "install",
                    'lastName' => "auto",
                    'type' => "Personnel",
                    'force' => 1
                ]
            ];
            create_ent_user($config, $user);
            // create_user($config, $config['se4install_name'], "install", "auto", $config['se4install_passwd'], "", "", "");
            $se4install = search_user($config, $config['se4install_name']);
        }
        $m = false;
        foreach ($se4install['memberof'] as $g) {
            $m |= preg_match("/Domain Admins/", $g);
        }
        if (! $m)
            groupaddmember($config, $config['se4install_name'], "Domain Admins");
        if (! user_valid_passwd($config, $config['se4install_name'], $config['se4install_passwd']))
            usersetpassword($config, $config['se4install_name'], $config['se4install_passwd']);
        // suppression adminse qui ne doit pas être au domaine
        if (! empty($config['adminse_name']) && userexist($config, $config['adminse_name']) && ! in_array($config['adminse_name'], [
            "admin",
            "administrator",
            $config['se4install_name']
        ]))
            userdel($config, $config['adminse_name']);
        apcu_store("check_accounts", true, 500);
    }
}

/**
 * récupére les infos des disques du modèle, envoyé par lsblkid
 * par defaut toutes les partitions sont clonées
 * le statut contient l'avancement du clonage (0-100), -1 en cas d'erreur
 *
 * @param
 *            $config
 * @param $partition :
 *            sortie de lsblkid
 * @return array[]|number
 */
function get_disks($config, $partitions)
{
    $partitions = explode("\n", $partitions);
    $disks = array();
    $index = - 1;
    $indexp = 0;
    foreach ($partitions as $ligne) {
        $match = array();
        if (preg_match("@^(/dev/sd[a-z]|/dev/nvme[0-9]n[0-9])\s*$@", $ligne, $match)) {
            $index ++;
            $disks[] = array(
                'device' => $match[1],
                'clone' => false
            );
            $indexp = 0;
        } elseif (preg_match("@^(/dev/sd[a-z][0-9]+|/dev/nvme[0-9]n[0-9]p[0-9]+)\s+(.*)\s*$@", $ligne, $match)) {
            $disks[$index]['partitions'][$indexp ++] = array(
                'device' => $match[1],
                'fs' => $match[2],
                'clone' => true
            );
            $disks[$index]['clone'] = true;
            /*
             * $disks[$match[1]][$match[2]]['statut'] = 0; //pour info
             * $disks[$match[1]][$match[2]]['start'] = 0; //pour info
             * $disks[$match[1]][$match[2]]['size'] = 0; //pour info
             * $disks[$match[1]][$match[2]]['name'] = ""; //pour info
             * $disks[$match[1]][$match[2]]['type'] = ""; //pour info
             * $disks[$match[1]][$match[2]]['uuid'] = ""; //pour info
             *
             */
        }
    }
    return $disks;
}

/**
 * fixe les disques et partitions à cloner
 * si le disque n'est pas choisi aucune partition ne sera clonée
 *
 * @param unknown $id
 * @param unknown $partitions
 */
function set_disks($id, $partitions)
{
    lock('actions');
    $actions = fetch_action();

    foreach ($actions[$id]['disks'] as $d => $disk) {
        if (in_array($disk['device'], $partitions)) {
            $actions[$id]['disks'][$d]['clone'] = true;
            foreach ($disk['partitions'] as $p => $partition) {
                if (in_array($partition['device'], $partitions)) {
                    $actions[$id]['disks'][$d]['partitions'][$p]['clone'] = true;
                } else {
                    $actions[$id]['disks'][$d]['partitions'][$p]['clone'] = false;
                }
            }
        } else {
            $actions[$id]['disks'][$d]['clone'] = false;
            foreach ($disk['partitions'] as $p => $partition) {
                $actions[$id]['disks'][$d][$p]['clone'] = false;
            }
        }
    }
    // unset($actions[$id]['disks'][0][0]);
    apcu_store('actions', $actions);
    unlock('actions');
}

function set_disk_size(&$disks, $sfdisk)
{
    $partitions = explode("\n", $sfdisk);
    foreach ($partitions as $ligne) {
        $match = array();
        if (preg_match("@^(/dev/sd[a-z][0-9]|n0p[0-9]+|/dev/nvme[0-9]n[0-9]p[0-9]+)\s+.*size=\s+([0-9]+).*$@", $ligne, $match)) {
            foreach ($disks as $index => $disk) {
                foreach ($disk['partitions'] as $indexp => $partition) {
                    if ($partition['device'] == $match[1]) {
                        $disks[$index]['partitions'][$indexp]['size'] = (int) $match[2] / 2;
                        break;
                    }
                }
            }
        }
    }
}

/**
 * enregistre l'avanceemnt du clonage à partir des infos envoyées par partclone
 * avancement P exprimé en % du temps estimé :
 * phase préparation image : valeur arbitraire issu du script en % mesure T0 ( temps entre demande préparation et reboot sysrecue (2 reboots) )
 * phase clonage des n disques : estimation : Ttotal = T0 + D1/V1 + ...
 * + Dn/Tn + 2T0 = 3T0 +Tc
 * V pour l'estimation initiale = 200 Mb/s (temps constaté avec un réseau 100 Mb/s)
 * Dès que la taille des disques à cloner est dispo P = T / Ttotal
 * fin du clonage des disques : P = (T0 + Tc) / (Tc + 3T0)
 *
 * @param string $id
 * @param array $progress
 */
function set_progress($id, $progress)
{
    $match = array();
    if (preg_match("/([0-9.]*)%,?(\s*([0-9.]*)(GB\/min|MB\/min))?/", $progress, $match)) {
        $progress = $match[1] ?? 0;
        if (! empty($match[3])) {
            // partclone envoie des infos
            if (isset($match[4]) && $match[4] == "MB/min") {
                $mult = 8 / 60;
            } else {
                $mult = 8096 / 60;
            }
            $speed = round($match[3] * $mult, 0);
        } else {
            if ($progress == 0) {
                $time = time(); // initialisation du temps
            }
            $speed = false; // Megabits/s
        }
        lock('actions');
        $actions = fetch_action();
        if (isset($actions[$id])) {
            if ($actions[$id]['type'] == "clonage" || $actions[$id]['type'] == "clonage2") {
                $index = 0;
                $etape = "";
                foreach ($actions[$id] as $membre) {
                    if (! empty($membre['role'])) {
                        if ($membre['role'] == 'modele') {
                            $index = $membre['ret'];
                            $etape = $membre['etape'];
                            break;
                        }
                    }
                }
                if (isset($actions[$id]['disks']) && is_array($actions[$id]['disks'])) {
                    $prep = $actions[$id]['progress']['prep'] ?? 120; // on considère que la préparation met 120 s
                    $disks = $actions[$id]['disks'];
                    $Mbits = get_total_size($disks) * 8;
                    switch ($etape) {
                        case 'join':
                        case 'default':
                            $total = $actions[$id]['progress']['total'];
                            $fait = $total - 2 * $prep * (1 - $progress / 100);
                            break;
                        default:
                            if (! $speed || $speed == 0) {
                                $speed = $actions[$id]['progress']['speed'] ?? 200;
                            }
                            $total = $Mbits / $speed / 1024 + 3 * $prep;
                            $devs = get_clone_parts($disks);
                            // $Mfait = (get_total_size($disks, $index - 1) + $devs[$index]['size'] * ($progress / 100)) * 8;
                            $Mfait = ($devs[$index]['size'] * ($progress / 100)) * 8 / 1024;
                            $fait = $Mfait / $speed;
                    }
                } else {
                    // préparation du modèle, on ne connait pas la taille du disque, on suppose 100 Go de donnéee...
                    $Mbits = 800000;

                    $speed = $actions[$id]['progress']['speed'] ?? 200;
                    $fait = time() - $actions[$id]['progress']['time'];
                    $total = $Mbits / $speed + $progress / 100 * $fait;
                }
            } else {
                // autre cas que clonage
                $total = 100;
                $fait = $progress;
            }

            if (isset($actions[$id]['progress']['time']))
                $tmp_time = $actions[$id]['progress']['time'];
            else
                $tmp_time = time();

            $actions[$id]['progress'] = array(
                'total' => $total, // temps estimé
                'fait' => $fait, // temps écoulé
                'avancement' => round($fait / $total * 100, 0), // %
                'time' => $time ?? $tmp_time, // heure
                'speed' => $speed // débit en Mb/s
            );
            apcu_store('actions', $actions);
        }
        unlock('actions');
    }
}

function get_clone_disks($disks)
{
    $list = array();
    foreach ($disks as $disk) {
        if ($disk['clone'])
            $list[] = $disk['device'];
    }
    return $list;
}

function get_clone_parts($disks)
{
    $list = array();
    foreach ($disks as $disk) {
        if ($disk['clone'])
            foreach ($disk['partitions'] as $part)
                if ($part['clone'])
                    $list[] = array(
                        'device' => $part['device'],
                        'fs' => $part['fs'],
                        'size' => $part['size']
                    );
    }
    return $list;
}

function get_total_size($disks, $index = "")
{
    $size = $i = 0;
    foreach ($disks as $disk) {
        if ($disk['clone'])
            foreach ($disk['partitions'] as $part)
                if ($part['clone']) {
                    if (isset($part['size'])) {
                        $size = $size + $part['size'];
                    }
                    if (! empty($index) && ++ $i == $index)
                        return $size;
                }
    }
    return $size;
}

/**
 * incremente le port udp utilisé par le clonage, de façon à etre unique et pair
 *
 * @param
 *            string
 * @return number
 */
function get_clone_port($id)
{
    $port = 9000;
    $ports = array();
    lock('actions');
    $actions = fetch_action();
    if (! empty($actions[$id]['port'])) {
        $port = $actions[$id]['port'];
    } else {
        foreach ($actions as $action) {
            if (! empty($action['port']))
                $ports[] = $action['port'];
        }
        while (in_array($port, $ports)) {
            $port = $port + 2;
        }
    }
    $actions[$id]['port'] = $port;
    apcu_store('actions', $actions);
    unlock('actions');

    return $port;
}

function get_action_member($config, $id, $role)
{
    $list = array();
    $action = fetch_action()[$id] ?? [];
    foreach ($action as $uuid => $membre) {
        if (! empty($membre['role'])) {
            if ($membre['role'] == $role) {
                $list[] = search_machine($config, $uuid)['cn'];
            }
        }
    }
    return $list;
}

function set_statut($id, $statut = "")
{
    lock('actions');
    $actions = fetch_action();
    $script = $actions[$id]['statut'] ?? "inconnu";
    if (isset($actions[$id])) {
        $etape = "";
        foreach ($actions[$id] as $membre) {
            if (! empty($membre['role'])) {
                if ($membre['role'] == 'modele') {
                    // $script = $membre['script'];
                    $etape = $membre['etape'] ?? "attente...";
                }
            }
        }
        if ((($actions[$id]['type'] != "clonage" && $actions[$id]['type'] != "clonage2") || $etape == "emit-partitions" || $etape == "change-hostname") && ! empty($statut)) {
            $script = $statut;
        }
        $actions[$id]['statut'] = $script;
        apcu_store('actions', $actions);
    }
    unlock('actions');
    return $script;
}

function wait_clone($id)
{
    $pret = true;
    $nb = $p = 0;
    $actions = fetch_action()[$id] ?? [];
    $etape = "";
    foreach ($actions as $membre) {
        if (! empty($membre['role'])) {
            if ($membre['role'] == 'modele') {
                $etape = $membre['ret'] + 1;
            }
        }
    }
    foreach ($actions as $membre) {
        if (! empty($membre['role'])) {
            if ($membre['role'] == 'clone') {
                $nb ++;
                $mp = (($membre['etape'] == "receive-partitions") && ($membre['ret'] == $etape));
                if ($mp)
                    $p ++;
                $pret = ($pret && $mp);
            }
        }
    }
    // il faut qu'au moins un clone ait été défini, sinon on attend
    $pret = $pret && ($nb > 0);
    return array(
        'pret' => $pret,
        'clones' => "(" . $p . "/" . $nb . ")"
    );
}

function toggle_action_keep($id)
{
    lock('actions');
    $actions = fetch_action();
    if (isset($actions[$id]['keep'])) {
        if ($actions[$id]['keep'])
            $actions[$id]['keep'] = false;
        else
            $actions[$id]['keep'] = true;
    } else {
        $actions[$id]['keep'] = true;
    }
    apcu_store('actions', $actions);
    unlock('actions');
}

function refresh_page($form, $duree = 30)
{
    $stylecss = "/elements/style_sheets/sambaedu.css";
    $html = "<meta http-equiv='Content-Type' content='text/html; charset=UTF-8' />
<link type='text/css' rel='stylesheet' href='$stylecss' />
<div id='decompte' style='float: right; border: 1px solid black;'></div>
<script type='text/javascript'>
    /**
     * Decompte le temps
     * @language Javascript
     * @Parametres
     * @return le decompte qui s'affiche
     */
    cpt=" . $duree . ";
    compte_a_rebours='y';
    function decompte(cpt){
        if(compte_a_rebours=='y'){
            document.getElementById('decompte').innerHTML=cpt;
            if(cpt>0){
                cpt--;
            }else{
                cpt = " . $duree . ";                  
                document.getElementById('" . $form . "').submit()
            }
            setTimeout(\"decompte(\"+cpt+\")\",1000);
        }else{
    	    document.getElementById('decompte').style.display='none';
        }
    }
    window.onload=function(){decompte(cpt)};

    document.addEventListener('DOMContentLoaded', function (event) {
        var scrollpos = localStorage.getItem('scrollpos');
        if (scrollpos) window.scrollTo(0, scrollpos);
    });
    window.onscroll = function (e) {
        localStorage.setItem('scrollpos', window.scrollY);
    };
</script>\n";
    return $html;
}

function form_actions_statut($type = "")
{
    $html = "<table class='table-bordered' width=\"80%\">\n";
    $html .= "<th colspan=7 class=menuheader style=\"height: 30\">Liste des actions</th>\n";

    $html .= "<tr><td class='menuheader' align=\"center\">Identifiant</td>";
    $html .= "<td class='menuheader' align=\"center\">Type</td>";
    $html .= "<td class='menuheader' align=\"center\">Boot iPXE</td>";
    $html .= "<td class='menuheader' align=\"center\">Etape</td>";
    $html .= "<td class='menuheader' align=\"center\">Avancement</td>";
    $html .= "<td class='menuheader' align=\"center\">Statut</td>";
    $html .= "<td class='menuheader' align=\"center\">Sélectionner</td>";
    $html .= "</tr>\n";
    $actions = fetch_action();

    foreach ($actions as $id => $action) {
        if (($id != "default") && (empty($type) || $type == $action['type'])) {
            $statut = set_statut($id);
            $etape = $script = "";
            foreach ($action as $val) {
                if (! empty($val['script']) && ! empty($val['etape'])) {
                    $etape = $val['etape'];
                    $script = $val['script'];
                    break;
                }
            }
            $progress = $action['progress']['avancement'] ?? 0;
            $html .= "<tr><td>" . $id . "</td>";
            $html .= "<td>" . $action['type'] . "</td>";
            $html .= "<td>" . $script . "</td>";
            $html .= "<td>" . $etape . "</td>";
            // $html .= "<td><label for=\"" . $id . "\">fait : " . $action['progress']['avancement'] . "%</label>";
            // $html .= "<progress id=\"" . $id . "\" max=\"100\" value=" . $action['progress']['avancement'] . "> " . $action['progress']['avancement'] . "% </progress></td>";
            $html .= "<td><progress max=\"100\" value=" . $progress . "> " . $progress . "% </progress>  " . $progress . " %</td>";
            $html .= "<td>" . $statut . "</td>";
            $html .= "<td align=\"center\"><input type=\"checkbox\" name=\"id[]\" value=\"" . $id . "\"></td>";
            $html .= "</tr>\n";
        }
    }
    $html .= "</TABLE>";
    // $html .= form_action_type($type);
    return $html;
}

function form_action_type($config, $type = "voir")
{
    $types = array(
        "voir" => "Voir l'action",
        "supprimer" => "Supprimer l'action",
        "clonage2" => "Clonage en direct avec Sysrescd",
        "install" => "Installation automatique des postes",
        "renomme" => "Renommage d'un poste",
        // "renomme_sysprep" => "Renommage d'un poste (sysprep)",
        "post" => "Post-intégration ou migration se3",
        "wpkg" => "Forcer wpkg en mode graphique",
        "reboot" => "Allumer ou rebooter"
    );
    if (! empty($config['ltsp_ip'])) {
        $types = array_merge($types, [
            'ltsp' => "Booter un client LTSP"
        ]);
    }
    asort($types);
    // Commenté pour le moment car aucune prise en compte dans actions.inc.php et donc ne fonctionne pas
    // $clonezilla_type = array("image" => "Sauvegarde et restauration d'une image Clonezilla");
    // $types = array_merge($types,$clonezilla_type);
    $html = "Choix du type d'actions <br>\n";
    $html .= "<SELECT NAME=\"type\" onchange=\"this.form.submit()\">";
    $html .= "<option value=\"\">Rafraîchir</option>";
    foreach ($types as $t => $txt) {
        $html .= "<option value=\"$t\"";
        if ($type == $t) {
            $html .= " selected>";
        } else {
            $html .= ">";
        }
        $html .= "$txt</option>";
    }
    $html .= "</SELECT>";
    $html .= " <input type=\"submit\" value=\"Valider\">\n";
    return $html;
}

// @TODO differencier le choix en fonction de l'étape en cours
function form_action_clonage($action = "submit", $current = [])
{
    // début
    $actions = array(
        'new' => "Nouveau clonage ou modèle "
    );
    // modele choisi
    $actions = array(
        'prepare' => "Préparer le modèle",
        'add' => "Ajouter des clones",
        'remove' => "Supprimer des clones",
        'start' => "(Re)Démarrer les clones",
        'del' => "Supprimer totalement",
        'submit' => "Valider/Actualiser"
    );
    // modele pret
    $actions = array(
        'keep' => "Conserver le modèle pour un clonage ultérieur",
        'add' => "Ajouter des clones",
        'remove' => "Supprimer des clones",
        'start' => "(Re)Démarrer les clones",
        'disk' => "Choisir les partitions à cloner",
        'del' => "Supprimer totalement",
        'submit' => "Valider/Actualiser"
    );

    $actions = array(
        'submit' => "Valider/Actualiser",
        'start' => "Démarrer",
        // 'prepare' => "Préparer le modèle avec sysprep si possible",
        'prepare2' => "Préparer le modèle sans sysprep",
        'keep' => "Conserver le modèle pour un clonage ultérieur",
        'reset' => "Redémarrer le clonage (sans refaire le modèle)",
        'new' => "Nouveau clonage ou modèle ",
        'add' => "Ajouter des clones",
        'remove' => "Supprimer des clones",
        'disk' => "Choisir les partitions à cloner",
        'reinit' => "Interrompre le clonage et remettre le modèle au domaine",
        'del' => "Supprimer totalement"
    );
    $html = "<SELECT NAME=\"action\" id=\"action_clonage\" onchange=\"this.form.submit()\">";
    $html .= "<option value=\"\">Choisir l'action...</option>";
    asort($actions);
    foreach ($actions as $a => $txt) {
        $html .= "<option value=\"$a\"";
        if ($action == $a) {
            $html .= " selected>";
        } else {
            $html .= ">";
        }
        $html .= "$txt</option>";
    }
    $html .= "</SELECT>";
    $html .= " <input type=\"submit\" value=\"Valider\">\n";

    return $html;
}

function form_choix_install($action = "")
{
    $actions = [];
    if (file_exists("/var/sambaedu/unattended/install/os/Win10/sources/boot.wim")) {
        $actions['wimboot10'] = "Installation auto Windows 10";
        $actions['wimboot10disk'] = "Installation auto Windows 10 avec partitionnment personnalisé";
        $actions['wimboot10perso'] = "Installation auto Windows 10 (hors domaine)";
    }
    if (file_exists("/var/sambaedu/unattended/install/os/Win11/sources/boot.wim")) {
        $actions['wimboot11'] = "Installation auto Windows 11";
    }
    if (file_exists("/var/sambaedu/unattended/install/os/debian-installer")) {
        $actions = array_merge($actions, array(
            'deb_mate' => "Installation auto Debian Mate",
            'deb_gnome' => "Installation auto Debian Gnome",
            'deb_cinnamon' => "Installation auto Debian Cinnamon",
            'deb_kde' => "Installation auto Debian KDE",
            'deb_base' => "Installation auto Debian terminal",
            'deb_gnome_perso' => "Installation auto Debian Gnome (hors domaine)"
        ));
    }
    if (file_exists("/var/sambaedu/unattended/install/os/ubuntu-installer")) {
        $actions = array_merge($actions, array(
            'ubuntu64' => "Installation auto Ubuntu (hors domaine)"
        ));
    }
    asort($actions);
    $html = "Choisir l'OS à installer\n";
    $html .= "<SELECT NAME=\"action\" onchange=\"submit()\">";
    $html .= "<option value=\"\">Installation...</option>";
    foreach ($actions as $a => $txt) {
        $html .= "<option value=\"$a\"";
        if ($action == $a) {
            $html .= " selected>";
        } else {
            $html .= ">";
        }
        $html .= "$txt</option>";
    }
    $html .= "</SELECT>";
    return $html;
}

function form_choix_role($role = "")
{
    $html = "choisir le rôle de la machine pour le clonage\n";
    $html .= "<SELECT NAME=\"role\" onchange=\"submit()\">";
    $html .= "<option value=\"\">Role...</option>";
    $roles = array(
        "modele",
        "clone"
    );
    foreach ($roles as $r) {
        $html .= "<option value=\"$r\"";
        if ($role == $r) {
            $html .= " selected>";
        } else {
            $html .= ">";
        }
        $html .= "$r</option>";
    }
    $html .= "</SELECT>";
    return $html;
}

function form_choix_serveur_ltsp($config, $script = "")
{
    $html = "choisir le serveur ltsp à utiliser\n";
    $html .= "<SELECT NAME=\"script\">";
    $html .= "<option value=\"\">choisir...</option>";
    groupadd($config, "serveurs_ltsp", $config['parcs_rdn'], "Serveurs de terminaux réseau LTSP", $config['suffix']);
    foreach (list_members_parc($config, "serveurs_ltsp", false) as $s) {
        $html .= "<option value=\"$s\"";
        if ($s == $script) {
            $html .= " selected>";
        } else {
            $html .= ">";
        }
        $html .= "$s</option>";
    }
    $html .= "</SELECT>";
    return $html;
}

function form_choix_reservation($action)
{
    $html = "Action dhcp :\n";
    $html .= "<SELECT NAME=\"action\" onchange=\"submit()\">";
    $html .= "<option value=\"\">choisir...</option>";
    $roles = array(
        "reservation",
        "dynamique"
    );
    foreach ($roles as $r) {
        $html .= "<option value=\"$r\"";
        if ($action == $r) {
            $html .= " selected>";
        } else {
            $html .= ">";
        }
        $html .= "$r</option>";
    }
    $html .= "</SELECT>";
    return $html;
}

function form_new_name($name, $clones)
{
    if (count($clones) == 1) {
        $html = "Choisir le nouveau nom pour la machine " . $clones[0] . " :\n";
        $html .= " <INPUT TYPE=\"text\" NAME=\"modele\" VALUE=\"" . $clones[0] . "\" SIZE=\"15\" >"; // onchange=submit()>";
    } else {
        $html = "Choisir le préfixe pour le nouveau nom des machines. Toutes les machines choisies pour être renommées auront cepréfixe de nom suivi de deux chiffres \n";
        $html .= " <INPUT TYPE=\"text\" NAME=\"modele\" VALUE=\"" . $name . "\" SIZE=\"15\" >-XX "; // onchange=submit()>";
    }
    foreach ($clones as $clone) {
        echo form_hidden("clones[]", $clone);
    }
    // $html .= "<input type=\"hidden\" name=\"name\" value=\"" . $name . "\">\n";
    $html .= "<input type=\"submit\" value=\"Renommer\">\n";
    return $html;
}

function form_action_membres($config, $id)
{
    $action = fetch_action()[$id] ?? [];
    $progress = $action['progress']['avancement'] ?? 0;
    $speed = $action['progress']['speed'] ?? 0;
    $html = "<table class='table-bordered' width=\"80%\">\n";
    $html .= "<th colspan=4 class=menuheader style=\"height: 30\">Liste des actions</th>\n";

    $html .= "<tr><td class='menuheader' align=\"center\">Identifiant</td>";
    $html .= "<td class='menuheader' align=\"center\">Type</td>";
    $html .= "<td class='menuheader' align=\"center\">Avancement</td>";
    $html .= "<td class='menuheader' align=\"center\">Statut</td>";
    $html .= "</tr>\n";
    $statut = $action['statut'] ?? "";
    $html .= "<tr><td>" . $id . "</td>";
    $html .= "<td>" . @$action['type'] . "</td>";
    if ($speed) {
        $html .= "<td align=\"center\"> <progress max=\"100\" value=" . $progress . "> " . $progress . "% </progress>  " . $progress . " % Débit = " . $speed . " Mbit/s<br> 
temps restant : " . $progress = round(($action['progress']['total'] - $action['progress']['fait']) / 60, 0) . " minutes</td>";
    } else {
        $html .= "<td align=\"center\"> <progress max=\"100\" value=" . $progress . "> " . $progress . "% </progress>  " . $progress . " % </td>";
    }
    $html .= "<td>" . $statut . "</td>";
    $html .= "</tr>\n";
    $html .= "</table>";

    $html .= "<table class='table-bordered' width=\"80%\">\n";
    $html .= "<th colspan=7 class=menuheader style=\"height: 30\">Liste des postes</th>\n";
    $html .= "<tr><td class='menuheader' align=\"center\">Role</td>";
    $html .= "<td class='menuheader' align=\"center\">Nom</td>";
    $html .= "<td class='menuheader' align=\"center\">Ip</td>";
    $html .= "<td class='menuheader' align=\"center\">Mac</td>";
    $html .= "<td class='menuheader' align=\"center\">Boot iPXE</td>";
    $html .= "<td class='menuheader' align=\"center\">étape</td>";
    $html .= "<td class='menuheader' align=\"center\">Sélectionner</td>";
    $html .= "</tr>\n";

    foreach ($action as $uuid => $membre) {
        if (! empty($membre['role'])) {
            $machine = get_action($config, $uuid);
            $etape = $membre['etape'] ?? "attente...";
            $html .= "<tr><td>" . $membre['role'] . "</td>";
            $html .= "<td>" . $machine['cn'] . "</td>";
            $html .= "<td>" . @$machine['iphostnumber'] . "</td>";
            $html .= "<td>" . $machine['networkaddress'] . "</td>";
            $html .= "<td>" . $membre['script'] . "</td>";
            $html .= "<td>" . $etape . "</td>";
            if ($membre['role'] == "modele") {
                if (@$action['keep'])
                    $html .= "<td align=\"center\">Conserver</td>";
                else
                    $html .= "<td></td>";
            } else {
                $html .= "<td align=\"center\"><input type=\"checkbox\" name=\"clones[]\" value=\"" . $uuid . "\"></td>";
            }
            $html .= "</tr>\n";
        }
    }
    $html .= "</TABLE>";
    $html .= "<br>";
    $html .= "<input type=\"hidden\" name=\"id\" value=\"" . $id . "\">\n";

    return $html;
}

function form_choix_poste($config, $action, $filtre, $choix, $role, $id, $parc = "")
{
    $html = "Choix des postes pour l'action $role :<p>";
    $list_parcs = search_parcs($config, "*");
    if (count($list_parcs) > 0) {
        $html .= "Choisir un parc (facultatif) : <SELECT NAME=\"parc\" SIZE=\"1\" onchange=\"this.form.submit()\">";
        $html .= "<option value=\"\">Sélectionner</option>";
        for ($loop = 0; $loop < count($list_parcs); $loop ++) {
            $html .= "<option value=\"" . $list_parcs[$loop]["name"] . "\"";
            if ($parc == $list_parcs[$loop]["name"]) {
                $html .= " selected";
            }
            $html .= ">" . $list_parcs[$loop]["name"] . "\n";
            $html .= "</option>";
        }
        $html .= "</SELECT><p>\n";
    }
    $html .= "Filtre sur le nom des machines (laisser vide pour toutes)  : <INPUT TYPE=\"text\" name=\"filtre\" VALUE=\"$filtre\" SIZE=\"14\">";
    $html .= "<input type=\"hidden\" name=\"action\" value=\"$action\">\n";
    $html .= "<input type=\"hidden\" name=\"role\" value=\"$role\">\n";
    $html .= "<input type=\"hidden\" name=\"id\" value=\"$id\">\n";
    $html .= "<input type=\"hidden\" name=\"choix\" value=\"1\">\n";
    $html .= "<br>";
    if (empty($filtre))
        $filtre = "*";
    else
        $filtre = "*" . $filtre . "*";
    if ($choix == 1) {
        $list_computer = array_column(search_ad($config, $filtre, "machine", $config['dn']['computers']), 'cn');
        if (! empty($parc)) {
            $mps = list_members_parc($config, $parc);
            $list_computer = array_uintersect($mps, $list_computer, "strcasecmp");
        }
        if (count($list_computer) > 15)
            $size = 15;
        else
            $size = count($list_computer);
        if (count($list_computer) == 1) {
            $html .= "Nom ou adresse IP de la machine : ";
            if ($role == "clone" || $role == "wpkg" || $role == "install" || $role == "renomme" || $role == "ltsp" || $role == "reboot") {
                $html .= "<input type=hidden name=\"clones[0]\" value=" . $list_computer[0] . ">" . $list_computer[0] . "\n";
                // elseif ($role == "install")
                // $html .= "<p><select size=\"" . $size . "\" name=\"name\">\n";
            } else {
                // $ma = get_action($config, $list_computer[0]);
                // if ($ma['action']['role'] != "modele") {
                // if (@$ma['action']['keep']) {
                // $html .= "<input type=hidden name=\"modele\" value=" . $list_computer[0] . ">" . $list_computer[0] . "\n";
                // } else {
                // $html .= "<input type=hidden name=\"modele\" value=" . $list_computer[0] . ">" . $list_computer[0] . "\n";
                // }
                // }
                $html .= "<input type=hidden name=\"modele\" value=" . $list_computer[0] . ">" . $list_computer[0] . "\n";
            }
        } elseif (count($list_computer) > 1) {
            sort($list_computer);
            $html .= "Nom ou adresse IP de la machine : ";
            if ($role == "clone" || $role == "wpkg" || $role == "install" || $role == "renomme" || $role == "ltsp" || $role == "reboot")
                $html .= "<p><select size=\"" . $size . "\" name=\"clones[]\" multiple>\n";
            // elseif ($role == "install")
            // $html .= "<p><select size=\"" . $size . "\" name=\"name\">\n";
            else
                $html .= "<p><select size=\"" . $size . "\" name=\"modele\">\n";
            foreach ($list_computer as $machine) {
                $ma = get_action($config, $machine);
                if (@$ma['action']['keep'] && @$ma['action']['role'] == "modele") {
                    if ($ma['action']['role'] == "modele")
                        $html .= "<option disabled value=\"" . $machine . "\">" . $machine . " (préparée)</option>\n";
                } else {
                    $html .= "<option value=\"" . $machine . "\">" . $machine . "</option>\n";
                }
            }
            $html .= "</select>\n";
            $html .= "<br>";
        }
    }
    $html .= " <input type=\"submit\" value=\"Valider\">\n";
    $html .= "<u onmouseover=\"return escape('Donner le début du nom ou de l\'adresse IP de la machine que vous souhaitez trouver.')\"><img name=\"action_image2\"  src=\"../elements/images/system-help.png\" alt=\"help\"></u> ";
    return $html;
}

function form_hidden($name, $val)
{
    return "<input type=\"hidden\" name=\"" . $name . "\" value=\"" . $val . "\">\n";
}

function form_choix_partitions($id)
{
    $action = fetch_action()[$id];
    $html = "<SELECT NAME=\"partitions[]\" SIZE=\"5\" MULTIPLE=\"true\">";
    $html .= "<option value=\"\">Sélectionner les disques et partitions...</option>";
    foreach ($action['disks'] as $disk) {
        $html .= "<option value=\"" . $disk['device'] . "\"";
        if ($disk['clone'])
            $html .= " selected";
        $html .= ">Copier le MBR de " . $disk['device'] . "</option>";
        foreach ($disk['partitions'] as $partition) {
            $html .= "<option value=\"" . $partition['device'] . "\"";
            if ($partition['clone'])
                $html .= " selected";
            $html .= "> Cloner " . $partition['device'] . " (" . $partition['fs'] . ", " . round($partition['size'] / (1024 * 1024), 1) . " G)</option>";
        }
    }
    $html .= "</SELECT>";
    $html .= "<input type=\"hidden\" name=\"id\" value=\"" . $id . "\">\n";
    return $html;
}

function form_action_id($type = "")
{
    $html = "<FORM action=\"lance_action.php\" method=\"post\">\n";
    "<SELECT NAME=\"id\" SIZE=\"20\">";
    if ($actions = fetch_action()) {
        foreach ($actions as $id => $action) {
            if (empty($type) || ($action['type'] == $type)) {
                $html .= "<option value=\"" . $id . "\"> Id : " . $id . " (" . $action['type'] . ")</option>";
            }
        }
    }
    $html .= "</SELECT>";
    $html .= "<input type=\"submit\" value=\"Valider\">\n";
    $html .= "</FORM>\n";
    return $html;
}

function select_actions($actions, $module = "")
{
    $option = "";
    foreach ($actions as $action) {
        if (empty($module) || ($action['module'] == $module)) {
            $option .= "<option value=\"" . $action['nom'] . "\">" . $action['description'] . "</option>";
        }
    }
    return $option;
}

/**
 * récupère le tableau action
 *
 * @return string[][]|string[][][]|number[][][]
 */
function fetch_action()
{
    if (! ($actions = apcu_fetch('actions'))) {
        $actions = array(
            'default' => array(
                'type' => 'default',
                'default' => array(
                    'role' => '',
                    'script' => "default",
                    'ret' => - 1
                )
            )
        );
        apcu_add('actions', $actions);
    }
    return $actions;
}

/**
 * Récupère l'action programmée pour la machine, au boot ipxe, ou lors d'actions à distance
 * on récupère l'action et un token unique commun à une action et qui permet d'authentifier la requête
 * si l'enregistrement ad n'existe pas, on renvoie une action avec l'@mac comme id
 * Cette fonction ne met rien en cache.
 * Elle n'ecrit pas dans l'Ad sauf si mac est passée et qu'il manque uuid dans l'ad (migration)
 *
 * @param array $config
 * @param string $uuid
 *            : uuid ip, ou nom de la machine
 * @param string $mac
 *            : mac ip, ou nom de la machine
 * @return array : tableau des attributes de la machine, avec les valeurs par défaut pour (action)
 *         et 'netbootguid' (token)
 */
function get_action(array $config, string $uuid = "", string $mac = "")
{
    if (! empty($uuid)) {
        $uuid = trim(strtolower($uuid));
        $machine = search_machine($config, $uuid, true, true);
    } else {
        $machine = [];
    }
    if (count($machine) == 0 && ! empty($mac)) {
        // pas trouvé l'uuid, on essaie avec le 2ème argument (mac, ou autre)
        $machine = search_machine($config, $mac, true, true);
    }
    if (count($machine) > 0 && ! empty($uuid) && strtolower($machine['cn']) != strtolower($uuid) && isset($machine['networkaddress']) && $machine['networkaddress'] != $uuid) {
        // on enregistre l'uuid si c'est bien un uuid
        $machine = register_machine_hardware($config, $machine, $uuid);
    }

    // recherche des actions en cours pour la machine
    $actions = fetch_action();
    if (count($machine) > 0) {
        // un enregistrement AD existe, on l'utilise
        $uuid = $machine['netbootguid'] ?? "";
    } else {
        $machine = array(
            'networkaddress' => $mac,
            'action' => [],
            'netbootguid' => $uuid,
            'memberof' => []
        );
    }
    foreach ($actions as $k => $a) {
        if (isset($a[$uuid])) {
            $machine['id'] = $k;
            $machine['action'] = $a;
            break;
        }
    }
    if (empty($machine['action'])) {
        $machine['action'] = $actions['default'];
        $machine['id'] = "default";
    }

    return $machine;
}

/**
 * enregistre l'action pour une machine.
 * - si accès ipxe : token + mac doivent être fournis, eventuellement id, action, role
 * - si accès sysrescuecd : mac suffit, on est déjà enregistré en phase ipxe
 * - si accès interface : choix de tous les parametres.
 *
 * @param array $config
 * @param string $uuid
 *            id unique machine (smbios)
 *            : nom ou ip ou mac de la machine
 * @param string $type
 *            : type d'action ipxe ( install | clonage | clonage2 | wpkg | ltsp | renomme | default )
 * @param string $id
 *            : identifiant de l'action à effectuer (généré automatiquement) commun à toutes les machines de l'action en cours
 * @param string $role
 *            : role de la machine le cas échéant (modele|clone|nouveau nom|windows|linux)
 * @param string $script
 *            : script à exécuter par ipxe (sysrecuecd|sysprep| post | serv_ltsp|deb-*|windows|default). Dans le cas du clonage, contient les noms des étapes intermédiaires.
 * @param int $ret
 *            : etape pour les scripts en plusieurs étapes côté client.
 *            
 * @return array $actions
 */
// function set_action(array $config, string $uuid, string $type = "", string $id = "", string $role = "", string $script = "", int $ret = - 255, array $disks = array(), bool $increment = false)
function set_action(array $config, string $uuid, array $params = [], bool $increment = false)
{
    // on recupere les valeurs
    lock('actions');
    $machine = get_action($config, $uuid);
    if (! isset($machine['netbootguid'])) {
        unlock('actions');
        return false;
    }
    if ($machine['id'] == "default" && ! empty($params['id']) && $params['id'] != "default") {
        $id = $params['id'];
    } else {
        $id = $machine['id'];
    }

    $uuid = $machine['netbootguid'];
    if (isset($params['type'])) {
        if ($params['type'] == "default") {
            // on supprime le poste des actions où il serait enregistré (mais pas toute l'action si clonage par exemple)
            delete_action_members($uuid);
            $id = "default";
        } else {
            // initialisation de l'id
            if ($id == "default" || $id == "new") {
                if (isset($machine['cn'])) {
                    $id = $params['type'] . "-" . $machine['cn'] . "@" . date_et_heure();
                } else {
                    $id = $params['type'] . "-" . $uuid . "@" . date_et_heure();
                }
            }
        }
    }
    $actions = fetch_action();
    $res = false;
    // mise a jour de actions
    if ($id != "default") {
        $actions[$id] = $actions[$id] ?? array();
        if (! empty($params['type'])) {
            $actions[$id]['type'] = $params['type'];
        } elseif (empty($actions[$id]['type'])) {
            $actions[$id]['type'] = 'default';
        }
        $actions[$id][$uuid] = $actions[$id][$uuid] ?? array();
        if (! empty($params['disks']) && count($params['disks']) > 0) {
            $actions[$id]['disks'] = $params['disks'];
        }
        if (! empty($params['role'])) {
            $actions[$id][$uuid]['role'] = $params['role'];
        }
        if (! empty($params['script'])) {
            $actions[$id][$uuid]['script'] = $params['script'];
        }
        if (! empty($params['etape'])) {
            $actions[$id][$uuid]['etape'] = $params['etape'];
        }
        $actions[$id][$uuid]['ret'] = $params['ret'] ?? $actions[$id][$uuid]['ret'] ?? - 1;
        if ($increment) {
            $actions[$id][$uuid]['ret'] ++;
        }
    }
    apcu_store('actions', $actions);
    unlock('actions');

    return get_action($config, $uuid);
}

function increment_action(array $config, string $cn)
{
    return set_action($config, $cn, [], true);
}

function delete_action_members(string $uuid)
{
    lock('actions');
    $actions = fetch_action();
    foreach ($actions as $key => $action) {
        if (isset($action[$uuid])) {
            unset($actions[$key][$uuid]);
        }
    }
    apcu_store('actions', $actions);
    unlock('actions');
    return $actions;
}

function delete_action(string $id)
{
    lock('actions');
    $actions = fetch_action();
    if (isset($actions[$id])) {
        unset($actions[$id]);
    }
    apcu_store('actions', $actions);
    unlock('actions');
    return $actions;
}

/**
 * verifie que l'action est authentifiée pour la machine
 *
 * @param array $config
 * @param string $cn
 *            :nom ou ip ou mac de la machine (ou n'importe quel attribut unique)
 * @param string $uuid
 *            : jeton d'identification
 * @return boolean
 */
function auth_action(array $config, string $cn, string $uuid)
{
    $machine = get_action($config, $uuid, $cn);
    return (! empty($machine['netbootguid']) && $machine['netbootguid'] == $uuid);
}

/**
 * retourne true pour le boot iPXE en cas d'autentification avec un compte computer_is_admin et que la machine existe en cache actions
 *
 * @param array $config
 * @param string $login
 * @param string $password
 * @return string $uuid : jeton unique, "" si non autorisé
 */
function login_action(array $config, $uuid, string $login, string $password)
{
    $res = false;
    if (user_valid_passwd($config, $login, $password) == 1 && have_right($config, SE_COMPUTER_INSTALL, $login)) {
        $machine = get_action($config, $uuid);
        if (! empty($machine['netbootguid'])) {
            $res = true;
        }
    }
    return $res;
}
?>
