<?php

/**
 * Fonctions pour l'automatisation NextCloud
 * @Projet SambaEdu
 * @Auteurs Equipe Sambaedu
 * @Licence Distribue sous la licence GPL
 * @Repertoire: includes/
 * file: cloud.inc.php
 *
 * Accès minimal à l'API NC pour permettre la création de partages (classes, groupes) et la récupération de la clé client pour le montage RClone
 * 
 * Utilisation de lib php webdav
 * 
 * Seuls les appels utiles sont implémentés.
 * 
 * Authentification
 * 
 * - login avec un compte et sa clé d'api
 * - récupération de la clé api pour un compte utilisateur en passant le mot de passe, ou via un compte admin si c'est possible
 * 
 * Dossiers (api webdav)
 * 
 * - liste des dossiers
 * - mkdir
 * - rm
 * - mv
 * 
 * Partages (api shares)
 * 
 * - liste de droits
 * - ajout d'un droit
 * - suppression d'un droit
 * 
 * 
 * les données de conf sont dans /etc/sambaedu/sambaedu.conf.d/cloud.conf
 * 
 * cloud_name = "nuage"
 * cloud_type = "webdav"
 * cloud_uri = "https://nuage.etab.fr/"
 *
 *
 * un compte admin sur le NC doit être défini dans la conf pour les opérations.
 *
 * cloud_api_user = "nomdeladmincloud"
 * cloud_api_password = "token"
 *
 *
 * la lib sabre/dav est utilisée pour faire les requêtes webdav
 *
 * les partages sont gérés par un utilisateur unique par établissement.
 */
use GuzzleHttp\Exception\GuzzleException;

function cloud_path($config, $user)
{
    switch (@$config['cloud_type']) {
        case "seafile":
            $path = $config['cloud_uri'];
            break;
        default:
            $path = $config['cloud_uri'] . "remote.php/dav/files/" . $user . "/";
    }
    return $path;
}

function get_nc_client_code($config, $user, $password)
{
    $info = false;
    require_once (dirname(__FILE__) . '/../vendor/autoload.php');
    $opt = [
        'base_uri' => $config['cloud_uri'],
        'cookies' => true,
        'headers' => [
            'OCS-APIRequest' => "true"
        ]
    ];
    curl_proxy_options($config, $opt);
    $client = new GuzzleHttp\Client($opt);
    try {
        $response = $client->request('GET', 'ocs/v2.php/core/getapppassword', [
            'auth' => [
                $user,
                $password
            ],
            'query' => [
                'format' => 'json'
            ]
        ]);
        if ($response) {
            $res = json_decode((string) $response->getBody(), true);
            if ($res['ocs']['meta']['status'] == "ok") {
                $info['code'] = $res['ocs']['data']['apppassword'];
                $response = $client->request('GET', 'ocs/v2.php/cloud/user', [
                    'auth' => [
                        $user,
                        $info['code']
                    ],
                    'query' => [
                        'format' => 'json'
                    ]
                ]);
                if ($response) {
                    $res = json_decode((string) $response->getBody(), true);
                    if ($res['ocs']['meta']['status'] == "ok") {
                        $info['id'] = $res['ocs']['data']['id'];
                        // $info['storageLocation'] = $res['ocs']['data']['storageLocation'];
                    }
                }
            }
        }
    } catch (\GuzzleHttp\Exception\RequestException | \GuzzleHttp\Exception\ConnectException $e) {
        return false;
    }
    return $info;
}

function test_cloud_login($config, $user = "", $password = "")
{
    $data = false;
    if (empty($user) && empty($password)) {
        $creds = [
            $config['cloud_api_user'],
            $config['cloud_api_password']
        ];
    } else {
        $creds = [
            $user,
            $password
        ];
    }

    require_once (dirname(__FILE__) . '/../vendor/autoload.php');
    $opt = [
        'base_uri' => $config['cloud_uri'],
        'cookies' => true,
        'headers' => [
            'OCS-APIRequest' => "true"
        ],
        'query' => [
            'format' => 'json'
        ]
    ];
    curl_proxy_options($config, $opt);
    $client = new GuzzleHttp\Client($opt);
    try {
        $response = $client->request('GET', 'ocs/v2.php/cloud/user?format=json', [
            'auth' => $creds
        ]);
        if ($response) {
            $res = json_decode((string) $response->getBody(), true);
            if ($res['ocs']['meta']['status'] == "ok") {
                $data = $res['ocs']['data'];
            }
        }
    } catch (\GuzzleHttp\Exception\RequestException | \GuzzleHttp\Exception\ConnectException $e) {
        return false;
    }
    return $data;
}

/**
 * affiche l'état de la connexion au cloud de l'établissement (celui qui héberge les partages péda)
 *
 * @param array $config
 * @param array $login
 * @param array $cloud
 * @return string
 */
function cloud_etab_access(array $config, array $login, array &$cloud = [])
{
    $cloud_password = $_POST['cloud_password'] ?? "";
    $reconnect = $_POST['cloud_reconnect'] ?? "";
    $html = "";
    if (isset($config['cloud_type'])) {
        $html .= "<H1>Accés aux partages de l'établissement : ";
        $html .= "<a href='" . $config['cloud_uri'] . "' title='Accés aux partages de l&lsquo;établissement'>";
        $html .= $config['cloud_uri'] . '</a></H1>';
        if ((! isset($config['etab_ou']) || $config['etab_ou'] == $_SESSION['etab']) && ! isset($config['no_cloud'])) {
            if (! test_rclone_config($config, $login['cn'])) {
                $html_form = '<form name="cloudpwd" action="" method="post">
<div class="parent">
<label>Vérifiez votre mot de passe :</label>
<input type="password" name="cloud_password" size="20" id="password">
<span id="password-field" class="fa fa-fw fa-eye field-icon toggle-password"></span>
</div>
<br>';
                $html_form .= '<input type="submit" value="Valider">';
                $html_form .= '</form>';
                if (! empty($cloud_password)) {
                    if (configure_user_cloud($config, $login['cn'], $cloud_password, $html)) {
                        $html .= "<p>L'accès aux dossier cloud " . $config['cloud_uri'] . " est maintenant configuré.<p>";
                    } else {
                        $html .= "<p>Le mot de passe est incorrect, veuillez recommencer<p>";
                        $html .= $html_form;
                    }
                } else {
                    $html .= "<center>Par mesure de sécurité l'accès automatique aux partages depuis les postes de l'établissement nécessite une nouvelle authentification";
                    $html .= $html_form;
                }
                $html .= '</center>';
            } else {
                $html .= '<center>';
                if (! empty($reconnect)) {
                    configure_user_cloud($config, $login['cn'], "", $html);
                }
                $cloud[] = $config['cloud_name'];
                $html .= "<center>Connexion activée. En cas de souci, si vous n'arrivez pas à accéder à vos dossiers cours et devoirs cliquez ci dessous.</center>";
                $html_form = '<form name="cloudpwd" action="" method="post">';
                $html_form .= '<input type="submit" name="cloud_reconnect" value="Reconnecter">';
                $html_form .= '</form>';
                $html .= $html_form;
                $html .= '</center>';
            }
        } else {
            $html .= "<center>L'accès n'est pas encore disponible.</center>";
        }
    }
    return $html;
}

function cloud_apps_access($config, $user, &$cloud = [])
{
    $html = "";
    if (! is_eleve($config, $user['cn']) && (! isset($config['etab_ou']) || (isset($config['etab_ou']) && $config['etab_ou'] == $_SESSION['etab'])) && ! isset($config['no_nuage_apps']) && ! isset($config['no_cloud'])) {
        $html = '<center>';
        $html .= "<H1>Profs : Accès à <a href='https://apps.education.fr'><img src='../elements/images/Logo-apps.svg' style='height:30px;' alt='portail.apps.education.fr'></a></H1>";

        $data = unserialize($_POST['data'] ?? NULL);
        $nuage = $_POST['nuage'] ?? "";
        $init = $_POST['init'] ?? "0";

        $conf = get_rclone_config($config, $user['cn']);
        if (isset($conf['nuage.apps']) && $init == 0) {
            $config['cloud_uri'] = parse_url($conf['nuage.apps']['url'], PHP_URL_SCHEME) . "://" . parse_url($conf['nuage.apps']['url'], PHP_URL_HOST) . "/";
            // $config['cloud_uri'] = "https://nuage.apps.education.fr/";
            $config['cloud_type'] = $conf['nuage.apps']['type'];
            $config['cloud_vendor'] = $conf['nuage.apps']['vendor'];
            $config['cloud_name'] = 'nuage.apps';
            $config['cloud_api_password'] = $conf['nuage.apps']['pass'];
            $config['cloud_api_user'] = $conf['nuage.apps']['user'];
            $html .= '<p>Vous êtes connecté sur <a href="' . $config['cloud_uri'] . '">' . $config['cloud_uri'] . '</a>  avec le compte
<b>' . $config['cloud_api_user'] . '</b>.<br>
Vous trouverez sur votre bureau une icône <b>Mes documents en ligne</b> et en l\'ouvrant un dossier <b>nuage.apps</b> vous donnant accès à vos données.<p>';
            $html .= 'Vous pouvez réinitialiser cette connexion en cas de problème 
<form name="cloud" action="" method="post">
<input type=hidden name=nuage value="">
<input type=hidden name=init value="1">
<input type=submit name="valider" value="Réinitialiser">
</form><br>
Pour plus d&lsquo;explications sur la connexion de votre ordinateur personnel à ce cloud : <a href="https://magistere.education.fr/f709">Formation à l&lsquo;usage du cloud</a>
';
            $cloud[] = "nuage.apps";
        } else {
            $config['cloud_uri'] = "https://" . $nuage . ".apps.education.fr/";
            // $config['cloud_uri'] = "https://nuage.apps.education.fr/";
            $config['cloud_type'] = "webdav";
            $config['cloud_vendor'] = "nextcloud";
            $config['cloud_name'] = "nuage.apps";

            require_once (dirname(__FILE__) . '/../vendor/autoload.php');

            $opt = [
                'base_uri' => $config['cloud_uri'],
                'cookies' => true,
                'headers' => [
                    'OCS-APIRequest' => "true",
                    'User-Agent' => "SambaEdu"
                ],
                'http_errors' => false
            ];
            curl_proxy_options($config, $opt);
            $html .= '<form name="cloud" action="" method="post">';
            if (! $data) {
                if (empty($nuage)) {
                    // étape 1 : choix du serveur
                    $html .= "<p>Pour activer la connexion automatique avec votre espace personnel sur apps.education.fr,<br>
Vous devez au préalable avoir créé votre compte sur apps.education.fr, et activé l&lsquo;application \"nuage\".<br>
Choisissez le serveur auquel vous êtes connectés :<br>
<select name=nuage onchange=\"this.form.submit()\">
    <option value=''>Choisir votre serveur</option>
    <option value='nuage01'>nuage01</option>
    <option value='nuage02'>nuage02</option>
    <option value='nuage03'>nuage03</option>
    <option value='nuage04'>nuage04</option>
    <option value='nuage05'>nuage05</option>
    <option value='nuage06'>nuage06</option>
    <option value='nuage07'>nuage07</option>
    <option value='nuage08'>nuage08</option>
    <option value='nuage09'>nuage09</option>
</select>";
                } elseif (empty($data['poll']['token'])) {
                    // Etape 2 : récup de l'url de la page de login et du token associé
                    $client = new GuzzleHttp\Client($opt);
                    try {
                        $response = $client->request('POST', 'index.php/login/v2', []);
                        if ($response) {
                            // on ouvre la page de login
                            $data = json_decode((string) $response->getBody(), true);
                            $url = $data['login'];
                            $html .= "<input type=hidden name=data value=" . serialize($data) . ">
<p>Pour activer la connexion automatique avec votre espace personnel sur apps.education.fr, Entrez vos identifiants dans la fenêtre qui va apparaître.<br>
Vous devez au préalable avoir créé votre compte sur apps.education.fr, et activé l&lsquo;application \"nuage\".<br>
Le mot de passe saisi sert juste à la vérification et ne sera pas enregistré<p>
<script type=\"text/javascript\">
    function openOnce(url, target){
      var winref = window.open('', target, 'resizable=yes,height=500,width=500');
      if(winref.location.href === 'about:blank'){
         winref.location.href = url;
         var refresh = true;
      }else if(winref.location.href === '" . $url . "/#/'){
         var refresh = true;
      }else{
         var refresh = false;
      }
      return winref;
    }
    openOnce('" . $url . "', 'apps.education');
    var wait = setTimeout(document.cloud.submit(), 29000);
</script>
<input type=hidden name=data value=" . serialize($data) . ">
<input type=hidden name=nuage value=" . $nuage . ">
<input type=\"submit\" name=\"valider\" value=\"Valider\">";
                        } else {
                            $html .= "erreur, pas de réponse !";
                        }
                    } catch (\GuzzleHttp\Exception\RequestException | \GuzzleHttp\Exception\ConnectException $e) {
                        $html .= "erreur : " . print_r($e, true);
                    }
                }
            } else {
                $client = new GuzzleHttp\Client($opt);
                try {
                    $response = $client->request('POST', $data['poll']['endpoint'], [
                        'form_params' => [
                            'token' => $data['poll']['token']
                        ]
                    ]);
                    if ($response->getStatusCode() == 200) {
                        $data = json_decode((string) $response->getBody(), true);
                        $config['cloud_uri'] = $data['server'] . "/";
                        $info = [
                            'id' => $data['loginName'],
                            'code' => $data['appPassword']
                        ];
                        $res = create_rclone_config($config, $info, $user);
                        if ($res === true) {
                            $html .= "<p>Votre connexion à <a href='https://apps.education.fr'>apps.education.fr</a> a bien été activée.<br>
Vous trouverez sur votre bureau un dossier <b>nuage.apps</b> vous donnant accès à vos données.";
                        } else {
                            $html .= $res;
                        }
                    } elseif ($response->getStatusCode() == 404) {
                        $html .= "Attente de l'authentification sur nuage<br> 
<input type=hidden name=data value=" . serialize($data) . ">
<script type=\"text/javascript\">
    var wait = setTimeout(document.cloud.submit(), 29000)
</script>
<input type=hidden name=nuage value=" . $nuage . ">
<input type=\"submit\" name=\"valider\" value=\"Valider\">";
                    } else {
                        $html .= "erreur : " . $response->getStatusCode();
                    }
                } catch (\GuzzleHttp\Exception\ConnectException $e) {
                    $html .= "erreur : <pre>" . print_r($e, true) . "</pre>";
                }
            }
            $html .= '</form>';
        }
        $html .= '</center>';
    }

    return $html;
}

function cloud_drive_idf_access($config, $user, &$cloud = [])
{
    $html = "";
    if (isset($config['openent_uri']) && $config['openent_uri'] == "https://ent.iledefrance.fr/" && (! isset($config['etab_ou']) || (isset($config['etab_ou']) && $config['etab_ou'] == $_SESSION['etab'])) && ! isset($config['no_cloud_drive_idf']) && ! isset($config['no_cloud'])) {
        $html = '<center>';
        $html .= "<H1>Accès à <a href='https://drive.monlycee.net/'>drive.monlycee.net</a></H1>";
        $drive_idf = $_POST['drive_idf'] ?? "";
        $data = unserialize($_POST['data'] ?? NULL);
        $init = $_POST['init_cloud_drive_idf'] ?? "0";

        $conf = get_rclone_config($config, $user['cn']);
        if (isset($conf['drive.monlycee.net']) && $init == 0) {
            $config['cloud_uri'] = "https://drive.monlycee.net/";
            $config['cloud_type'] = "webdav";
            $config['cloud_vendor'] = "nextcloud";
            $config['cloud_name'] = 'drive.monlycee.net';
            $config['cloud_api_password'] = $conf['drive.monlycee.net']['pass'];
            $config['cloud_api_user'] = $conf['drive.monlycee.net']['user'];
            $html .= '<p>Vous êtes connecté sur <a href="' . $config['cloud_uri'] . '">' . $config['cloud_uri'] . '</a>  avec le compte
<b>' . $config['cloud_api_user'] . '</b>.<br>
Vous trouverez sur votre bureau une icône <b>Mes documents en ligne</b> et en l\'ouvrant un dossier <b>drive.monlycee.net</b> vous donnant accès à vos données.<p>';
            $html .= 'Vous pouvez réinitialiser cette connexion en cas de problème
<form name="cloud_drive_idf" action="" method="post">
<input type=hidden name=init_cloud_drive_idf value="1">
<input type=submit name="valider" value="Réinitialiser">
</form><br>
';
            $cloud[] = "drive.monlycee.net";
        } else {
            $config['cloud_uri'] = "https://drive.monlycee.net/";
            $config['cloud_type'] = "webdav";
            $config['cloud_vendor'] = "nextcloud";
            $config['cloud_name'] = "drive.monlycee.net";

            require_once (dirname(__FILE__) . '/../vendor/autoload.php');

            $opt = [
                'base_uri' => $config['cloud_uri'],
                'cookies' => true,
                'headers' => [
                    'OCS-APIRequest' => "true",
                    'User-Agent' => "SambaEdu"
                ],
                'http_errors' => false
            ];
            curl_proxy_options($config, $opt);
            $html .= '<form name="cloud_drive_idf" action="" method="post">';
            if (! $data) {
                if (empty($drive_idf)) {
                    // étape 1 :
                    $html .= "<p>Pour activer la connexion automatique avec votre espace personnel sur  <b>" . $config['cloud_uri'] . "</b>, cliquer sur<br>
<input type=submit name=drive_idf onchange=\"this.form.submit()\" value=\"Valider\">
";
                } elseif (empty($data['poll']['token'])) {
                    // Etape 2 : récup de l'url de la page de login et du token associé
                    $client = new GuzzleHttp\Client($opt);
                    try {
                        $response = $client->request('POST', 'index.php/login/v2', []);
                        if ($response) {
                            // on ouvre la page de login
                            $data = json_decode((string) $response->getBody(), true);
                            $url = $data['login'];
                            $html .= "<input type=hidden name=data value=" . serialize($data) . ">
<p>Pour activer la connexion automatique avec votre espace personnel sur <b>" . $config['cloud_uri'] . "</b>, Entrez vos identifiants dans la fenêtre qui va apparaître.<br>
Le mot de passe saisi sert juste à la vérification et ne sera pas enregistré<p>
<script type=\"text/javascript\">
    function openOnce(url, target){
      var winref = window.open('', target, 'resizable=yes,height=500,width=500');
      if(winref.location.href === 'about:blank'){
         winref.location.href = url;
         var refresh = true;
      }else if(winref.location.href === '" . $url . "/#/'){
         var refresh = true;
      }else{
         var refresh = false;
      }
      return winref;
    }
    openOnce('" . $url . "', 'drive.monlycee.net');
    var wait = setTimeout(document.cloud_drive_idf.submit(), 29000);
</script>
<input type=hidden name=data value=" . serialize($data) . ">
<input type=\"submit\" name=\"valider\" value=\"Valider\">";
                        } else {
                            $html .= "erreur, pas de réponse !";
                        }
                    } catch (\GuzzleHttp\Exception\RequestException | \GuzzleHttp\Exception\ConnectException $e) {
                        $html .= "erreur : " . print_r($e, true);
                    }
                }
            } else {
                $client = new GuzzleHttp\Client($opt);
                try {
                    $response = $client->request('POST', $data['poll']['endpoint'], [
                        'form_params' => [
                            'token' => $data['poll']['token']
                        ]
                    ]);
                    if ($response->getStatusCode() == 200) {
                        $data = json_decode((string) $response->getBody(), true);
                        $config['cloud_uri'] = $data['server'] . "/";
                        $info = [
                            'id' => $data['loginName'],
                            'code' => $data['appPassword']
                        ];
                        $res = create_rclone_config($config, $info, $user);
                        if ($res === true) {
                            $html .= "<p>Votre connexion à <a href='https://drive.monlycee.net'>drive.monlycee.net</a> a bien été activée.<br>
Vous trouverez sur votre bureau un dossier <b>drive.monlycee.net</b> vous donnant accès à vos données.";
                        } else {
                            $html .= $res;
                        }
                    } elseif ($response->getStatusCode() == 401 || $response->getStatusCode() == 404) {
                        $html .= "Attente de l'authentification sur drive.monlycee.net<br>
<input type=hidden name=data value=" . serialize($data) . ">
<script type=\"text/javascript\">
    var wait = setTimeout(document.cloud_drive_idf.submit(), 29000)
</script>
<input type=\"submit\" name=\"valider\" value=\"Valider\">";
                    } else {
                        $html .= "erreur : " . $response->getStatusCode();
                    }
                } catch (\GuzzleHttp\Exception\ConnectException $e) {
                    $html .= "erreur : <pre>" . print_r($e, true) . "</pre>";
                }
            }
            $html .= '</form>';
        }
        $html .= '</center>';
    }

    return $html;
}

function get_seafile_client_code($config, $user, $password)
{
    $info = false;
    require_once (dirname(__FILE__) . '/../vendor/autoload.php');
    $opt = [
        'base_uri' => $config['cloud_uri'],
        'cookies' => true
    ];
    curl_proxy_options($config, $opt);
    $client = new GuzzleHttp\Client($opt);
    try {
        $response = $client->request('POST', 'api2/auth-token/', [
            'form_params' => [
                'username' => $user . "@" . $config['domain'],
                'password' => $password
            ]
        ]);
        if ($response) {
            $res = json_decode((string) $response->getBody(), true);
            if ($res['token']) {
                $info['code'] = $res['token'];
                $info['id'] = $user;
            }
        }
    } catch (\GuzzleHttp\Exception\RequestException | \GuzzleHttp\Exception\ConnectException $e) {
        return false;
    }
    return $info;
}

function test_rclone_config($config, $login)
{
    // $cache = apcu_fetch("rclone_test");
    // if (! isset($cache[$login])) {
    $ret = 0;
    $rclone_config = "/home/$login/.config/rclone/rclone.conf";
    exec("sudo stat " . $rclone_config . " 2>&1", $out, $ret);
    if ($ret == 0) {
        $rclone = "rclone lsjson " . $config['cloud_name'] . ":/ --max-depth=1 --retries 1 --config=$rclone_config 2>/dev/null";
        exec(sudo_proxy($config) . $rclone, $out, $ret);
        $test = ($ret == 0);
    } else {
        $test = false;
    }
    // lock("rclone_test");
    // $cache = apcu_fetch("rclone_test");
    // if (! is_array($cache)) {
    // $cache = [];
    // }
    // $cache[$login] = $test;
    // apcu_store("rclone_test", $cache);
    // unlock("rclone_test");
    // }
    return $test;
}

function get_rclone_config($config, $user)
{
    $rclone_config = "/home/$user/.config/rclone/rclone.conf";
    $rclone = "rclone config dump --config=$rclone_config";
    exec(sudo_proxy($config) . $rclone, $out, $ret);
    if ($ret == 0) {
        return json_decode(implode(' ', $out), true);
    }
    return false;
}

function create_rclone_config($config, $info, $user)
{
    $rclone_config = "/home/" . $user['cn'] . "/.config/rclone/rclone.conf";
    system("sudo mkdir -p /home/" . trim($user['cn']) . "/.config/rclone/");
    system("sudo chown -R " . $user['cn'] . " /home/" . $user['cn'] . "/.config/");

    switch (@$config['cloud_type']) {
        case "seafile":
            $rclone = "rclone config create " . $config['cloud_name'] . " " . $config['cloud_type'] . " vendor=sambaedu url=\"" . cloud_path($config, $user['cn']) . "\" token=" . $info['code'] . " --obscure --config=$rclone_config";
            break;
        default:
            if (@$config['cloud_name'] != "nuage.apps" && @$config['cloud_name'] != "drive.monlycee.net") {
                $rclone = "rclone config create " . $config['cloud_name'] . " " . $config['cloud_type'] . " vendor=sambaedu url=\"" . cloud_path($config, urlencode($user['Id NC'])) . "\" user=\"" . $user['cn'] . "\" pass=" . @$info['code'] . " --obscure --config=$rclone_config";
            } else {
                $rclone = "rclone config create " . $config['cloud_name'] . " " . $config['cloud_type'] . " vendor=sambaedu url=\"" . cloud_path($config, urlencode($info['id'])) . "\" user=\"" . $info['id'] . "\" pass=" . @$info['code'] . " --obscure --config=$rclone_config";
            }
    }
    exec(sudo_proxy($config) . $rclone, $out, $ret);
    if ($ret == 0) {
        exec("sudo chown " . $user['cn'] . " $rclone_config");
        $c = false;
        foreach ($user['memberof'] as $m) {
            if (preg_match("/=Cloud,/", $m)) {
                $c = true;
                break;
            }
        }
        if (! $c) {
            groupaddmember($config, $user, "Cloud", false);
        }
        $ret = test_rclone_config($config, $user['cn']);
    } else {
        $ret = false;
    }
    return $ret;
}

/**
 * Configure l'ensemble des identifiants cloud pour l'utilisateur
 *
 * @param unknown $config
 * @param unknown $login
 * @param string $password
 * @param string $html
 * @param boolean $stack
 * @return boolean
 */
function configure_user_cloud($config, $login, $password = "", &$html = "", $stack = false)
{
    // if (false) { // empty($password) && test_rclone_config($config, $login)) {
    // $m = [];
    // $rclone_id = "";
    // $rclone = get_rclone_config($config, $login);
    // if (isset($rclone[$config['cloud_name']]['url'])) {
    // // faut-il tester la cnx rclone ?
    // preg_match("#^" . $config['cloud_uri'] . "remote.php/dav/files/(.*)$#", $rclone[$config['cloud_name']]['url'], $m);
    // if (! empty($m[1]) && test_rclone_config($config, $login)) {
    // $rclone_id = $m[1];
    // $user = search_user($config, $login, "all", true);
    // if (empty($user['Id NC']) || $rclone_id != ! $user['Id NC']) {
    // $ad = $user;
    // $ad['Id NC'] = $ad['Id NC'] ?? "";
    // $ent = [
    // 'Id NC' => $rclone_id
    // ];
    // $mod = [
    // 'ad' => $ad,
    // 'ent' => $ent
    // ];
    // modify_ad_user($config, $mod, $html);
    // $html .= "Id NC modifié pour " . $login . " : " . $rclone_id . "<br>";
    // }
    // $user['Id NC'] = $rclone_id;
    // } else {
    // $html .= "pas d'activation rclone pour " . $login . "<br>";
    // }
    // }
    // $ret = true;
    // } else {
    if ($stack) {
        lock('cloud');
        $cloud = apcu_fetch("cloud");
        if (! is_array($cloud)) {
            $cloud = [];
        }
        $cloud[] = [
            'login' => $login,
            'password' => $password
        ];
        apcu_store('cloud', $cloud);
        unlock('cloud');
        return true;
    }
    $user = search_user($config, $login, "all", true);
    $i = $user['Id NC'] ?? "";
    // $user = nc_users_ids($config, $user)[0];
    $ret = true;
    switch (@$config['cloud_type']) {
        case "seafile":
            // $code = $password;
            $info = get_seafile_client_code($config, $user, $password);
            break;
        case "webdav":
            $info = get_nc_client_code($config, $user['cn'], $password);
            if ($info) {
                $user['Id NC'] = $info['id'];
                $info['storageLocation'] = cloud_path($config, $user['Id NC']);
            }
            break;
        default:
            $info = false;
    }
    if ($info) {
        $ret = create_rclone_config($config, $info, $user);
    } else {
        $ret = false;
    }
    // }
    if ($ret && $user['Id NC'] != $i) {
        $ad = $user;
        $ad['Id NC'] = $ad['Id NC'] ?? "";
        $ent = [
            'Id NC' => $user['Id NC']
        ];
        $mod = [
            'ad' => $ad,
            'ent' => $ent
        ];
        $ret = modify_ad_user($config, $mod, $html, false);
    }
    if ($ret) {
        $html .= "Activation cloud pour " . $login . "<br>";
    } else {
        $html .= "pas d'activation cloud pour " . $login . "<br>";
    }

    return $ret;
}

function apply_user_cloud($config, $tmax = 30, &$html = "")
{
    lock('cloud');
    $cloud = apcu_fetch("cloud");
    if (! is_array($cloud)) {
        $cloud = [];
    } else {
        apcu_delete('cloud');
    }
    unlock('cloud');
    $t = time();
    while (count($cloud) > 0 && (time() - $t) < $tmax) {
        $c = array_shift($cloud);
        configure_user_cloud($config, $c['login'], $c['password'], $html);
    }
    lock('cloud');
    if (is_array($nc = apcu_fetch("cloud"))) {
        $cloud = array_merge($cloud, $nc);
    }
    $count = count($cloud);
    apcu_store('cloud', $cloud);
    unlock('cloud');
    return $count;
}

function get_nc_userdata($config, $user)
{
    $data = false;
    require_once (dirname(__FILE__) . '/../vendor/autoload.php');
    $opt = [
        'base_uri' => $config['cloud_uri'],
        'cookies' => true,
        'headers' => [
            'OCS-APIRequest' => "true"
        ]
    ];
    curl_proxy_options($config, $opt);
    $client = new GuzzleHttp\Client($opt);
    try {
        $response = $client->request('GET', 'ocs/v2.php/cloud/users/' . $user, [
            'auth' => [
                $config['cloud_api_user'],
                $config['cloud_api_password']
            ],
            'query' => [
                'format' => 'json'
            ]
        ]);
        if ($response) {
            $res = json_decode((string) $response->getBody(), true);
            if ($res['ocs']['meta']['status'] == "ok") {
                $data = $res['ocs']['data'];
            }
        }
    } catch (\GuzzleHttp\Exception\RequestException | \GuzzleHttp\Exception\ConnectException $e) {
        return false;
    }
    return $data;
}

/**
 * crée un utilisateur API pour un site et renvoie son id et code api.
 * le résultat est enegistreré dans un fichier pour déploient ultérieur sur les sites.
 * ne pas utiliser pour autre chose que créer un compte api ! et uniquement sur le
 * serv central lors du provisionnement des sites
 *
 * @param unknown $config
 * @param unknown $user
 *            (uai du site)
 * @return boolean
 */
function create_nc_api_user($config, $user)
{
    $name = substr($user, 3);
    if (! file_exists("/etc/sambaedu/cloud/" . $user . ".conf")) {
        $password = create_random_password(20, true, false);
        require_once (dirname(__FILE__) . '/../vendor/autoload.php');
        $opt = [
            'base_uri' => $config['cloud_uri'],
            'cookies' => true,
            'headers' => [
                'OCS-APIRequest' => "true"
            ]
        ];
        curl_proxy_options($config, $opt);
        $client = new GuzzleHttp\Client($opt);
        try {
            $response = $client->request('POST', 'ocs/v1.php/cloud/users', [
                'auth' => [
                    $config['cloud_api_user'],
                    $config['cloud_api_password']
                ],
                'form_params' => [
                    'userid' => "api-" . $name,
                    'password' => $password,
                    'displayName' => "Api " . $user,
                    'groups' => [
                        'admin'
                    ],
                    'quota' => "100GB",
                    'format' => 'json'
                ]
            ]);
            if ($response) {
                $res = json_decode((string) $response->getBody(), true);
                if ($res['ocs']['meta']['status'] == "ok") {
                    $info = get_nc_client_code($config, "api-" . $name, $password);
                    if ($info) {
                        $file = 'cloud_api_user = "api-' . $name . '"
cloud_type = "webdav"
cloud_name = "moncollege95"
cloud_password = "' . $password . '"
cloud_api_password = "' . $info['code'] . '"
cloud_api_id = "' . $info['id'] . '"
cloud_uri = "' . $config['cloud_uri'] . '"
';

                        return ! empty(file_put_contents("/etc/sambaedu/cloud/" . $user . ".conf", $file));
                    }
                }
            }
            return false;
        } catch (\GuzzleHttp\Exception\RequestException | \GuzzleHttp\Exception\ConnectException $e) {
            return false;
        }
    } else {
        return true;
    }
}

/**
 * Fixe le quota pour un utilisateur NC
 *
 * 0 illimité, 100MO, 1GO...
 *
 *
 * @param unknown $config
 * @param string $user
 * @param string $quota
 * @return boolean
 */
function set_nc_user_quota($config, $user, $quota = 0)
{
    require_once (dirname(__FILE__) . '/../vendor/autoload.php');
    $opt = [
        'base_uri' => $config['cloud_uri'],
        'cookies' => true,
        'headers' => [
            'OCS-APIRequest' => "true"
        ]
    ];
    curl_proxy_options($config, $opt);
    $client = new GuzzleHttp\Client($opt);
    try {
        $response = $client->request('PUT', 'ocs/v2.php/cloud/users/' . $user, [
            'auth' => [
                $config['cloud_api_user'],
                $config['cloud_api_password']
            ],
            'form_params' => [
                'key' => 'quota',
                'value' => $quota,
                'format' => 'json'
            ]
        ]);
        if ($response) {
            $res = json_decode((string) $response->getBody(), true);
            if ($res['ocs']['meta']['status'] == "ok") {
                return true;
            }
        }
        return false;
    } catch (\GuzzleHttp\Exception\RequestException | \GuzzleHttp\Exception\ConnectException $e) {
        return $e;
    }
}

/**
 * efface un utilisateur NC
 *
 *
 * @param unknown $config
 * @param string $user
 * @return boolean
 */
function delete_nc_user($config, $user)
{
    require_once (dirname(__FILE__) . '/../vendor/autoload.php');
    $opt = [
        'base_uri' => $config['cloud_uri'],
        'cookies' => true,
        'headers' => [
            'OCS-APIRequest' => "true"
        ]
    ];
    curl_proxy_options($config, $opt);
    $client = new GuzzleHttp\Client($opt);
    try {
        $response = $client->request('DELETE', 'ocs/v1.php/cloud/users/' . $user, [
            'auth' => [
                $config['cloud_api_user'],
                $config['cloud_api_password']
            ],
            'form_params' => [
                'format' => 'json'
            ]
        ]);
        if ($response) {
            $res = json_decode((string) $response->getBody(), true);
            if ($res['ocs']['meta']['status'] == "ok") {
                return true;
            }
        }
        return false;
    } catch (\GuzzleHttp\Exception\RequestException | \GuzzleHttp\Exception\ConnectException $e) {
        return $e;
    }
}

function search_nc_user($config, $user)
{
    if (is_array($user)) {
        $u = $user['cn'];
    } else {
        $u = $user;
    }

    if (! get_nc_userdata($config, $u)) {

        if (! is_array($user)) {
            $ad = search_user($config, $user);
        } else {
            $ad = $user;
        }
        if (! $ad) {
            return false;
        }
        $fullname = $ad['fullname'] ?? $ad['nom'] ?? explode('@', $ad['mail'])[0] ?? explode('@', $ad['userprincipalname'])[0];
        require_once (dirname(__FILE__) . '/../vendor/autoload.php');
        $opt = [
            'base_uri' => $config['cloud_uri'],
            'cookies' => true,
            'headers' => [
                'OCS-APIRequest' => "true"
            ]
        ];
        curl_proxy_options($config, $opt);
        $client = new GuzzleHttp\Client($opt);
        try {
            $response = $client->request('GET', 'ocs/v2.php/core/autocomplete/get', [
                'auth' => [
                    $config['cloud_api_user'],
                    $config['cloud_api_password']
                ],
                'query' => [
                    'format' => 'json',
                    'search' => $fullname,
                    'limit' => 1
                ]
            ]);
            if ($response) {
                $res = json_decode((string) $response->getBody(), true);
                if ($res['ocs']['meta']['status'] == "ok") {
                    $u = $res['ocs']['data'][0]['id'] ?? false;
                }
            }
        } catch (\GuzzleHttp\Exception\RequestException | \GuzzleHttp\Exception\ConnectException $e) {
            return false;
        }
    }
    return $u;
}

/**
 * met à jour dans l'ad les id NC si besoin.
 *
 * si user est passé on force le changement de Id NC dans l'AD
 *
 * @param unknown $config
 * @param string $user
 * @param number $bloc
 * @param string $html
 * @return mixed|unknown
 */
function nc_users_ids($config, $user = "", $bloc = 50, &$html = "")
{
    // $html = "";
    if (! empty($config['cloud_uri'])) {
        $force = false;
        $hash = $user;
        // apcu_delete(new APCUIterator('#^nc_#'));
        if (apcu_fetch("ldap_cache_invalid")) {
            apcu_delete(new APCUIterator('#^nc_#'));
            apcu_delete("ldap_cache_invalid");
        }
        $nc_users = apcu_fetch("nc_users_" . $hash);
        $reste = apcu_fetch('nc_reste_' . $hash);
        if ((is_array($nc_users) && count($nc_users) > 0) && (! is_array($reste) || count($reste) == 0)) {
            return $nc_users;
        }
        $time = time();
        if (! is_array($nc_users)) {
            $nc_users = [];
        }
        if (! is_array($reste) || count($reste) > 0) {
            // ils reste des comptes à traiter
            if (empty($user)) {
                if (! is_array($reste)) {
                    if (empty($config['etab_ou'])) {
                        // recherche de tous les utilisateurs
                        $reste = search_ad($config, "*", "user", $config['dn']['people']);
                    } else {
                        // recherche de tous les membres du groupe etab, car un utilisateur peut être localisé dans un autre etab
                        $reste = search_ad($config, "*", "memberof", $config['etab_ou']);
                    }
                }
            } else {
                $reste = [
                    $user
                ];
                $force = true;
            }
            foreach ($reste as $k => $user) {
                // on teste les données rclone : si le compte rclone est ok le compte sera fonctionnel pour NC, pas la peine d'en faire plus.
                $m = [];
                $rclone_id = "";
                $rclone = get_rclone_config($config, $user['cn']);
                if (isset($rclone[$config['cloud_name']]['url'])) {
                    // faut-il tester la cnx rclone ?
                    preg_match("#^" . $config['cloud_uri'] . "remote.php/dav/files/(.*)/?$#U", $rclone[$config['cloud_name']]['url'], $m);
                    if (! empty($m[1]) && test_rclone_config($config, $user['cn'])) {
                        $rclone_id = $m[1];
                        if (empty($user['Id NC']) || $rclone_id != $user['Id NC']) {
                            $ad = $user;
                            $ad['Id NC'] = $ad['Id NC'] ?? "";
                            $ent = [
                                'Id NC' => $rclone_id
                            ];
                            $mod = [
                                'ad' => $ad,
                                'ent' => $ent
                            ];
                            $html .= "Test Rclone : Id NC modifié pour " . $user['cn'] . " : " . $user['Id NC'] . "->" . $rclone_id . "<br>";
                            modify_ad_user($config, $mod, $html);
                        }
                        $user['Id NC'] = $rclone_id;
                    } else {
                        $html .= "pas d'activation rclone pour " . $user['cn'] . "<br>";
                    }
                }
                if (! $force && ! empty($user['Id NC'])) {
                    $nc_users = array_merge($nc_users, $user);
                    unset($reste[$k]);
                }
                if (time() > $time + 30) {
                    apcu_store("nc_users_" . $hash, $nc_users);
                    apcu_store("nc_reste_" . $hash, $reste);
                    $html .= "analyse rclone : " . count($nc_users) . " reste : " . count($reste) . " en " . (time() - $time) . " s<br>";
                    return count($reste);
                }
            }
        }
        if (! is_array($nc_users)) {
            $nc_users = [];
        }
        $todo = array_slice($reste, 0, $bloc);
        $reste = array_slice($reste, $bloc);
        // echo "a faire : " . count($todo) . " reste : " . count($reste) . "<br>";
        require_once (dirname(__FILE__) . '/../vendor/autoload.php');
        $opt = [
            'base_uri' => $config['cloud_uri'],
            'cookies' => true,
            'headers' => [
                'OCS-APIRequest' => "true"
            ]
        ];
        curl_proxy_options($config, $opt);
        $client = new GuzzleHttp\Client($opt);
        $requests = function ($todo, $reste) use ($client) {
            foreach ($todo as $k => $user) {
                yield $k => function () use ($client, $user) {
                    $config = [];
                    $config = get_config($config, true);
                    return $client->getAsync('ocs/v2.php/core/autocomplete/get', [
                        'auth' => [
                            $config['cloud_api_user'],
                            $config['cloud_api_password']
                        ],
                        'query' => [
                            'format' => 'json',
                            'search' => $user['cn'] . "@",
                            'limit' => 1
                        ],
                        'connect_timeout' => 2
                    ]);
                };
            }
        };

        $pool = new GuzzleHttp\Pool($client, $requests($todo, $reste, $html), [
            'concurrency' => 30,
            'fulfilled' => function (GuzzleHttp\Psr7\Response $response, $index) use (&$todo) {
                // this is delivered each successful response
                $res = json_decode((string) $response->getBody(), true);
                if ($res['ocs']['meta']['status'] == "ok") {
                    $todo[$index]['New Id NC'] = $res['ocs']['data'][0]['id'];
                    // echo $todo[$index]['cn'] . "->".$todo[$index]['Id NC'] . "<br>";
                }
            },
            'rejected' => function (GuzzleException $reason, $index) use (&$todo, &$reste, &$html) {
                // this is delivered each failed request
                // echo "erreur : " . $reason->getMessage();
                $reste[] = $todo[$index];
                $html .= $todo[$index]['cn'] . "-> à faire<br>";
                unset($todo[$index]);
            }
        ]);

        // Initiate the transfers and create a promise
        $promise = $pool->promise();

        // Force the pool of requests to complete.
        $promise->wait();
        foreach ($todo as $k => $user) {
            $user['Id NC'] = $user['Id NC'] ?? "";
            if (! empty($user['New Id NC']) && $user['New Id NC'] != $user['Id NC']) {
                $ad = $user;
                $ent = [
                    'Id NC' => $user['New Id NC']
                ];

                $mod = [
                    'ad' => $ad,
                    'ent' => $ent
                ];
                modify_ad_user($config, $mod, $html);
                $todo[$k]['Id NC'] = $user['New Id NC'];
                $html .= "Test NC : Id NC modifié pour " . $user['cn'] . " : " . $user['New Id NC'] . "<br>";
            }
        }
        $nc_users = array_merge($nc_users, $todo);
        apcu_store("nc_users_" . $hash, $nc_users);
        apcu_store("nc_reste_" . $hash, $reste);
        $html .= "fait : " . count($nc_users) . " reste : " . count($reste) . " en " . (time() - $time) . " s<br>";
        if (count($reste) == 0) {
            return $nc_users;
        } else {
            return count($reste);
        }
    }
}

define('OCS_GROUP', 1);
define('OCS_USER', 0);
define('OCS_PERM_RO', 1);
define('OCS_PERM_RW', 15); // RW pour les dossiers
define('OCS_PERM_W', 3);

// RW pour les fichiers
function get_nc_all_shares($config)
{
    $data = false;
    require_once (dirname(__FILE__) . '/../vendor/autoload.php');
    $opt = [
        'base_uri' => $config['cloud_uri'],
        'cookies' => true,
        'headers' => [
            'OCS-APIRequest' => "true"
        ]
    ];
    curl_proxy_options($config, $opt);
    $client = new GuzzleHttp\Client($opt);
    try {
        $response = $client->request('GET', 'ocs/v2.php/apps/files_sharing/api/v1/shares', [
            'auth' => [
                $config['cloud_api_user'],
                $config['cloud_api_password']
            ],
            'query' => [
                'format' => 'json'
            ]
        ]);
        if ($response) {
            $res = json_decode((string) $response->getBody(), true);
            if ($res['ocs']['meta']['status'] == "ok") {
                $data = $res['ocs']['data'];
            }
        }
    } catch (\GuzzleHttp\Exception\RequestException | \GuzzleHttp\Exception\ConnectException $e) {
        return $e;
    }
    return $data;
}

function get_nc_shares($config, $path, $sub = 'false', &$client = NULL)
{
    $data = false;
    require_once (dirname(__FILE__) . '/../vendor/autoload.php');
    if (! $client) {
        $opt = [
            'base_uri' => $config['cloud_uri'],
            'cookies' => true,
            'headers' => [
                'OCS-APIRequest' => "true"
            ]
        ];
        curl_proxy_options($config, $opt);
        $client = new GuzzleHttp\Client($opt);
    }
    try {
        $response = $client->request('GET', 'ocs/v2.php/apps/files_sharing/api/v1/shares', [
            'auth' => [
                $config['cloud_api_user'],
                $config['cloud_api_password']
            ],
            'query' => [
                'path' => $path,
                'format' => 'json',
                'subfiles' => $sub
            ]
        ]);
        if ($response) {
            $res = json_decode((string) $response->getBody(), true);
            if ($res['ocs']['meta']['status'] == "ok") {
                $data = $res['ocs']['data'];
            }
        }
    } catch (\GuzzleHttp\Exception\RequestException | \GuzzleHttp\Exception\ConnectException $e) {
        return false;
    }
    return $data;
}

function create_nc_share($config, $path, $share, &$data = [])
{
    $u = $share['share_with'];
    // if ($share['share_type'] == OCS_GROUP ){ // pourquoi ? c'est super lent !!! || $u = search_nc_user($config, $u)) {

    require_once (dirname(__FILE__) . '/../vendor/autoload.php');
    $opt = [
        'base_uri' => $config['cloud_uri'],
        'cookies' => true,
        'headers' => [
            'OCS-APIRequest' => "true"
        ]
    ];
    curl_proxy_options($config, $opt);
    $client = new GuzzleHttp\Client($opt);
    try {
        $response = $client->request('POST', 'ocs/v2.php/apps/files_sharing/api/v1/shares', [
            'auth' => [
                $config['cloud_api_user'],
                $config['cloud_api_password']
            ],
            'form_params' => [
                'path' => $path,
                'shareType' => $share['share_type'],
                'shareWith' => $u,
                'permissions' => $share['permissions'],
                'format' => 'json'
            ]
        ]);
        if ($response) {
            $res = json_decode((string) $response->getBody(), true);
            $data = $res['ocs']['data'];
            if ($res['ocs']['meta']['status'] == "ok") {
                return true;
            } else {
                return false;
            }
        }
    } catch (\GuzzleHttp\Exception\RequestException | \GuzzleHttp\Exception\ConnectException $e) {
        return false;
    }
    return true;
    // } else {
    // return false;
    // }
}

/**
 * change les permissions pour un partage nc
 *
 * @param unknown $config
 * @param unknown $id
 * @param unknown $share
 * @param array $data
 * @return boolean
 */
function modify_nc_share($config, $id, $share, &$data = [])
{
    // if ($u = search_nc_user($config, $share['share_with'])) {
    require_once (dirname(__FILE__) . '/../vendor/autoload.php');
    $opt = [
        'base_uri' => $config['cloud_uri'],
        'cookies' => true,
        'headers' => [
            'OCS-APIRequest' => "true"
        ]
    ];
    curl_proxy_options($config, $opt);
    $client = new GuzzleHttp\Client($opt);
    try {
        $response = $client->request('PUT', 'ocs/v2.php/apps/files_sharing/api/v1/shares/' . $id, [
            'auth' => [
                $config['cloud_api_user'],
                $config['cloud_api_password']
            ],
            'query' => [
                'permissions' => $share['permissions'],
                'format' => 'json'
            ]
        ]);
        if ($response) {
            $res = json_decode((string) $response->getBody(), true);
            $data = $res['ocs']['data'];
            if ($res['ocs']['meta']['status'] == "ok") {
                return true;
            } else {
                return false;
            }
        }
    } catch (\GuzzleHttp\Exception\RequestException | \GuzzleHttp\Exception\ConnectException $e) {
        return false;
    }
    return true;
}

function delete_nc_share($config, $id)
{
    require_once (dirname(__FILE__) . '/../vendor/autoload.php');
    $opt = [
        'base_uri' => $config['cloud_uri'],
        'cookies' => true,
        'headers' => [
            'OCS-APIRequest' => "true"
        ]
    ];
    curl_proxy_options($config, $opt);
    $client = new GuzzleHttp\Client($opt);
    try {
        $response = $client->request('DELETE', 'ocs/v2.php/apps/files_sharing/api/v1/shares/' . $id, [
            'auth' => [
                $config['cloud_api_user'],
                $config['cloud_api_password']
            ],
            'query' => [
                'format' => 'json'
            ]
        ]);
        if ($response) {
            $res = json_decode((string) $response->getBody(), true);
            if ($res['ocs']['meta']['status'] == "ok") {
                return true;
            } else {
                return false;
            }
        }
    } catch (\GuzzleHttp\Exception\RequestException | \GuzzleHttp\Exception\ConnectException $e) {
        return false;
    }
    return true;
}

// webdav
function dav_proxy_options($config, &$opt = [])
{
    if (! empty($config['server_proxy']) && ! empty($config['proxy_address']) && ! empty($config['proxy_port'])) {
        $opt['proxy'] = "http://" . $config['proxy_address'] . ":" . $config['proxy_port'];
    }
}

/**
 * liste l'ensemble des dossiers (pas les fichiers !) avec leurs partages éventuels
 *
 * fonction destinée à la synchro des dossiers groupes. Pas forcément adaptée à autre chose.
 * retourne un tableau avec les élements compatibles avec la création ou la suppression.
 *
 * @param unknown $config
 * @param string $path
 * @param unknown $client
 * @param number $depth
 * @return \Sabre\HTTP\ClientHttpException|array
 */
function list_nc_files($config, $path = "", &$client = NULL, $depth = 3)
{
    require_once (dirname(__FILE__) . '/../vendor/autoload.php');

    $user = $config['cloud_api_user'];

    if (! $client) {

        $opt = array(
            'baseUri' => cloud_path($config, $user),
            // 'userName' => $user,
            'userName' => $config['cloud_api_user'],
            'password' => $config['cloud_api_password']
        );
        dav_proxy_options($config, $opt);
        $client = new Sabre\DAV\Client($opt);
    }

    try {
        $data = $client->propfind(str_replace(" ", "%20", ltrim($path, "/")), array(
            '{http://owncloud.org/ns}share-types'
            // '{http://owncloud.org/ns}owner-display-name',
            // '{http://owncloud.org/ns}fileid'
        ), $depth);
        // return $data;
    } catch (Sabre\HTTP\ClientException | Sabre\HTTP\ClientHttpException $e) {
        return $e;
    }
    // $shares = get_nc_all_shares($config);
    $list = [];
    foreach ($data as $dav_path => $info) {
        $dav_path = urldecode($dav_path);
        $m = [];
        preg_match("!^/remote\.php/dav/files/" . $user . "(/?" . $path . ".*)/$!", $dav_path, $m);
        $relative_path = $m[1] ?? "";
        if (! empty($relative_path)) {
            if (! empty($info['{http://owncloud.org/ns}share-types'])) {
                $shares = get_nc_shares($config, $relative_path, 'false');
                foreach ($shares as $s) {
                    $list[$relative_path][$s['id']] = [
                        'type' => $s['share_type'],
                        'permissions' => $s['permissions'],
                        'share_with' => $s['share_with']
                    ];
                }
            } else {
                // pas de partage défini : héritage du parent
                $list[$relative_path] = [];
            }
        }
    }
    uksort($list, 'path_sort');
    return $list;
}

/**
 * liste l'ensemble des fichiers, mais sans les partages eventuels
 *
 * @param unknown $config
 * @param string $path
 * @param unknown $client
 * @param number $depth
 * @return \Sabre\HTTP\ClientHttpException|array
 */
function list_nc_folders($config, $path = "", $depth = 3, &$client = NULL)
{
    require_once (dirname(__FILE__) . '/../vendor/autoload.php');
    if (! $client) {

        $user = $config['cloud_api_user'];
        $opt = array(
            'baseUri' => cloud_path($config, $user),
            // 'userName' => $user,
            'userName' => $config['cloud_api_user'],
            'password' => $config['cloud_api_password']
        );
        dav_proxy_options($config, $opt);
        $client = new Sabre\DAV\Client($opt);
    }

    try {
        $data = $client->propfind(str_replace(" ", "%20", ltrim($path, "/")), array(
            // '{http://owncloud.org/ns}share-types'
            // '{http://owncloud.org/ns}owner-display-name',
            // '{http://owncloud.org/ns}fileid',
            '{DAV:}getcontentlength'
            // '{http://owncloud.org/ns}permissions'
        ), $depth);
        // return $data;
    } catch (Sabre\HTTP\ClientException | Sabre\HTTP\ClientHttpException $e) {
        return $e;
    }
    uksort($data, 'path_sort');
    return $data;
}

function create_nc_folder($config, $path, &$client = NULL)
{
    require_once (dirname(__FILE__) . '/../vendor/autoload.php');

    if (! $client) {
        $user = $config['cloud_api_user'];
        $opt = array(
            'baseUri' => cloud_path($config, $user),
            'userName' => $user,
            'password' => $config['cloud_api_password']
        );
        dav_proxy_options($config, $opt);
        $client = new Sabre\DAV\Client($opt);
    }

    try {
        $data = $client->request('MKCOL', str_replace(" ", "%20", ltrim($path, "/")));
        if ($data['statusCode'] == '201') {
            return true;
        } else {
            return $data['statusCode'];
        }
    } catch (Sabre\HTTP\ClientException | Sabre\HTTP\ClientHttpException $e) {
        // $m = "Erreur :" . print_r($e, true);
        return false;
    }
}

function delete_nc_folder($config, $path, &$client = NULL)
{
    require_once (dirname(__FILE__) . '/../vendor/autoload.php');

    if (! $client) {
        $user = $config['cloud_api_user'];
        $opt = array(
            'baseUri' => cloud_path($config, $user),
            'userName' => $user,
            'password' => $config['cloud_api_password']
        );
        dav_proxy_options($config, $opt);
        $client = new Sabre\DAV\Client($opt);
    }
    try {
        $res = true;
        // récupération de la liste des fichiers avec 20 niveau de récursion
        $data = $client->propfind(str_replace(" ", "%20", ltrim($path, "/")), array(
            '{http://owncloud.org/ns}fileid'
        ), 20);
        if (is_array($data) && count($data)) {
            // tri des dossiers décroissant
            uksort($data, 'path_sort_desc');
            foreach ($data as $dav_path => $info) {
                $m = [];
                $dav_path = urldecode($dav_path);
                preg_match("!/remote\.php/dav/files/" . $user . "/(.*)$!", $dav_path, $m);
                $path = $m[1] ?? "";
                // effacement de l'objet
                $del = $client->request('DELETE', str_replace(" ", "%20", ltrim($path, "/")));
                $res &= ($del['statusCode'] == '201');
            }
        }
        return $res;
    } catch (Sabre\HTTP\ClientException | Sabre\HTTP\ClientHttpException $e) {
        return false;
    }
}

function copy_nc_file_recursive($config, $source, $dest, &$html, &$client = NULL)
{
    require_once (dirname(__FILE__) . '/../vendor/autoload.php');

    if (! $client) {
        $user = $config['cloud_api_user'];
        $opt = array(
            'baseUri' => cloud_path($config, $user),
            'userName' => $user,
            'password' => $config['cloud_api_password']
        );
        dav_proxy_options($config, $opt);
        $client = new Sabre\DAV\Client($opt);
    }
    try {
        $res = true;
        // récupération de la liste des fichiers avec 20 niveau de récursion
        $data = $client->propfind(str_replace(" ", "%20", ltrim($source, "/")), array(
            '{http://owncloud.org/ns}fileid'
        ), 20);
        if (is_array($data) && count($data)) {
            // tri des dossiers croissant
            uksort($data, 'path_sort');
            foreach ($data as $dav_path => $info) {
                $m = [];
                $dav_path = urldecode($dav_path);
                preg_match("!/remote\.php/dav/files/" . $user . "/(.*)$!", $dav_path, $m);
                $path = $m[1] ?? "";
                // copie de l'objet
                $data = $client->request('COPY', str_replace(" ", "%20", ltrim($path, "/")), NULL, [
                    'Destination' => cloud_path($config, $user) . str_replace(" ", "%20", ltrim($dest, "/")),
                    'Overwrite' => 'T'
                ]);
                $res &= ($data['statusCode'] == '201' || $data['statusCode'] == '204');
            }
        }
        return $res;
    } catch (Sabre\HTTP\ClientException | Sabre\HTTP\ClientHttpException $e) {
        return false;
    }
}

function copy_nc_file($config, $source, $dest, &$html, &$client = NULL)
{
    require_once (dirname(__FILE__) . '/../vendor/autoload.php');

    if (! $client) {
        $user = $config['cloud_api_user'];
        $opt = array(
            'baseUri' => cloud_path($config, $user),
            'userName' => $user,
            'password' => $config['cloud_api_password']
        );
        dav_proxy_options($config, $opt);
        $client = new Sabre\DAV\Client($opt);
    }
    try {
        $data = $client->request('COPY', str_replace(" ", "%20", ltrim($source, "/")), NULL, [
            'Destination' => cloud_path($config, $user) . str_replace(" ", "%20", ltrim($dest, "/")),
            'Overwrite' => 'T'
        ]);
        $res = ($data['statusCode'] == '201' || $data['statusCode'] == '204');
        return $res;
    } catch (Sabre\HTTP\ClientException | Sabre\HTTP\ClientHttpException $e) {
        return false;
    }
}

function move_nc_folder($config, $path, $newpath, &$client = NULL)
{
    require_once (dirname(__FILE__) . '/../vendor/autoload.php');
    if (! $client) {
        $user = $config['cloud_api_user'];
        $opt = array(
            'baseUri' => cloud_path($config, $user),
            'userName' => $user,
            'password' => $config['cloud_api_password']
        );
        dav_proxy_options($config, $opt);
        $client = new Sabre\DAV\Client($opt);
    }
    try {
        $data = $client->request('MOVE', str_replace(" ", "%20", ltrim($path, "/")), NULL, [
            'Destination' => cloud_path($config, $user) . str_replace(" ", "%20", ltrim($newpath, "/")),
            'Overwrite' => 'T'
        ]);
        if ($data['statusCode'] == '201' || $data['statusCode'] == '204') {
            return true;
        } else {
            return false;
        }
    } catch (Sabre\HTTP\ClientException | Sabre\DAV\Exception $e) {
        return false;
    }
}

/**
 *
 * @param unknown $config
 * @param unknown $file
 *            : /source/du fichier/fichier.ext
 * @param unknown $dest
 *            : destination/du/fichier.ext
 * @param unknown $client
 * @return boolean
 */
function put_nc_file($config, $file, $dest, &$html = "", &$client = NULL)
{
    require_once (dirname(__FILE__) . '/../vendor/autoload.php');
    if (! $client) {
        $user = $config['cloud_api_user'];
        $opt = array(
            'baseUri' => cloud_path($config, $user),
            'userName' => $user,
            'password' => $config['cloud_api_password']
        );
        dav_proxy_options($config, $opt);
        $client = new Sabre\DAV\Client($opt);
    }
    try {
        $handle = fopen($file, "r");
        $contents = fread($handle, filesize($file));
        fclose($handle);
        // $html .= "put_nc : " . $dest . " " . filesize($file) . "<br>";
        $data = $client->request('PUT', str_replace(" ", "%20", ltrim($dest, "/")), $contents);
        if ($data['statusCode'] == '201') {
            // $html .= "OK<br>";
            return true;
        } else {
            $html .= "<pre>" . print_r($data, true) . "</pre>";
            return true;
        }
    } catch (Sabre\HTTP\ClientException | Sabre\HTTP\ClientHttpException $e) {
        $html .= "Erreur :" . print_r($e, true);
        return false;
    }
}

/**
 *
 * @param unknown $config
 * @param unknown $source
 *            /chemin/du/dossier
 * @param unknown $dest
 *            dossier/de/destination (dossier devient destination)
 * @param unknown $client
 */
function put_nc_files_recursive($config, $source, $dest, &$html = "", &$client = NULL)
{
    $res = true;
    if (is_dir($source) === true) {
        $res &= create_nc_folder($config, $dest, $client);
        $html .= "create : " . $dest . "<br>";

        $files = new RecursiveIteratorIterator(new RecursiveDirectoryIterator($source), RecursiveIteratorIterator::SELF_FIRST);
        foreach ($files as $file) {
            // Ignore "." and ".." folders
            if (in_array(substr($file, strrpos($file, '/') + 1), array(
                '.',
                '..'
            ))) {
                continue;
            }
            $file = realpath($file);
            if (is_dir($file) === true) {
                $res &= create_nc_folder($config, str_replace($source, $dest, $file), $client);
                // $html .= "create : " . str_replace($source, $dest, $file) . "<br>";
            } elseif (is_file($file) === true) {
                $//html .= "put : " . $file . "=>" . str_replace($source, $dest, $file) . "<br>";
                $res &= put_nc_file($config, $file, str_replace($source, $dest, $file), $html, $client);
            }
        }
    }
    return $res;
}

// gestions des dossiers échange
function list_nc_echanges($config, $prof)
{
    $echanges = [];
    if (is_array($prof)) {
        $classes = $prof['memberof'];
    } else {
        $classes = search_user($config, $prof)['memberof'];
    }
    foreach ($classes as $classe) {
        $m = array();
        $profs = [];

        if (preg_match("/CN=(Equipe|Matiere)_(.*),OU=equipe.*/i", $classe, $m)) {
            $nom = $m[2];
            // if (empty($config['no_group_shares']) && $m[1] == "Equipe") {
            if ($m[1] == "Equipe") {
                $dir = "@" . $m[2];
                $matiere = $m[2];
                $root = "/Groupes";
                foreach (search_ad($config, $classe, "group")[0]['member'] as $p) {
                    $prof = search_user($config, $p);
                    if (preg_match("/" . $config['dn']['Profs'] . "/i", $prof['dn'])) {
                        $profs[] = $prof;
                    }
                }
                $classe = $m[2];
            } elseif (empty($config['no_matiere_shares']) && $m[1] == "Matiere") {
                list ($matiere, $classe) = explode("@", $nom);
                $dir = $nom;
                $root = "/Matieres";
                foreach (list_matiere_profs($config, $nom) as $p) {
                    $profs[] = search_user($config, $p);
                }
            } else {
                continue;
            }
            if (! empty($classe) && count(search_ad($config, $classe, "classe")) > 0) {
                $type = "Classe";
            } else {
                $type = "Cours";
            }
            $etab_path = $root . "/" . $dir . $config['suffix'] . "/_Echange@" . $nom;
            $res = get_nc_shares($config, $etab_path);
            $echanges[$nom] = [
                'id' => 0,
                'type' => $type,
                'classe' => $classe,
                'matiere' => $matiere,
                'etab_path' => $etab_path,
                'profs' => $profs
            ];
            if (is_array($res) && count($res) > 0) {
                foreach ($res as $s) {
                    if ($s['share_type'] == OCS_GROUP && $s['permissions'] == OCS_PERM_RW && $s['share_with'] == $type . "_" . $classe . $config['suffix']) {
                        $echanges[$nom]['id'] = $s['id'];
                        break;
                    }
                }
            }
        }
    }

    return $echanges;
}

function set_nc_echanges($config, &$echanges)
{
    $res = true;
    foreach ($echanges as $nom => $e) {
        if ($e['id'] == 0) {
            $res &= delete_nc_share($config, $e['id']);
        } else {
            $s = [
                'share_type' => OCS_GROUP,
                'share_with' => $e['type'] . "_" . $nom . $config['suffix'],
                'permissions' => OCS_PERM_RW
            ];
            $res &= create_nc_share($config, "/Groupes/@" . $nom . "/_Echange@" . $nom, $s);
        }
        $shares = get_nc_shares($config, "/Groupes/@" . $nom . "/_Echange@" . $nom);
        if (count($shares) > 0) {
            foreach ($shares as $s) {
                if ($s['share_type'] == OCS_GROUP && $s['permissions'] == OCS_PERM_RW && $s['share_with'] == $e['type'] . "_" . $nom . $config['suffix']) {
                    $echanges[$nom] = [
                        'id' => $s['id'],
                        'type' => $e['type']
                    ];
                    $res &= $res;
                    break;
                }
            }
        }
    }
    return $res;
}

/**
 * Bascule un partage échange entre l'état partage et non partagé
 *
 * @param unknown $config
 * @param unknown $nom
 * @param unknown $echanges
 *            liste des Echanges en cache
 * @return boolean
 */
function toggle_nc_echange($config, $nom, &$echanges, &$html)
{
    $res = false;
    $e = $echanges[$nom];
    if ($e['id'] != 0) {
        // on désactive le partage avec les élèves uniquement
        $res = delete_nc_share($config, $e['id']);
        $echanges[$nom]['id'] = 0;
    } else {
        // on active le partage RW avec les éleves
        $elements = explode("/", $e['etab_path']);
        $res1 = create_nc_folder($config, "/" . $elements[1]);
        $res1 = create_nc_folder($config, "/" . $elements[1] . "/" . $elements[2]);
        $res1 = create_nc_folder($config, $e['etab_path']);
        $s = [
            'share_type' => OCS_GROUP,
            'share_with' => $e['type'] . "_" . $e['classe'] . $config['suffix'],
            'permissions' => OCS_PERM_RW
        ];
        if ($res = create_nc_share($config, $e['etab_path'], $s)) {
            $res = get_nc_shares($config, $e['etab_path']);
            if (count($res) > 0) {
                foreach ($res as $s) {
                    if ($s['share_type'] == OCS_GROUP && $s['permissions'] == OCS_PERM_RW && $s['share_with'] == $e['type'] . "_" . $e['classe'] . $config['suffix']) {
                        $echanges[$nom]['id'] = $s['id'];
                        $classes = search_ad($config, $e['classe'], strtolower($e['type']));
                        // $html .= "<pre>" . print_r($classe, true) . "</pre>";
                        if (isset($classes[0]['member'])) {
                            foreach ($classes[0]['member'] as $eleve) {
                                $eleve = ldap_dn2cn($eleve);
                                if (test_rclone_config($config, $eleve)) {
                                    if (is_array($e['not_activated'])) {
                                        unset($e['not_activated'][array_search($eleve, $e['not_activated'])]);
                                    }
                                    rclone_move($config, $eleve, "/_Echange@" . $nom, "/Echanges/" . $e['matiere']);
                                } else {
                                    $e['not_activated'][] = $eleve;
                                }
                            }
                            $echanges[$nom]['not_activated'] = $e['not_activated'];
                        }
                    } elseif ($s['share_type'] == OCS_USER && $s['permissions'] == OCS_PERM_RW) {
                        if (($k = array_search($s['share_with'], array_column($e['profs'], 'Id NC')) !== false)) {
                            unset($e['profs'][$k]);
                        } else {
                            // suppr des partages à des utilisateurs ne faisant pas partie de l'équipe
                            delete_nc_share($config, $s['id']);
                        }
                    }
                }
                if (! empty($config['etab_ou'])) {
                    $etab_root = etab_to_name($config, $config['etab_ou']) . "/";
                } else {
                    $etab_root = "";
                }
                $user_path = "/Mes groupes/" . $etab_root . $nom . "/Echange";
                if (is_array($e['profs'])) {
                    foreach ($e['profs'] as $p) {
                        $s = [
                            'share_type' => OCS_USER,
                            'share_with' => $p['Id NC'],
                            'permissions' => OCS_PERM_RW
                        ];
                        $res = create_nc_share($config, $e['etab_path'], $s);
                        rclone_move($config, $p['cn'], "/_Echange@" . $nom, $user_path);
                    }
                }
            }
        }
    }
    return $res;
}

// gestions des dossiers devoirs

/**
 * liste les devoirs donnés par un prof, ainsi que leur état :
 * - desactive : non partagé avec les élèves.
 * On vérifie juste le nom du dossier
 * - active : sujet copié et partagé en écriture. on teste l'existence d'au moins un sous dossier partagé en ecriture
 * - verrouille : sujet rendu, visible en lecture seule pour les éleves. On test l'existence sous dossier.
 *
 * la présence éventuelle d'un sujet à distribuée doit faite en allant le chercher chez le prof, là où il l'a posé.
 * On le fait côté serveur avec rclone.
 *
 * cette fonction ne touche pas aux travaux eleves
 *
 *
 * @param unknown $config
 * @param unknown $prof
 * @return string[][]
 */
function list_nc_devoirs($config, $prof, &$html)
{
    $res = true;
    $devoirs = $classes = [];
    if (is_array($prof)) {
        $classes = $prof['memberof'];
    } else {
        $classes = search_user($config, $prof)['memberof'];
    }
    // recherche d'un sujet éventuel à distribuer
    // utilisation de rclone avec les identifiants de l'utilisateur
    if (! rclone_command($config, $prof['cn'])) {
        return false;
    }
    if (! empty($config['etab_ou'])) {
        $etab_root = etab_to_name($config, $config['etab_ou']) . "/";
    } else {
        $etab_root = "";
    }
    $rclone = "rclone lsjson \"" . $config['cloud_name'] . ":/Mes groupes/" . $etab_root . "\" --max-depth 4 --fast-list --exclude=@* --no-mimetype --no-modtime --retries-sleep 10s";
    $client = NULL;

    if (rclone_command($config, $prof['cn'], $rclone, $out)) {
        $my_files = json_decode(implode("\n", $out), true);
        // $html .= "<pre>" . print_r($my_files, true) . "</pre>";

        $out = [];
        // recherche des nouveau devoirs chez l'utilisateur
        foreach ($my_files as $mf) {
            $m = [];
            if (preg_match("#^(.*)@(.*)/Devoirs/((.*)(\..*)?)(/(.*)(\..*)?)?$#U", $mf['Path'], $m)) {
                if (! empty($m[4])) {
                    // $html .= "<pre>" . print_r($mf, true) . "</pre>";

                    $m[5] = $m[5] ?? "";
                    $devoirs[$m[4]]['matiere'] = $m[1];
                    $devoirs[$m[4]]['classe'] = str_replace($config['suffix'], "", $m[2]);
                    $devoirs[$m[4]]['sujet'] = $m[4] . $m[5];
                    $devoirs[$m[4]]['eleves'] = [];
                    $devoirs[$m[4]]['extension'] = $m[5] ?? "";
                    if (! empty($m[7]) && $m[7] == "Sujet") {
                        $devoirs[$m[4]]['del'] = true;
                    }
                    if ($m[4] . $m[5] == $mf['Name']) {
                        $devoirs[$m[4]]['dir'] = $mf['IsDir'];
                    }
                    // $html .= "<pre>" . print_r($devoirs[$m[4]], true) . "</pre>";
                }
            }
        }
        foreach ($devoirs as $nom => $devoir) {
            if (isset($devoir['del'])) {
                unset($devoirs[$nom]);
            }
        }
        if (count($devoirs) > 0) {
            foreach ($devoirs as $nom => $devoir) {
                $etab_path = "/Matieres/" . $devoir['matiere'] . "@" . $devoir['classe'] . $config['suffix'] . "/Devoirs/" . $nom;
                // un sujet est disponible dans le dossier : déplacement vers :/tmp, puis copie dans le dossier etab, partage avec le prof, puis déplacement du dossier du partage pour le prof.
                // c'est compliqué, mais pas le choix, sinon on perd le lien en cas de modif par le prof
                if ($devoir['dir']) {
                    // / un dossier sujet à récuperer est dispo : on le copie en vers /tmp.
                    $user_path = "/Mes groupes/" . $etab_root . $devoir['matiere'] . "@" . $devoir['classe'] . "/Devoirs/" . $devoir['sujet'];
                    $tmp_path = "/tmp/" . $devoir['sujet'];
                    $rclone = "rclone copy \"" . $config['cloud_name'] . ":" . $user_path . "\" \"" . $tmp_path . "\"";
                    $res &= rclone_command($config, $prof['cn'], $rclone, $out);
                    $rclone = "rclone delete \"" . $config['cloud_name'] . ":" . $user_path . "\" --rmdirs";
                    $res &= rclone_command($config, $prof['cn'], $rclone, $out);
                    // copie du dossier vers etab
                    $res &= create_nc_folder($config, $etab_path);
                    $res &= create_nc_folder($config, $etab_path . "/Sujet");
                    $res &= put_nc_files_recursive($config, $tmp_path, $etab_path . "/Sujet", $html, $client);
                    exec("sudo rm -fr " . $tmp_path);
                    // partage du sujet avec le prof
                    $s = [
                        'share_type' => OCS_USER,
                        'share_with' => $prof['Id NC'],
                        'permissions' => OCS_PERM_RW
                    ];
                    $res &= create_nc_share($config, $etab_path . "/Sujet", $s);
                    // déplacement du partage côté prof.
                    $rclone = "rclone move \"" . $config['cloud_name'] . ":/Sujet\" \"" . $config['cloud_name'] . ":" . $user_path . "/Sujet\"";
                    rclone_command($config, $prof['cn'], $rclone, $out);
                    // devoir déjà existant : on ne fait rien
                } else {
                    // un fichier sujet est dispo, on crée un dossier du même nom sans extension et on copie le fichier dedans sous le nom Sujet.xxx
                    $user_path = "/Mes groupes/" . $etab_root . $devoir['matiere'] . "@" . $devoir['classe'] . "/Devoirs/" . $devoir['sujet'];
                    $tmp_path = "/tmp/" . $devoir['sujet'];

                    $rclone = "rclone copyto \"" . $config['cloud_name'] . ":" . $user_path . "\" \"" . $tmp_path . "\"";
                    rclone_command($config, $prof['cn'], $rclone, $out);
                    $rclone = "rclone delete \"" . $config['cloud_name'] . ":" . $user_path . "\"";
                    rclone_command($config, $prof['cn'], $rclone, $out);
                    // copie du dossier vers etab
                    create_nc_folder($config, $etab_path, $client);
                    put_nc_file($config, $tmp_path, $etab_path . "/Sujet" . $devoir['extension'], $html, $client);
                    exec("sudo rm -f " . $tmp_path);
                    // partage du sujet avec le prof
                    $s = [
                        'share_type' => OCS_USER,
                        'share_with' => $prof['Id NC'],
                        'permissions' => OCS_PERM_RW
                    ];
                    $res &= create_nc_share($config, $etab_path . "/Sujet" . $devoir['extension'], $s);
                    // déplacement du partage côté prof.
                    $user_path = "/Mes groupes/" . $etab_root . $devoir['matiere'] . "@" . $devoir['classe'] . "/Devoirs/" . $nom;
                    rclone_move($config, $prof['cn'], "Sujet" . $devoir['extension'], $user_path . "/Sujet" . $devoir['extension']);
                }
            }
        }
    }
    // analyse côté serveur
    foreach ($classes as $classe) {
        $m = array();
        if (preg_match("/^CN=Matiere_((.*)@(.*)),OU=Equipe.*/i", $classe, $m)) {
            $mc = $m[1];
            $c = $m[3];
            $matiere = $m[2];
            if (count($g = search_ad($config, $c, "classe")) > 0 && isset($g[0]['member'])) {
                $type = "Classe";
                $nb = count($g[0]['member']);
            } elseif (count($g = search_ad($config, $c, "cours")) > 0 && isset($g[0]['member'])) {
                $type = "Cours";
                $nb = count($g[0]['member']);
            }
            $base = "Matieres/" . $mc . $config['suffix'] . "/Devoirs";
            // $html .= $base . "<br>";
            $res = list_nc_folders($config, $base, 3, $client);
            if (is_array($res) && count($res) > 0) {
                // $html .= "<pre>" . print_r($res, true) . "</pre>";
                foreach ($res as $p => $shares) {
                    $m = [];
                    $p = urldecode($p);
                    // $html .= "<pre>" . print_r($p, true) . "</pre>";
                    if (preg_match("#/remote\.php/dav/files/" . $config['cloud_api_user'] . "(/" . $base . "/(.*)/(.*)(/(.*)?|))$#U", $p, $m)) {
                        // $html .= "<pre>" . print_r($res, true) . "</pre>";
                        if (empty($m[5]) && ! empty($m[3])) {
                            $e = [];

                            if (preg_match("#^Sujet(\..*)?#", $m[3], $e)) {
                                $devoirs[$m[2]]['type'] = $type;
                                $devoirs[$m[2]]['effectif'] = $nb;
                                $devoirs[$m[2]]['classe'] = $c;
                                $devoirs[$m[2]]['matiere'] = $matiere;
                                $devoirs[$m[2]]['dir'] = ($m[3] == "Sujet");
                                $devoirs[$m[2]]['extension'] = $e[1] ?? "";
                                $devoirs[$m[2]]['not_activated'] = $devoirs[$m[2]]['not_activated'] ?? [];
                            } else {
                                // on est sur un fichier/dossier eleve matiere@classe@eleve
                                $mm = [];
                                if (preg_match("/^" . $matiere . "@" . $m[2] . "@(.*)$/U", $m[3], $mm)) {
                                    $devoirs[$m[2]]['eleves'][] = $mm[1];
                                }
                            }
                        }
                    }
                }
            }
        }
    }
    // assignaation du statut en fonction des droits
    foreach ($devoirs as $nom => $devoir) {
        if (! empty($nom)) {
            $devoir['eleves'] = $devoir['eleves'] ?? [];
            $devoirs[$nom]['eleves'] = $devoir['eleves'];
            if (count($devoir['eleves']) == 0) {
                $devoirs[$nom]['statut'] = "desactive";
            } else {
                if (! empty($devoir['extension'])) {
                    foreach ($devoir['eleves'] as $k => $eleve) {
                        $devoir['eleves'][$k] = str_replace($devoir['extension'], "", $eleve);
                    }
                }
                $devoirs[$nom]['eleves'] = $devoir['eleves'];
                $shares = get_nc_shares($config, "Matieres/" . $devoir['matiere'] . "@" . $devoir["classe"] . $config['suffix'] . "/Devoirs/" . $nom . "/" . $devoir['matiere'] . "@" . $nom . "@" . $devoir['eleves'][0] . $devoir['extension']);
                if (is_array($shares) && count($shares)) {
                    foreach ($shares as $s) {
                        if ($s['share_with'] != $prof['Id NC']) {
                            if ($s['permissions'] == OCS_PERM_RW || $s['permissions'] == OCS_PERM_W) {
                                $devoirs[$nom]['statut'] = "active";
                            } else {
                                $devoirs[$nom]['statut'] = "verrouille";
                            }
                            break;
                        }
                    }
                } else {
                    $devoirs[$nom]['statut'] = "verrouille";
                }
            }
        } else {
            $devoirs[$nom]['statut'] = "desactive";
        }
    }
    // $html .= "<pre>" . print_r($out, true) . "</pre>";
    // $html .= "<pre>" . print_r($devoirs, true) . "</pre>";

    return $devoirs;
}

/**
 * Bascule un partage devoirs entre les différents états
 *
 * On ne crée par un devoir : la création est faite par le prof en déposant un fichier ou un dossier nommé
 * "Sujet" (.extension dans le cas d'un fichier simple)
 * dans un dossier du nom du devoir dans le partage Devoirs de la matière de classe de son choix.
 *
 *
 *
 * si action = active : copie de sujet vers nom.prenom si existe pas déjà, et partage RW.
 * si action = verrouille : partage RW
 * si action = desactive : suppression partage
 * si action = supprime : suppression du dossier devoir complet
 *
 * @param unknown $config
 * @param
 *            array &$devoirs
 * @param string $nom
 * @param string $action
 *
 * @return boolean
 */
function toggle_nc_devoir($config, &$devoirs, $nom, $prof, $action, &$html = "")
{
    $res = true;
    $devoir = $devoirs[$nom];
    if ($devoir['dir']) {
        $rw = OCS_PERM_RW;
    } else {
        $rw = OCS_PERM_W;
    }

    // $html .= "<pre>" . print_r($devoir, true) . "</pre>";
    $classe = search_ad($config, $devoir['classe'], strtolower($devoir['type']));
    // $html .= "<pre>" . print_r($classe, true) . "</pre>";
    if (isset($classe[0]['member'])) {
        foreach ($classe[0]['member'] as $eleve) {
            $eleve = ldap_dn2cn($eleve);
            if (test_rclone_config($config, $eleve)) {
                $eleves[] = $eleve;
                unset($devoir['not_activated'][array_search($eleve, $devoir['not_activated'])]);
            } else {
                $devoir['not_activated'][] = $eleve;
            }
        }
        $devoirs[$nom]['not_activated'] = $devoir['not_activated'];
    }
    if (! empty($config['etab_ou'])) {
        $etab_root = etab_to_name($config, $config['etab_ou']) . "/";
    } else {
        $etab_root = "";
    }
    // emplacement du devoir pour l'etab
    $etab_path = "/Matieres/" . $devoir['matiere'] . "@" . $devoir['classe'] . $config['suffix'] . "/Devoirs/" . $nom;
    switch ($action) {
        case "active":
        case "rafraichir":
            // emplacement du dossier devoir pour le prof. C'est toujours un dossier.
            $prof_path = "/Mes groupes/" . $etab_root . $devoir['matiere'] . "@" . $devoir['classe'] . "/Devoirs/" . $nom;
            foreach (list_matiere_profs($config, $devoir['matiere'] . "@" . $devoir['classe']) as $p) {
                $profs[] = search_user($config, $p);
            }
            foreach ($eleves as $eleve) {
                $eleve_id = search_user($config, $eleve)['Id NC'];
                // l'élève voit le devoir sur /Mes devoirs/MATIERE/NOM(.ext)
                $eleve_path = "/Mes devoirs/" . $devoir['matiere'] . "/" . $nom . $devoir['extension'];
                $eleve_share_name = $devoir['matiere'] . "@" . $nom . "@" . $eleve . $devoir['extension'];
                // $html .= $eleve_path . "<br>";
                if (in_array($eleve, $devoir['eleves'])) {
                    $shares = get_nc_shares($config, $etab_path . "/" . $eleve_share_name);
                    // $html .= "<pre>" . print_r($shares, true) . "</pre>";
                    $ps = $profs;
                    if (is_array($shares) && count($shares) > 0) {
                        $mod = false;
                        foreach ($shares as $s) {
                            if ($s['share_type'] == OCS_USER && $s['permissions'] == OCS_PERM_RO && $s['share_with'] == $eleve_id) {
                                $s = [
                                    'share_type' => OCS_USER,
                                    'share_with' => $eleve_id,
                                    'permissions' => $rw,
                                    'id' => $s['id']
                                ];
                                modify_nc_share($config, $s['id'], $s);
                                $mod = true;
                                // break;
                            }
                            if ($k = array_search($s['share_with'], array_column($ps, "Id NC")) && $s['permissions'] == $rw) {
                                unset($ps[$k]);
                            }
                        }
                        if (! $mod) {
                            // partage désactivé
                            $s = [
                                'share_type' => OCS_USER,
                                'share_with' => $eleve_id,
                                'permissions' => $rw
                            ];
                            create_nc_share($config, $etab_path . "/" . $eleve_share_name, $s);
                            // déplacement côté élève
                            $res &= rclone_move($config, $eleve, $eleve_share_name, $eleve_path);
                        }
                        // en cas d'ajout d'un prof à la matiere
                        foreach ($ps as $p) {
                            $s = [
                                'share_type' => OCS_USER,
                                'share_with' => $p['Id NC'],
                                'permissions' => $rw
                            ];
                            $res = create_nc_share($config, $etab_path . "/" . $eleve_share_name, $s);
                            $res &= rclone_move($config, $p['cn'], $eleve_share_name, $prof_path . "/" . inverse_login($eleve) . $devoir['extension']);
                        }
                    }
                } else {
                    // on crée les dossiers eleves
                    // $html .= $etab_path . "/Sujet => " . $etab_path . "/" . $eleve . "<br>";
                    if ($devoir['dir']) {
                        copy_nc_file($config, $etab_path . "/Sujet", $etab_path . "/" . $eleve_share_name, $html);
                    } else {
                        copy_nc_file($config, $etab_path . "/Sujet" . $devoir['extension'], $etab_path . "/" . $eleve_share_name, $html);
                    }
                    // partage du sujet avec le prof et l'élève
                    foreach ($profs as $p) {
                        $s = [
                            'share_type' => OCS_USER,
                            'share_with' => $p['Id NC'],
                            'permissions' => $rw
                        ];
                        $res = create_nc_share($config, $etab_path . "/" . $eleve_share_name, $s);
                    }
                    $s = [
                        'share_type' => OCS_USER,
                        'share_with' => $eleve_id,
                        'permissions' => $rw
                    ];
                    $res &= create_nc_share($config, $etab_path . "/" . $eleve_share_name, $s);
                    // déplacement du partage du devoir eleve côté profs en le renommant nom.prenom.
                    foreach (list_matiere_profs($config, $devoir['matiere'] . "@" . $devoir['classe']) as $p) {
                        $res &= rclone_move($config, $p, $eleve_share_name, $prof_path . "/" . inverse_login($eleve) . $devoir['extension']);
                    }
                    // déplacement côté élève en le remommant nom(.ext)
                    $res &= rclone_move($config, $eleve, $eleve_share_name, $eleve_path);
                    $devoir['eleves'][] = $eleve;
                    $devoirs[$nom]['eleves'] = $devoir['eleves'];
                }
            }
            $devoirs[$nom]['statut'] = "active";
            break;
        case "verrouille":
            foreach ($eleves as $eleve) {
                $eleve_id = search_user($config, $eleve)['Id NC'];
                // l'élève voit le devoir sur /Mes devoirs/MATIERE/NOM(.ext)
                $eleve_path = "/Mes devoirs/" . $devoir['matiere'] . "/" . $nom . $devoir['extension'];
                $eleve_share_name = $devoir['matiere'] . "@" . $nom . "@" . $eleve . $devoir['extension'];
                if (in_array($eleve, $devoir['eleves'])) {
                    $shares = get_nc_shares($config, $etab_path . "/" . $eleve_share_name);
                    // on passe en RW
                    if (is_array($shares) && count($shares) > 0) {
                        foreach ($shares as $s) {
                            if ($s['share_type'] == OCS_USER && $s['permissions'] != OCS_PERM_RO && $s['share_with'] == $eleve_id) {
                                $s = [
                                    'share_type' => OCS_USER,
                                    'share_with' => $eleve_id,
                                    'permissions' => OCS_PERM_RO,
                                    'id' => $s['id']
                                ];
                                modify_nc_share($config, $s['id'], $s);
                                // déplacement côté élève est-ce utile ?
                                // $rclone = "rclone moveto \"" . $config['cloud_name'] . ":/" . $eleve . $devoir['extension'] . "\" \"" . $eleve_path . "\"";
                                // $res &= rclone_command($config, $eleve, $rclone, $out);
                                $mod = true;
                                break;
                            }
                        }
                        if (! $mod) {
                            // partage désactivé
                            $s = [
                                'share_type' => OCS_USER,
                                'share_with' => $eleve_id,
                                'permissions' => OCS_PERM_RO
                            ];
                            create_nc_share($config, $etab_path . "/" . $eleve_share_name, $s);
                            $res &= rclone_move($config, $eleve, $eleve_share_name, $eleve_path);
                        }
                    }
                }
            }
            $devoirs[$nom]['statut'] = "verrouille";
            break;
        case "desactive":
            foreach ($eleves as $eleve) {
                $eleve_id = search_user($config, $eleve)['Id NC'];
                // l'élève voit le devoir sur /Mes devoirs/MATIERE/NOM(.ext)
                $eleve_path = "/Mes devoirs/" . $devoir['matiere'] . "/" . $nom . $devoir['extension'];
                $eleve_share_name = $devoir['matiere'] . "@" . $nom . "@" . $eleve . $devoir['extension'];
                // $html .= $eleve_path . "<br>";
                if (in_array($eleve, $devoir['eleves'])) {
                    $shares = get_nc_shares($config, $etab_path . "/" . $eleve_share_name);
                    // on passe en RW
                    if (is_array($shares) && count($shares) > 0) {
                        foreach ($shares as $s) {
                            // suppression
                            if (is_array($shares) && count($shares) > 0) {
                                foreach ($shares as $s) {
                                    if ($s['share_type'] == OCS_USER && $s['share_with'] == $eleve_id) {
                                        delete_nc_share($config, $s['id']);
                                        break;
                                    }
                                }
                            }
                        }
                    }
                }
            }
            $devoirs[$nom]['statut'] = "desactive";
            break;
        case "supprime":
            // emplacement du dossier devoir pour le prof. C'est toujours un dossier.
            $prof_path = $config['cloud_name'] . ":/Mes groupes/" . $etab_root . $devoir['matiere'] . "@" . $devoir['classe'] . "/Devoirs/" . $nom;
            $rclone = "rclone delete --rmdirs \"" . $prof_path . "\"";
            $res &= rclone_command($config, $prof['cn'], $rclone, $out);
            $rclone = "rclone rmdirs \"" . $prof_path . "\"";
            $res &= rclone_command($config, $prof['cn'], $rclone, $out);
            delete_nc_folder($config, $etab_path);
            unset($devoirs['nom']);
    }
    return $res;
}

/**
 * Active ou met à jour un partage matiere@classe
 *
 * les dossiers cours seront créés et partagés avec les differents profs et eleves.
 *
 *
 *
 *
 * @param unknown $config
 * @param array $prof
 * @param string $nom
 * @param string $html
 *
 * @return array $unactivated_eleves
 *         liste des élèves n'ayant pas activé leur accès.
 */
function update_nc_matieres($config, $path, $prof, &$html = "")
{
    list ($m, $classe) = explode("@", $path);
    $matiere = str_replace("/Matieres/", "", $m);
    // nom avec suffix pour path etab
    $mc = $matiere . "@" . $classe . $config['suffix'];
    // nom affiché et ad
    $nom = $matiere . "@" . $classe;
    if (count($g = search_ad($config, $classe, "classe")) > 0 && isset($g[0]['member'])) {
        $type = "Classe";
        $nb = count($g[0]['member']);
        $eleves = $g[0]['member'];
    } elseif (count($g = search_ad($config, $classe, "cours")) > 0 && isset($g[0]['member'])) {
        $type = "Cours";
        $nb = count($g[0]['member']);
        $eleves = $g[0]['member'];
    }
    $unactivated_eleves = [];
    foreach ($eleves as $eleve) {
        $eleve = ldap_dn2cn($eleve);
        if (test_rclone_config($config, $eleve)) {
            $activated_eleves[] = $eleve;
            unset($unactivated_eleves[array_search($eleve, $unactivated_eleves)]);
        } else {
            $unactivated_eleves[] = $eleve;
        }
    }
    if (! rclone_command($config, $prof['cn'])) {
        return false;
    }
    if (! empty($config['etab_ou'])) {
        $etab_root = etab_to_name($config, $config['etab_ou']) . "/";
    } else {
        $etab_root = "";
    }
    // emplacement matieres pour l'etab
    $etab_path = "/Matieres/" . $mc;
    update_nc_partages($config, $html, false, $etab_path);
    // liste actuelle

    // déplacement des partages côté profs
    foreach (list_matiere_profs($config, $nom) as $prof) {
        rclone_move($config, $prof, "_Travail@" . $mc, "Mes groupes/" . $etab_root . $nom . "/Cours");
        $rclone = "rclone mkdir \"" . $config['cloud_name'] . ":/Mes groupes/" . $etab_root . $nom . "/Devoirs\"";
        rclone_command($config, $prof, $rclone, $out);
    }
    foreach ($activated_eleves as $eleve) {
        rclone_move($config, $eleve, "_Travail@" . $mc, "Mes cours/" . $matiere);
    }
    return $unactivated_eleves;
}

// gestion globale des partages
function path_sort($a, $b)
{
    return count(explode('/', $a)) - count(explode('/', $b));
}

function path_sort_desc($a, $b)
{
    return count(explode('/', $b)) - count(explode('/', $a));
}

/**
 * calcule le tableau des différences entre deux états des partages a et b
 *
 * Les partages sont listés à plat, le niveau de récursion est donc géré à l'étape de la génération de a et b.
 *
 * génère un tableau de la forme ['path' => ['id'=> [share]]]
 * si 'path'=> [], on déplace ou efface le dossier
 * si 'id' = 0, on crée le partage
 * si 'id' != 0 :
 * si share = [] on supprime le partage
 * sinon on le modifie
 *
 * Ne modifie pas le partage des dossiers Echange, mais les crée ou supprime si besoin.
 *
 * @param array $aa
 *            : état souhaité (appartenance aux groupes pour l'ajout)
 * @param array $bb
 *            : état à soustraire (Arbo NC pour l'ajout)
 * @param boolean $del
 *            : si true on prend en compte les dossiers dans archives pour la suppression
 * @return []
 */
function diff_shares($config, $aa, $bb, $remove = false, $del = false, $migrate = false)
{
    $diff = [];
    // recherche des dossiers à restaurer
    foreach ($bb as $pb => $b) {
        $m = [];
        // On regarde les dossiers dans archives uniquement
        if (preg_match("#^/Archives/([^/]*)@([^/]*)@(Archives|Groupes|Matieres)$#", $pb, $m)) {
            $prb = $pb;
            if ($m[3] == "Archives") {
                $m[3] = "Groupes";
            }
            $pb = "/" . $m[3] . "/@" . $m[2] . "/" . $m[1] . "@" . $m[2];
            foreach ($aa as $pa => $a) {
                if ($pa == $pb && ! $migrate) {
                    $diff[$prb] = $a;
                    // on ne tient pas compte du dossier pouvant exister
                    unset($bb[$pb]);
                    unset($aa[$pb]);
                }
            }
        }
    }
    foreach ($aa as $pa => $a) {
        $opa = preg_replace("/-[0-9]{4}[a-z]/", "", $pa);
        $m = false;
        // pas de prise en compte des dossiers déjà dans Archives pour la suppression,
        if (! preg_match("#/Archives/|/Direction/#", $pa) || $del) {
            foreach ($bb as $pb => $b) {
                // pas de prise en compte des suffixes pour le diff
                $opb = preg_replace("/-[0-9]{4}[a-z]/", "", $pb);
                if ($opa == $opb) {
                    if ($remove) {
                        // pas de prise en compte des partages pour la suppression
                        $m = true;
                    } elseif ($migrate) {
                        // transition laissé juste pour référence
                        if ($pa != $pb && preg_match("|^/Groupes/@.*-[0-9]{4}[a-z]$|", $pa)) {
                            $diff[$pa] = $a;
                        }
                        $m = true;
                    } else {
                        // pas de changement des partages pour Echanges et Devoirs, c'est géré ailleurs
                        if (preg_match("#/_Echange@|/Devoirs#", $pa)) {
                            $m = true;
                        } elseif (! preg_match("#/Direction/#", $pa)) {
                            // première étape : on retire les correspondances des partages des listes a et b
                            $m = true;
                            if (count($a) > 0) {
                                // au moins un partage voulu
                                foreach ($a as $ida => $sa) {
                                    if (count($b) > 0) {
                                        // au moins un partage existant
                                        foreach ($b as $idb => $sb) {
                                            // on compare le partage
                                            $s = array_diff($sa, $sb);
                                            if (count($s) == 0) {
                                                // partage identique : on retire les partages des listes pour la suite de l'analyse
                                                unset($b[$idb]);
                                                unset($a[$ida]);
                                                break;
                                            }
                                        }
                                    }
                                }
                            }
                            // deuxième étape : il reste des partages souhaités dans la liste a
                            // if (count($a) > 0) {
                            // au moins un partage voulu
                            // $m = true;
                            if (count($b) > 0) {
                                // au moins un partage existant
                                foreach ($b as $idb => $sb) {
                                    // on efface l'ancien partage $idb nc qui est != 0 si il contenait des infos de partage
                                    if (count($sb) > 0 && $idb != 0) {
                                        $diff[$pa][$idb] = []; // array vide pour effacer. Si tous les partages sont éffacés on est en heritage
                                    }
                                }
                                if (count($diff[$pa]) == 0 && count($a) == 0) {
                                    // heritage : on ne veut pas de partages
                                    $diff[$pa] = [];
                                }
                            }
                            // si $b est vide, on est sur un chemin qui hérite des droits du parent
                            // ajout : on ne fait rien
                            // soustraction : non plus
                            if (count($a) > 0) {
                                foreach ($a as $ida => $sa) {
                                    // on ajoute le partage demandé
                                    // ne sera pris en compte que pour l'ajout
                                    // if (count($sa) > 0 || $ida = "-") {
                                    $diff[$pa][$ida] = $sa;
                                    // }
                                }
                            }
                        }
                    }
                    break;
                } elseif (preg_match("#/_Echange@|/Devoirs#", $pa)) {
                    $m = true;
                }
            }
            // la correspondance n'a pas été trouvée précedemment, on ajoute donc le partage demandé
            if (! $m && ! $migrate) // && count($a) > 0)
                $diff[$pa] = $a;
        }
    }
    uksort($diff, 'path_sort');
    return $diff;
}

/**
 * mise à jour de l'arborescence du partage pour un groupe.
 * crée les dossiers manquants :
 * Groupes/@groupe -> partagé avec Equipe_groupe
 * Groupes/@groupe/eleve@groupe -> partagé avec eleve
 * déplace les dossiers en trop dans Archive/eleve@groupe@archive -> partagé avec eleve
 * restaure les dossiers Archive/eleve@groupe@archive -> Groupes/@groupe/eleve@groupe Si besoin
 *
 * La mise a jour de l'arborescence côté utilisateur est faite par un script de logon sur le montage rclone
 *
 * fonction agissant sur un cache
 *
 * @param array $config
 * @param string $html
 * @param bool $delete
 * @return boolean
 */
function update_nc_partages($config, &$html, $delete = false, $matiere = "")
{
    $end = false;
    $time = time();
    if (isset($config['etab_ou'])) {
        // par défaut on prend la conf cloud de l'etab défini par etab_ou
        $config = get_config($config, true, $config['etab_ou']);
        if (file_exists("/etc/sambaedu/cloud/" . $config['etab_ou'] . ".conf")) {
            // recup de la conf cloud de l'étab virtuel si il existe.
            $config = array_merge($config, parse_ini_file("/etc/sambaedu/cloud/" . $config['etab_ou'] . ".conf"));
        }
    } else {
        $config['etab_ou'] = 0;
    }
    $html .= $matiere . "<br>";
    if (empty($matiere)) {
        $partages = apcu_fetch('partages_' . $matiere . $config['etab_ou']);
    } else {
        $partages = false;
    }
    $client = NULL;
    if (! $partages) {
        $liste = list_nc_files($config, $matiere, $client, 3);
        $groupes = list_partages_groups($config, $matiere);
        if ($groupes === false) {
            return false;
        }
        $migrate = diff_shares($config, $groupes, $liste, false, false, true);
        $add = diff_shares($config, $groupes, $liste);
        $remove = []; // diff_shares($config, $liste, $groupes, true, $delete);
                       // $html .= count($remove) . " répertoires à supprimer <br>";
        $html .= count($add) . " répertoires à créer ou modifier<br>";
        // $html .= "<pre>" . print_r($remove, true) . "</pre>\n";
        // $html .= "<pre>" . print_r($add, true) . "</pre>\n";
        // return true;
    } else {
        $migrate = $partages['migrate'];
        $add = $partages['add'];
        $remove = $partages['remove'];
        $html .= count($remove) . " répertoires restants à supprimer<br>";
        $html .= count($add) . " répertoires restants à créer ou modifier<br>";
    }
    // if (empty($matiere)) {
    // return true;
    // }
    // on commence par renommer les dossiers avec le suffix si besoin.
    while (count($migrate) > 0 && time() < $time + 25) {
        $p = key($migrate);
        array_shift($migrate);
        $op = preg_replace("/-[0-9]{4}[a-z]/", "", $p);
        $html .= "deplacement de $op vers  $p <br>\n";
        $res = move_nc_folder($config, $op, $p);
        if ($res) {
            $html .= "OK<br>\n";
        } else {
            $html .= " ERREUR  $res<br>\n";
        }
    }
    while (count($remove) > 0 && time() < $time + 25) {
        $p = key($remove);
        array_shift($remove);
        $m = [];
        unset($n);
        preg_match("!/Groupes/@([^/]*)/?([^@]*)@?(.*)!", $p, $m);
        if (! empty($m[2])) {
            switch ($m[2]) {
                case "Prof":
                    $n = "/Groupes/@" . $m[1] . "/_Equipe@" . $m[3];
                    $html .= "deplacement de $p vers  $n <br>\n";
                    $res = move_nc_folder($config, $p, $n);
                    break;
                case "Echange":
                case "Travail":
                case "Equipe":
                    $n = "/Groupes/@" . $m[1] . "/_" . $m[2] . "@" . $m[3];
                    $html .= "deplacement de $p vers  $n <br>\n";
                    $res = move_nc_folder($config, $p, $n);
                    break;
                default:
                    $html .= "Archivage de $p vers  /Archives/" . $m[2] . "@" . $m[3] . "@Groupes<br>\n";
                    $res = move_nc_folder($config, $p, "/Archives/" . $m[2] . "@" . $m[3] . "@Groupes");
            }
            if ($res) {
                $html .= "OK<br>\n";
                if (isset($n)) {
                    unset($add[$n]); // $add[$n] ???
                }
            } else {
                $html .= " ERREUR  $res<br>\n";
            }
        } elseif (! empty($m[1])) {
            $html .= "effacement de $p <br>\n";
            $res = delete_nc_folder($config, $p);
            if ($res) {
                $html .= "OK<br>\n";
            } else {
                $html .= " ERREUR  $res<br>\n";
            }
        }
        if ($delete) {
            $m = [];
            preg_match("!/Archives/(.*)!", $p, $m);
            if (! empty($m[1])) {
                $html .= "effacement de $p  ";
                $res = delete_nc_folder($config, $p);
                if ($res) {
                    $html .= "OK<br>\n";
                } else {
                    $html .= " ERREUR  $res<br>\n";
                }
            }
        }
        $m = [];
        if (preg_match("!/Matieres/([^/]*)!", $p, $m)) {
            $html .= "Archivage de $p vers  /Archives/" . $m[1] . "@Matieres<br>\n";
            $res = move_nc_folder($config, $p, "/Archives/" . $m[1] . "@Matieres");
            if ($res) {
                $html .= "OK<br>\n";
            } else {
                $html .= " ERREUR  $res<br>\n";
            }
        }
    }
    $client = NULL;
    while (count($add) > 0 && time() < $time + 25) {
        $res = false;
        $p = array_key_first($add);
        $share = array_shift($add);
        $m = [];
        if (preg_match("#^/Archives/([^/]*)@([^/]*)@(Groupes|Matieres)$#", $p, $m)) {
            $tmp = $p . ".tmp";
            $newp = "/Groupes/@" . $m[2] . "/" . $m[1] . "@" . $m[2];
            // $res1 = delete_nc_folder($config, $newp);
            $res1 = move_nc_folder($config, $p, $tmp);
            $res1 &= move_nc_folder($config, $newp, $p);
            $res1 &= move_nc_folder($config, $tmp, $newp);
            if ($res1) {
                $html .= "restauration de $newp OK, <br>\n";
            } else {
                $html .= "restauration de $newp ERREUR<br>\n";
            }
        } elseif (! preg_match("#^/Groupes/@.*/$#", $p, $m)) {
            $res1 = create_nc_folder($config, $p, $client);
            if ($res1) {
                $msg = "creation de $p, \n";
            } else {
                $msg = "creation de $p ERREUR  $res1, \n";
            }
            if (is_array($share)) {
                $res = true;
                foreach ($share as $id => $s) {
                    if (count($s) == 0) {
                        if ($id != 0) {
                            $res &= delete_nc_share($config, $id);
                        }
                        // } else {
                        // $res &= modify_nc_share($config, $id, $s);
                        // }
                    } else {
                        $res &= create_nc_share($config, $p, $s);
                    }
                    // $html .= "<pre>" . print_r($data, true) . "</pre>\n";
                }
            }
            if ($res) {
                $html .= "partage de $p OK<br>\n";
            } else {
                if (! $res1) {
                    $html .= $msg;
                }
                $html .= "partage de $p ERREUR<br>\n";
                // $client2 = NULL;
                // $html .= "<pre>" . print_r(get_nc_shares($config, $p, $client2), true) . "</pre>\n";
            }
        }
    }
    $partages = [
        'add' => $add,
        'remove' => $remove,
        'migrate' => $migrate
    ];
    apcu_store('partages_' . $matiere . $config['etab_ou'], $partages, 1000);
    if (count($migrate) + count($add) + count($remove) == 0) {
        $end = true;
    }
    if ($end) {
        apcu_delete('partages_' . $matiere . $config['etab_ou']);
    }
    return $end;
}

/**
 * execute la commande rclone dans le contexte de l'utilisateur
 *
 *
 *
 * @param unknown $config
 * @param unknown $user
 *            : login
 * @param unknown $command
 *            : commande rclone. Si vide teste la présence d'une conf pour l'utilisateur
 * @return boolean
 */
function rclone_command($config, $user, $command = "", &$out = [])
{
    $ret = 0;
    $rclone_config = "/home/" . $user . "/.config/rclone/rclone.conf";
    if (empty($command)) {
        exec("sudo stat " . $rclone_config . " 2>&1", $out, $ret);
        if ($ret == 0) {
            $out = [];
        }
    } else {
        $rclone = $command . " --retries 1 --config " . $rclone_config . " 2>/dev/null";
        // $rclone = $command . " --retries 0 --retries-sleep 10s --config " . $rclone_config . " 2>/dev/null";
        exec(sudo_proxy($config) . $rclone, $out, $ret);
    }
    if ($ret == 0) {
        return true;
    } else {
        // trigger_error("rclone : " . $rclone . " : " . implode(" ", $out));
        return false;
    }
}

function rclone_file_exists($config, $user, $file)
{
    $rclone = "rclone lsjson --stat \"" . $config['cloud_name'] . ":/" . ltrim($file, "/") . "\"";
    return (rclone_command($config, $user, $rclone, $out));
}

/**
 * deplace un element (partage nc en général) de la racine vers le chemin indiqué
 *
 * automatisation de l'ensemble des opérations, et quelques optimisations.
 *
 * @param unknown $config
 * @param unknown $user
 * @param unknown $source
 *            element à la racine du home
 * @param unknown $dest
 *            chemin de la destinaton
 *            
 * @return boolean
 */
function rclone_move($config, $user, $source, $dest)
{
    $res = true;
    if (! rclone_file_exists($config, $user, $source) || rclone_file_exists($config, $user, $dest)) {
        return true;
    }
    $p = explode("/", pathinfo($dest)['dirname']);
    $path = "";
    foreach ($p as $d) {
        $path .= "/" . $d;
        $rclone = "rclone mkdir \"" . $config['cloud_name'] . ":" . $path . "\"";
        $res &= rclone_command($config, $user, $rclone, $out);
    }
    $rclone = "rclone moveto \"" . $config['cloud_name'] . ":/" . ltrim($source, "/") . "\" \"" . $config['cloud_name'] . ":/" . ltrim($dest, "/") . "\"";
    $res &= rclone_command($config, $user, $rclone, $out);
    return $res;
}

/**
 * met a jour le dossier perso de l'utilisateur sur NC
 *
 * Crée l'arborescence côté utilisateur, en renommant si besoin les liens de partages définis par le compte api-etab
 * Aucune donnée n'est modifiée par cette fonction.
 *
 * les modifications sont faites via rclone côté serveur, avec les identifiants de l'utilisateur. Cela suppose que son compte soit activé.
 *
 * motifs des fichiers à déplacer :
 * Matiere@Devoir@Login -> Eleve : /Mes devoirs/Matiere/Nom
 * _Travail@Matiere@Groupe -> Eleve /Mes cours/Matiere Prof : /Mes groupes/(etab)/matiere@classe/Cours
 * _Echange@Matiere@Groupe -> Eleve /Echanges/Matiere Prof : /Mes groupes/(etab)/matiere@classe/Echange
 *
 *
 * @param unknown $config
 * @param unknown $info
 * @return boolean
 */
function update_user_dir($config, $info)
{
    $rclone = "rclone lsjson " . $config['cloud_name'] . ":/ --max-depth 1 --no-mimetype --no-modtime";
    $ret = rclone_command($config, $info['user']['cn'], $rclone, $out);
    if ($ret) {
        if (! empty($config['etab_ou'])) {
            $etab_root = etab_to_name($config, $config['etab_ou']) . "/";
        } else {
            $etab_root = "";
        }

        $list = json_decode(implode("\n", $out), true);
        foreach ($list as $file) {
            $m = [];
            if (in_array('Eleves', $info['list_u'])) {
                if (preg_match("#(.*)@(.*)@(.*)(\..*)?$#", $file['Path'], $m)) {
                    switch ($m[1]) {
                        case "_Travail":
                            $ret = rclone_move($config, $info['user']['cn'], $m[0], "/Mes cours/" . $m[2]);
                            break;
                        case "_Echange":
                            $ret = rclone_move($config, $info['user']['cn'], $m[0], "/Echanges/" . $m[2]);
                            break;
                        default:
                            if ($m[3] == $info['user']['cn']) {
                                $ret = rclone_move($config, $info['user']['cn'], $m[0], "/Mes devoirs/" . $m[1] . "/" . $m[2] . $m[4]);
                            }
                    }
                }
            } elseif (in_array('Profs', $info['list_u'])) {
                if (preg_match("#(.*)@(.*)@(.*)(\..*)?$#", $file['Path'], $m)) {
                    switch ($m[1]) {
                        case "_Travail":
                            $ret = rclone_move($config, $info['user']['cn'], $m[0], "/Mes groupes/" . $etab_root . $m[2] . "@" . $m[3] . "/Cours");
                            break;
                        case "_Echange":
                            $ret = rclone_move($config, $info['user']['cn'], $m[0], "/Mes groupes/" . $etab_root . $m[2] . "@" . $m[3] . "/Echange");
                            break;
                        default:
                    }
                }
            }
        }
    }
    return $ret;
}

/**
 * met a jour le dossier perso de l'utilisateur sur NC pour les matieres
 *
 * cette cete fonction nécessite $info, généré à partir du script de logon.
 *
 *
 * Crée l'arborescence côté utilisateur, en renommant si besoin les liens de partages définis par le compte api-etab
 * Aucune donnée n'est modifiée par cette fonction.
 *
 * Les partages apparaissent à la racine de l'utilisateur, sous le forme $partage@$classe$suffix, on les bouge dans l'arborescence vue par l'utilisateur
 *
 * les modifications sont faites via rclone côté serveur, avec les identifiants de l'utilisateur. Cela suppose que son compte CLoud soit activé.
 *
 *
 * @param unknown $config
 * @param unknown $info
 * @return boolean
 */
function update_user_dir_matieres($config, $info)
{
    $file = "# script pour " . $info['user']['cn'] . "\n";
    $rclone_config = "/home/" . $info['user']['cn'] . "/.config/rclone/rclone.conf";
    exec("sudo stat " . $rclone_config . " 2>&1", $out, $ret);
    unset($out);
    if ($ret == 0) {
        $rclone = "rclone lsjson " . $config['cloud_name'] . ":/ --max-depth 2 --no-mimetype --no-modtime --retries-sleep 10s --config " . $rclone_config . " 2>/dev/null";
        $file .= $rclone . "\n";
        exec(sudo_proxy($config) . $rclone, $out, $ret);
        if ($ret == 0) {
            $file .= "out : " . implode("\n", $out) . "\n";
            $out = json_decode(implode("\n", $out), true);
            $file .= "json : " . print_r($out, true) . "\n";

            // cas d'un prof
            if (in_array('Profs', $info['list_u'])) {
                if (! in_array("Mes groupes", array_column($out, 'Path'))) {
                    $rclone = "rclone mkdir \"" . $config['cloud_name'] . ":/Mes groupes\"  --config=" . $rclone_config . " 2>/dev/null";
                    $file .= $rclone . "\n";
                    exec(sudo_proxy($config) . $rclone, $out1, $ret);
                }
                $file .= "groupes : " . print_r($info['list_ue'], true) . "\n";
                foreach ($info['list_ue'] as $groupe) { // groupes avec prefix
                    $m = [];
                    if (preg_match("/Matiere_(.*)@((.*)(-[0-9]{4}[a-z])|(.*))/", $groupe, $m)) {
                        if (! empty($m[4])) {
                            // multi-etab
                            $uai = etab_suffix2uai($config, $m[4]);
                            $nom = "/" . etab_to_name($config, $uai);
                            if (! in_array("Mes groupes/" . $nom, array_column($out, 'Path'))) {
                                // création du sous dossier au nom de l'étab (ETAB_XXX)
                                $rclone = "rclone mkdir \"" . $config['cloud_name'] . ":/Mes groupes" . $nom . "\"  --config=" . $rclone_config . " 2>/dev/null";
                                $file .= $rclone . "\n";
                                exec(sudo_proxy($config) . $rclone, $out1, $ret);
                            }
                        } else {
                            $nom = "";
                        }
                        $matiere_e = $m[1] . "@" . $m[2]; // nom avec etab
                        $matiere = $m[1] . "@" . $m[3]; // nom sans etab

                        if (in_array($matiere_e, array_column($out, 'Path'))) {
                            // nouveau partage
                            if (in_array("Mes groupes" . $nom . "/" . $matiere, array_column($out, 'Path'))) {
                                // un dossier existe déjà : ce n'est pas normal... on le renomme
                                $rclone = "rclone move \"" . $config['cloud_name'] . ":Mes groupes" . $nom . "/" . $matiere . "\" \"" . $config['cloud_name'] . ":Mes groupes" . $nom . "/" . $matiere . "_conflit\" --config=" . $rclone_config . " 2>/dev/null";
                                exec(sudo_proxy($config) . $rclone, $out1, $ret);
                            }
                            $rclone = "rclone move \"" . $config['cloud_name'] . $matiere_e . "\" \"" . $config['cloud_name'] . ":Mes groupes" . $nom . "/" . $matiere . "\"  --config=" . $rclone_config . " 2>/dev/null";
                            exec(sudo_proxy($config) . $rclone, $out1, $ret);
                        }
                    }
                }
            } elseif (in_array('Administratifs', $info['list_u'])) {
                // on ne fait rien pour le moment !
            } else {
                // eleve :
                if (! in_array("Mes travaux", array_column($out, 'Path'))) {
                    $rclone = "rclone mkdir \"" . $config['cloud_name'] . ":Mes travaux\"  --config=" . $rclone_config . " 2>/dev/null";
                    $file .= $rclone . "\n";

                    exec(sudo_proxy($config) . $rclone, $out1, $ret);
                }
                if (! in_array("Travail demandé", array_column($out, 'Path'))) {
                    $rclone = "rclone mkdir \"" . $config['cloud_name'] . ":Travail demandé\"  --config=" . $rclone_config . " 2>/dev/null";
                    $file .= $rclone . "\n";
                    exec(sudo_proxy($config) . $rclone, $out1, $ret);
                }
                if (empty($config['no_echange_shares']) && ! in_array("Echange", array_column($out, 'Path'))) {
                    $rclone = "rclone mkdir \"" . $config['cloud_name'] . ":Echange\"  --config=" . $rclone_config . " 2>/dev/null";
                    $file .= $rclone . "\n";
                    exec(sudo_proxy($config) . $rclone, $out1, $ret);
                }
                $file .= print_r($info, true) . "\n";
                $paths = array_column($out, 'Path');
                sort($paths, SORT_ASC);
                foreach ($paths as $p) {
                    $m = [];
                    if (preg_match("/^((Devoir_(.*)#(.*))|(.*))@(.*)(\..*)?$/", $p, $m)) { // dossier @ matiere
                        switch ($m[1]) {
                            case "_Travail":
                                $base = "Travail demandé/" . $m[6];
                            case "_Echange":
                                $base = "Echange/" . $m[6];
                                if (! in_array($base, array_column($out, 'Path'))) {
                                    $rclone = "rclone move \"" . $config['cloud_name'] . ":" . $p . "\" \"" . $config['cloud_name'] . ":" . $base . "\"  --config=" . $rclone_config . " 2>/dev/null";
                                    $file .= $rclone . "\n";
                                    exec(sudo_proxy($config) . $rclone, $out1, $ret);
                                }
                                break;
                            default: // par défaut c'est les partages Devoir_truc#machin@matiere => Mes travaux/matiere/truc
                                      // le partage est sur Devoir_truc#machin@matiere, pas sur le dossier matier
                                if ($m[4] == $info['user']['cn']) {
                                    $base = "Mes travaux/" . $m[6];
                                    if (! in_array($base, array_column($out, 'Path'))) {
                                        // on crée le dossier matiere si besoin
                                        $rclone = "rclone mkdir \"" . $config['cloud_name'] . ":Travail demandé\"  --config=" . $rclone_config . " 2>/dev/null";
                                        $file .= $rclone . "\n";
                                        exec(sudo_proxy($config) . $rclone, $out1, $ret);
                                    }
                                    // on renomme le devoir
                                    if (! in_array($base . "/" . $m[3] . $m[7], array_column($out, 'Path'))) {
                                        $rclone = "rclone move \"" . $config['cloud_name'] . ":" . $p . "\" \"" . $config['cloud_name'] . ":" . $base . "/" . $m[3] . $m[7] . "\"  --config=" . $rclone_config . " 2>/dev/null";
                                        $file .= $rclone . "\n";
                                        exec(sudo_proxy($config) . $rclone, $out1, $ret);
                                    }
                                }
                        }
                    }
                }
            }
            // file_put_contents("/tmp/rclone.log", $file);
            return true;
        } else {
            return false;
        }
    } else {
        return false;
    }
}

/**
 * génération du script qui va déplacer les dossiers partagés de l'élève dans son arborescence.
 *
 * commentaires OBSOLETE : la construction de l'arborescence côté utilisateur est maintenant faite en direct lors des modifications des partages
 *
 * Lancé par les scripts applications, une fois le partage monté par rclone.
 * Nextcloud place les dossiers partagés à la racine, on les déplace pour que cela soit compréhensible.
 * Pour éviter les collisions avec des fichiers de l'utilisateur, ils sont nommés :
 * Groupes/@groupe/Travail@groupe
 * /x@groupe
 * /...@groupe
 * En cas de suppression des dossiers par l'utilisateur, ils sont remis en place au logon suivant (ce ne sont que des liens vers les données réelles du NC)
 *
 * l'élève verra par exemple :
 * Mes travaux\1G1
 * \PhysSpe
 * Travail demandé\1G1
 * \PhysSpe
 * Echanges\1G1
 * \PhysSpe
 *
 *
 * le prof verra par exemple :
 * Mes groupes\1G1\...
 * \TSTI1\Profs@TSTI1
 * \Travail@TSTI1
 * \Echange@TSTI1
 * \x@TSTI1
 * \y@TSTI1
 * \NSI\...
 * \I2D\...
 *
 * Il n'est pas possible pour les profs de renommer les sous dossiers individuels car cela poserait des conflits pour les élèves. seuls les liens de partage peuvent être renommés.
 *
 * Les droits et la visibilité sont définis par l'API, et donc si le dossier échange par exemple est non partagé il ne sera pas vu et créé.
 *
 * Le montage est unique pour les profs.
 *
 * Pour les élèves un montage séparé est fait pour chaque sous-dossier afin de bien gérer les droits des partages
 *
 * @param unknown $config
 * @param unknown $id
 * @return string
 */
function genere_script_partage_utilisateur($config, $id)
{
    $perso = $config['cloud_perso_name'] ?? "Mes Documents";
    $script = "";
    // recup de l'info concernant l'utilisateur générée lors du logon
    $info = apcu_fetch("apps." . $id);
    if (! empty($info) && in_array("rclone", $info['liste_applications'])) {
        $ret = 0;
        $out = [];
        $rclone_config = "/home/" . $info['user']['cn'] . "/.config/rclone/rclone.conf";
        exec("sudo stat " . $rclone_config . " 2>&1", $out, $ret);
        if ($ret == 0) {
            $rclone = "rclone listremotes --config=$rclone_config 2>/dev/null";
            $remotes = [];
            exec(sudo_proxy($config) . $rclone, $remotes, $ret);
            $i = 0;
            foreach ($remotes as $remote) {
                if ($info['os'] == "windows") {
                    $path = '%userprofile%\\nuages';
                    $script .= "if not exist " . $path . " (md " . $path . ")\r\n";
                }
                $remote = rtrim($remote, ":");
                if (! empty($config['cloud_name']) && $remote == $config['cloud_name']) {
                    if ($ret = update_user_dir($config, $info)) {
                        $script .= rclone_mount($config, $info, $remote, "", $i);
                        // plus nécessaire : les partages devoirs echange et cours peuvent être accédés directement sans risque, vu qu'ils sont mis à jour sur le NC

                        // if (in_array('Profs', $info['list_u']) || in_array('Administratifs', $info['list_u'])) {
                        // //
                        // // profs : montage de la racine du home NC
                        // //
                        // $script .= rclone_mount($config, $info, $remote, "", $i);
                        // } else {
                        // //
                        // // eleves
                        // //
                        // if ($info['os'] == "windows") {
                        // $path = '%userprofile%\\nuages\\' . $config['cloud_name'];
                        // $script .= "md " . $path . "\r\n";
                        // } else {
                        // // $script .= "mkdir -p /home/\$USER/nuages/" . $config['cloud_name'] . "\n";
                        // }
                        // $script .= rclone_mount($config, $info, $remote, $perso, $i);
                        // $script .= rclone_mount($config, $info, $remote, "Mes travaux", $i);
                        // if (empty($config['no_echange_shares'])) {
                        // $script .= rclone_mount($config, $info, $remote, "Echange", $i);
                        // }
                        // $script .= rclone_mount($config, $info, $remote, "Travail demandé", $i, "--read-only");
                        // }
                    }
                } else {
                    $script .= rclone_mount($config, $info, $remote, "", $i);
                }
            }
        }
    }
    return $script;
}

function rclone_proxy($config, $info, $i)
{
    if ($info['os'] == "windows") {
        if ($config['proxy_type'] != "aucun") {
            $proxy = "echo set http_proxy=http://" . $config['proxy_address'] . ":" . $config['proxy_port'] . ">%temp%/rclone-$i.cmd\r\n";
            $proxy .= "echo set https_proxy=http://" . $config['proxy_address'] . ":" . $config['proxy_port'] . ">>%temp%/rclone-$i.cmd\r\n";
        } else {
            $proxy = "echo rem pas de proxy>%temp%/rclone-$i.cmd\r\n";
        }
    } else {
        if ($config['proxy_type'] != "aucun") {
            $proxy = "   Environment=http_proxy=http://" . $config['proxy_address'] . ":" . $config['proxy_port'] . "\n";
            $proxy .= "   Environment=https_proxy=http://" . $config['proxy_address'] . ":" . $config['proxy_port'] . "\n";
        } else {
            $proxy = "    # Pas de proxy\n";
        }
    }
    return $proxy;
}

function pin_script($path)
{
    $script = "pwsh.exe -ExecutionPolicy Bypass -NoLogo -NonInteractive -NoProfile -Command \"\$Q = New-Object -ComObject shell.application;";
    $script .= "\$Path = \$Q.Namespace('" . $path . "');\$Path.Self.Path;if(-not (\$Q.Namespace('shell:::{679f85cb-0220-4080-b29b-5540cc05aab6}').Items()";
    $script .= "| ? {\$_.Path -eq \$Path.Self.Path})){\$Path.Self.InvokeVerb('pintohome')}\"\r\n";
    return iconv("UTF-8", "Windows-1252", $script);
}

function rclone_mount($config, $info, $remote, $sub, &$i, $args = "")
{
    if ($info['os'] == "windows") {
        if (! empty($sub)) {
            $path = "%userprofile%\\nuages\\" . $remote . "\\" . $sub;
        } else {
            $path = "%userprofile%\\nuages\\" . $remote;
        }
        $script = "rem montage de " . $path . "\r\n";
        // $script .= iconv("UTF-8", "Windows-1252", "pwsh.exe -ExecutionPolicy Bypass -NoLogo -NonInteractive -NoProfile -Command \"\$o = new-object -com shell.application; \$u=\$o.Namespace('shell:::{679F85CB-0220-4080-B29B-5540CC05AAB6}').Items() | Where-Object { \$_.Name -EQ '" . $remote . "' }; if (\$u -ne \$null) {\$u.InvokeVerb('unpinfromhome')}\"\r\n");
        if (! empty($config['cloud_name']) && $remote == $config['cloud_name']) {
            if (in_array('Eleves', $info['list_u'])) {
                $p = $path . "\\Mes cours";
                $script .= pin_script($p);
                if (empty($config['no_devoir_shares'])) {
                    $p = $path . "\\Mes devoirs";
                    $script .= pin_script($p);
                }
                if (empty($config['no_echange_shares'])) {
                    $p = $path . "\\Echanges";
                    $script .= pin_script($p);
                }
            } elseif (in_array('Profs', $info['list_u'])) {
                $p = $path . "\\Mes groupes";
                $script .= pin_script($p);
            } else {
                $p = $path . "\\Direction";
                $script .= pin_script($p);
            }
            if (in_array("Cloud_etab", $info['list_u'])) {
                $script .= pin_script("shell:Personal");
            }
        } else {
            $script .= pin_script($path);
        }
        $script .= "IF EXIST \"" . $path . "\" (RD /S /Q \"" . $path . "\")\r\n";
        $script .= rclone_proxy($config, $info, $i);
        $script .= "ECHO IF EXIST \"%programfiles%\\Winget\\Links\\rclone.exe\" (SET rclone=\"%programfiles%\\Winget\\Links\\rclone.exe\") ELSE (set rclone=\"%programfiles%\\rclone\\rclone.exe\")>>%temp%\\rclone-$i.cmd\r\n";
        $script .= "ECHO %%rclone%% mount \"" . $remote . ":" . $sub . "\" \"" . $path . "\" " . $args . " --config \\\\%SE4FS%\\users\\%username%\\.config\\rclone\\rclone.conf --vfs-cache-mode full --cache-dir %temp%\\rclone --no-console>>%temp%\\rclone-$i.cmd\r\n";
        $script .= "START /MIN %temp%\\rclone-$i.cmd\r\n";
        // $script .= "Set _seconds=0\r
        // :waitloop$i\r
        // Set /a \"_seconds=_seconds+5\">nul\r
        // PING -n 1 127.0.0.1>nul\r
        // if exist \"" . $path . "\" goto nextstep$i\r
        // if %_seconds%==30 goto nextstep$i\r
        // goto waitloop$i
        // :nextstep$i\r
        // ";
        // $script .= "if not exist \"" . $path . "\" (exit 1)\r\n";
        $script = iconv("UTF-8", "CP850", $script);
        
        $script .= "\r\n";
    } else {
        if (! empty($sub)) {
            $path = "%h/nuages/" . $remote . "/" . $sub;
        } else {
            $path = "%h/nuages/" . $remote;
        }
        $script = "mkdir -p \"\$HOME/nuages/" . $remote . "/" . $sub . "\"\n";
        $script .= "echo \"[Unit]
	Description=$path (rclone)
	AssertPathIsDirectory='" . $path . "'
	# Make sure we have network enabled
	After=network.target
	    
[Service]
	Type=simple
";
        $script .= rclone_proxy($config, $info, $i);
        $script .= "    ExecStart=/usr/bin/rclone mount '" . $remote . ":" . $sub . "' '" . $path . "' --vfs-cache-mode full " . $args . "
            
	# Perform lazy unmount
	ExecStop=/usr/bin/fusermount -zu '" . $path . "'
	
	# Restart the service whenever rclone exists with non-zero exit code
	Restart=on-failure
	RestartSec=15
	
[Install]
	# Autostart after reboot
	WantedBy=default.target\" > \$HOME/.config/systemd/user/rclone-$i.service
	
systemctl --user daemon-reload
systemctl --user enable --now rclone-$i.service\n";
    }
    $i ++;
    return $script;
}

/**
 * construit un tableau de l'arborescence des partages cloud à déployer pour un ou tous les utilisateurs, ou un groupe/matiere
 *
 * ces partages seront créés sur le NC de l'établissement, avec le compte établissement.
 *
 * * L'arborescence peut être de type "classes/groupes" ou de type "matieres".
 *
 * * pour chaque dossier, en plus du dossier travail demandé, plusieurs options sont possibles :
 * - dossier echanges (via interface echanges)
 * - dossiers individuels élèves
 * - travail à rendre (via interface devoirs)
 *
 * * les partages Administratifs sont également créés, avec des droits de partage individuels.
 *
 *
 * en multi-étab : /Groupes/@GRP-suffix/....
 * en mono-étab : /Groupes/@GRP/...
 *
 * Le -suffix sera remplacé par " (nom de l'etab ville)" lors de la construction de l'arborescence des profs
 *
 * @param array $config
 * @param string $matiere
 *            chemin vers le dossier matiere à lister /Matieres/M@C. Si un chemin autre est passé, il agit comme un filtre sur le résultat. /Matieres pour avoir que les matieres par exemple.
 * @return array|bool
 */
function list_partages_groups($config, $matiere = "*")
{
    $shares = [
        '/Groupes' => [],
        '/Archives' => [],
        '/Direction' => [],
        '/Matieres' => []
    ];
    $c = [];
    if (empty($matiere)) {
        $matiere = "*";
    }
    $config['etab_ou'] = $config['etab_ou'] ?? "0";
    // $users = search_ad($config, $user, "user", $config['dn']['people']);
    // if ($users = nc_users_ids($config)) {
    if ($users = search_ad($config, $matiere, "user", $config['dn']['people'])) {
        $m = [];
        if (preg_match("#^/Matieres/((.*)@(.*))?$#", $matiere, $m)) {
            // partage matière : on n'analyse que les profs concernés
            $profs = list_matiere_profs($config, $m[1]);
            $matiere = $m[2];
            $classe = str_replace($config['suffix'], "", $m[3]);
        }

        foreach ($users as $user) {
            if (! empty($profs)) {
                if (! in_array($user['cn'], $profs)) {
                    continue;
                }
            }
            if (isset($user['memberof']) && isset($user['Id NC'])) {
                $u = $user['Id NC'];
                $d = false;
                // première passe : Evaluation de l'appartenance aux groupes classe/groupe
                foreach ($user['memberof'] as $groupe) {
                    $m = array();
                    if (preg_match("/CN=(Classe|Equipe|Cours)_(.*),OU=(Classes|Equipes|Cours).*/i", $groupe, $m) && ldap_dn2uai($groupe) == $config['etab_ou']) {
                        // jamais vu cette classe
                        if (! isset($c[$m[2]])) {
                            $shares['/Groupes/@' . $m[2] . $config['suffix']] = [
                                [
                                    'share_type' => OCS_GROUP,
                                    'permissions' => OCS_PERM_RW,
                                    'share_with' => "Equipe_" . $m[2] . $config['suffix']
                                ]
                            ];
                            if (empty($config['no_echange'])) {
                                $shares['/Groupes/@' . $m[2] . $config['suffix'] . '/_Echange@' . $m[2]] = [ // [
                                                                                                              // 'share_type' => OCS_GROUP,
                                                                                                              // 'permissions' => OCS_PERM_RW,
                                                                                                              // 'share_with' => $m[1] . "_" . $m[2] . $config['suffix']
                                                                                                              // ]
                                ];
                            }
                            // premiere fois qu'on la voit : création d'un index du nom de le classe
                            $c[$m[2]] = [];
                        }
                        if ($m[1] == "Equipe") {
                            if (! isset($c[$m[2]][$m[1]])) {
                                // cas d'un prof
                                $shares['/Groupes/@' . $m[2] . $config['suffix'] . '/_Equipe@' . $m[2]] = [];
                                $c[$m[2]][$m[1]] = true;
                            }
                        } else {
                            if (! isset($c[$m[2]]['type'])) {
                                $shares['/Groupes/@' . $m[2] . $config['suffix'] . '/_Travail@' . $m[2]] = [
                                    [
                                        'share_type' => OCS_GROUP,
                                        'permissions' => OCS_PERM_RO,
                                        'share_with' => $m[1] . "_" . $m[2] . $config['suffix']
                                    ]
                                ];
                                $c[$m[2]]['type'] = $m[1];
                            }
                            $shares['/Groupes/@' . $m[2] . $config['suffix'] . '/' . $user['cn'] . "@" . $m[2]] = [
                                [
                                    'share_type' => OCS_USER,
                                    'permissions' => OCS_PERM_RW,
                                    'share_with' => $u
                                ]
                            ];
                        }
                    } elseif (! $d && preg_match("/CN=(Direction|VieScol|Secretariat),.*/i", $groupe, $m)) {
                        $d = true;
                        $shares['/Direction'][] = [
                            'share_type' => OCS_USER,
                            'permissions' => OCS_PERM_RW,
                            'share_with' => $u
                        ];
                    }
                }
                foreach ($user['memberof'] as $groupe) {
                    // deuxième passe : evaluation partage du répertoire /Matieres/$matiere@$classe(suffix) avec la classe (ro) et le prof(rw).
                    $m = [];
                    $matiere = $matiere ?? ".*";
                    $classe = $classe ?? ".*";
                    if (preg_match("/cn=Matiere_(" . $matiere . ")@(" . $classe . ")," . $config['dn']['equipes'] . "/i", $groupe, $m)) {
                        $classe_sam = $m[2] . $config['suffix'];
                        if (! isset($c[$m[2]]['type'])) {
                            $c[$m[2]]['type'] = type_group($config, $m[2])['type'];
                        }
                        $type = $c[$m[2]]['type'];
                        $path = '/Matieres/' . $m[1] . '@' . $classe_sam;
                        // partage de la racine
                        $shares[$path] = [];
                        if (empty($config['no_echange'])) {
                            // les partages pour échange ne sont pas pris en compte par l'algo
                            $shares[$path . '/_Echange@' . $m[1] . '@' . $classe_sam] = [];
                        }
                        $travail = $path . '/_Travail@' . $m[1] . '@' . $classe_sam;
                        $shares[$travail] = $shares[$travail] ?? [];
                        add_array_unique([
                            'share_type' => OCS_GROUP,
                            'permissions' => OCS_PERM_RO,
                            'share_with' => ucfirst($type) . "_" . $classe_sam
                        ], $shares[$travail]);
                        add_array_unique([
                            'share_type' => OCS_USER,
                            'permissions' => OCS_PERM_RW,
                            'share_with' => $u
                        ], $shares[$travail]);
                        // ajout devoirs
                        if (empty($config['no_works'])) {
                            // dossier pour les devoirs, son remplissage est fait par la gestion des devoirs.
                            $shares[$path . '/Devoirs'] = [];
                        }
                    }
                }
            }
        }
        // suppression eventuelle des partages en fonction de la conf
        foreach ($shares as $p => $share) {
            if (! empty($config['no_group_shares']) && preg_match("|/Groupes|", $p)) {
                unset($shares[$p]);
            }
            if (! empty($config['no_matiere_shares']) && preg_match("|/Matieres|", $p)) {
                unset($shares[$p]);
            }
        }
        uksort($shares, 'path_sort');
        return $shares;
    } else {
        return false;
    }
}

function form_partages_next($type = "commit")
{
    $types = array(
        "force" => "Forcer le rafraichissement",
        "clean" => "Nettoyer les partages",
        "commit" => "Rafraîchir les partages"
    );
    $html = "Sélectionner l'étape suivante :  <SELECT NAME=\"action\" >"; // onchange='this.form.submit()'>";
    foreach ($types as $t => $txt) {
        $html .= "<option value=\"$t\"";
        if ($type == $t) {
            $html .= " selected>";
        } else {
            $html .= ">";
        }
        $html .= "$txt</option>";
    }
    $html .= "<input type=\"submit\" value=\"Valider\">";
    $html .= "</SELECT><br>";
    return $html;
}

/**
 * copie les données d'un utilisateur depuis son partage Samba vers le Cloud
 *
 * * initialise le compte perso de l'utilisateur avec l'arborescence d'accès aux documents perso, et également aux dossiers partagés.
 *
 * * copie si besoin les documents présents sur le serveur vers cette arborescence
 *
 * Les opérations sont faites avec les identifiants rclone de l'utilisateur, en utilisant rclone lancé par un script batch
 *
 *
 *
 * @param unknown $config
 * @param unknown $user
 * @param unknown $data
 */
function migrate_to_nc($config, $user, $data, $dest = "")
{
    // Nom par défaut de l'espace perso monté dans "Mes Documents"
    $perso = $config['cloud_perso_name'] ?? "Mes Documents";
    $conf = "/home/" . $user['cn'] . "/.config/rclone/rclone.conf";
    switch ($data) {
        case 'home':
        case 'etab':
            $data = $config['cloud_name'];
        case 'nuage.apps':
            $path = "/home/" . $user['cn'] . "/Docs";
            $dest = $data . ":/" . $perso;
            $command = "if [ -f \"" . $conf . "\" ]; then\n";
            $command .= "rclone mkdir \"" . $dest . "\" --config=" . $conf . "\n";
            $command .= "[ -d \"" . $path . "\" ] && rclone copy " . $path . " \"" . $dest . "\" --config=" . $conf . "\n";
            $command .= "fi\n";
            // $path = "/home/" . $user['cn'] . "/photos";
            // $dest = $m[1] . ":/Photos";
            // $command = "rclone copy " . $path . " " . $dest . " -config=/home/" . $user['cn'] . "/.config/rclone.conf";
            break;
        case 'drive.monlycee.net':
            $path = "/home/" . $user['cn'] . "/Docs";
            $dest = $data . ":/" . $perso;
            $command = "if [ -f \"" . $conf . "\" ]; then\n";
            $command .= "rclone mkdir \"" . $dest . "\" --config=" . $conf . "\n";
            $command .= "[ -d \"" . $path . "\" ] && rclone copy " . $path . " \"" . $dest . "\" --config=" . $conf . "\n";
            $command .= "fi\n";
            // $path = "/home/" . $user['cn'] . "/photos";
            // $dest = $m[1] . ":/Photos";
            // $command = "rclone copy " . $path . " " . $dest . " -config=/home/" . $user['cn'] . "/.config/rclone.conf";
            break;
        case 'classe':
            $classe = list_classes($config, $user)[0];
            $path = "/var/www/sambaedu/Classes/Classe_" . $classe . "/" . inverse_login($user['cn']);
            $dest = $config['cloud_name'] . ":/Mes travaux/" . $classe;
            $command = "rclone copy " . $path . " " . $dest . " --config=" . $conf . "\n";
            break;
        default:
            $path = $data;
            $command = "rclone copy " . $path . " " . $dest . " --config=" . $conf . "\n";
    }
    batch_command($command);
    batch_write();
}

function form_cloud_migrate($config, $user, $cloud)
{
    if (isset($_POST['migrate']) && $_POST['migrate'] == $config['cloud_uri']) {
        $c = "etab";
    } else {
        $c = $_POST['migrate'] ?? "";
    }
    $html = "<center>";
    $html .= "<H1>Hébergement de l'espace personnel</H1>";
    $cloud_perso = "";
    $config['cloud_perso_name'] = $config['cloud_perso_name'] ?? "Mes Documents";
    foreach ($user['memberof'] as $g) {
        $m = [];
        if (preg_match("/cn=Cloud_(.*),ou=.*/iU", $g, $m)) {
            $cloud_perso = $m[1];
            if ($cloud_perso == "etab") {
                $cloud_perso = $config['cloud_uri'];
            }
            break;
        }
    }
    if (! empty($cloud_perso)) {
        $html .= "Espace personnel en ligne activé ! Le dossier <b>" . $config['cloud_perso_name'] . "</b> est hébergé sur <b>" . $cloud_perso . "</b>.";
    } elseif (! empty($_POST['migrate'])) {
        if (groupaddmember($config, $user['cn'], "Cloud_" . $c)) {
            migrate_to_nc($config, $user, $c);
            $html .= "Vos données personnelles sont en train d'être transférées vers le serveur " . $_POST['migrate'] . "<br>";
        } else {
            $html .= "Erreur, transfert vers " . $_POST['migrate'] . " impossible.<br>";
        }
    } elseif (! empty($cloud)) {
        $html .= '<center>Choisir le serveur vers lequel vous voulez migrer vos données personnelles<br>
Attention cette migration aura pour effet de copier tous les fichiers présents dans Mes documents (K:\\Docs) vers le serveur choisi. Le Dossier <b>' . $config['cloud_perso_name'] . '</b> pointera ensuite vers ce serveur.
En cas de coupure de la connexion internet, vos documents seront inaccessibles !<br>
Cette opération est définitive.<br>
<form name="cloud-migrate" action="" method="post">';
        foreach ($cloud as $c) {
            if ($c == $config['cloud_name']) {
                $c = $config['cloud_uri'];
            }
            $html .= '<input type=submit name="migrate" value="' . $c . '">';
        }
        $html .= '</form>';
    }
    $html .= "</center>";
    return $html;
}
?>
