<?php
/**
 * manipulation des parametres de conf
 * @Projet LCS / SambaEdu

 * @Auteurs Equipe SambaEdu

 * @Note Ce fichier de fonction doit être appelé par un include dans entete.inc.php
 * @Note Ce fichier est complete a l'installation

 * @Licence Distribué sous la licence GPL
 */

/**
 *
 * file: config.php
 *
 * @Repertoire: includes/
 */

// Liste des caracteres speciaux pour les mots de passe
$char_spec = "_#@£%§:!?*$-";

// wol / shutdown
define('SAMBAEDU_NO_WOL', 1);
define('SAMBAEDU_NO_WOL_LOG', 2);
define('SAMBAEDU_NO_SHUTDOWN', 4);
define('SAMBAEDU_NO_SHUTDOWN_LOG', 8);
define('SAMBAEDU_SHUTDOWN_ERROR', 16);
define('SAMBAEDU_NO_IPXE', 32);
// logon/logoff
define('SAMBAEDU_NO_LOGON_LOG', 64);
define('SAMBAEDU_NO_LOGOFF_LOG', 128);
// applications
define('SAMBAEDU_STARTUP_APP_ERROR', 256);
define('SAMBAEDU_SHUTDOWN_APP_ERROR', 512);
define('SAMBAEDU_LOGON_APP_ERROR', 1024);
define('SAMBAEDU_LOGOFF_APP_ERROR', 2048);
define('SAMBAEDU_LOGON_SYS_APP_ERROR', 4096);
define('SAMBAEDU_LOGOFF_SYS_APP_ERROR', 8192);
// wpkg
define('SAMBAEDU_WPKG_RUNNING', 16384);
define('SAMBAEDU_WPKG_ERROR', 32768);
// Cloud
define('SAMBAEDU_CLOUD_ERROR', 0x10000);
define('SAMBAEDU_SLOW_NET_ERROR', 0x20000);

function db_connect($config, $etab = 'localhost')
{
    if (empty($etab)) {
        $etab = "localhost";
    }
    if ($etab != "localhost") {
        $etab = "se4fs-" . $etab;
    }
    return mysqli_connect($etab, 'sambaedu', $config['sql_passwd'], 'sambaedu');
}

/**
 * met en place un verrou pour l'accès à la conf afin d'éviter les accès concurrents
 *
 * @param
 *            $mode
 * @return resource|boolean
 */
function lock_conf($mode)
{
    $lock = "/var/lock/sambaedu.lock";
    if ($fp = fopen($lock, "c")) {
        $startTime = microtime();
        do {
            $canWrite = flock($fp, $mode);
            // If lock not obtained sleep for 0 - 100 milliseconds, to avoid collision and CPU load
            if (! $canWrite)
                usleep(round(rand(0, 100) * 1000));
        } while ((! $canWrite) and ((microtime() - $startTime) < 1000));

        // file was locked so now we can store information
        if ($canWrite) {
            return $fp;
        }
    }
    return false;
}

function etab_suffix($uai)
{
    if (preg_match("/[0-9]{7}[a-z]/i", $uai)) {
        return strtolower("-" . substr($uai, 3));
    } else {
        return "";
    }
}

/**
 * Lire la conf de se4 et de ses modules
 * fonction pour récupérer la conf de se4 ou des modules de façon recursive dans /etc/sambaedu/
 * possibilité de parser l'ancienne conf afin de pouvoir comparer les valeurs
 *
 * @Paramètres : "nom du module", "sambaedu" ou "all"
 * @returns array["parametre"]
 */
function get_config_file(string $module = "sambaedu", $global, bool $old = false, bool $table = false)
{
    if ($old) {
        $ext = "-old";
    } else {
        $ext = "";
    }
    $config = array();
    if ($module == "all") {
        $config = get_config_file("sambaedu", $global, $old, $table);
        if ($handle = opendir('/etc/sambaedu/sambaedu.conf.d')) {
            unset($module);
            while (false !== ($entry = readdir($handle))) {
                if ($entry != "." && $entry != "..") {
                    $module = preg_replace("/\.conf" . $ext . "/", "", $entry);
                    if (isset($module)) {
                        $config = array_merge($config, get_config_file($module, $global, $old, $table));
                    }
                }
            }
            closedir($handle);
        }
    } elseif ($module == "sambaedu") {
        $conf_file = "/etc/sambaedu/sambaedu.conf" . $ext;
        $config = parse_ini_file($conf_file);
        if ($table) {
            $t = [];
            foreach ($config as $p => $v) {
                $t[$p] = [
                    'module' => $module,
                    'valeur' => $v
                ];
            }
            $config = $t;
        } else {
            $config['etablissements_rdn'] = $config['etablissements_rdn'] ?? "OU=etablissements";
            $config['matieres_rdn'] = $config['matieres_rdn'] ?? "OU=Matieres";
            // $config['dn']['admin'] = "cn=" . $config['ldap_admin_name'] . "," . $config['admin_rdn'] . "," . $config['ldap_base_dn'];
            if ($global === false && ! empty($config['etab_ou'])) {
                $global = $config['etab_ou'];
            }
            if (! preg_match("/[0-9]{7}[a-z]/i", $global)) {
                $config['suffix'] = "";
                // $config['etab_ou'] = "0"; WTF !!!
            } else {
                $config['etab_ou'] = $global;
                $config['people_rdn'] = "OU=" . $config['etab_ou'] . "," . $config['people_rdn'];
                $config['groups_rdn'] = "OU=" . $config['etab_ou'] . "," . $config['groups_rdn'];
                $config['equipements_rdn'] = "OU=" . $config['etab_ou'] . "," . $config['equipements_rdn'];
                $config['delegations_rdn'] = "OU=" . $config['etab_ou'] . "," . $config['delegations_rdn'];
                $config['parcs_rdn'] = "OU=" . $config['etab_ou'] . "," . $config['parcs_rdn'];
                $config['computers_rdn'] = "OU=" . $config['etab_ou'] . "," . $config['computers_rdn'];
                $config['suffix'] = etab_suffix($config['etab_ou']);
            }
            $config['dn']['people'] = $config['people_rdn'] . "," . $config['ldap_base_dn'];
            $config['dn']['Eleves'] = "OU=Eleves," . $config['people_rdn'] . "," . $config['ldap_base_dn'];
            $config['dn']['Profs'] = "OU=Profs," . $config['people_rdn'] . "," . $config['ldap_base_dn'];
            $config['dn']['Administratifs'] = "OU=Administratifs," . $config['people_rdn'] . "," . $config['ldap_base_dn'];
            $config['dn']['groups'] = $config['groups_rdn'] . "," . $config['ldap_base_dn'];
            $config['dn']['rights'] = $config['rights_rdn'] . "," . $config['ldap_base_dn'];
            $config['dn']['equipements'] = $config['equipements_rdn'] . "," . $config['ldap_base_dn'];
            $config['dn']['etablissements'] = $config['etablissements_rdn'] . "," . $config['ldap_base_dn'];
            $config['dn']['delegations'] = $config['delegations_rdn'] . "," . $config['ldap_base_dn'];
            $config['dn']['matieres'] = $config['matieres_rdn'] . "," . $config['ldap_base_dn'];
            $config['dn']['trash'] = $config['trash_rdn'] . "," . $config['ldap_base_dn'];
            $config['dn']['parcs'] = $config['parcs_rdn'] . "," . $config['ldap_base_dn'];
            $config['dn']['computers'] = $config['computers_rdn'] . "," . $config['ldap_base_dn'];
            $config['dn']['autres'] = $config['other_groups_rdn'] . "," . $config['groups_rdn'] . "," . $config['ldap_base_dn'];
            $config['dn']['projets'] = $config['projets_rdn'] . "," . $config['groups_rdn'] . "," . $config['ldap_base_dn'];
            $config['dn']['cours'] = $config['cours_rdn'] . "," . $config['groups_rdn'] . "," . $config['ldap_base_dn'];
            $config['dn']['classes'] = $config['classes_rdn'] . "," . $config['groups_rdn'] . "," . $config['ldap_base_dn'];
            $config['dn']['equipes'] = $config['equipes_rdn'] . "," . $config['groups_rdn'] . "," . $config['ldap_base_dn'];
            // if (!($global) && ! empty($config['etab_ou'])){
            // $config['ldap_base_dn'] = "OU=" . $config['etab_ou'] . "," . $config['ldap_base_dn'];
            // }
        }
    } else {
        $conf_file = "/etc/sambaedu/sambaedu.conf.d/$module.conf" . $ext;
        if (file_exists($conf_file)) {
            if ($lock = lock_conf(LOCK_SH)) {
                $config = parse_ini_file($conf_file);
                if ($table) {
                    $t = [];
                    foreach ($config as $p => $v) {
                        $t[$p] = [
                            'module' => $module,
                            'valeur' => $v
                        ];
                    }
                    $config = $t;
                }
                fclose($lock);
            }
        }
    }
    return ($config);
}

function compare_config($module = "all")
{
    $conf = get_config_file($module, true, false, false);
    $old_conf = get_config_file($module, true, true, false);
    $conf_diff = @array_diff_assoc($conf, $old_conf);
    $conf_reverse_diff = @array_diff_assoc($old_conf, $conf);
    $diff = array_merge($conf_reverse_diff, $conf_diff);
    if ($old_diff = apcu_fetch('diff')) {
        $diff = array_merge($old_diff, $diff);
    }
    return $diff;
}

/*
 * fonction pour récupérer la conf dans un tableau persistant 120s
 * @Parametres : [$force = true] pour forçer la lecture
 * @returns : $config = tableau des parametres
 */
function get_config(array $config = [], bool $force = true, $global = false, $module = "all")
{
    if (isset($config['bind'])) {
        $bind = $config['bind'];
    }
    $config = get_config_file($module, $global, false, false);
    if (empty($config)) {
        die("Erreur de lecture de la configuration se4");
    }

    if (! session_name()) {
        session_name("Sambaedu");
    }
    if (! isset($_SESSION)) {
        @session_start([
            'cookie_lifetime' => 86400
        ]);
    }
    $config['login'] = $_SESSION['login'] ?? "";

    // augmentation du temps de cache des données d'import si besoin via la conf
    $config['ent_timeout'] = $config['ent_timeout'] ?? 600;
    if (isset($bind)) {
        $config['bind'] = $bind;
    } else {
        /**
         * Connexion avec l'AD en GSSAPI
         *
         * @description Connexion GSSAPI utilisée par les fonctions de plus haut niveau search_ad, modify_ad, move_ad, etc...
         */
        $url = ad_url($config, "ldaps");
        $ds = ldap_connect($url);
        if ($ds) {
            ldap_set_option($ds, LDAP_OPT_PROTOCOL_VERSION, 3);
            ldap_set_option($ds, LDAP_OPT_REFERRALS, 0);
            ldap_set_option($ds, LDAP_OPT_NETWORK_TIMEOUT, 60);

            // $r = ldap_sasl_bind($ds, null, null, 'GSSAPI');
            $r = ldap_bind($ds, $config['ldap_admin_name'] . "@" . $config['domain'], $config['ldap_admin_passwd']);
            if (! $r) {
                die("Echec de l'Authentification AD : " . ldap_error($ds));
                $ds = false;
            }
        } else {
            die("Erreur de connection kerberos au serveur AD");
            $ds = false;
        }
        $config['bind'] = $ds;
    }
    return ($config);
}

/*
 * fonction pour executer une action système en cas de changement d'un parametre de conf
 * exécute un sudo script shell
 * @Parametres : $param
 *
 * @Parametres : module ( defaut = "base" )
 * @returns : $ret
 */
function set_config_action($param)
{
    exec("sudo -c '/usr/share/sambaedu/scripts/config_action.sh $param'", $ret);
    return ($ret);
}

/**
 * Ecriture d'un paramètre de conf
 *
 * @description fonction pour écrire la conf dans les fichiers de conf  :retourne le tableau $config et met à jour le cache
 * @Parametres : $param : parametre a fixer, $value : valeur à positionner, $module = "sambaedu" : module SE4 concerné
 * @returns : $config ou false
 */
function set_config($config, $param, $value = "", $module = "sambaedu")
{
    /*
     * while (apcu_fetch('config_lock')) {
     * sleep(1);
     * }
     */
    $new_config = get_config_file($module, true, false, false);
    if ($param != "etab_ou") {
        if (empty($value)) {
            if (isset($new_config[$param]))
                unset($new_config[$param]);
        } else {
            $new_config[$param] = $value;
        }
    }
    $content = "";
    foreach ($new_config as $key => $v) {
        if (! ($key == "dn" || $key == "login" || empty($v) || empty($key))) {
            $content .= $key . " = \"" . $v . "\"\n";
        }
    }
    // write it into file
    if ($module == "sambaedu") {
        $conf_file = "/etc/sambaedu/sambaedu.conf";
        // $conf_file_tmp = "/etc/sambaedu/sambaedu.conf.tmp";
    } else {
        $conf_file = "/etc/sambaedu/sambaedu.conf.d/$module.conf";
        // $conf_file_tmp = "/etc/sambaedu/sambaedu.conf.d/$module.conf.tmp";
    }
    if ($lock = lock_conf(LOCK_EX)) {
        if (! $handle = fopen($conf_file, "w")) {
            fclose($lock);
            die("Erreur d'ecriture de la configuration se4 : $module $param $value");
        }
        $res = fwrite($handle, $content);
        fclose($handle);
        fclose($lock);
    } else {
        die("Erreur d'ecriture de la configuration se4 : $module $param $value");
    }
    return get_config($config, true);
}

/**
 * Initilialisation d'un paramètre de conf
 *
 * @description initilise la valeur d'un parametre s'il n'existe pas dans la conf
 * @Parametres : parametre a initaliser, valeur, module ( defaut = "sambaedu" )
 * @returns : array() $config
 */
function init_param(&$config, $nom, $valeur, $module = "sambaedu")
{
    if (! isset($config[$nom])) {
        $config = set_config($config, $nom, $valeur, $module);
    }
}

function set_param(&$config, $nom, $valeur, $module = "sambaedu")
{
    $config = set_config($config, $nom, $valeur, $module);
    return $valeur;
}

/**
 * Lecture d'un paramètre de conf
 *
 * @description Lit un paramètre dans la conf
 * @parametres $config, $nom
 * @returns ""|string
 */
function get_param($config, $nom)
{
    if (isset($config[$nom])) {
        return $config[$nom];
    } else {
        return "";
    }
}

/**
 * remplace de façon récursive dans tous les fichiers de conf la valeur du parametre $param par $valeur (recherche par regexp)
 *
 * @param unknown $config
 * @param unknown $param
 * @param unknown $valeur
 */
function replace_param($config, $param, $valeur)
{
    $config_table = get_config_file("all", true, false, true);
    $old_v = $config_table[$param]['valeur'];
    foreach ($config_table as $p => $table) {
        if (preg_match("|" . $old_v . "|", $table['valeur']) == 1) {
            $config = set_config($config, $p, preg_replace("|" . $old_v . "|", $valeur, $table['valeur']), $table['module']);
        }
    }
    return $config;
}

/**
 *
 * @description remplace ###_NOM_### par $param['nom'] dans un fichier en mémoire. Peut s'utiliser directement avec $config
 * @parametres array $params, array $lignes tableau du fichier à remplacer
 * @returns mixed
 */
function write_param(array $params, array &$lignes)
{
    $l = $lignes;
    foreach ($params as $param => $valeur) {
        if (is_string($valeur) || is_int($valeur))
            $lignes = preg_replace("/###_" . strtoupper($param) . "_###/", $valeur, $lignes);
    }
    return ($l != $lignes);
}

// fonctions pour exécution asynchrone de scripts
/**
 *
 * @description Mise en fil d'attente de commandes programmée pour exécution asynchrone
 * @parametres $command commande à lancer
 * @returns
 */
function batch_command($command)
{
    $batch = apcu_fetch("batch_" . session_id());
    $batch .= $command . "\n";
    apcu_store("batch_" . session_id(), $batch, 500);
}

/**
 * name
 *
 * @description Lancement programmé des commandes mises en fil d'attente via batch_command
 * @parametres $queue fast / normal / slow pour un lancement dans la mn / 10mn / à 21h00
 * @returns
 */
function batch_write($queue = "normal")
{
    $batch = apcu_fetch("batch_" . session_id()) ?? "";
    if (file_exists("/tmp/admin_script_$queue.sh")) {
        $script = file_get_contents("/tmp/admin_script_$queue.sh");
    } else {
        $script = "#!/bin/bash\n";
        $script .= "rm -f /tmp/admin_script_$queue.sh\n";
    }
    $script .= $batch;
    file_put_contents("/tmp/admin_script_$queue.sh", $script);
    system("sudo chmod 755 /tmp/admin_script_$queue.sh");
    apcu_delete("batch_" . session_id());
}

/**
 * détermine l'url de l'ad à utiliser pour les opérations
 * se base sur le sambaedu.conf pour tester les serveurs à utiliser.
 * par défaut uitlise d'abord le local (se4ad_etab_name).
 *
 * attention, on ne doit pas faire de mixentre des machines sur site et hors site
 *
 * mode : ldap pour ldes requetes ldap
 * ldaps, dns, sambatool
 *
 * @param unknown $config
 * @param string $mode
 * @return string|array|string
 *
 * @todo a refaire à partir es infos des sites ?
 */
function ad_url($config, $mode = "ldap", $master = false)
{
    if ($master || empty($url = apcu_fetch("ad_url_" . $mode))) {
        $dns = $config['se4ad_ip'];
        if ($master) {
            $kdc[1] = $config['se4ad_ip'];
        } else {
            if (! empty($config['se4ad_etab_ip'])) {
                $kdc[0] = $config['se4ad_etab_ip'];
                $kdc[1] = $config['se4ad_ip'];
            } else {
                $kdc[0] = $config['se4ad_ip'];
            }
            // on lit resolv.conf
            $resolv = array_merge($kdc, file("/etc/resolv.conf"));
            foreach ($resolv as $line) {
                $m = [];
                if (preg_match("/([0-9]*\.[0-9]*\.[0-9]*\.[0-9]*)/", $line, $m)) {
                    // on teste le bon fonctionnement d'au moins 1 AD pour pouvoir recup la liste des AD
                    $res = 0;
                    $out = [];
                    exec("host -W1 " . $config['domain'] . " " . $m[1] . " 2>&1", $out, $res);
                    if ($res == 0) {
                        $dns = $m[1];
                        // on met les dns locaux en premier
                        $out = array_merge($kdc, $out);
                        foreach ($out as $l) {
                            $i = [];
                            if (preg_match("/^(.*has address )?([0-9]*\.[0-9]*\.[0-9]*\.[0-9]*)/", $l, $i)) {
                                $re = 0;
                                system("host -W1 " . $config['domain'] . " " . $i[2] . " >/dev/null 2>&1", $re);
                                if ($re == 0) {
                                    if (! in_array($i[2], $kdc)) {
                                        $kdc[] = $i[2];
                                        // mieux avec plusieurs ?
                                        // break;
                                    }
                                    if (! empty($config['etab_ou']) || count($kdc) > 4) {
                                        // on en a assez cela suffit !
                                        break;
                                    }
                                } else {
                                    if ($k = array_search($i[2], $kdc)) {
                                        unset($kdc[$k]);
                                    }
                                }
                            }
                        }
                        break;
                    }
                }
            }
        }
        $url = "";
        unset($resolv);
        $resolv[0] = "domain " . $config['domain'];
        foreach ($kdc as $n) {
            switch ($mode) {
                case "sambatool":
                    $res = 0;
                    $out = [];
                    // exec("host -r " . $n . " " . $dns . "| grep pointer", $out, $res);
                    exec("host " . $n . " " . $dns . "| grep pointer", $out, $res);
                    if ($res == 0) {
                        foreach ($out as $l) {
                            $m = [];
                            if (preg_match("/pointer (.*)\..*$/", $l, $m)) {
                                $url .= "-H ldap://" . $m[1] . " ";
                                break;
                            }
                        }
                    } else {
                        $url .= "-H ldap://" . $config['se4ad_name'] . " ";
                    }
                    break 2;
                case "dns":
                    $res = 0;
                    $out = [];
                    // exec("host -r " . $n . " " . $dns . "| grep pointer", $out, $res);
                    exec("host " . $n . " " . $dns . "| grep pointer", $out, $res);
                    if ($res == 0) {
                        foreach ($out as $l) {
                            $m = [];
                            if (preg_match("/pointer (.*)\..*$/", $l, $m)) {
                                $url .= $m[1];
                                break;
                            }
                        }
                    } else {
                        $url .= $config['se4ad_name'] . " ";
                    }
                    break 2;
                case "ldaps":
                    $url .= "ldaps://" . $n . " ";
                    break 2;
                default:
                    $url .= "ldap://" . $n . " ";
                    break 2;
            }
            $resolv[] = "nameserver " . $n;
        }
        if (! $master && count($resolv) > 1) {
            file_put_contents("/tmp/resolv.conf", implode("\n", $resolv) . "\n");
            exec("sudo cp -f /tmp/resolv.conf /etc/resolv.conf");
            exec("sudo chown root /etc/resolv.conf");
            apcu_store("ad_url_" . $mode, $url, 300);
        }
    }
    return $url;
}

// configuration du proxy

/**
 * ajout des options du proxy pour curl
 *
 * @param unknown $config,
 *            $opt options curl
 */
function curl_proxy_options($config, &$opt = [])
{
    if (! empty($config['server_proxy']) && ! empty($config['proxy_address']) && ! empty($config['proxy_port'])) {
        $opt['curl'] = [
            CURLOPT_PROXY => "http://" . $config['proxy_address'] . ":" . $config['proxy_port'],
            // CURLOPT_HTTPPROXYTUNNEL => true,
            CURLOPT_SSL_VERIFYPEER => false,
            CURLOPT_SSL_VERIFYHOST => false
        ];
    } else {
        $opt['curl'] = [
            CURLOPT_SSL_VERIFYPEER => false,
            CURLOPT_SSL_VERIFYHOST => false
        ];
    }
    $opt['curl'][CURLOPT_CONNECTTIMEOUT_MS] = 2000;
}

function set_git_proxy($config)
{
    if (! empty($config['server_proxy']) && ! empty($config['proxy_address']) && ! empty($config['proxy_port'])) {
        $command = "git config --global http.proxy http://" . $config['proxy_address'] . ":" . $config['proxy_port'];
    } else {
        $command = "git config --global --unset http.proxy";
    }
    exec($command);
}

function set_oauth_proxy($config, &$opt)
{
    if (! empty($config['server_proxy']) && ! empty($config['proxy_address']) && ! empty($config['proxy_port'])) {
        $opt['proxy'] = "http://" . $config['proxy_address'] . ":" . $config['proxy_port'];
        $opt['verify'] = false;
    }
}

function sudo_proxy($config)
{
    if (! empty($config['server_proxy']) && ! empty($config['proxy_address']) && ! empty($config['proxy_port'])) {
        $ret = "sudo env https_proxy=http://" . $config['proxy_address'] . ":" . $config['proxy_port'] . " ";
    } else {
        $ret = "sudo ";
    }
    return $ret;
}

function replace_element($config, $name, $default)
{
    if (! empty($config[$name])) {
        return $config[$name];
    } else {
        return $default;
    }
}

function header_authorize(&$config)
{
    $login = $config['login'];
    $html = "";
    // Prise en compte de la page demandee initialement - leb 25/6/2005
    if ($login == "") {
        $request = $_SERVER['PHP_SELF'];
        if ($_SERVER['QUERY_STRING'] != "") {
            $request .= "?" . $_SERVER['QUERY_STRING'];
        }
        $html .= "<script>\n";
        // request n'est plus pris en compte cf 4a10ef9c
        // $html .= " window.location.href = '/auth.php?request=" . rawurlencode($request) . "';\n";
        $html .= "    window.location.href = '/auth.php';\n";
        $html .= "</script>\n";
        echo $html;
        // Pour prevenir une poursuite dans le cas ou javascript serait desactive sur le client
        die();
    } else {
        if (! session_name()) {
            session_name("Sambaedu");
            session_start();
        }
        /*
         * $html .= '
         * <!DOCTYPE html>
         * <HTML>
         * <HEAD>
         *
         * <STYLE type="text/css">
         * body {
         * background: url(' . replace_element($config, "LogoFond", "/elements/images/fond_SE4.png") . ') #f0f1f6 bottom right
         * no-repeat fixed;
         * }
         * </STYLE>
         *
         * <title>Interface d\'administration de SambaEdu</title>
         * <meta http-equiv="Content-Type" content="text/html; charset=UTF-8">
         * <link rel="stylesheet" href="/elements/style_sheets/sambaedu.css"
         * type="text/css">
         *
         * <script language="javascript" type="text/javascript">
         *
         * function getconfirm() {
         * if(confirm("Confirmez-vous la suppression ?")) {
         * return true;
         * } else {
         * return false;
         * }
         * }
         * function getconfirm_all() {
         * if(confirm("ATTENTION, en supprimant un utilisateur de son groupe principal, \nil est supprimé de tous les autres groupes !! \nConfirmez-vous la suppression ?")) {
         * return true;
         * } else {
         * return false;
         * }
         * }
         * function getformatconfirm() {
         * if(confirm("Confirmez-vous le formatage ? (cela peut etre long)")) {
         * return true;
         * } else {
         * return false;
         * }
         * }
         * function getlongconfirm() {
         * if(confirm("Cette operation peut etre longue...")) {
         * return true;
         * } else {
         * return false;
         * }
         * }
         *
         * function popuprecherche(page,nom,option){
         * window.open(page,nom,option);
         * }
         *
         * function Reporter(l) {
         * var nouveau=l.options[l.options.selectedIndex].value;
         * window.opener.document.forms["visu"].elements["nouveau"].value=nouveau;
         * }
         * </script>
         * <script type="text/javascript" src="/elements/js/prototype.js"></script>
         * ';
         * // if (isset($autorefresh)) {
         * // $html .= '<meta http-equiv="Refresh" CONTENT="' . $autorefresh . '";URL="' . $url . '">';
         * // }
         * $html .= '</head>
         * <body>
         * ';
         */

        $html .= "<h3 align=\"right\">Bonjour $login&emsp;</h3>";
        if (have_right($config, SE_USER_ADMIN, $login) || have_right($config, SE_COMPUTER_ADMIN, $login)) {
            $etabs = activated_etabs($config);
        } else {
            $etabs = my_etabs($config, $login);
        }
        sort_by_key('uai', $etabs);
        $choix = "";
        // $etabs = list_etabs($config);
        // $etabs[0] = 0;
        $etab = $config['etab_ou'] ?? 0;
        $etab_choix = $_POST['etab'] ?? $_GET['etab'] ?? $_SESSION['etab'] ?? $etab;
        $_SESSION['etab'] = $etab_choix;
        $index = array_search($etab_choix, array_column($etabs, 'uai'));
        if (count($etabs) > 1) {
            $choix .= "Choisir l'établissement à administrer";
            $choix .= "<form name = 'etab' action='" . $_SERVER['PHP_SELF'] . "' method='post' >\n";
            // $choix .= "<SELECT NAME=\"etab\" SIZE=\"1\" onchange=submit()>";

            $choix .= '<datalist id="ListEtab">';
            foreach ($etabs as $e) {
                $choix .= "<option value=\"" . $e['uai'] . "\"";
                if ($etabs[$index]['uai'] == $e['uai']) {
                    // $choix .= " selected";
                    $ActualValue = $e['uai'] . " " . etab_to_name($config, $e['uai']);
                }
                if ($e['uai'] == 0) {
                    $choix .= ">Domaine entier\n";
                    $ActualValue = "Domaine entier";
                } else {
                    $choix .= ">" . $e['uai'] . " " . etab_to_name($config, $e['uai']);
                    if (! $e['activated']) {
                        $choix .= " <b>virtuel</b>";
                    }
                    $choix .= "\n";
                }
                $choix .= "</option>";
            }
            $choix .= "</datalist>";
            $choix .= '<input list="ListEtab" id="etab" name="etab"   size="50" value="' . $ActualValue . '" onfocus="this.value=' . "''" . ';" />';
            $choix .= '<button class="btn btn-primary" type="button"  onClick="submit();">
                                    <i>Choisir</i>
                                </button>';
            $choix .= "</form>\n";

            $html .= $choix;
        }
        // on redirige vers l'etablissement si possible
        if ($index !== false && $etabs[$index]['activated'] && $etabs[$index]['uai'] != $etab) {
            if ($etabs[$index]['uai'] == 0) {
                $html .= '<script language="javascript">
    top.location.href = "/index.php";
</script>';
            } else {
                $html .= '<script language="javascript">
    top.location.href = "/' . $etabs[$index]['uai'] . '/index.php";
</script>';
            }
        }
        $config = get_config($config, true, $_SESSION['etab']);
    }
    return $html;
}

function header_authorize_script($config)
{
    $se4_key = $_POST['se4_key'] ?? $_GET['se4_key'] ?? "";
    if (! ($_SERVER['REMOTE_ADDR'] == $config['se4fs_ip'] && $se4_key == $config["se4_key"])) {
        header($_SERVER["SERVER_PROTOCOL"] . " 403 Forbidden");
        header("Location:/interdit.html");
        exit();
    } else {
        @session_name("Sambaedu");
        @session_start();
        header("Content-type: text/plain");
    }
}

function put_ini_file($config, $file, $has_section = false, $write_to_file = true)
{
    $fileContent = '';
    if (! empty($config) && is_array($config)) {
        foreach ($config as $i => $v) {
            if ($has_section) {
                $fileContent .= "\n[$i]" . PHP_EOL . put_ini_file($v, $file, false, false);
            } else {
                if (is_array($v)) {
                    foreach ($v as $t => $m) {
                        if ($i == "php_flag") {
                            $fileContent .= $i . "[" . $t . "]=" . (is_numeric($m) ? $m : $m) . PHP_EOL;
                        } else {
                            $fileContent .= $i . "[" . $t . "]=" . (is_numeric($m) ? $m : '"' . $m . '"') . PHP_EOL;
                        }
                    }
                } else
                    $fileContent .= $i . "=" . (is_numeric($v) ? $v : '"' . $v . '"') . PHP_EOL;
            }
        }
    }

    if ($write_to_file && strlen($fileContent))
        return file_put_contents($file, $fileContent, LOCK_EX);
    else
        return $fileContent;
}

function get_fpm_conf(string $file)
{
    $php_version = PHP_MAJOR_VERSION . "." . PHP_MINOR_VERSION;
    $php_conf_path = "/etc/php/" . $php_version;
    $old = parse_ini_file($php_conf_path . $file, true);
    return $old;
}

function set_fpm_conf(array $conf, string $file, bool $section = true)
{
    $php_version = PHP_MAJOR_VERSION . "." . PHP_MINOR_VERSION;
    $php_conf_path = "/etc/php/" . $php_version;
    $old = parse_ini_file($php_conf_path . $file, true);
    $new = array_replace_recursive($old, $conf);
    // $mod = ! empty(array_diff($new, $old));
    if ($mod = ($new != $old)) {
        system("sudo chown www-admin " . $php_conf_path . $file);
        $mod = put_ini_file($new, $php_conf_path . $file, $section, true);
        system("sudo chown root " . $php_conf_path . $file);
    }
    return $mod;
}

function configure_fpm($config)
{
    $fpm = get_fpm_conf("/fpm/pool.d/sambaedu.conf");
    // mono-étab
    $memory_limit = "2G";
    $apc_shm_size = "1024M";
    if (count(activated_etabs($config)) > 1) {
        // quasi illimité pour serv central
        $memory_limit = "16G";
        $apc_shm_size = "4096M";
    }
    if (! empty($config['etab_ou'])) {
        // les serv d'etab peuvent être amenés à manipuler la base AD totale.
        $memory_limit = "3G";
        $apc_shm_size = "2048M";
    }
    $fpm['sambaedu'] = array_merge($fpm['sambaedu'], [
        'pm' => "dynamic",
        'pm.max_children' => "40",
        'pm.start_servers' => "15",
        'pm.min_spare_servers' => "15",
        'pm.max_spare_servers' => "25",
        'pm.max_requests' => "200",
        'php_value' => [
            'include_path' => ".:/var/www/sambaedu/includes:/var/www/sambaedu/central/php/includes",
            'max_execution_time' => "300",
            'max_input_vars' => "3000",
            'upload_max_filesize' => "8M",
            'memory_limit' => $memory_limit,
            'apc.shm_size' => $apc_shm_size,
            'apc.ttl' => "1200",
            'date.timezone' => "Europe/Paris"
        ]
    ]);
    // on active le cache redis si une clé est présente
    if (! empty($config['redis_key'])) {
        $fpm['sambaedu']['php_value'] = array_merge($fpm['sambaedu']['php_value'], [
            'session.save_handler' => "redis",
            'session.save_path' => "tcp://se4fs:6379?auth=" . $config['redis_key']
        ]);
    }
    $mod = set_fpm_conf($fpm, "/fpm/pool.d/sambaedu.conf");
    $mod &= set_debug($config);
    if ($mod) {
        batch_command("systemctl reload php" . PHP_MAJOR_VERSION . "." . PHP_MINOR_VERSION . "-fpm.service");
        batch_write("fast");
    }
    return $mod;
}

setlocale(LC_ALL, "C");

// Paramètres LDAP
// $config = [];
// $config = get_config($config, true);

// set_php_proxy($config);
?>