<?php

/**
 * Fonctions gérant l'assignation de GPO à des OU (user ou salles)

 * @Projet SambaEdu

 * @Auteurs Equipe Sambaedu, denis bonnenfant

 * 
 * @Note: Ce fichier de fonction doit etre appele par un include

 * @Licence Distribue sous la licence GPL
 */

/**
 * liste les groupes d'une delegation
 *
 * @param unknown $config
 * @param unknown $delegation
 * @return string[][]|unknown[][]
 */
function list_delegation_salles($config, $delegation)
{
    $res = search_ad($config, $delegation, "delegation");
    $list = array();
    if (count($res) > 0) {
        foreach ($res as $link) {
            $type = ldap_dn2cn(ldap_dn2oudn(ldap_dn2oudn($link['dn'])));
            $gpo = ldap_dn2cn(ldap_dn2oudn(ldap_dn2oudn(ldap_dn2oudn($link['dn']))));
            $ous = gpolistcontainers($config, $gpo);
            foreach ($ous as $key => $ou_dn) {
                $ou = search_ad($config, ldap_dn2cn($ou_dn), "ou");
                if (count($ou) > 0) {
                    $type_ou = "user";
                } else {
                    $type_ou = "machine";
                }
                if (strcasecmp($type, $type_ou) != 0) {
                    unset($ous[$key]);
                }
            }
            $list[] = array(
                'gpo' => $gpo,
                'type' => strtolower($type),
                'links' => $ous
            );
        }
    }
    return $list;
}

/**
 * liste toutes les delegations d'un groupe, avec tous les details des gpo
 *
 * @param unknown $config
 * @param unknown $salle
 * @return array|array[]
 */
function list_salle_delegations($config, $salle, $fast = true)
{
    $delegations = array();
    $ou = search_parcs($config, $salle, "salle");
    $type = "machine";
    if (count($ou) == 0) {
        $ou = search_ad($config, $salle, "ou");
        $type = "user";
    }
    if (count($ou) > 0) {
        $gpos = gpogetlink($config, $ou[0]['dn']);
        foreach ($gpos as $key => $gpo) {
            // $type_link = strtolower(ldap_dn2cn(ldap_dn2oudn(ldap_dn2oudn($gpo['dn']))));
            // if ($type == $type_link) {
            $delegations = array_merge(search_ad($config, "*", "delegation", "cn=" . $gpo['uuid']), $delegations);
            // }
        }
        foreach ($delegations as $key => $delegation) {
            if (isset($delegations[$key])) {
                $type_ou = strtolower(ldap_dn2cn(ldap_dn2oudn(ldap_dn2oudn($delegation['dn']))));
                if ($type_ou != $type) {
                    unset($delegations[$key]);
                } else {
                    $delegations[$key]['type'] = $type_ou;
                    $delegations[$key]['gpo'] = ldap_dn2cn(ldap_dn2oudn(ldap_dn2oudn(ldap_dn2oudn($delegation['dn']))));
                    $delegations[$key] = array_merge(list_delegations($config, $delegation['delegationname'], $fast)[0], $delegations[$key]);
                }
            }
        }
    }
    return $delegations;
}

/**
 * crée l'delegation dans une gpo
 *
 * @param unknown $config
 * @param unknown $delegation
 *            nom
 * @param unknown $gpo
 *            guid
 * @param unknown $type
 *            user|machine
 * @return string|boolean
 */
function add_delegation_policy($config, $delegation, $gpo, $type)
{
    $sid = get_sid($config, $delegation);
    $data = list_delegation_policies($config, $gpo);

    if (empty($data)) {
        $data = [
            'Unicode' => [
                "unicode=yes"
            ],
            [
                'Version' => [
                    "signature=\"\$CHICAGO\$\"",
                    "Revision=1"
                ],
                'Group Membership' => [
                    "*" . $sid . "__Memberof = *S-1-5-32-544",
                    "*" . $sid . "__Members ="
                ]
            ]
        ];
    } else {
        $data['Group Membership'][$delegation]['Memberof'] = [
            "*S-1-5-32-544"
        ];
    }

    $parent = "CN=PusheddelegationConnections,CN=" . ucfirst($type) . ",CN=" . $gpo . ",CN=Policies,CN=System," . $config['ldap_base_dn'];
    $exist = search_ad($config, "PusheddelegationConnections", "container", "cn=" . $type . ",cn=" . $gpo);
    $guid = guid();
    $attrs = array(
        // 'objectClass' => array(
        // "top",
        'objectclass' => "msPrint-ConnectionPolicy",
        // ),
        'cn' => $guid,
        'name' => $guid,
        'objectCategory' => "CN=ms-Print-ConnectionPolicy,CN=Schema,CN=Configuration," . $config['ldap_base_dn'],
        // 'uNCName' => "\\\\" . $config['se4fs_name'] . "." . $config['domain'] . "\\" . $delegation,
        // 'servername' => "\\\\" . $config['se4fs_name'] . "." . $config['domain'],
        'uNCName' => "\\\\" . $config['se4fs_name'] . "\\" . $delegation,
        'servername' => "\\\\" . $config['se4fs_name'],
        'delegationName' => $delegation,
        'printAttributes' => 0
    );
    $dn = "cn=" . $guid . "," . $parent;
    // $config['bind'] = bind_ad_gssapi($config);
    if (! $exist) {
        $container = array(
            'cn' => ucfirst($type),
            'objectclass' => "container"
        );
        $newdn = "CN=" . ucfirst($type) . ",CN=" . $gpo . ",CN=Policies,CN=System," . $config['ldap_base_dn'];
        ldap_add($config['bind'], $newdn, $container);
        $err = ldap_error($config['bind']);
        $container = array(
            'cn' => "PusheddelegationConnections",
            'objectclass' => "container"
        );
        $newdn = "CN=PusheddelegationConnections,CN=" . ucfirst($type) . ",CN=" . $gpo . ",CN=Policies,CN=System," . $config['ldap_base_dn'];
        ldap_add($config['bind'], $newdn, $container);
        $err .= ldap_error($config['bind']);
    }
    $ret = ldap_add($config['bind'], $dn, $attrs);
    // ldap_close($config['bind']);
    if ($ret) {
        // ajout extension 180... correspond a la console gpo et CC13... a la console delegation
        if ($type == "user") {
            $attrs2['gpcuserextensionnames'] = "[{8A28E2C5-8D06-49A4-A08C-632DAA493E17}{180F39F3-CF17-4C68-8410-94B71452A22D}{CC13E3F3-D6D7-4A7C-A806-085502AA8281}]]";
        } else {
            $attrs2['gpcmachineextensionnames'] = "[{8A28E2C5-8D06-49A4-A08C-632DAA493E17}{180F39F3-CF17-4C68-8410-94B71452A22D}]";
        }
        @modify_ad($config, $gpo, "gpo", $attrs2, "add");
        increment_gpo($config, $gpo);
        return true;
    } else {
        return false;
    }
}

/**
 * liste les delegations de groupes locaux dans une gpo
 *
 * @param unknown $config
 * @param unknown $gpo
 * @return array data for gpttmpl.ini
 */
function list_delegation_policies($config, $gpo)
{
    $data = read_gpo_sysvol($config, $gpo, MACHINE_SECEDIT);
    foreach ($data['Group Membership'] as $key => $value) {

        $g = explode("__", $key);
        $v = explode(",", $value);
        $data['Group Membership'][$g[0]][$g[1]] = $v;
    }
    return $data;
}

/**
 * supprime l'delegation dans une gpo.
 * Ne supprime pas la gpo ni ses liens
 *
 * @param unknown $config
 * @param unknown $delegation
 *            nom
 * @param unknown $gpo
 *            guid
 * @param unknown $type
 *            user|machine
 * @return boolean
 */
function del_delegation_policy($config, $delegation, $gpo, $type)
{
    $p = search_ad($config, $delegation, "delegation", "cn=" . $type . ",cn=" . $gpo);
    if (count($p) > 0) {
        $dn = $p[0]['dn'];
        // $config['bind'] = bind_ad_gssapi($config);
        $ret = ldap_delete($config['bind'], $dn);
        // ldap_close($config['bind']);
        increment_gpo($config, $gpo);
        return $ret;
    }
    return true;
}

/**
 * assigne une delegation à un groupe (salle ou eleves|profs|administratifs) en créant une gpo si besoin, en la liant au groupe
 *
 * @param unknown $config
 * @param unknown $delegation
 * @param unknown $salle
 * @return boolean|unknown
 */
function add_delegation_salle($config, $delegation, $salle)
{
    $type = "";
    $attrs = array();
    // si parc, gpo machine, si ou people, user
    $ou = search_parcs($config, $salle, "salle");
    if (count($ou) > 0) {
        $type = "machine";
    } // ad
      // on ajoute au groupe ad correspondant, potentiellement pour les droits et les vlans
    groupaddmember($config, $delegation, $salle . $config['suffix']);
    // existe-t-il déjà des gpo utilisables dans ce groupe ? on utilise le premier dispo
    $gpos = list_salle_delegations($config, $salle);
    if (count($gpos) > 0) {
        foreach ($gpos as $key => $gpo) {
            if ($gpo['delegationname'] == $delegation) {
                // déjà en place
                return true;
            } else {
                $list = list_delegation_policies($config, $gpo['gpo']);
                $p = $list[$type] ?? array();
                $nb_delegations = count($p);

                if (($nb_delegations > 0) && ($p[0]['count'] == 1)) {
                    // on peut ajouter
                    return add_delegation_policy($config, $delegation, $gpos[$key]['gpo'], $type);
                }
            }
        }
    } else {
        // existe-t-il des stratégies utilisables déjà configurées pour d'autres groupes ?
        $gpos = list_delegation_salles($config, $delegation);
        if (count($gpos) > 0) {
            foreach ($gpos as $key => $gpo) {
                $list = list_delegation_policies($config, $gpo['gpo']);
                $p = $list[$type] ?? array();
                $nb_delegations = count($p);

                if ($nb_delegations == 0) {
                    // on peut ajouter
                    add_delegation_policy($config, $delegation, $gpos[$key]['gpo'], $type);
                    return gposetlink($config, $ou[0]['dn'], $gpos[$key]['gpo']);
                } elseif ($nb_delegations == 1) {

                    return gposetlink($config, $ou[0]['dn'], $gpos[$key]['gpo']);
                }
            }
        }
        // nouvelle gpo
        $gpo = gpocreate($config, "se4_delegations_" . $salle . "_" . $delegation);
        if ($gpo) {
            // ajout extension console secedit

            $attrs['gpcmachineextensionnames'] = "[{827D319E-6EAC-11D2-A4EA-00C04F79F83A}{803E14A0-B4FB-11D0-A0D0-00A0C90F574B}]";
            modify_ad($config, $gpo, "gpo", $attrs);
            add_delegation_policy($config, $delegation, $gpo, $type);
            return gposetlink($config, $ou[0]['dn'], $gpo);
        }
        return false;
    }
    return false;
}

/**
 * supprime une delegation d'un groupe (salle ) :
 * - la gpo contient plusieurs delegations, et est liée a un seul groupe : on enlève l'delegation de la gpo
 * - la gpo contient plusieurs delegations, et et liée à plusieurs groupes : on supprime le lien et on recrée une gpo pour ce groupe avec les delegations restantes
 * - la gpo contient une seule delegation et est liée a un seul groupe : on supprime la gpo
 * - la gpo contient une seule delegation et est liée plusieurs groupe : on supprime le lien
 *
 * @param unknown $config
 * @param unknown $delegation
 * @param unknown $salle
 * @return boolean|unknown
 */
function remove_delegation_salle($config, $delegation, $salle)
{
    $type = $autre_type = "";
    // si parc, gpo machine, si ou people, user
    $ou = search_parcs($config, $salle, "salle");
    if (count($ou) > 0) {
        $type = "machine";
        $autre_type = "user";
    } else {
        $ou = search_ad($config, $salle, "ou");
        if (count($ou) > 0) {
            $type = "user";
            $autre_type = "machine";
        }
    }
    groupdelmember($config, $delegation, $salle);
    $gpos = list_salle_delegations($config, $salle);
    if (count($gpos) > 0) {
        foreach ($gpos as $gpo) {
            $list = list_delegation_policies($config, $gpo['gpo']);
            $p = $list[$type];
            $ap = $list[$autre_type] ?? array();
            $nb_delegations = count($p);
            if ($nb_delegations == 1) {
                if ($p[0]['count'] == 1) {
                    if ((count($ap) == 0) || ($ap[0]['count'] == 0)) {
                        // aucun autre lien, on enleve la gpo
                        gpodellink($config, $ou[0]['dn'], $gpo['gpo']);
                        return gpodel($config, $gpo['gpo']);
                    } else {
                        // des liens dans d'autres types, on efface et on enleve le lien
                        del_delegation_policy($config, $delegation, $gpo['gpo'], $type);
                        return gpodellink($config, $ou[0]['dn'], $gpo['gpo']);
                    }
                } else {
                    // plusieurs liens
                    return gpodellink($config, $ou[0]['dn'], $gpo['gpo']);
                }
            } else {
                if ($p[0]['count'] == 1) {
                    // pas d'autre liens, on peut supprimer
                    return del_delegation_policy($config, $delegation, $gpo['gpo'], $type);
                } else {
                    // cela se complique, on enleve le lien pour ce groupe
                    gpodellink($config, $ou[0]['dn'], $gpo['gpo']);
                    // on recree une gpo avec les delegations restantes
                    $ret = false;
                    foreach ($list as $np) {
                        if ($np['delegationname'] != $delegation) {
                            $ret = add_delegation_salle($config, $np['delegationname'], $salle);
                        }
                    }
                    return $ret;
                }
            }
        }
    }
    return true;
}
?>
