<?php

/**
 * Fonctions utiles
 * @Version 02-2020
 * @Projet SambaEdu
 * Fonctions Interface Homme/Machine
 * @Auteurs Denis Bonnenfant @ Equipe Sambaedu
 * @Note: Ce fichier de fonction doit etre appele par un include
 * @Licence Distribue sous la licence GPL
 */

/**
 *
 * file: fonc_outils.inc.php
 *
 * @Repertoire: includes/
 */
use GuzzleHttp\Exception\GuzzleException;

require_once "logs.inc.php";

function ldapTimeToUnixTime($ldapTime)
{
    $secsAfterADEpoch = $ldapTime / 10000000;
    $ADToUnixConverter = ((1970 - 1601) * 365 - 3 + round((1970 - 1601) / 4)) * 86400;
    return intval($secsAfterADEpoch - $ADToUnixConverter);
}

function unixTimeToLdapTime($unixTime)
{
    $ADToUnixConverter = ((1970 - 1601) * 365 - 3 + round((1970 - 1601) / 4)) * 86400;
    $secsAfterADEpoch = intval($ADToUnixConverter + $unixTime);
    return $secsAfterADEpoch * 10000000;
}

/**
 *
 * Fonctions qui retourne le nom d'une machine en fonction de son adresse IP
 *
 * @Parametres Adresse IP de la machine
 *
 * @return Le nom de la machine
 *        
 */
function avoir_nom($ipHost)
{ // Retourne le nom d'une machine a partir de l'adresse IP ou 0 si pas
    $mp_curr = search_ad("(&(ipHostNumber=$ipHost)(objectClass=ipHost))", "computers");
    if (isset($mp_curr[0]["cn"])) {
        $mpenc = $mp_curr[0]['cn'];
        return $mpenc;
    } else {
        return 0;
    }
}

/**
 * Demarre, eteint ou reboote un poste
 *
 * @Parametres action: wol, reboot, shutdown
 *             nom : nom du poste
 *
 * @return
 *
 */
function start_poste($config, $action, $name, $force = false, $silent = false)
{
    $name = strtolower($name);

    if ($name == $config['se4fs_name'] || $name == $config['se4ad_name'])
        return "Ca va pas ?";

    $machine = search_machine($config, $name, true);
    $ret = false;
    $out = array();
    $msg = "";
    $status = get_machine_status($config, $name);
    $ip = $machine['iphostnumber'] ?? $name;
    $ping = fping($ip);
    switch ($action) {
        case "start":
        case "wol":
            if (! $ping || $status['state'] == "off" || $force) {
                if (empty($machine['networkaddress'])) {
                    log_connexion($config, "", $name, "wol", "startup", "", SAMBAEDU_NO_WOL);
                    $msg .= "On n'allume pas  <b>$name</b> : elle n'a pas d'adresse mac enregistrée !";
                    break;
                }
                $reseau = get_vlan($config, @$machine['iphostnumber']);
                $msg .= "Mise en marche de la machine  <b>$name</b> : ";
                if ($reseau !== false) {
                    exec("/usr/bin/wakeonlan -i " . long2ip($reseau['broadcast']) . " " . $machine['networkaddress'], $out);
                }
                $msg .= end($out) . "<br>";
                if (isset($config['wol_broadcast'])) {
                    exec("/usr/bin/wakeonlan -i " . $config['wol_broadcast'] . " " . $machine['networkaddress'], $out);
                    $msg .= end($out) . "<br>";
                }
                if (! $ping) {
                    log_connexion($config, "", $name, "wol", "startup");
                }
            } else {
                $msg .= "On n'allume pas  <b>$name</b> : elle est déjà allumée !";
            }
            break;

        case "reboot":
            if ($ping && ($status['state'] != "off" || $force)) {
                $msg .= "On reboote avec l'action <b>" . $action . "</b> le poste <b>" . $name . "</b> :<br>\n";
                // if ($status['state'] != "linux") {
                if ($ping == "windows") {
                    // machine windows
                    if ($silent) {
                        $commande = "/usr/bin/net rpc shutdown --use-kerberos=required -t 2 -f -r -C 'Reboot demande par le serveur sambaEdu' -S " . $name . " >/dev/null 2>&1 &";
                    } else {
                        $commande = "/usr/bin/net rpc shutdown --use-kerberos=required -t 2 -f -r -C 'Reboot demande par le serveur sambaEdu' -S " . $name . " 2>&1";
                    }
                    exec($commande, $out, $ret);
                    if ($ret != 0) {
                        $msg .= "<b>Attention, reboot impossible</b> : " . end($out) . "<br><br>";
                        log_connexion($config, "", $name, "windows", "shutdown", "", SAMBAEDU_SHUTDOWN_ERROR);
                        $msg .= start_poste($config, "wol", $name, true);
                    } else {
                        log_connexion($config, "", $name, "windows", "shutdown");
                        $msg .= end($out);
                    }
                    $msg .= "<br><br>";
                } elseif ($ping == "linux") {
                    // poste linux : ssh...
                    exec("/usr/bin/ssh -i /etc/sambaedu/id_rsa -o StrictHostKeyChecking=no -o ConnectTimeout=1 root@" . $machine['cn'] . " shutdown -r now", $out, $ret);
                    $msg .= end($out) . "<br><br>";
                    if ($ret != 0) {
                        $msg .= "On reboote avec l'action <b>" . $action . "</b> le poste <b>" . $name . "</b> :<br>\n";
                        $msg .= "<b>Attention, reboot impossible</b>, on essaye de la démarrer <br><br>";
                        log_connexion($config, "", $name, "linux", "shutdown", "", SAMBAEDU_SHUTDOWN_ERROR);
                        $msg .= start_poste($config, "wol", $name, true);
                    } else {
                        log_connexion($config, "", $name, "linux", "shutdown");
                        $msg .= "OK<br>\n";
                    }
                }
            } else {
                $msg .= "<b>$name</b>  est éteinte : on l'allume : <br>\n";
                $msg .= start_poste($config, "wol", $name, true);
            }
            break;

        case "stop":
        case "shutdown":
            if ($force || (isset($status['user']) && count($status['user']) == 0)) {
                if ($ping) {
                    $msg .= "On éteint avec l'action <b>" . $action . "</b> le poste <b>" . $name . "</b> : <br>\n";
                }
                // if ($status['state'] != "linux") {
                if ($ping == "windows") {

                    // machine windows
                    if ($silent) {
                        $commande = "/usr/bin/net rpc shutdown --use-kerberos=required -t 30 -f -C \"Arret demande par le serveur sambaEdu\" -S " . $name . " >/dev/null 2>&1 &";
                    } else {
                        $commande = "/usr/bin/net rpc shutdown --use-kerberos=required -t 30 -f -C \"Arret demande par le serveur sambaEdu\" -S " . $name . " 2>&1";
                    }
                    exec($commande, $out, $ret);
                    if ($ret != 0) {
                        $msg .= "<b>Attention, arrêt impossible</b> : " . end($out) . "<br><br>";
                        log_connexion($config, "", $name, "windows", "shutdown", "", SAMBAEDU_SHUTDOWN_ERROR);
                    } else {
                        log_connexion($config, "", $name, "windows", "shutdown");
                        $msg .= end($out);
                    }
                    $msg .= "<br><br>";
                } elseif ($ping == "linux") {
                    // poste linux : ssh...
                    exec("/usr/bin/ssh -i /etc/sambaedu/id_rsa -o StrictHostKeyChecking=no -o ConnectTimeout=1 root@" . $machine['cn'] . " shutdown -h now", $out, $ret);
                    $msg .= end($out) . "<br><br>";
                    if ($ret != 0) {
                        log_connexion($config, "", $name, "linux", "shutdown", "", SAMBAEDU_SHUTDOWN_ERROR);
                        $msg .= "On éteint avec l'action <b>" . $action . "</b> le poste <b>" . $name . "</b> : <br>\n";
                        $msg .= "<b>Attention, arrêt impossible</b>, la machine est injoignable ! <br><br>";
                    } else {
                        log_connexion($config, "", $name, "linux", "shutdown");
                        $msg .= "OK<br>\n";
                    }
                }
            } else {
                $msg .= "On n'éteint pas le <b> le poste <b>" . $name . "</b> : un utilisateur est connecté<br>\n";
            }
            break;
    }
    return $msg;
}

/**
 * Demarre, eteint ou reboote un poste
 *
 * @Parametres action: wol, reboot, shutdown
 *             nom : nom du poste
 *
 * @return
 *
 */
function start_poste_async($config, $action, array &$machine, $force = false, $silent = false)
{
    $name = strtolower($machine['cn']);

    if ($name == $config['se4fs_name'] || $name == $config['se4ad_name'])
        return "Ca va pas ?";

    $ret = false;
    $out = array();
    $msg = "";
    $status = get_machine_status($config, $name);
    $ip = $machine['iphostnumber'] ?? $name;
    $ping = fping($ip);
    switch ($action) {
        case "status":
            $code = $status['code'] ?? 200;
            $msg = $status['message'] ?? "";
            break;
        case "start":
        case "wol":
            if (! $ping || $status['state'] == "off" || $force) {
                if (empty($machine['networkaddress'])) {
                    log_connexion($config, "", $name, "wol", "startup", "", SAMBAEDU_NO_WOL);
                    $msg .= "On n'allume pas  <b>$name</b> : elle n'a pas d'adresse mac enregistrée !";
                    $code = 203;
                    break;
                }
                $reseau = get_vlan($config, @$machine['iphostnumber']);
                $msg .= "Mise en marche de la machine  <b>$name</b> : ";
                if ($reseau !== false) {
                    exec("/usr/bin/wakeonlan -i " . long2ip($reseau['broadcast']) . " " . $machine['networkaddress'], $out);
                }
                $msg .= end($out) . "<br>";
                if (isset($config['wol_broadcast'])) {
                    exec("/usr/bin/wakeonlan -i " . $config['wol_broadcast'] . " " . $machine['networkaddress'], $out);
                    $msg .= end($out) . "<br>";
                }
                if (! $ping) {
                    log_connexion($config, "", $name, "wol", "startup");
                    $code = 202;
                } else {
                    $code = 201;
                }
            } else {
                $msg .= "On n'allume pas  <b>$name</b> : elle est déjà allumée !";
                $code = 200;
            }
            break;

        case "reboot":
            if ($ping && ($status['state'] != "off" || $force)) {
                $msg .= "On reboote avec l'action <b>" . $action . "</b> le poste <b>" . $name . "</b> :<br>\n";
                // if ($status['state'] != "linux") {
                if ($ping == "windows") {
                    // machine windows
                    if ($silent) {
                        $commande = "/usr/bin/net rpc shutdown --use-kerberos=required -t 2 -f -r -C 'Reboot demande par le serveur sambaEdu' -S " . $name . " >/dev/null 2>&1 &";
                    } else {
                        $commande = "/usr/bin/net rpc shutdown --use-kerberos=required -t 2 -f -r -C 'Reboot demande par le serveur sambaEdu' -S " . $name . " 2>&1";
                    }
                    exec($commande, $out, $ret);
                    if ($ret != 0) {
                        $msg .= "<b>Attention, reboot impossible</b> : " . end($out) . "<br><br>";
                        log_connexion($config, "", $name, "windows", "shutdown", "", SAMBAEDU_SHUTDOWN_ERROR);
                        $msg .= start_poste($config, "wol", $name, true);
                        $code = 203;
                    } else {
                        log_connexion($config, "", $name, "windows", "shutdown");
                        $msg .= end($out);
                        $code = 201;
                    }
                    $msg .= "<br><br>";
                } elseif ($ping == "linux") {
                    // poste linux : ssh...
                    exec("/usr/bin/ssh -i /etc/sambaedu/id_rsa -o StrictHostKeyChecking=no -o ConnectTimeout=1 root@" . $machine['cn'] . " shutdown -r now", $out, $ret);
                    $msg .= end($out) . "<br><br>";
                    if ($ret != 0) {
                        $msg .= "On reboote avec l'action <b>" . $action . "</b> le poste <b>" . $name . "</b> :<br>\n";
                        $msg .= "<b>Attention, reboot impossible</b>, on essaye de la démarrer <br><br>";
                        log_connexion($config, "", $name, "linux", "shutdown", "", SAMBAEDU_SHUTDOWN_ERROR);
                        $msg .= start_poste($config, "wol", $name, true);
                        $code = 203;
                    } else {
                        log_connexion($config, "", $name, "linux", "shutdown");
                        $msg .= "OK<br>\n";
                        $code = 201;
                    }
                }
            } else {
                $msg .= "<b>$name</b>  est éteinte : on l'allume : <br>\n";
                $msg .= start_poste($config, "wol", $name, true);
                $code = 201;
            }
            break;

        case "stop":
        case "shutdown":
            if ($force || (isset($status['user']) && count($status['user']) == 0)) {
                // if ($status['state'] != "linux") {
                if ($ping == "windows") {
                    $msg .= "On éteint avec l'action <b>" . $action . "</b> le poste <b>" . $name . "</b> : <br>\n";

                    // machine windows
                    if ($silent) {
                        $commande = "/usr/bin/net rpc shutdown --use-kerberos=required -t 30 -f -C \"Arret demande par le serveur sambaEdu\" -S " . $name . " >/dev/null 2>&1 &";
                    } else {
                        $commande = "/usr/bin/net rpc shutdown --use-kerberos=required -t 30 -f -C \"Arret demande par le serveur sambaEdu\" -S " . $name . " 2>&1";
                    }
                    exec($commande, $out, $ret);
                    if ($ret != 0) {
                        $msg .= "<b>Attention, arrêt impossible</b> : " . end($out) . "<br><br>";
                        log_connexion($config, "", $name, "windows", "shutdown", "", SAMBAEDU_SHUTDOWN_ERROR);
                        $code = 203;
                    } else {
                        log_connexion($config, "", $name, "windows", "shutdown");
                        $msg .= end($out);
                        $code = 201;
                    }
                    $msg .= "<br><br>";
                } elseif ($ping == "linux") {
                    $msg .= "On éteint avec l'action <b>" . $action . "</b> le poste <b>" . $name . "</b> : <br>\n";

                    // poste linux : ssh...
                    exec("/usr/bin/ssh -i /etc/sambaedu/id_rsa -o StrictHostKeyChecking=no -o ConnectTimeout=1 root@" . $machine['cn'] . " shutdown -h now", $out, $ret);
                    $msg .= end($out) . "<br><br>";
                    if ($ret != 0) {
                        log_connexion($config, "", $name, "linux", "shutdown", "", SAMBAEDU_SHUTDOWN_ERROR);
                        $msg .= "On éteint avec l'action <b>" . $action . "</b> le poste <b>" . $name . "</b> : <br>\n";
                        $msg .= "<b>Attention, arrêt impossible</b>, la machine est injoignable ! <br><br>";
                    } else {
                        log_connexion($config, "", $name, "linux", "shutdown");
                        $msg .= "OK<br>\n";
                    }
                } else {
                    $msg .= "action <b>" . $action . "</b> le poste <b>" . $name . "</b> est déjà eteint <br>\n";
                    $code = 200;
                }
            } else {
                $msg .= "On n'éteint pas le <b> le poste <b>" . $name . "</b> : un utilisateur est connecté<br>\n";
                $code = 203;
            }
            break;
    }
    $status['code'] = $code;
    $status['message'] = $msg;
    $machine = array_merge($machine, $status);
    return $code;
}

/**
 * fonction permettant l'appel asynchrone du statut et des actions sur les postes passées dans $list
 * les requètes seront faites en // sur les serveurs d'etab des machines
 *
 * @param unknown $config
 * @param unknown $action
 *            (status|start|stop|reboot) @TODO a étendre ?
 * @param unknown $list
 *            liste AD des machines, statut mis à jour par la fonction.
 * @param array $reste
 *            machines non traitées ( pas de réponse du serveur ?)
 */
function async_action($config, $action, &$list, &$reste = [])
{
    // appel en parallèle de chaque machine, sauf pour celles qui sont ok en cache.
    require_once (dirname(__FILE__) . '/../vendor/autoload.php');
    $opt = [
        'base_uri' => $config['se4_url'] . "/",
        'cookies' => true
    ];
    curl_proxy_options($config, $opt);
    $client = new GuzzleHttp\Client($opt);
    $requests = function ($list, $action, $reste) use ($client) {
        foreach ($list as $k => $machine) {
            if (! isset($machine['status']['code']) || $machine['status']['code'] != 200) {
                yield $k => function () use ($client, $machine, $action) {
                    $config = [];
                    $config = get_config($config, true);
                    if (! empty($machine['etab'])) {
                        $path = $machine['etab'] . "/";
                    } else {
                        $path = "/";
                    }
                    return $client->postAsync($path . 'api2/action/machine', [
                        'form_params' => [
                            'machine' => json_encode($machine, JSON_PRETTY_PRINT | JSON_INVALID_UTF8_IGNORE),
                            'action' => $action
                        ]
                    ]);
                };
            }
        }
    };

    $pool = new GuzzleHttp\Pool($client, $requests($list, $action, $reste), [
        'concurrency' => 100,
        'fulfilled' => function (GuzzleHttp\Psr7\Response $response, $index) use (&$list, &$reste) {
            // this is delivered each successful response
            $res = json_decode((string) $response->getBody(), true);
            $list[$index] = $res;
        },
        'rejected' => function (GuzzleException $reason, $index) use (&$list, &$reste) {
            // this is delivered each failed request
            echo "erreur : " . $reason->getMessage();
            // echo "<pre>" . print_r($list, true) . "</pre>";
            $reste[] = $list[$index];
            unset($list[$index]);
        }
    ]);

    // Initiate the transfers and create a promise
    $promise = $pool->promise();

    // Force the pool of requests to complete.
    $promise->wait();
    // $res = $parc;
}

/**
 * Demarre, eteint ou reboote un parc
 *
 * @Parametres action: wol, reboot, shutdown
 *             parc : nom du parc
 *
 * @return
 *
 */
function start_parc($config, $action, $parc, $force = false, $silent = true)
{
    $liste = list_members_parc($config, $parc, false);
    sort($liste);
    $ret = "";
    foreach ($liste as $value) {
        $ret .= start_poste($config, $action, $value, $force, $silent) . "<br>\n";
        // usleep(300000);
    }
    return $ret;
}

/**
 * Affiche la date dans le format j/m/a
 *
 * @Parametres la date
 *
 * @return la date au format j/m/a
 */
function affiche_date($date)
{
    list ($a, $m, $j) = preg_split("/-/", $date);
    return "$j/$m/$a";
}

/**
 * Retourne une liste sous forme d'un tableau
 *
 * @Parametres $liste
 *
 * @return la tableau
 */
function liste_tab($liste)
{
    $t = preg_split("/\|/", $liste);
    for ($i = 0; $i < count($t); $i = $i + 2) {
        $cle = $t[$i];
        $val = $t[$i + 1];
        $tab[$cle] = $val;
    }
    return $tab;
}

/**
 * Retourne un select pour choisir la date
 *
 * @Parametres date et param
 * @return string select au sens HTML
 *        
 */
function choix_date($date, $param)
{
    $tab_mois = array(
        9 => "Septembre",
        10 => "Octobre",
        11 => "Novembre",
        12 => "Décembre",
        1 => "Janvier",
        2 => "Février",
        3 => "Mars",
        4 => "Avril",
        5 => "Mai",
        6 => "Juin",
        7 => "Juillet",
        8 => "Août"
    );
    list ($an, $mois, $jour) = preg_split("/-/", $date);
    $html = "<select name=\"jour_$param\">";
    // <option value=\"jour\"> jour</option>";
    for ($i = 1; $i <= 31; $i ++) {
        $ii = ($i < 10 ? "0$i" : $i);
        $html .= "<option value=\"$ii\" " . ($ii == $jour ? "selected" : "") . "> $ii</option>";
    }
    $html .= "</select>";
    $html .= "<select name=\"mois_$param\"> ";
    // <option value=\"mois\"> mois</option>";
    for ($i = 1; $i <= 12; $i ++) {
        $ii = ($i < 10 ? "0$i" : $i);
        $html .= "<option value=\"$ii\" " . ($ii == $mois ? "selected" : "") . "> $tab_mois[$i]</option>";
    }
    $html .= "</select>";
    $html .= "<select name=\"an_$param\"> ";
    // <option value=\"an\"> année</option>";
    $html .= "<option value=\"$an\" selected> $an</option>";
    $an_suivant = $an + 1;
    $html .= "<option value=\"$an_suivant\" > $an_suivant</option>";
    $html .= "</select>";
    return $html;
}

/**
 * a partir d'un uid d'élève fournit le nom inversé de son répertoire classe
 * en cas de login prenom.nom, renvoie nom.prenom, sinon renvoie le login
 *
 * @Parametres $uid
 *
 * @return $rep
 */
function inverse_login($login)
{
    $tab = preg_split('/\./', $login);
    if (count($tab) == 2) {
        $rep = $tab[1] . "." . $tab[0];
    } else {
        $rep = $login;
    }
    return $rep;
}

function affiche_all_groups($config, $align, $afftype, $cours_gr = [])
{
    // // Etablissement des listes des groupes disponibles
    $uai = $config['etab_ou'] ?? 0;
    if ($afftype == "central") {
        $list_groups = search_ad($config, "*", "group_fast", "all");
    } else {
        $list_groups = search_ad($config, "*", "group_fast");
    }
    // $html .= $config['dn']['groups'];
    // Etablissement des sous listes de groupes :
    $j = 0;
    $k = 0;
    $l = 0;
    $i = 0;
    $classe = [];
    $cours = $matiere = $equipe = $autres = $rights = array();
    for ($loop = 0; $loop < count($list_groups); $loop ++) {
        // Cours
        if (preg_match("/Cours_/", $list_groups[$loop]["cn"]) && $list_groups[$loop]["etab"] == $uai) {
            $cours[$i]["cn"] = $list_groups[$loop]["cn"];
            $cours[$i]["description"] = $list_groups[$loop]["description"] ?? $list_groups[$loop]["cn"];
            $i ++;
        }
        // // Classe
        if (preg_match("/Classe_/", $list_groups[$loop]["cn"]) && $list_groups[$loop]["etab"] == $uai) {
            $classe[$j]["cn"] = $list_groups[$loop]["cn"];
            $classe[$j]["description"] = $list_groups[$loop]["description"] ?? $list_groups[$loop]["cn"];
            $j ++;
        } // Matiere
        elseif (preg_match("/Matiere_/", $list_groups[$loop]["cn"]) && $list_groups[$loop]["etab"] == $uai) {
            $matiere[$l]["cn"] = $list_groups[$loop]["cn"];
            $matiere[$l]["description"] = $list_groups[$loop]["description"] ?? $list_groups[$loop]["cn"];
            $l ++;
        } // Equipe
        elseif (preg_match("/Equipe_/", $list_groups[$loop]["cn"]) && $list_groups[$loop]["etab"] == $uai) {
            $equipe[$k]["cn"] = $list_groups[$loop]["cn"];
            $equipe[$k]["description"] = $list_groups[$loop]["description"] ?? $list_groups[$loop]["cn"];
            $k ++;
        } elseif (preg_match("/" . $config['rights_rdn'] . "/i", $list_groups[$loop]["dn"])) {
            $rights[$k]["cn"] = $list_groups[$loop]["cn"];
            $rights[$k]["description"] = $list_groups[$loop]["description"] ?? $list_groups[$loop]["cn"];
            $k ++;
        } // Autres
          // elseif (! preg_match("/^overfill/", $list_groups[$loop]["cn"]) && ! preg_match("/^lcs-users/", $list_groups[$loop]["cn"]) && ! preg_match("/nogroup/", $list_groups[$loop]["cn"]) && ! preg_match("/Cours_/", $list_groups[$loop]["cn"]) && ! preg_match("/Matiere_/", $list_groups[$loop]["cn"]) && ! preg_match("/root/", $list_groups[$loop]["cn"]) && ! preg_match("/invites/", $list_groups[$loop]["cn"]) && ! preg_match("/^machines/", $list_groups[$loop]["cn"])) {
          // $autres[$m]["cn"] = $list_groups[$loop]["cn"];
          // $autres[$m]["description"] = $list_groups[$loop]["description"] ?? "";
          // $m ++;
          // }
    }

    // Affichage des boites de sélection des groupes sur lesquels fixer les quotas + choix d'un user spécifique
    $html = "
		<table align=\"$align\" border=\"0\" cellspacing=\"10\" style='float: none' >
		<tr>
		<td>Classes</td>";
    $html .= "<td>Mati&#232res</td>";
    if ($afftype == "cours")
        $html .= "<td>Cours</td>";
    $html .= "<td>Equipes</td>
		<td>Autres</td>";
    if ($afftype == "user")
        $html .= "<td>Utilisateur spécifique</td>";
    $html .= "</tr>
		<tr>
		<td valign=\"top\">
        
		<select name= \"classe_gr[]\" size=\"8\" multiple=\"multiple\">\n";
    for ($loop = 0; $loop < count($classe); $loop ++) {
        $html .= "<option value=" . $classe[$loop]["cn"] . ">" . $classe[$loop]["description"];
    }
    $html .= "</select>";
    $html .= "</td>\n";

    $html .= "<td valign=\"top\">\n";
    $html .= "<select name= \"matiere_gr[]\" size=\"8\" multiple=\"multiple\">\n";
    for ($loop = 0; $loop < count($matiere); $loop ++) {
        $html .= "<option value=" . $matiere[$loop]["cn"] . ">" . $matiere[$loop]["description"];
    }
    $html .= "</select></td>\n";
    // $html .= "<td valign=\"top\">";

    if ($afftype == "cours") {
        $html .= "<td valign=\"top\">\n";
        $html .= "<select name= \"cours_gr[]\" value=\"$cours_gr\" size=\"10\" multiple=\"multiple\">";
        foreach ($cours as $c) {
            $html .= "<option value=" . $c['cn'] . ">" . $c["description"];
        }
        $html .= "</select>";
        $html .= "</td>";
    }

    $html .= "<td valign=\"top\">\n";
    $html .= "<select name= \"equipe_gr[]\" size=\"8\" multiple=\"multiple\">\n";
    foreach ($equipe as $e) {
        $html .= "<option value=" . $e["cn"] . ">" . $e["description"];
    }
    $html .= "</select></td>\n";
    $html .= "<td valign=\"top\">";
    $autres = array_merge([['cn' => "Eleves"], ['cn' => "Profs"], ['cn' => "Administratifs"]], filter_group_autres($config));
    $autres_projets = filter_group_projets($config);
    $html .= "<select name=\"autres_gr[]\" size=\"8\" multiple=\"multiple\">";
    $html .= "<option value=administratifs>administratifs";
    $html .= "<option value=eleves>eleves";
    $html .= "<option value=profs>profs";
    foreach ($autres as $a) {
        $html .= "<option value=" . $a["cn"] . ">" . $a["cn"];
    }
    foreach ($autres_projets as $p) {
        $html .= "<option value=" . $p["cn"] . ">" . $p["cn"];
    }
    if (count($rights) > 0) {
        foreach ($rights as $r) {
            $html .= "<option value=" . $r["cn"] . ">" . $r["cn"];
        }
    }
    $html .= "</select></td>\n";
    if ($afftype == "user")
        $html .= "<td valign=\"top\"><INPUT TYPE=\"TEXT\" NAME=\"user\" size=15></td>\n";
    $html .= "</tr><tr></tr></table>\n\n";
    return $html;
}

?>
