<?php

/**

 * Librairie de fonctions utilisees dans l'interface d'administration parcs
 * @Projet SambaEdu

 * @Auteurs Equipe Sambaedu

 * @Version

 * @Note: Ce fichier de fonction doit etre appele par un include

 * @Licence Distribue sous la licence GPL

 */

/**
 *
 * @Repertoire: includes/
 * file: fonc_parc.php
 *
 */

/**
 *
 * test si une machine cliente repond au ping
 *
 * @Parametres
 * @ string $ip nom ou adresse a pinguer Attention, si on denne le nom on dépend de la résolution dns
 * @return string|bool
 */
function fping($ip, $wait = 0.1)
{
    // verifie si une machine est online
    // port 445 pour les win et 22 pour les linux
    // renvoit true si ok
    for ($timeout = 0.002; $timeout <= $wait; $timeout = $timeout * 4) {

        if (ssh($ip, $timeout)) {
            return "linux";
        } elseif (smb($ip, $timeout)) {
            return "windows";
        }
    }
    return false;
}

/**
 *
 * teste la connexion smb sur une machine cliente
 *
 * @Parametres $ip adresse a tester
 * @return
 */
function smb($ip, $t)
{
    $fp = @fsockopen($ip, 445, $errno, $errstr, $t);
    if (! $fp) {
        return false;
    } else {
        return true;
    }
}

/**
 *
 * teste la connexion dns sur une machine cliente
 *
 * @Parametres $ip adresse a tester
 * @return
 */
function dns($ip, $t)
{
    $fp = @fsockopen($ip, 53, $errno, $errstr, $t);
    if (! $fp) {
        return false;
    } else {
        return true;
    }
}

/**
 *
 * test la connexion ssh sur une machine cliente
 *
 * @Parametres $ip adresse a tester
 * @return
 */
function ssh($ip, $t)
{
    $fp = @fsockopen($ip, 22, $errno, $errstr, $t);
    if (! $fp) {
        return false;
    } else {
        return true;
    }
}

/**
 *
 * test si une machine cliente repond sur le port 3389
 *
 * @Parametres $ip adresse a tester
 * @return construit un fichier ts dans /tmp pour cette machine si c'est le cas
 */
function ts($ip, $t = 0.05)
{
    for ($timeout = 0.001; $timeout <= $t; $timeout = $timeout * 2) {

        $fp = @fsockopen($ip, 3389, $errno, $errstr, $timeout);

        if ($fp) {
            return true;
        }
    }
    return false;
}

/**
 *
 * test si une machine cliente repond sur le port 5900 vnc
 *
 * @Parametres $ip adresse a tester
 * @return construit un fichier vnc dans /tmp pour cette machine si c'est le cas
 */
function vnc($ip, $t = 0.05)
{
    for ($timeout = 0.001; $timeout <= $t; $timeout = $timeout * 2) {

        $fp = @fsockopen($ip, 5900, $errno, $errstr, $timeout);

        if ($fp) {
            return true;
        }
    }
    return false;
}

/**
 *
 * test du ping sur une machine cliente
 *
 * @Parametres $ip adresse a tester
 * @return
 */
function ping($ip)
{
    exec("ping -q -w 1 $ip", $out, $ret);
    if ($ret == 0) {
        return true;
    } else {
        return false;
    }
}

/**
 *
 * pour l'affichage des heures dans le menu planification des allumage/extinction postes
 *
 * @Parametres
 */
function heure_deroulante($config, $parcf, $jourf, $type_actionf)
{
    $authlink = db_connect($config);
    if ($parcf) {
        $resultf = mysqli_query($authlink, "select heure from actions where action='$type_actionf' and parc='$parcf' and jour='$jourf';") or die("Impossible d'effectuer la requete");

        if ($resultf) {
            $rowf = mysqli_fetch_row($resultf);
            $heuref = $rowf[0] ?? "00:00";
        }
    }
    $heuref = $heuref ?? "00:00";
    $timef = explode(":", $heuref);
    // $heure_act=date("H");
    // $minute_act=date("m");
    for ($if = 0; $if < 24; $if ++) {
        $selectedf = "";
        for ($jf = 0; $jf < 4; $jf ++) {
            $mf = $jf * 15;
            $selectedf = "";
            if (($if == $timef[0]) and ($mf == $timef[1])) {
                $selectedf = "SELECTED";
            } // elseif ($heure_act==$i-1) { $selected="SELECTED"; }
            if ($mf < 10) {
                echo "<option $selectedf >$if:0$mf</option>\n";
            } else {
                echo "<option $selectedf>$if:$mf</option>\n";
            }
            $selectedf = "";
        }
    }
    mysqli_close($authlink);
}

// test si une action d'allumage extinction est prevu sur un parc donne
// un jour donne ( retourne checked pour un select)
function jour_check($config, $parcf, $jourf, $type_actionf)
{
    $authlink = db_connect($config);
    if ($parcf) {
        $resultf = mysqli_query($authlink, "select heure from actions where parc='$parcf' and jour='$jourf' and action='$type_actionf';") or die("Impossible d'effectuer la requete");
        if ($resultf) {
            if (mysqli_num_rows($resultf) > 0) {
                return "checked";
            }
        }
    }
}

/**
 *
 * affiche le tableau detaillant l'allumage des machines et la prise de controle
 *
 * @Parametres
 * @return
 */
function detail_parc($config, $parc, $filtrecomp = "")
{
    require_once ("printers.inc.php");

    echo "\n<br>\n<CENTER>\n";
    echo "<table class='table-bordered' width=\"60%\">\n<tr class=menuheader style=\"height: 30\">\n";
    echo "<td align=\"center\"></td>\n";
    echo "<td align=\"center\">STATIONS CONCERNEES</td>\n";
    echo "<td align=\"center\">ETAT</td>\n";
    echo "<td align=\"center\">CONNEXIONS</td>\n";
    echo "<td align=\"center\">CONTROLE</td></tr>\n";

    $mp_all = list_members_parc($config, $parc);

    // Filtrage selon critere
    if (empty($filtrecomp)) {
        $mp = $mp_all;
    } else {
        $lmloop = 0;
        $mpcount = count($mp_all);
        for ($loop = 0; $loop < $mpcount; $loop ++) {
            $mach = $mp_all[$loop];
            if (preg_match("/$filtrecomp/", $mach))
                $mp[$lmloop ++] = $mach;
        }
    }

    if (count($mp) > 0) {
        sort($mp);
        for ($loop = 0; $loop < count($mp); $loop ++) {
            $mpenc = urlencode($mp[$loop]);
            $mp_en_cours = urldecode($mpenc);
            $mp_curr = search_machine($config, $mp_en_cours);
            if (isset($mp_curr[0]["ipHostNumber"])) {
                $iphost = $mp_curr[0]["ipHostNumber"];
            }
            // Test si on a une imprimante ou une machine
            $resultat = search_printers($config, $mp_en_cours, "printer");
            $suisje_printer = "0";
            for ($loopp = 0; $loopp < count($resultat); $loopp ++) {
                if ($mp_en_cours == $resultat[$loopp]['printername']) {
                    $suisje_printer = "1";
                    continue;
                }
            }

            // On teste si la machine a des connexions actives
            // en fonction de la version de samba
            // On ne rentre dedans que si on est pas une imprimante

            if ($suisje_printer != "1") {
                $smb_sess = get_smbsess($config, $mp_en_cours);
                $etat_session = $smb_sess['html'];

                if (isset($mp_curr[0]["iphostnumber"])) {
                    $iphost = $mp_curr[0]["iphostnumber"];
                    $ping = fping($iphost);

                    // On teste si la machine est en marche ou pas
                    if ($ping) {
                        $etat = "La machine est actuellement allumée, cliquez pour l'éteindre <a href=\"action_machine.php?machine=$mp_en_cours&action=shutdown&parc=$parc&retour=action_parc\"  onclick=\"if (window.confirm('Etes-vous sur de vouloir éteindre la machine " . $mp_en_cours . " ?'))
                                	   	{return true;}
                				else {return false;}\"/><img type=\"image\" border=\"0\" src=\"../elements/images/enabled.png\"></a>\n";
                    } else {
                        $etat = "La machine est actuellement éteinte, cliquez pour l'allumer<a href=\"action_machine.php?machine=" . $mp_en_cours . "&action=wol&parc=" . $parc . "&retour=action_parc\"><img type=\"image\" border=\"0\" src=\"../elements/images/disabled.png\"></a>\n";
                    }

                    echo "<td align=center ><a href=show_histo.php?selectionne=2&amp;mpenc=$mp_en_cours>$mp_en_cours</A></td>\n<td align=center >$etat</td>\n<td align=center>$etat_session </td>\n";
                    echo "<td align=\"center\">";
                    if ($ping) {
                        $ts = ts($iphost, 0.1);
                        if ($ts) {
                            echo $ts;
                        } else {
                            echo "<img type=\"image\" border=\"0\" src=\"../elements/images/disabled.png\">\n";
                        }
                    } else {
                        echo "<img type=\"image\" border=\"0\" src=\"../elements/images/disabled.png\">\n";
                    }
                    echo "</td></tr>\n";
                }
            }
        }
    }
    echo "</table>\n";
    echo "</center>\n";
}

/**
 *
 * Affiche un tableau des imprimantes dans un parc
 *
 * @Parametres $parc le nom du parc a tester
 * @return
 */
function detail_parc_printer($config, $parc, $filtrecomp = "")
{
    // include "printers.inc.php";
    $mp_all = list_members_parc($config, $parc);

    // Filtrage selon critere
    if (empty($filtrecomp))
        $mp = $mp_all;
    else {
        $lmloop = 0;
        $mpcount = count($mp_all);
        for ($loop = 0; $loop < $mpcount; $loop ++) {
            $mach = $mp_all[$loop];
            if (preg_match("/$filtrecomp/", $mach))
                $mp[$lmloop ++] = $mach;
        }
    }

    if (count($mp) > 0) {
        sort($mp);

        $tableau_printer = "\n<br>\n<CENTER>\n";
        $tableau_printer .= "<table class='table-bordered' width=\"60%\">\n<tr class=menuheader style=\"height: 30\">\n";
        $tableau_printer .= "<td align=\"center\"></td>\n";
        $tableau_printer .= "<td align=\"center\">Imprimantes</td>\n";
        $tableau_printer .= "<td align=\"center\">Etat</td>\n";

        $tableau_printer .= "<td align=\"center\">Travaux d'impression</td>\n";
        $tableau_printer .= "</tr>\n";

        $suisje_printer = "0";
        for ($loop = 0; $loop < count($mp); $loop ++) {
            $mpenc = urlencode($mp[$loop]);
            $mp_en_cours = urldecode($mpenc);
            $mp_curr = search_machine($config, $mp_en_cours);

            // Test si on a une imprimante ou une machine
            $resultat = search_printers($config, $mp_en_cours, "printer");
            for ($loopp = 0; $loopp < count($resultat); $loopp ++) {
                if ($mp_en_cours == $resultat[$loopp]['printername']) {
                    $suisje_printer = "1";
                    $tableau_printer .= "<tr>";
                    $tableau_printer .= "<td align=\"center\"><img style=\"border: 0px solid ;\" src=\"../elements/images/printer.png\" title=\"Imprimante\" alt=\"Imprimante\"></TD>";

                    $tableau_printer .= "<TD align=\"center\"><A href='../printers/view_printers.php?one_printer=$mp_en_cours'>$mp_en_cours</A></TD>\n";

                    $printer = $mp_en_cours;
                    $sys = exec("/usr/bin/lpstat -p $printer | grep enabled");
                    $sys2 = exec("/usr/bin/lpstat -a $printer | grep not");

                    if (($sys != "") and ($sys2 == "")) {
                        $tableau_printer .= "<TD align=\"center\"><u onmouseover=\"this.T_SHADOWWIDTH=5;this.T_STICKY=1;return escape('$printer est actuellement active.')\"><img type=\"image\" border=\"0\" src=\"../elements/images/enabled.png\"></u>\n";
                    } else {
                        if ($sys2 == "") {
                            $tableau_printer .= "<TD align=\"center\"><u onmouseover=\"this.T_SHADOWWIDTH=5;this.T_STICKY=1;return escape('$printer est actuellement inactive. Les documents sont stockés dans la file d impression.')\"><img type=\"image\" border=\"0\" src=\"../elements/images/disabled.png\"></u>\n";
                        } else {
                            $tableau_printer .= "<TD align=\"center\"><u onmouseover=\"this.T_SHADOWWIDTH=5;this.T_STICKY=1;return escape('$printer rejette actuellement tout travail d impression.')\"><img type=\"image\" border=\"0\" src=\"../elements/images/disabled.png\"></u>\n";
                        }
                    }
                    $tableau_printer .= "</TD>\n";

                    // Travaux d'impression
                    $sys = exec("/usr/bin/lpstat -o $printer");
                    if ($sys != "") {
                        $tableau_printer .= "<TD align=\"center\"><img type=\"image\" border=\"0\" src=\"../elements/images/enabled.png\">\n";
                    } else {
                        $tableau_printer .= "<TD align=\"center\"><img type=\"image\" border=\"0\" src=\"../elements/images/disabled.png\">\n";
                    }

                    $tableau_printer .= "</tr>\n";
                    continue;
                }
            }
        }
        $tableau_printer .= "</table></center>";
        if ($suisje_printer == "1") {
            echo $tableau_printer;
            echo "<br>";
        }
    }
}

/*
 * fonction generant un tableau global a partir de smbstatus
 * on cache le resultat avec apc 1 minutes afin de reduire la charge de la recherche des machines
 *
 * c'est nul, et super lent... a refaire autrement
 *
 * @Parametres : aucun
 * @ smbstatus["machine"]["login"=>]
 * ["ip"=>]
 */
function smbstatus(array $config)
{
    /*
     * while (apcu_fetch('smb_lock')) {
     * sleep(1);
     * }
     */
    if (! $status = apcu_fetch('smb_login')) {
        // apcu_add('smb_lock', 1, 300);
        $status = array();
        $filename = "/tmp/smbstatus";
        $h = fopen($filename, "r");
        $leases = import_dhcp_leases($config);
        if ($h) {
            while (! feof($h)) {
                $fields = explode(" ", fgets($h));
                if (count($fields) == 2) {
                    $ip = rtrim($fields[1]);
                    $host = $ip; // gethostbyaddr($ip);
                    if ($host == $ip) {
                        if ($lease = get_dhcp_lease($config, $leases, $ip)) {
                            $host = $lease['name'];
                        }
                    }
                    $machine = search_machine($config, $host, false);
                    if (count($machine) == 0) {
                        $machine = search_machine($config, $fields[0], false);
                    }
                    if (count($machine) > 0) {
                        if (strcasecmp($machine['samaccountname'] ?? "", $fields[0])) {
                            $user = $fields[0];
                        } else {
                            $user = "wpkg";
                        }
                        $host = $machine['cn'];
                    } else {
                        $user = $fields[0];
                        $host = $ip;
                    }
                    $status[] = array(
                        'login' => $user,
                        'ip' => $ip,
                        'machine' => $host
                    );
                }
            }
            fclose($h);
        }
        apcu_add('smb_login', $status, 300);
        // apcu_delete('smb_lock');
    }
    return ($status);
}

function machine_smbstatus(array $config, $machine, $os)
{
    $user = "";
    $computer = search_machine($config, $machine, false);
    if ($os == "linux") {
        $out = [];
        exec("/usr/bin/ssh -i /etc/sambaedu/id_rsa -o StrictHostKeyChecking=no -o ConnectTimeout=1 root@" . $machine . " users", $out, $ret);
        if ($ret == 0 && isset($out[0])) {
            $users = explode(" ", $out[0]);
            foreach ($users as $user) {
                if ($user != "root" && $user != "Debian-gdm") {
                    break;
                }
            }
        }
    } else {
        if (! empty($computer['iphostnumber'])) {
            $filename = "/tmp/smbstatus";
            if (! file_exists($filename)) {
                exec("/usr/bin/sudo /usr/share/sambaedu/sbin/smbstatus.sh");
            }
            $h = fopen($filename, "r");
            if ($h) {
                while (! feof($h)) {
                    $line = fgets($h);
                    if (preg_match("/" . $computer['iphostnumber'] . "$/", $line)) {
                        $fields = explode(" ", $line);
                        if (count($fields) == 2) {
                            $u = rtrim($fields[0]);
                            // if (preg_match("/^.*\\$$/", $u)) {
                            if (strcasecmp($u, $computer['samaccountname']) == 0) {
                                $user = "wpkg";
                                // } elseif ($u != "root" && $u != "Debian-gdm") {
                            } else {
                                $user = search_user($config, $u);
                                if (! empty($user['cn'])) {
                                    $user = $user['cn'];
                                    break;
                                }
                            }
                        }
                    }
                }
                fclose($h);
            }
        }
    }
    return $user;
}

/*
 * fonction destinee a connaitre l'etat de la session samba d'une machine
 * @Parametres $device : Nom de la machine
 * obsolete
 */
function get_smbsess(array $config, string $machine)
{
    if ($machine) {

        $status = get_machine_status($config, $machine);
        $login = false;
        $etat_session = "";
        if (empty($status['user'][0]['name'])) {
            $etat_session = "<img type=\"image\" src=\"../elements/images/disabled.png\">\n";
        } elseif ($status['user'][0]['name'] == $config['se4install_name']) {
            $texte = $status['user'][0]['name'] . " connecté";
            $etat_session .= "<img onmouseout=\"UnTip();\" onmouseover=\"Tip('" . $texte . "',WIDTH,250,SHADOW,true,DURATION,5000);\" src=\"../elements/images/travaux2.png\" border=\"0\" />";
        } else {
            $texte = $status['user'][0]['name'] . " connecté";
            $etat_session .= "<img onmouseout=\"UnTip();\" onmouseover=\"Tip('" . $texte . "',WIDTH,250,SHADOW,true,DURATION,5000);\" src=\"../elements/images/enabled.png\" border=\"0\" />";
        }
        return array(
            'login' => $status['user'][0]['name'],
            'html' => $etat_session
        );
    } else {
        return false;
    }
}

/*
 * fonction destinee a connaitre l'etat de la session samba d'une machine
 * @Parametres $user : Nom de l'utilisateur
 *
 * obsolete
 *
 * @Return : array machines
 */
function get_smbmachine(array $config, string $user)
{
    $status = smbstatus($config);
    $machines = array();
    foreach ($status as $session) {
        if (strcasecmp($session['login'], $user) == 0) {
            $machines[] = $session['machine'];
        }
    }
    return $machines;
}

function get_os_icon($config, $machine)
{
    if (is_printer($config, $machine)) {
        if (count(list_printers($config, $machine['cn'], true)) > 0) {
            $icone = "printer.png";
            $retourOs = "imprimante";
        } else {
            $icone = "unknown.png";
            $retourOs = "équipement";
        }
    } else {
        if (! isset($machine['state'])) {
            $resultat = get_machine_status($config, $machine['cn']);
        } else {
            $resultat['state'] = $machine['state'];
        }

        $installed_os = get_os($config, $machine);
        switch (count($installed_os)) {
            case 2:
                $os = "dual_boot";
                break;
            case 1:
                $os = $installed_os[0];
                break;
            default:
                $os = "";
        }
        if (isset($resultat['state'])) {
            if ($resultat['state'] == "linux") {
                $machine['operatingsystem'] = "Linux";
            }
            if ($resultat['state'] != "off") {
                $os = $resultat['state'];
            }
        }
        switch ($os) {
            case "dual_boot":
                $icone = "dual_boot.png";
                $retourOs = "dual_boot";
                break;
            case "linux":
                $icone = "linux.png";
                $retourOs = "linux";
                break;
            case "windows":
                $retourOs = $machine['operatingsystem'] ?? "";
                if ($retourOs == "") {
                    $icone = "computer_disable.png";
                } elseif (preg_match("/Windows XP.*/", $retourOs)) {
                    $icone = "winxp.png";
                } elseif (preg_match("/Windows 7.*/", $retourOs)) {
                    $icone = "win7.png";
                } elseif (preg_match("/Windows.*10.*/", $retourOs)) {
                    $icone = "win10.png";
                } elseif (preg_match("/Windows.*11.*/", $retourOs)) {
                    $icone = "win11.png";
                } elseif (preg_match("/Windows Vista.*/", $retourOs)) {
                    $icone = "winvista.png";
                } else {
                    $icone = "unknown.png";
                }
                break;
            default:
                $icone = "computer_disable.png";
                $retourOs = "eteint";
        }
    }
    return [
        'os' => $retourOs,
        'icone' => $icone
    ];
}

/*
 * Fonction renvoyant l'année scolaire en cours
 *
 * @Parametres : aucun
 * @Return : année scolaire (ex : 2023-2024)
 */
function get_annee_scolaire()
{
    $jour = date('Y-m-d');
    $annee1 = date('Y', strtotime($jour));
    if ($jour < date($annee1 . '-09-01')) {
        $annee2 = $annee1 - 1;
        $annee = $annee2 . '-' . $annee1;
    } else {
        $annee2 = $annee1 + 1;
        $annee = $annee1 . '-' . $annee2;
    }
    return $annee;
}

/*
 * Fonction renvoyant les fichiers nécessaires à l'API Vacances scolaires
 *
 * @Parametres : config
 * @Return : array files
 */
function get_api_file($config)
{
    $retour = array(
        'file_academies' => '0',
        'file_vacances' => '0',
        'file_feries' => '0'
    );

    require_once ('../vendor/autoload.php');
    $opt = [
        'cookies' => false
    ];
    curl_proxy_options($config, $opt);
    $client = new GuzzleHttp\Client($opt);

    $url = 'https://data.education.gouv.fr/api/explore/v2.1/catalog/datasets/fr-en-calendrier-scolaire';

    try {
        $response = $client->request('GET', $url);
        (string) $data = $response->getBody();
        $json = json_decode($data, true);
        $api_modified = new DateTime($json['metas']['default']['modified'], new DateTimeZone($json['metas']['default']['timezone']));
        if (empty($config['api_calendrier_scolaire_modified']) || empty($config['api_calendrier_scolaire_timezone']) || $api_modified > new DateTime($config['api_calendrier_scolaire_modified'], new DateTimeZone($config['api_calendrier_scolaire_timezone']))) {
            $modified = true;
            set_config($config, 'api_calendrier_scolaire_modified', $json['metas']['default']['modified'], 'api-calendrier-scolaire');
            set_config($config, 'api_calendrier_scolaire_timezone', $json['metas']['default']['timezone'], 'api-calendrier-scolaire');
        } else {
            $modified = false;
        }
    } catch (GuzzleHttp\Exception\ConnectException | Exception $e) {
        trigger_error("Erreur get_api_file GET $url : " . $e);
    }

    $url = $url . '/records?';
    $url_academies = $url . 'group_by=location&order_by=location&refine=annee_scolaire%3A%22' . get_annee_scolaire() . '%22';
    if (! empty($config['academie'])) {
        $url_vacances = $url . 'order_by=end_date&timezone=Europe%2FParis&refine=annee_scolaire%3A%22' . get_annee_scolaire() . '%22&refine=location%3A%22' . urlencode($config['academie']) . '%22';
    }
    $url_feries = 'https://calendrier.api.gouv.fr/jours-feries/metropole/' . date('Y') . '.json';

    $dir = '/etc/sambaedu/api-calendrier-scolaire';
    $file_academies = $dir . '/academies-' . get_annee_scolaire() . '.json';
    if (! empty($config['academie'])) {
        $file_vacances = $dir . '/vacances-' . $config['academie'] . '-' . get_annee_scolaire() . '.json';
    } else {
        $file_vacances = '';
    }
    $file_feries = $dir . '/feries-metropole-' . date('Y') . '.json';
    if (! file_exists($dir)) {
        if (! mkdir($dir)) {
            return $retour;
        }
    }

    if (! $modified && file_exists($file_academies)) {
        $retour['file_academies'] = $file_academies;
    } else {
        foreach (glob($dir . '/academies-*') as $file) {
            unlink($file);
        }
        try {
            $client->request('GET', $url_academies, [
                'sink' => $file_academies
            ]);
            if (file_exists($file_academies)) {
                $retour['file_academies'] = $file_academies;
            }
        } catch (GuzzleHttp\Exception\ConnectException | Exception $e) {
            trigger_error("Erreur get_api_file GET $url_academies : " . $e);
        }
    }

    if (! $modified && file_exists($file_vacances)) {
        $retour['file_vacances'] = $file_vacances;
    } else {
        foreach (glob($dir . '/vacances-*') as $file) {
            unlink($file);
        }
        if (! empty($config['academie'])) {
            try {
                $client->request('GET', $url_vacances, [
                    'sink' => $file_vacances
                ]);
                if (file_exists($file_vacances)) {
                    $retour['file_vacances'] = $file_vacances;
                }
            } catch (GuzzleHttp\Exception\ConnectException | Exception $e) {
                trigger_error("Erreur get_api_file GET $url_vacances : " . $e);
            }
        }
    }

    if (file_exists($file_feries)) {
        $retour['file_feries'] = $file_feries;
    } else {
        foreach (glob($dir . '/feries-*') as $file) {
            unlink($file);
        }
        try {
            $client->request('GET', $url_feries, [
                'sink' => $file_feries
            ]);
            if (file_exists($file_feries)) {
                $retour['file_feries'] = $file_feries;
            }
        } catch (GuzzleHttp\Exception\connectException | Exception $e) {
            trigger_error("Erreur get_api_file GET $url_feries : " . $e);
        }
    }

    return $retour;
}

/*
 * Fonction renvoyant la liste des académies pour l'API Vacances scolaires
 *
 * @Parametres : config
 * @Return : array academies
 */
function get_academies($config)
{
    if (get_api_file($config)['file_academies'] == '0') {
        return '0';
    } else {
        $raw = file_get_contents(get_api_file($config)['file_academies']);
    }
    $json = json_decode($raw, true);
    $academies = array();
    foreach ($json['results'] as $cle => $valeur) {
        $academies[$cle] = $valeur['location'];
    }
    return $academies;
}

/*
 * Fonction retournant l'état et le nom des vacances pour l'API Vacances scolaires
 *
 * @Parametres : config
 * @Return : array etat (0=échec, 1=boulot, 2=vacances), nom des vacances
 */
function get_vacances($config)
{
    if (get_api_file($config)['file_vacances'] == '0') {
        return [
            'etat' => '0',
            'description' => ''
        ];
    } else {
        $raw = file_get_contents(get_api_file($config)['file_vacances']);
    }
    $json = json_decode($raw, true);
    $jour = date('Y-m-d');
    $etat = '1';
    $description = '';
    foreach ($json['results'] as $vacances) {
        if ($vacances['population'] != 'Élèves') {
            if ($vacances['start_date'] != $vacances['end_date']) {
                if ($jour >= date('Y-m-d', strtotime($vacances['start_date'])) and $jour < date('Y-m-d', strtotime($vacances['end_date']))) {
                    $etat = '2';
                    break;
                }
            } else {
                if ($jour >= date('Y-m-d', strtotime($vacances['start_date'])) and $jour < date('Y-m-d', strtotime('+1 day', strtotime($vacances['end_date'])))) {
                    $etat = '2';
                    break;
                }
            }
        }
    }
    if ($etat == '2') {
        $description = $vacances['description'];
    }
    return [
        'etat' => $etat,
        'description' => $description
    ];
}

/*
 * Fonction renvoyant l'état et le nom des jours fériés pour l'API Vacances scolaires
 *
 * @Parametres : config
 * @Return : array etat (0=échec, 1=boulot, 2=férié), nom du jour férié
 */
function get_feries($config)
{
    if (get_api_file($config)['file_feries'] == '0') {
        return [
            'etat' => '0',
            'description' => ''
        ];
    } else {
        $raw = file_get_contents(get_api_file($config)['file_feries']);
    }
    $json = json_decode($raw, true);
    $jour = date('Y-m-d');
    foreach ($json as $ferie => $description) {
        if ($ferie == $jour) {
            return [
                'etat' => '2',
                'description' => $description
            ];
        }
    }
    return [
        'etat' => '1',
        'description' => ''
    ];
}

function export_machines($config, $uai = 0)
{
    if (! empty($uai)) {
        $config = set_config($config, true, $uai);
    }
    $computers = search_ad($config, "*", "machine_fast");
    $lignes[] = "uai,machine,mac";
    foreach ($computers as $c) {
        $lignes[] = $c['etab'] . "," . $c['cn'] . "," . $c['networkaddress'];
    }
    return implode("\n", $lignes);
}
?>