<?php

/**
 * Librairie de fonctions utilisees dans l'interface d'administration

 * @Projet SambaEdu

 * @Note: Ce fichier de fonction doit etre appele par un include

 * @Licence Distribue sous la licence GPL
 */

/**
 *
 * file: functions.inc.php
 *
 * @Repertoire: includes/
 */

/**
 * Retourne l'IP distante
 * Utilisation de $_SERVER['REMOTE_ADDR'] pour éviter l'IP spoofing
 *
 * @Parametres
 * @return
 */
function remote_ip()
{
    if (getenv("HTTP_CLIENT_IP")) {
        $ip = getenv("HTTP_CLIENT_IP");
    } elseif (filter_var($_SERVER['REMOTE_ADDR'], FILTER_VALIDATE_IP)) { // (getenv("HTTP_X_FORWARDED_FOR")) {
        $ip = $_SERVER['REMOTE_ADDR'];
    } /*
       * else {
       * $ip = getenv("REMOTE_ADDR");
       * }
       */
    if (filter_var($ip, FILTER_VALIDATE_IP)) {
        return $ip;
    } else {
        return "";
    }
}

// =================================================
function open_session($config, $login, $passwd, $newpasswd = "")
{
    $res = false;
    // Initialisation
    $auth_ldap = 0;
    // if (! empty($config['login'])) {
    // session_destroy();
    // }
    $auth_ldap = user_valid_passwd($config, $login, $passwd, $newpasswd);
    if ($auth_ldap == 0) {
        // Log en cas d'echec
        $fp = fopen("/var/log/sambaedu/auth.log", "a");
        if ($fp) {
            fputs($fp, "[" . $auth_ldap . "] " . date("Y-m-d H:i:s") . "|remote ip : " . remote_ip() . "|Login : " . $login . "|TimeStamp srv : " . time() . "\n");
            fclose($fp);
        }
    }
    if ($auth_ldap > 0) {
        if (! session_name()) {
            session_name("Sambaedu");
            // session_start();
        }
        if (empty(session_id())) {
            session_start([
                'cookie_lifetime' => 86400
                // 'login' => $login,
                // 'passwd' => $passwd,
            ]);
        }
        $_SESSION['login'] = $login;
        $_SESSION['passwd'] = $passwd;
        $res = true;
    }

    return $res;
}

// =================================================

/**
 * Ferme la session en cours
 *
 * @Parametres
 * @return
 */
function close_session()
{
    // Destruction session php Sambaedu
    // session_name("Sambaedu");
    if (session_id()) {
        // On detruit toutes les variables de session
        // unset($_SESSION);
        // On detruit la session sur le serveur.
        session_destroy();
    }
    // Destruction du cookie de session
    setcookie("Sambaedu", "", time() - 3600, "/", "", 0);
}

function menuprint($config, $liens, $menu)
{
    // $rights = apcu_fetch("rights");
    $rights = list_rights($config, $config['login'], true);
    $getintlevel = getintlevel();
    // var_dump($liens);
    for ($idmenu = 0; $idmenu < count($liens); $idmenu ++) {
        echo "<div id=\"menu$idmenu\" style=\"position:absolute; left:10px; top:12px; width:200px; z-index:" . $idmenu . " ";
        if ($idmenu != $menu) {
            echo "; visibility: hidden";
        }
        echo "\">\n";

        echo "
        <table width=\"200\" border=\"0\" cellspacing=\"3\" cellpadding=\"6\">\n";
        for ($menunbr = 1; $menunbr < count($liens); $menunbr ++) {
            $menutarget = "main";
            // niveau minimal des droits requis pour voir le menu (SE_NO_RIGHT : on montre a tout le monde)
            $rightname = $liens[$menunbr]['right'];

            $level = $liens[$menunbr]['level'];
            
            $afftest = ($rightname == 0 || ($rights & $rightname) > 0);
            if ($level > $getintlevel)
                $afftest = false;
            if ($afftest)
                if (($idmenu == $menunbr) && ($idmenu != 0)) {
                    echo "
                <tr>
                    <td class=\"menuheader\">
                        <p style='margin:2px; padding-top:2px; padding-bottom:2px'><a href=\"javascript:;\" onClick=\"P7_autoLayers('menu0');return false\">&#10514;&nbsp;</a>
                        <a href=\"javascript:;\" onClick=\"P7_autoLayers('menu" . $menunbr . "');return false\">" . $liens[$menunbr]['text'] . "</a></p>
                    </td>
                    </tr>
                    <tr>
                    <td class=\"menucell\">";
                    foreach ($liens[$menunbr]['sub'] as $sub) {
                        // Test des droits pour affichage
                        $subrightname = $sub['right'];
                        $level = $sub['level'];
                            $afftest = ($subrightname == 0 || ($rights & $subrightname) > 0); // au moins un droit correspondant
                        if ($level > $getintlevel)
                            $afftest = false;
                        if ($afftest) {
                            echo "&#8226; &nbsp;"; // "<img src=\"elements/images/typebullet.png\" width=\"30\" height=\"11\" alt=\"\">";
                            echo "<a href=\"" . $sub['link'] . "\" TARGET='$menutarget'>" . $sub['text'] . "</a><br>\n";
                        }
                    }
                    echo "
                    </td></tr>\n";
                } else {
                    echo "
                <tr>
                    <td class=\"menuheader\">
                    <p style='margin:2px; padding-top:2px; padding-bottom:2px'><a href=\"javascript:;\" onClick=\"P7_autoLayers('menu" . $menunbr . "');return false\">
                    &#10515;&nbsp;" . $liens[$menunbr]['text'] . "</a></p>
                    </td></tr>\n";
                }
        } // for menunbr : boucle d'affichage des entrees de menu principales
        $lien = "<a href=\"./user/index.php\" TARGET='user/index.php'>Mode Utilisateur</a><br>\n";
        echo "
                <tr>
                    <td class=\"menuheader\">
                    <p style='margin:2px; padding-top:2px; padding-bottom:2px'>" . $lien . "</p>
                    </td></tr>\n";
        echo "
        </table>
</div>\n";
    } // for idmenu : boucle d'affichage des differents calques
}

// function menuprint

/**
 * Retourne le niveau de l'interface
 *
 * @Parametres
 * @return
 */
function getintlevel()
{
    /* Lis le niveau d'interface dans la table session */
    // session_name("Sambaedu");
    // @session_start();
    $ret = $_SESSION['level'] ?? 4;
    return $ret;
}

// =================================================

/**
 * Change le niveau d'interface dans la table session
 *
 * @Parametres
 * @return
 */
function setintlevel($new_level)
{
    // session_name("Sambaedu");
    // @session_start();
    $_SESSION['level'] = $new_level;
}

function mktable($title, $content)
{
    $html = "<H3>$title</H3>";
    $html .= $content;
    return $html;
}
/**
 * Fonction de crob destinee a afficher les variables transmises d'une page a l'autre: GET, POST et SESSION
 *
 * Usage : debug_var(); en début du script à debugguer
 *
 * @Parametres
 * @return
 */
$debug_var_count = array();

function tab_debug_var($chaine_tab_niv1, $tableau, $pref_chaine, $cpt_debug)
{
    // global $cpt_debug;
    global $debug_var_count;

    echo " (<a href='#' onclick=\"tab_etat_debug_var[$cpt_debug]=tab_etat_debug_var[$cpt_debug]*(-1);affiche_debug_var('container_debug_var_$cpt_debug',tab_etat_debug_var[$cpt_debug]);return false;\">*</a>)\n";

    echo "<table id='container_debug_var_$cpt_debug' summary=\"Tableau de debug\">\n";
    foreach ($tableau as $post => $val) {
        echo "<tr><td valign='top'>" . $pref_chaine . "['" . $post . "']=</td><td>" . $val;

        if (is_array($tableau[$post])) {

            tab_debug_var($chaine_tab_niv1, $tableau[$post], $pref_chaine . '[' . $post . ']', $cpt_debug);

            $cpt_debug ++;
        } elseif (isset($debug_var_count[$chaine_tab_niv1])) {
            $debug_var_count[$chaine_tab_niv1] ++;
        }

        echo "</td></tr>\n";
    }
    echo "</table>\n";
}

function debug_var()
{
    global $debug_var_count;

    $debug_var_count['POST'] = 0;
    $debug_var_count['GET'] = 0;

    $debug_var_count['COOKIE'] = 0;

    // Fonction destinée à afficher les variables transmises d'une page à l'autre: GET, POST et SESSION
    echo "<div style='border: 1px solid black; background-color: white; color: black;'>\n";

    $cpt_debug = 0;

    echo "<p><strong>Variables transmises en POST, GET, SESSION,...</strong> (<a href='#' onclick=\"tab_etat_debug_var[$cpt_debug]=tab_etat_debug_var[$cpt_debug]*(-1);affiche_debug_var('container_debug_var_$cpt_debug',tab_etat_debug_var[$cpt_debug]);return false;\">*</a>)</p>\n";

    echo "<div id='container_debug_var_$cpt_debug'>\n";
    $cpt_debug ++;

    echo "<p>Variables envoyées en POST: ";
    if (count($_POST) == 0) {
        echo "aucune";
    } else {
        echo "(<a href='#' onclick=\"tab_etat_debug_var[$cpt_debug]=tab_etat_debug_var[$cpt_debug]*(-1);affiche_debug_var('container_debug_var_$cpt_debug',tab_etat_debug_var[$cpt_debug]);return false;\">*</a>)";
    }
    echo "</p>\n";
    echo "<blockquote>\n";
    echo "<div id='container_debug_var_$cpt_debug'>\n";
    $cpt_debug ++;

    echo "<script type='text/javascript'>
	tab_etat_debug_var=new Array();
        
	function affiche_debug_var(id,mode) {
		if(document.getElementById(id)) {
			if(mode==1) {
				document.getElementById(id).style.display='';
			}
			else {
				document.getElementById(id).style.display='none';
			}
		}
	}
</script>\n";

    /*
     * echo "<table summary=\"Tableau de debug\">\n";
     * foreach($_POST as $post => $val){
     * //echo "\$_POST['".$post."']=".$val."<br />\n";
     * //echo "<tr><td>\$_POST['".$post."']=</td><td>".$val."</td></tr>\n";
     * echo "<tr><td valign='top'>\$_POST['".$post."']=</td><td>".$val;
     *
     * if(is_array($_POST[$post])) {
     * echo " (<a href='#' onclick=\"tab_etat_debug_var[$cpt_debug]=tab_etat_debug_var[$cpt_debug]*(-1);affiche_debug_var('container_debug_var_$cpt_debug',tab_etat_debug_var[$cpt_debug]);return false;\">*</a>)";
     * echo "<table id='container_debug_var_$cpt_debug' summary=\"Tableau de debug\">\n";
     * foreach($_POST[$post] as $key => $value) {
     * echo "<tr><td>\$_POST['$post'][$key]=</td><td>$value</td></tr>\n";
     * }
     * echo "</table>\n";
     * //echo "<script type='text/javascript'>affiche_debug_var('debug_var_$post',tab_etat_debug_var[$cpt_debug]);</script>\n";
     * $cpt_debug++;
     * }
     *
     * echo "</td></tr>\n";
     * }
     * echo "</table>\n";
     */

    echo "<table summary=\"Tableau de debug\">\n";
    foreach ($_POST as $post => $val) {
        echo "<tr><td valign='top'>\$_POST['" . $post . "']=</td><td>";
        if (is_array($val)) {
            echo "Array";
        } else {
            echo $val;
        }

        if (is_array($_POST[$post])) {
            tab_debug_var('POST', $_POST[$post], '$_POST[' . $post . ']', $cpt_debug);

            $cpt_debug ++;
        } else {
            $debug_var_count['POST'] ++;
        }

        echo "</td></tr>\n";
    }
    echo "</table>\n";

    echo "<p>Nombre de valeurs en POST: <b>" . $debug_var_count['POST'] . "</b></p>\n";
    echo "</div>\n";
    echo "</blockquote>\n";

    echo "<p>Variables envoyées en GET: ";
    if (count($_GET) == 0) {
        echo "aucune";
    } else {
        echo "(<a href='#' onclick=\"tab_etat_debug_var[$cpt_debug]=tab_etat_debug_var[$cpt_debug]*(-1);affiche_debug_var('container_debug_var_$cpt_debug',tab_etat_debug_var[$cpt_debug]);return false;\">*</a>)";
    }
    echo "</p>\n";
    echo "<blockquote>\n";
    echo "<div id='container_debug_var_$cpt_debug'>\n";
    $cpt_debug ++;
    echo "<table summary=\"Tableau de debug sur GET\">";
    foreach ($_GET as $get => $val) {
        // echo "\$_GET['".$get."']=".$val."<br />\n";
        // echo "<tr><td>\$_GET['".$get."']=</td><td>".$val."</td></tr>\n";

        echo "<tr><td valign='top'>\$_GET['" . $get . "']=</td><td>";
        if (is_array($val)) {
            echo "Array";
        } else {
            echo $val;
        }

        if (is_array($_GET[$get])) {
            tab_debug_var('GET', $_GET[$get], '$_GET[' . $get . ']', $cpt_debug);

            $cpt_debug ++;
        } else {
            $debug_var_count['GET'] ++;
        }

        echo "</td></tr>\n";
    }
    echo "</table>\n";
    echo "</div>\n";
    echo "</blockquote>\n";

    echo "<p>Variables envoyées en SESSION: ";
    if (count($_SESSION) == 0) {
        echo "aucune";
    } else {
        echo "(<a href='#' onclick=\"tab_etat_debug_var[$cpt_debug]=tab_etat_debug_var[$cpt_debug]*(-1);affiche_debug_var('container_debug_var_$cpt_debug',tab_etat_debug_var[$cpt_debug]);return false;\">*</a>)";
    }
    echo "</p>\n";
    echo "<blockquote>\n";
    echo "<div id='container_debug_var_$cpt_debug'>\n";
    $cpt_debug ++;
    echo "<table summary=\"Tableau de debug sur SESSION\">";
    foreach ($_SESSION as $variable => $val) {
        // echo "\$_SESSION['".$variable."']=".$val."<br />\n";
        echo "<tr><td>\$_SESSION['" . $variable . "']=</td><td>" . $val . "</td></tr>\n";
    }
    echo "</table>\n";
    echo "</div>\n";
    echo "</blockquote>\n";

    echo "<p>Variables envoyées en SERVER: ";
    if (count($_SERVER) == 0) {
        echo "aucune";
    } else {
        echo "(<a href='#' onclick=\"tab_etat_debug_var[$cpt_debug]=tab_etat_debug_var[$cpt_debug]*(-1);affiche_debug_var('container_debug_var_$cpt_debug',tab_etat_debug_var[$cpt_debug]);return false;\">*</a>)";
    }
    echo "</p>\n";
    echo "<blockquote>\n";
    echo "<div id='container_debug_var_$cpt_debug'>\n";
    $cpt_debug ++;
    echo "<table summary=\"Tableau de debug sur SERVER\">";
    foreach ($_SERVER as $variable => $valeur) {
        // echo "\$_SERVER['".$variable."']=".$valeur."<br />\n";
        echo "<tr><td>\$_SERVER['" . $variable . "']=</td><td>" . $valeur . "</td></tr>\n";
    }
    echo "</table>\n";
    echo "</div>\n";
    echo "</blockquote>\n";

    echo "<p>Variables envoyées en FILES: ";
    if (count($_FILES) == 0) {
        echo "aucune";
    } else {
        echo "(<a href='#' onclick=\"tab_etat_debug_var[$cpt_debug]=tab_etat_debug_var[$cpt_debug]*(-1);affiche_debug_var('container_debug_var_$cpt_debug',tab_etat_debug_var[$cpt_debug]);return false;\">*</a>)";
    }
    echo "</p>\n";
    echo "<blockquote>\n";
    echo "<div id='container_debug_var_$cpt_debug'>\n";
    $cpt_debug ++;

    echo "<table summary=\"Tableau de debug\">\n";
    foreach ($_FILES as $key => $val) {
        echo "<tr><td valign='top'>\$_FILES['" . $key . "']=</td><td>";

        if (is_array($val)) {
            echo "Array";
        } else {
            echo $val;
        }

        if (is_array($_FILES[$key])) {
            tab_debug_var('FILES', $_FILES[$key], '$_FILES[' . $key . ']', $cpt_debug);

            $cpt_debug ++;
        }

        echo "</td></tr>\n";
    }
    echo "</table>\n";

    echo "</div>\n";
    echo "</blockquote>\n";

    echo "<p>Variables COOKIES: ";
    if (count($_COOKIE) == 0) {
        echo "aucune";
    } else {
        echo "(<a href='#' onclick=\"tab_etat_debug_var[$cpt_debug]=tab_etat_debug_var[$cpt_debug]*(-1);affiche_debug_var('container_debug_var_$cpt_debug',tab_etat_debug_var[$cpt_debug]);return false;\">*</a>)";
    }
    echo "</p>\n";
    echo "<blockquote>\n";
    echo "<div id='container_debug_var_$cpt_debug'>\n";
    $cpt_debug ++;
    echo "<table summary=\"Tableau de debug sur COOKIE\">";
    foreach ($_COOKIE as $get => $val) {

        echo "<tr><td valign='top'>\$_COOKIE['" . $get . "']=</td><td>" . $val;

        if (is_array($_COOKIE[$get])) {
            tab_debug_var('COOKIE', $_COOKIE[$get], '$_COOKIE[' . $get . ']', $cpt_debug);

            $cpt_debug ++;
        } else {
            $debug_var_count['COOKIE'] ++;
        }

        echo "</td></tr>\n";
    }
    echo "</table>\n";
    echo "</div>\n";
    echo "</blockquote>\n";

    echo "<script type='text/javascript'>
	// On masque le cadre de debug au chargement:
	//affiche_debug_var('container_debug_var',var_debug_var_etat);
	
	//for(i=0;i<tab_etat_debug_var.length;i++) {
	for(i=0;i<$cpt_debug;i++) {
		if(document.getElementById('container_debug_var_'+i)) {
			affiche_debug_var('container_debug_var_'+i,-1);
		}
		// Variable destinée à alterner affichage/masquage
		tab_etat_debug_var[i]=-1;
	}
</script>\n";

    echo "</div>\n";
    echo "</div>\n";
}
?>
