<?php

function read_gpei_xml($config, $gpei, &$users, $uai = "")
{
    // read entire file into string
    $xmlfile = file_get_contents($gpei);

    $xml = new DOMDocument();
    $xml->formatOutput = false;
    $xml->preserveWhiteSpace = false;
    $xml->loadXML($xmlfile);
    $xpath = new DOMXPath($xml);

    $users = array_merge($users, read_attrs($xpath->query('/ficAlimMENESR/addRequest/attributes/attr')));
    $users = array_merge($users, read_attrs($xpath->query('/ficAlimMENESR/modifyRequest/modifications/modification')));

    foreach ($xpath->query('/ficAlimMENESR/deleteRequest/identifier/id') as $id) {
        $users[] = [
            'Id GPEI' => $id->nodeValue,
            'type' => "trash"
        ];
    }
    if (! empty($uai) && $uai != 0) {
        foreach ($users as $id => $user) {
            $found = false;
            if (isset($user['etabs']) && is_array($user['etabs'])) {
                foreach ($user['etabs'] as $i) {
                    $found |= ($uai == etab_uai($config, $i));
                }
            }
            if (! $found && $user['type'] != "Etab") {
                unset($users[$id]);
            }
        }
    }

    return count($users);
}

function read_attrs($reqs)
{
    $user = [];
    $i = 0;
    foreach ($reqs as $i => $req) {
        // if ($i ++ > 100)
        // break;
        $id = $req->parentNode->parentNode->getElementsByTagName('identifier')
            ->item(0)
            ->getElementsByTagName('id')
            ->item(0)->nodeValue;
        $cat = $req->parentNode->parentNode->getElementsByTagName('operationalAttributes')
            ->item(0)
            ->getElementsByTagName('attr')
            ->item(0)->nodeValue;
        switch ($cat) {
            case "PersEducNat":
                $user[$id]['type'] = "Teacher";
                break;
            case "Eleve":
                $user[$id]['type'] = "Student";
                break;
            case "EtabEducNat":
                $user[$id]['type'] = "Etab";
                break;
        }
        $attr = $req->getAttribute('name');
        $vals = $req->getElementsByTagName('value');
        foreach ($vals as $v) {
            if (is_object($v->firstChild)) {
                $val = $v->firstChild->nodeValue;
                switch ($attr) {
                    case "GPEIPersonDateNaissance":
                        $key = "birthDate";
                        $user[$id][$key] = map_birthdate($val, "gpei");
                        break;
                    case "GPEIPersonJointure":
                    case "GPEIStructureJointure":
                        $key = "Id GPEI";
                        $user[$id][$key] = $val;
                        break;
                    case "GPEIPersonNom":
                        $key = "lastName";
                        $user[$id][$key] = $val;
                        $user[$id]['lastName_norm'] = simplify_name($val);
                        break;
                    case "GPEIPersonPrenom":
                        $key = "firstName";
                        $user[$id][$key] = $val;
                        $user[$id]['firstName_norm'] = simplify_firstname($val);
                        break;
                    case "GPEIPersonMail":
                    case "GPEIStructureMailSI":
                        $key = "email";
                        $user[$id][$key] = $val;
                        break;
                    case "GPEIPersonStructRattach":
                        $key = "structure";
                        $user[$id][$key] = $val;
                        break;
                    case "GPEIEleveStructRattachId":
                        $key = "Id Siecle";
                        $user[$id][$key] = $val;
                        break;
                    case "GPEIPersonFonctions":
                        $tab = explode('$', $val);
                        $key = "functions";
                        $user[$id][$key][] = [
                            'etab' => $tab[0],
                            'fonc' => $tab[1], // DIR, ADF, EDU, AED, MDS
                            'fonction' => $tab[2],
                            'id' => $tab[3],
                            'discipline' => $tab[4]
                        ];
                        add_array_unique($tab[4], $user[$id]['aafFunctions']);
                        break;
                    case "GPEIEnsDiv":
                    case "GPEIElvDiv":
                        $key = "allClasses";
                        $tab = explode('$', $val);
                        if (count($tab) > 0 && ! empty($tab[0])) {
                            $user[$id][$key][] = [
                                'etab' => $tab[0],
                                'name' => $tab[1],
                                'description' => $tab[2] ?? $tab[1]
                            ];
                        }
                        break;
                    case "GPEIEnsGpes":
                    case "GPEIElvGpes":
                        $key = "allGroups";
                        $tab = explode('$', $val);
                        if (count($tab) > 0 && ! empty($tab[0])) {
                            $user[$id][$key][] = [
                                'etab' => $tab[0],
                                'name' => $tab[1],
                                'description' => $tab[2] ?? $tab[1]
                            ];
                        }
                        break;
                    case "GPEIEnsMatieres":
                    case "GPEIElvMatieres":
                        $tab = explode('$', $val);
                        if (count($tab) > 0 && ! empty($tab[0])) {
                            $key = "allMatieres";
                            $user[$id][$key][] = [
                                'etab' => $tab[0],
                                'id' => $tab[1],
                                'name' => $tab[2]
                            ];
                        }
                        break;
                    case "GPEIStructureUAI":
                        $key = "UAI";
                        $user[$id][$key] = strtolower($val);
                        break;
                    case "GPEIStructureNomCourant":
                        $key = "name";
                        $user[$id][$key] = explode("$", $val)[1];
                        break;
                    case "GPEIStructureTypeStruct":
                        $key = "typeEtab";
                        $user[$id][$key] = $val;
                        break;
                    case "GPEIServAcAcademie":
                        $key = "academie";
                        $user[$id][$key] = $val;
                        break;
                    case "GPEIStructureVille":
                        $key = "city";
                        $user[$id][$key] = $val;
                        break;
                    case "GPEIStructureCodePostal":
                        $key = "code";
                        $user[$id][$key] = $val;
                        break;
                    case "GPEIStructureTelephone":
                        $key = "phone";
                        $user[$id][$key] = $val;
                        break;
                    case "GPEIEtablissementBassin":
                        $tab = explode('$', $val);
                        if (count($tab) > 0) {
                            $key = "bassin";
                            $user[$id][$key] = [
                                'id' => $tab[0],
                                'name' => $tab[1]
                            ];
                        }
                        break;
                }
            }
        }
        if (isset($user[$id]['structure'])) {
            $user[$id]['etabs'][0] = $user[$id]['structure'];
        } else {
            if (isset($user[$id]['functions'])) {
                foreach ($user[$id]['functions'] as $f => $fonction) {
                    if ($fonction['fonc'] != "-") {
                        $user[$id]['structure'] = $fonction['etab'];
                        $user[$id]['etabs'][0] = $user[$id]['structure'];
                        break;
                    }
                }
            }
        }
        if (isset($user[$id]['structure']) && isset($user[$id]['functions'])) {
            $user[$id]['fonction'] = $user[$id]['fonction'] ?? "Autres";
            $user[$id]['type'] = $user[$id]['type'] ?? "Personnel";
            $user[$id]['update'] = false;
            
            foreach ($user[$id]['functions'] as $f => $fonction) {
                // boucle sur les fonctions
                if ($fonction['etab'] == $user[$id]['structure']) {
                    // etablissement de rattachement : on définit la fonction principale
                    switch ($fonction['fonc']) {
                        case 'ENS':
                            $user[$id]['fonction'] = "";
                            $user[$id]['type'] = "Teacher";
                            $user[$id]['update'] = true;
                            break;
                        case 'DOC': // Documentaliste
                            $user[$id]['fonction'] = "Documentaliste";
                            $user[$id]['type'] = "Teacher";
                            $user[$id]['update'] = true;
                            break;
                        case 'DIR': // principal, adjoint, gestionnaire
                            $user[$id]['fonction'] = "Direction";
                            $user[$id]['type'] = "Personnel";
                            $user[$id]['update'] = true;
                            break;
                        case 'ADF': // secrétaires
                            $user[$id]['fonction'] = "Secretariat";
                            $user[$id]['type'] = "Personnel";
                            $user[$id]['update'] = true;
                            break;
                        case 'PSY':
                            $user[$id]['fonction'] = "Medical";
                            $user[$id]['type'] = "Personnel";
                            $user[$id]['update'] = false;
                            break;
                        case 'MDS': // medecin, infirmier
                            $user[$id]['fonction'] = "Medical";
                            $user[$id]['type'] = "Personnel";
                            $user[$id]['update'] = true;
                            break;
                        case 'AES': // AESH
                        case 'ASH': // AESH
                            $user[$id]['fonction'] = "AESH";
                            $user[$id]['type'] = "Teacher";
                            $user[$id]['update'] = false;
                            break;
                        case 'AED': // AED
                        case 'ADE': // ???
                            $user[$id]['fonction'] = "Autres";
                            $user[$id]['type'] = "Personnel";
                            $user[$id]['update'] = false;
                            break;
                        case 'EDU': // CPE
                            $user[$id]['fonction'] = "VieScol";
                            $user[$id]['type'] = "Personnel";
                            $user[$id]['update'] = true;
                            break;
                        default:
                            // on ne met à jour que si la fonction n'est pas déjà défini
                    }
                    // unset($user[$id]['functions'][$f]);
                    // break;
                } elseif ($fonction['fonc'] != "-") {
                    add_array_unique($fonction['etab'], $user[$id]['etabs']);
                }
            }
        }
    }
    return $user;
}

function sync_gpei($config, &$html = "")
{
    $html .= "<h1>Import gpei</h1>\n";
    $dossier_tmp_import_comptes = sys_get_temp_dir() . "/gpei";
    if (! is_dir($dossier_tmp_import_comptes)) {
        mkdir($dossier_tmp_import_comptes, "700");
    }
    exec("sudo chmod 700 " . $dossier_tmp_import_comptes);
    exec("sudo chmod 700 " . $dossier_tmp_import_comptes . "/*");
    exec("sudo chown -R www-admin " . $dossier_tmp_import_comptes);
    if (! empty($config['gpei_url'])) {
        // on est sur le central
        $ssh = "/usr/bin/ssh -i /etc/sambaedu/id_rsa " . $config['gpei_url'];
        $list = [];
        $html .= exec($ssh . " ls *.zip", $list);
        // recherche du dernier complet et des derniers deltas
        $complet = $delta = [];
        foreach ($list as $k => $gpei) {
            $m = [];
            if (preg_match("/^.*_GPEI_(Complet|Delta)_([0-9]{8})\.zip$/", $gpei, $m)) {
                if ($m[1] == "Complet") {
                    $complet[$k] = $gpei;
                } else {
                    $delta[$k] = $gpei;
                }
            }
        }
        sort($complet);
        if (count($complet) > 0) {
            // on prend le plus récent complet pour commencer
            $file = array_pop($complet);
            // on efface les anciens complets qui ne servent à rien
            foreach ($complet as $k => $c) {
                if (file_exists($dossier_tmp_import_comptes . "/" . $c)) {
                    unlink($dossier_tmp_import_comptes . "/" . $c);
                }
                unset($list[$k]);
            }
            $m = [];
            if (preg_match("/^.*_GPEI_Complet_([0-9]{8})\.zip$/", $file, $m)) {
                $date = $m[1];
                foreach ($delta as $k => $d) {
                    $m = [];
                    if (preg_match("/^.*_GPEI_Delta_([0-9]{8})\.zip$/", $d, $m)) {
                        // on efface les anciens deltas
                        if ($m[1] < $date) {
                            if (file_exists($dossier_tmp_import_comptes . "/" . $d)) {
                                unlink($dossier_tmp_import_comptes . "/" . $d);
                            }
                            unset($list[$k]);
                        }
                    }
                }
            }
        }
        // liste des derniers fichers téléchargés
        if (file_exists($dossier_tmp_import_comptes . "/list.json")) {
            $oldlist = json_decode(file_get_contents($dossier_tmp_import_comptes . "/list.json"), true);
            foreach ($oldlist as $k => $f) {
                if (! preg_match("/^.*_GPEI_.*_[0-9]{8}\.zip$/", $f)) {
                    unset($oldlist[$k]);
                }
            }
        } else {
            $oldlist = [];
        }
        // $html .= print_r($oldlist, true);
        // $html .= print_r($list, true);
        $new = array_diff($list, $oldlist);
        // $html .= "Récupération de " . count($new) . " fichiers gpei<br>\n";
        // $html .= print_r($new, true);
        // return $html;
        foreach ($new as $gpei) {
            $scp = "scp -i /etc/sambaedu/id_rsa -o StrictHostKeyChecking=no -o UserKnownHostsFile=/dev/null " . $config['gpei_url'] . ":~/" . $gpei . " " . $dossier_tmp_import_comptes;
            system($scp);
        }

        $dl_list = scandir($dossier_tmp_import_comptes);
        // on retire les intrus
        foreach ($dl_list as $k => $f) {
            if (! preg_match("/^.*_GPEI_.*_[0-9]{8}\.zip$/", $f)) {
                unset($dl_list[$k]);
            }
        }
        // répartiteur de charge : on répartit équitablement l'import sur les serveurs des colleges actifs
        $activated = activated_etabs($config);
        sort($activated);
        $workers[0] = [];
        $n = 1;
        foreach ($activated as $k => $etab) {
            if ($n < $config['gpei_workers'] && $etab['activated'] && fping("se4fs-" . $etab['uai']) && fping("se4ad-" . $etab['uai'])) {
                $workers[$etab['uai']] = [
                    $etab['uai']
                ];
                $n ++;
                unset($activated[$k]);
                // echo "se4fs-" . $etab['uai'] . "<br>";
            }
        }
        $nb = ceil(count($activated) / count($workers));
        $html .= "repartition de " . $nb . " etabs sur " . count($workers) . "<br>";
        foreach ($workers as $u => $worker) {
            $n = count($worker);
            foreach ($activated as $k => $etab) {
                if ($n ++ < $nb) {
                    $workers[$u][] = $etab['uai'];
                    unset($activated[$k]);
                } else {
                    break;
                }
            }
        }
        foreach ($workers as $worker => $etabs) {
            foreach ($etabs as $etab) {
                if ($worker != 0) {
                    system("ssh -i /etc/sambaedu/id_rsa -o StrictHostKeyChecking=no -o UserKnownHostsFile=/dev/null root@se4fs-" . $worker . " mkdir -p " . $dossier_tmp_import_comptes . "/" . $etab);
                    system("ssh -i /etc/sambaedu/id_rsa -o StrictHostKeyChecking=no -o UserKnownHostsFile=/dev/null root@se4fs-" . $worker . " chmod 700 " . $dossier_tmp_import_comptes . "/*");
                    // copie du cron pour être sûr que les etabs vont bien faire les sync successives pour chaque étab assigné.
                    $scp = "scp -i /etc/sambaedu/id_rsa -o StrictHostKeyChecking=no -o UserKnownHostsFile=/dev/null /etc/cron.d/sambaedu-web-common root@se4fs-" . $worker . ":/etc/cron.d/sambaedu-web-common";
                    system($scp);
                    if ($n = count($dl_list) > 0) {
                        $html .= " envoi au serveur " . $worker . " de $n fichiers pour " . $etab . "<br>\n";
                        foreach ($dl_list as $gpei) {
                            $scp = "scp -i /etc/sambaedu/id_rsa -o StrictHostKeyChecking=no -o UserKnownHostsFile=/dev/null " . $dossier_tmp_import_comptes . "/" . $gpei . " root@se4fs-" . $worker . ":" . $dossier_tmp_import_comptes . "/" . $etab . "/" . $gpei;
                            system($scp);
                        }
                    }
                    system("ssh -i /etc/sambaedu/id_rsa -o StrictHostKeyChecking=no -o UserKnownHostsFile=/dev/null root@se4fs-" . $worker . " chown -R www-admin " . $dossier_tmp_import_comptes);
                } else {
                    if (! is_dir($dossier_tmp_import_comptes . "/" . $etab)) {
                        mkdir($dossier_tmp_import_comptes . "/" . $etab);
                    }
                    foreach ($dl_list as $gpei) {
                        copy($dossier_tmp_import_comptes . "/" . $gpei, $dossier_tmp_import_comptes . "/" . $etab . "/" . $gpei);
                    }
                }
            }
        }

        // foreach (activated_etabs($config) as $etab) {
        // if ($etab['activated'] && fping("se4fs-" . $etab['uai'])) {
        // system("ssh -i /etc/sambaedu/id_rsa -o StrictHostKeyChecking=no -o UserKnownHostsFile=/dev/null root@se4fs-" . $etab['uai'] . " mkdir -p " . $dossier_tmp_import_comptes . "/" . $etab['uai']);
        // system("ssh -i /etc/sambaedu/id_rsa -o StrictHostKeyChecking=no -o UserKnownHostsFile=/dev/null root@se4fs-" . $etab['uai'] . " chmod 700 " . $dossier_tmp_import_comptes . "/*");
        // if ($n = count($dl_list) > 0) {
        // $html .= " envoi au serveur " . $etab['uai'] . " de $n fichiers<br>\n";
        // foreach ($dl_list as $gpei) {
        // $scp = "scp -i /etc/sambaedu/id_rsa -o StrictHostKeyChecking=no -o UserKnownHostsFile=/dev/null " . $dossier_tmp_import_comptes . "/" . $gpei . " root@se4fs-" . $etab['uai'] . ":" . $dossier_tmp_import_comptes . "/" . $etab['uai'] . "/" . $gpei;
        // system($scp);
        // }
        // }
        // system("ssh -i /etc/sambaedu/id_rsa -o StrictHostKeyChecking=no -o UserKnownHostsFile=/dev/null root@se4fs-" . $etab['uai'] . " chown -R www-admin " . $dossier_tmp_import_comptes);
        // } else {
        // if (! is_dir($dossier_tmp_import_comptes . "/" . $etab['uai'])) {
        // mkdir($dossier_tmp_import_comptes . "/" . $etab['uai']);
        // }
        // foreach ($dl_list as $gpei) {
        // copy($dossier_tmp_import_comptes . "/" . $gpei, $dossier_tmp_import_comptes . "/" . $etab['uai'] . "/" . $gpei);
        // }
        // }
        // }
        // on supprime les fichiers utilisés
        foreach ($dl_list as $gpei) {
            unlink($dossier_tmp_import_comptes . "/" . $gpei);
        }
        // on ecrit la liste pour la prochaine fois
        file_put_contents($dossier_tmp_import_comptes . "/list.json", json_encode($list));
    }

    // serveurs de site, ou etabs non joignables sur le central
    foreach (scandir($dossier_tmp_import_comptes) as $uai) {
        $file = [];
        if (preg_match("/[0-9]{7}[a-z]/", $uai)) {
            $html .= "Recherche des fichiers GPEI restant à traiter pour $uai :<br>\n";
            $dossier_tmp_import_comptes_uai = $dossier_tmp_import_comptes . "/" . $uai;
            $list = scandir($dossier_tmp_import_comptes_uai);
            $complet = $delta = [];
            $cd = 0;
            foreach ($list as $gpei) {
                $m = [];
                if (preg_match("/^.*_GPEI_(Complet|Delta)_([0-9]{8})\.zip$/", $gpei, $m)) {
                    if ($m[1] == "Complet") {
                        $complet[] = $gpei;
                    } else {
                        $delta[] = $gpei;
                        $cd ++;
                    }
                }
            }
            sort($complet);
            if (count($complet) > 0) {
                // on prend le plus récent complet pour commencer
                $cplt = array_pop($complet);
                $file[] = $cplt;
                // on efface les anciens complets qui ne servent à rien
                foreach ($complet as $c) {
                    unlink($dossier_tmp_import_comptes_uai . "/" . $c);
                }
                $m = [];
                if (preg_match("/^.*_GPEI_Complet_([0-9]{8})\.zip$/", $cplt, $m)) {
                    $date = $m[1];
                    foreach ($delta as $d) {
                        $m = [];
                        if (preg_match("/^.*_GPEI_Delta_([0-9]{8})\.zip$/", $d, $m)) {
                            // on efface les anciens deltas
                            if ($m[1] < $date) {
                                unlink($dossier_tmp_import_comptes_uai . "/" . $d);
                                $cd --;
                            }
                        }
                    }
                }
            }
            if ($cd > 0) {
                sort($delta);
                foreach ($delta as $d) {
                    $file[] = $d;
                }
            }
            // on boucle sur la liste ordonée des fichiers
            if (count($file) > 0) {
                apcu_store('uai', $uai);
                $config = get_config($config, true, $uai);
                $f = array_shift($file);
                $html .= import_archive($config, $dossier_tmp_import_comptes_uai . "/" . $f, "gpei");
                unlink($dossier_tmp_import_comptes_uai . "/" . $f);

                // passage au traitement
                // apcu_store('ent_html', $html);
                return true;
            } else {
                // on boucle sur l'etab suivant
                $html .= " Pas de fichier GPEI à traiter pour $uai <br>\n";
            }
        }
    }
    // apcu_store('ent_html', $html);
    return false;
}
?>