<?php

/*
 * fonctions de manipulation des GPO pour sambaedu
 * @Projet SambaEdu
 *
 * @Note: Ce fichier de fonction doit etre appele par un include
 *
 * @Licence Distribue sous la licence GPL
 *
 * file: gpo.inc.php
 *
 * @Repertoire: includes/
 */
define('REG_NONE', 0);
define('REG_SZ', 1);
define('REG_EXPAND_SZ', 2);
define('REG_BINARY', 3);
define('REG_DWORD', 4);
define('REG_DWORD_BIG_ENDIAN', 5);
define('REG_LINK', 6);
define('REG_MULTI_SZ', 7);
define('REG_RESOURCE_LIST', 8);
define('REG_FULL_RESOURCE_DESCRIPTOR', 9);
define('REG_RESOURCE_REQUIREMENTS_LIST', 10);
define('REG_QWORD', 11);
define('REG_QWORD_LITTLE_ENDIAN', 11);

define('REGFILE_SIGNATURE', "PReg");
define('REGFILE_VERSION', "\1\0\0\0");
define('UTF32_BIG_ENDIAN_BOM', chr(0x00) . chr(0x00) . chr(0xFE) . chr(0xFF));
define('UTF32_LITTLE_ENDIAN_BOM', chr(0xFF) . chr(0xFE) . chr(0x00) . chr(0x00));
define('UTF16_BIG_ENDIAN_BOM', chr(0xFE) . chr(0xFF));
define('UTF16_LITTLE_ENDIAN_BOM', chr(0xFF) . chr(0xFE));
define('UTF8_BOM', chr(0xEF) . chr(0xBB) . chr(0xBF));

// ACLS par défaut pour les gpo

// define('GPO_SDDL', "O:DAG:DAD:P(A;OICI;FA;;;DA)(A;OICI;FA;;;EA)(A;OICIIO;FA;;;CO)(A;OICI;FA;;;DA)(A;OICI;FA;;;SY)(A;OICI;0x1200a9;;;AU)(OA;OICI;;edacfd8f-ffb3-11d1-b41d-00a0c968f939;;AU)(A;OICI;0x1200a9;;;ED)");
define('GPO_SDDL', "O:DAG:DAD:P(A;OICI;FA;;;DA)(A;OICI;FA;;;EA)(A;OICIIO;FA;;;CO)(A;OICI;FA;;;DA)(A;OICI;FA;;;SY)(A;OICI;0x1200a9;;;AU)(A;OICI;0x1200a9;;;ED)");
// valeur attendue par sysvolcheck (racine):
// O:DAG:DAD:P(A;OICI;FA;;;DA)(A;OICI;FA;;;EA)(A;OICIIO;FA;;;CO)(A;OICI;FA;;;DA)(A;OICI;FA;;;SY)(A;OICI;0x1200a9;;;AU)(OA;OICI;;edacfd8f-ffb3-11d1-b41d-00a0c968f939;;AU)(A;OICI;0x1200a9;;;ED)
// valeur correcte racine:
// O:DAG:DAD:PAI(A;OICI;FA;;;DA)(A;OICI;FA;;;EA)(A;OICIIO;FA;;;CO)(A;OICI;FA;;;DA)(A;OICI;FA;;;SY)(A;OICI;0x1200a9;;;AU)(A;OICI;0x1200a9;;;ED)
// sous dossier
// O:BAG:DUD:AI(A;OICIID;FA;;;DA)(A;OICIID;FA;;;EA)(A;ID;FA;;;BA)(A;OICIIOID;FA;;;CO)(A;OICIID;FA;;;SY)(A;OICIID;0x1200a9;;;AU)(A;OICIID;0x1200a9;;;ED)
// fichier
// O:BAG:DUD:AI(A;ID;FA;;;DA)(A;ID;FA;;;EA)(A;ID;FA;;;BA)(A;ID;FA;;;SY)(A;ID;0x1200a9;;;AU)(A;ID;0x1200a9;;;ED)
// structures des GPO/GPP bien connues

define('GPT_INI', [
    'type' => "ini",
    'target' => "ini",
    'path' => '/',
    'file' => "GPT.INI"
]);

define('USER_GPO', [
    'type' => "gpo",
    'target' => "user",
    'path' => '/User/',
    'file' => "Registry.pol"
]);
define('MACHINE_GPO', [
    'type' => "gpo",
    'target' => "machine",
    'file' => "Registry.pol",
    'path' => '/Machine/'
]);
define('USER_PRINTERS', [
    'type' => "gpp",
    'target' => "user",
    'path' => '/User/Preferences/Printers/',
    'file' => "Printers.xml",
    'clsid' => "{1F577D12-3D1B-471e-A1B7-060317597B9C}"
]);
define('USER_SHORTCUTS', [
    'type' => "gpp",
    'target' => "user",
    'path' => '/User/Preferences/Shortcuts/',
    'file' => "Shortcuts.xml",
    'clsid' => "{872ECB34-B2EC-401b-A585-D32574AA90EE}"
]);
define('USER_REGISTRY', [
    'type' => "gpp",
    'target' => "user",
    'path' => '/User/Preferences/Registry/',
    'file' => "Registry.xml",
    'clsid' => "{9CD4B2F4-923D-47f5-A062-E897DD1DAD50}"
]);
define('MACHINE_REGISTRY', [
    'type' => "gpp",
    'target' => "machine",
    'path' => '/Machine/Preferences/Registry/',
    'file' => "Registry.xml",
    'clsid' => "{9CD4B2F4-923D-47f5-A062-E897DD1DAD50}"
]);

define('MACHINE_SECEDIT', [
    'type' => "ini",
    'target' => "machine",
    'path' => '/Machine/Preferences/Machine/Microsoft/Windows NT/SecEdit',
    'file' => "GptTmpl.inf",
    'clsid' => ""
]);

function detect_utf_encoding($text)
{
    $first2 = substr($text, 0, 2);
    $first3 = substr($text, 0, 3);
    $first4 = substr($text, 0, 3);

    if ($first3 == UTF8_BOM)
        return 'UTF-8';
    elseif ($first4 == UTF32_BIG_ENDIAN_BOM)
        return 'UTF-32BE';
    elseif ($first4 == UTF32_LITTLE_ENDIAN_BOM)
        return 'UTF-32LE';
    elseif ($first2 == UTF16_BIG_ENDIAN_BOM)
        return 'UTF-16BE';
    elseif ($first2 == UTF16_LITTLE_ENDIAN_BOM)
        return 'UTF-16LE';
    return 'UTF-8';
}

/*
 * Fonction de comparaison pour trier les listes des GPO
 * par la clé displayname
 *
 *
 */
function compare_list_gpo_by_name($gpo1, $gpo2)
{
    return strcasecmp($gpo1['displayname'], $gpo2['displayname']);
}

function dword2int($str)
{
    $bytes = unpack("nlow/nhigh", $str);
    if (count($bytes) == 2) {
        return $bytes['low'] + 65536 * $bytes['high'];
    } else {
        return NULL;
    }
}

function int2dword(int $int)
{
    $l = $int % 65536;
    $h = intdiv($int, 65536);
    return pack("v2", $l, $h);
}

function dstr2str($str)
{
    // return iconv("UTF-16LE", "UTF-8", substr($str, 1, strlen($str) - 1));
    // supprime le premier \0 issu du split et \0\0 à la fin
    // return iconv("UTF-16LE", "UTF-8", substr($str, 1, strlen($str) - 2));
    return preg_replace("/\\0/", "", $str);
}

function str2dstr($str)
{
    return iconv("UTF-8", "UTF-16LE", $str);
}

function char2dchar($str)
{
    return iconv("UTF-8", "UTF-16LE", $str);
}

function get_domain_sid($config, $user = "")
{
    if (empty($user)) {
        $user = $config['ldap_admin_name'];
    }
    $out = exec("wbinfo --name-to-sid " . $user);
    return preg_replace("/-[0-9]+ .*$/", "", $out);
}

function get_sid($config, $user)
{
    $out = exec("wbinfo --name-to-sid " . $user);
    return preg_replace("/ .*$/", "", $out);
}

/**
 * lit un fichier gpo .pol, le résultat est rangé dans un tableau, les chaines de caractere sont en utf-8
 * le tableau peut directement être utilié pour recontruire un fichier .pol
 * seuls les valeurs REG_DWORD et REG_SZ sont réellement parsées
 * Les données non manipulables simplement sont laissées telles quelles.
 *
 * @param unknown $file
 * @return array
 */
function read_pol($file)
{
    $pol = fopen($file, 'rb');
    $info = fread($pol, filesize($file));
    fclose($pol);
    $gpo = array();
    mb_regex_encoding("UTF-8");
    mb_internal_encoding("UTF-16LE");

    # code repris en partie de https://github.com/jtpereyda/regpol

    $magic_string = "\x50\x52\x65\x67\x01\x00\x00\x00";

    if (strpos($info, $magic_string) !== 0) {
        // on est sur une representation json
        $gpo = json_decode($info, true);
        // echo "$info<br>\n";
        return $gpo;
    }

    $body = substr($info, strlen($magic_string));

    while (strlen($body) > 0) {
        if (substr($body, 0, 2) !== "\x5B\x00") {
            echo "Error: Entry does not start with \"[\"<br>\n";
            // echo "$body<br>\n";
            break;
        }
        $body = substr($body, 2);

        // key
        list ($key, $body) = explode("\x3B\x00", $body, 2);
        $key = dstr2str($key);

        // value
        list ($value, $body) = explode("\x3B\x00", $body, 2);
        $value = dstr2str($value);

        // type
        $regtype = substr($body, 0, 4);
        $body = substr($body, 4 + 2); // 4 bytes + 2 bytes pour le délimiteur
        $regtype = unpack("V", $regtype)[1];
        // $regtype = dword2int($regtype);

        // size
        $size = substr($body, 0, 4);
        $body = substr($body, 4 + 2); // 4 bytes + 2 bytes pour le délimiteur
        $size = unpack("V", $size)[1];
        // $size = dword2int($size);

        // data
        $data = substr($body, 0, $size);
        $body = substr($body, $size);

        $cle = array(
            'key' => $key,
            'value' => $value,
            'type' => $regtype,
            'size' => $size
        );

        if (($cle['type'] == REG_SZ) || ($cle['type'] == REG_EXPAND_SZ) || ($cle['type'] == REG_MULTI_SZ)) {
            $cle['data'] = dstr2str($data);
        } elseif ($cle['type'] == REG_DWORD) {
            $cle['data'] = dword2int($data);
        } else {
            $cle['data'] = $data;
        }

        // echo "gpo : <pre>" . print_r($cle, true) . "</pre><br>\n";
        $gpo[] = $cle;

        if (substr($body, 0, 2) !== "\x5D\x00") {
            echo "Error: Entry does not end with \"]\"<br>\n";
            // echo "$body<br>\n";
            break;
        }
        $body = substr($body, 2);
    }

    return $gpo;
}

/**
 * ecrit un fichier gpo .pol, à partir d'un tableau
 * seuls les valeurs REG_DWORD et REG_SZ sont réellement parsées
 * Les données non manipulables simplement sont laissées telles quelles.
 *
 * @param unknown $file
 * @return array
 */
function write_pol($file, $gpo)
{
    $info = REGFILE_SIGNATURE . REGFILE_VERSION;
    foreach ($gpo as $cle) {
        if (($cle['type'] == REG_SZ) || ($cle['type'] == REG_EXPAND_SZ) || ($cle['type'] == REG_MULTI_SZ)) {
            $data = str2dstr($cle['data'] . "\0");
        } elseif ($cle['type'] == REG_DWORD) {
            $data = int2dword($cle['data']);
        } else {
            $data = $cle['data'];
        }
        $size = strlen($data);

        $info .= char2dchar("[");
        $info .= str2dstr($cle['key'] . "\0");
        $info .= char2dchar(";");
        $info .= str2dstr($cle['value'] . "\0");
        $info .= char2dchar(";");
        $info .= int2dword($cle['type']);
        $info .= char2dchar(";");
        $info .= int2dword($size);
        $info .= char2dchar(";");
        $info .= $data;
        $info .= char2dchar("]");
    }
    $pol = fopen($file, 'wb');
    $ret = fwrite($pol, $info);
    fclose($pol);
    return $ret;
}

function read_pref($file)
{}

/**
 * extrait les données d'une clé du registre.
 * Le résultat est un tableau des chaines séparées par des ;
 *
 * @param unknown $config
 * @param unknown $pol
 *            table extrait du .pol
 * @param unknown $key
 *            nom de la clé
 * @return array
 */
function get_pol_key($pol, $key)
{
    $i = array_search($key, array_column($pol, "value"));
    return explode(";", $pol[$i]['data']);
}

/**
 * Ecrit les données d'une clé du registre.
 * a tableau des chaines
 *
 * @param unknown $pol
 *            table extrait du .pol
 * @param unknown $key
 *            nom de la clé
 * @param unknown $replace
 * @return string $data
 */
function change_pol_key(&$pol, $key, string $data, $replace = true)
{
    $i = array_search($key, array_column($pol, "value"));
    $old = explode(";", $pol[$i]['data']);
    if ($replace) {
        $new = $data;
    } else {
        $new = implode(";", array_merge($old, $data));
    }
    $pol[$i]['data'] = $new;
    return $data;
}

function read_gpo_json()
{
    if (file_exists("/etc/sambaedu/applications/gpos.json") && $txt = file_get_contents("/etc/sambaedu/applications/gpos.json")) {
        return json_decode($txt, true);
    }
    return [];
}

function write_gpo_json($gpo, $type, $key, $data)
{
    $local_gpos = read_gpo_json();
    $local_gpos[$gpo['displayname']]['local'][$key] = [
        'type' => $type,
        'data' => $data
    ];
    file_put_contents("/etc/sambaedu/applications/gpos.json", json_encode($local_gpos, JSON_PRETTY_PRINT));
}

function read_gpp_shortcuts($config, $xml)
{}

/**
 * modification du parametrage du proxy à partir des parametres de conf
 * Le parametrage est réécrit complètement, on ne remplace pas les valeurs existantes
 * parametrage proxy windows (IE8 et 10) et mozilla
 *
 * @param unknown $config
 */
function change_proxy_file($config, $line)
{
    if ($config['proxy_type'] != "automatique" && ! empty($config['proxy_address']) && ! empty($config['proxy_port'])) {
        $config['proxy_url'] = "http://" . $config['proxy_address'] . ":" . $config['proxy_port'];
    }
    if (! empty($config['proxy_url']) && empty($config['proxy_type'])) {
        $config['proxy_type'] = "manuel";
    } elseif ($config['proxy_type'] != "automatique") {
        $config['proxy_type'] = "";
    }
    switch ($config['proxy_type']) {
        case "automatique":
            $config['proxy_autoconfigurl'] = $config['proxy_url'];
            // $config['proxy_server'] = "http://" . @$config['proxy_address'] . ":" . @$config['proxy_port'];
            $config['proxy_server'] = "";
            $config['proxy_auto'] = 1;
            $config['proxy_enable'] = 0;
            break;
        case "manuel":
            $config['proxy_autoconfigurl'] = "";
            $config['proxy_auto'] = 0;
            $config['proxy_server'] = $config['proxy_url'];
            $config['proxy_enable'] = 1;
            break;
        default:
            $config['proxy_autoconfigurl'] = "";
            $config['proxy_auto'] = 0;
            $config['proxy_server'] = "";
            $config['proxy_enable'] = 0;
            $config['proxy_url'] = "";
    }
    $xml = new DOMDocument();
    $xml->formatOutput = false;
    $xml->preserveWhiteSpace = false;
    $xml->loadXML($line);
    $xpath = new DOMXPath($xml);

    // Vintage

    if ($key = $xpath->query("/InternetSettings/IE8/Properties/Reg[@name='ProxyOverride']"))
        $key[0]->setAttribute('value', $config['proxy_url'] . ";<local>");
    if ($key = $xpath->query("/InternetSettings/IE8/Properties/Reg[@name='AutoConfigURL']"))
        $key[0]->setAttribute('value', $config['proxy_autoconfigurl']);
    if ($key = $xpath->query("/InternetSettings/IE8/Properties/Reg[@name='ProxyEnable']"))
        $key[0]->setAttribute('value', $config['proxy_enable']);
    if ($key = $xpath->query("/InternetSettings/IE8/Properties/Reg[@name='ProxyServer']"))
        $key[0]->setAttribute('value', $config['proxy_server']);
    // if ($key = $xpath->query("/InternetSettings/IE8/Properties/Reg[@name='AutoDetect']"))
    // $key[0]->setAttribute('value', $config['proxy_auto']);

    if ($key = $xpath->query("/InternetSettings/IE10/Properties/Reg[@name='ProxyOverride']"))
        $key[0]->setAttribute('value', $config['proxy_url'] . ";<local>");

    if ($key = $xpath->query("/InternetSettings/IE10/Properties/Reg[@name='AutoConfigURL']"))
        $key[0]->setAttribute('value', $config['proxy_autoconfigurl']);
    if ($key = $xpath->query("/InternetSettings/IE10/Properties/Reg[@name='ProxyEnable']"))
        $key[0]->setAttribute('value', $config['proxy_enable']);
    if ($key = $xpath->query("/InternetSettings/IE10/Properties/Reg[@name='ProxyServer']"))
        $key[0]->setAttribute('value', $config['proxy_server']);
    if ($key = $xpath->query("/InternetSettings/IE10/Properties/Reg[@name='AutoDetect']"))
        $key[0]->setAttribute('value', $config['proxy_auto']);

    return $xml->saveXML();
}

function change_proxy_pol($config)
{}

function generalise_gpo($config, $source, $export = false)
{
    $pol_list = [
        $source . "/User/Registry.pol",
        $source . "/Machine/Registry.pol"
    ];
    $params = [
        "domain",
        "samba_domain",
        "se4fs_name",
        "se4ad_name",
        "domain_sid",
        "se4install_name",
        "ldap_base_dn"
    ];

    $config['domain_sid'] = get_domain_sid($config);

    foreach ($pol_list as $pol) {
        $npol = $pol;
        $del = false;
        $p = explode("/", $pol);
        $f = array_pop($p);
        if (file_exists(implode("/", $p) . "/" . strtolower($f))) {
            $pol = implode("/", $p) . "/" . strtolower($f);
            $del = true;
        }
        if (file_exists($pol)) {
            $gpo = read_pol($pol);
            foreach ($gpo as $i => $cle) {
                foreach ($params as $param) {
                    $gpo[$i]['key'] = preg_replace("/" . $config[$param] . "/", "###_" . strtoupper($param) . "_###", $gpo[$i]['key']);
                    $gpo[$i]['value'] = preg_replace("/" . $config[$param] . "/", "###_" . strtoupper($param) . "_###", $gpo[$i]['value']);
                    $gpo[$i]['data'] = preg_replace("/" . $config[$param] . "/", "###_" . strtoupper($param) . "_###", $gpo[$i]['data']);
                }
            }
            if ($del) {
                unlink($pol);
            }
            if ($export) {
                file_put_contents($npol, json_encode($gpo, JSON_PRETTY_PRINT));
            } else {
                write_pol($npol, $gpo);
            }
        }
    }

    if (is_dir($source) === true) {
        $files = new RecursiveIteratorIterator(new RecursiveDirectoryIterator($source), RecursiveIteratorIterator::SELF_FIRST);

        foreach ($files as $file) {

            // Ignore "." and ".." folders
            if (in_array(substr($file, strrpos($file, '/') + 1), array(
                '.',
                '..',
                '.pol'
            )))
                continue;

            $file = realpath($file);

            if (is_file($file) === true) {
                $proxy_xml = preg_match("/InternetSettings\.xml/", $file);
                if ($proxy_xml != 1) {
                    $content = file_get_contents($file);
                    $enc = detect_utf_encoding($content);

                    if ($enc == 'UTF-16LE') {
                        $content = mb_convert_encoding($content, "UTF-8", "UTF-16LE");
                    }
                    foreach ($params as $param) {

                        $content_chunks = explode($config[$param], $content);
                        $content = implode("###_" . strtoupper($param) . "_###", $content_chunks);
                    }
                    $content_chunks = explode(">" . strtoupper($config['samba_domain']) . "\\", $content);
                    $content = implode(">###_SAMBA_DOMAIN_###\\", $content_chunks);
                    if ($enc == 'UTF-16LE') {
                        $content = mb_convert_encoding($content, "UTF-16LE", "UTF-8");
                    }
                    file_put_contents($file, $content);
                }
            }
        }
    }
}

function specialise_gpo($config, $source, $export = false)
{
    $pol_list = [
        $source . "/User/Registry.pol",
        $source . "/Machine/Registry.pol"
    ];
    $params = [
        "domain",
        "samba_domain",
        "se4fs_name",
        "se4ad_name",
        "domain_sid",
        "se4install_name",
        "ldap_base_dn"
    ];
    $config['domain_sid'] = get_domain_sid($config);

    foreach ($pol_list as $pol) {
        if (file_exists($pol)) {
            $gpo = read_pol($pol);
            foreach ($gpo as $i => $cle) {
                foreach ($params as $param) {
                    $gpo[$i]['key'] = preg_replace("/###_" . strtoupper($param) . "_###/", $config[$param], $gpo[$i]['key']);
                    $gpo[$i]['value'] = preg_replace("/###_" . strtoupper($param) . "_###/", $config[$param], $gpo[$i]['value']);
                    $gpo[$i]['data'] = preg_replace("/###_" . strtoupper($param) . "_###/", $config[$param], $gpo[$i]['data']);
                }
                change_proxy_pol($config);
            }
            write_pol($pol, $gpo);
        }
    }
    if (is_dir($source) === true) {
        $files = new RecursiveIteratorIterator(new RecursiveDirectoryIterator($source), RecursiveIteratorIterator::SELF_FIRST);

        foreach ($files as $file) {

            // Ignore "." and ".." folders
            if (in_array(substr($file, strrpos($file, '/') + 1), array(
                '.',
                '..',
                '.pol'
            )))
                continue;

            $file = realpath($file);

            if (is_file($file) === true) {
                $proxy_xml = preg_match("/InternetSettings\.xml/", $file);
                $content = file_get_contents($file);
                $enc = detect_utf_encoding($content);
                if ($enc == 'UTF-16LE') {
                    $content = mb_convert_encoding($content, "UTF-8", "UTF-16LE");
                }
                foreach ($params as $param) {
                    $content_chunks = explode("###_" . strtoupper($param) . "_###", $content);
                    $content = implode($config[$param], $content_chunks);
                }
                if ($proxy_xml == 1) {
                    $content = change_proxy_file($config, $content);
                }
                if ($enc == 'UTF-16LE') {
                    $content = mb_convert_encoding($content, "UTF-16LE", "UTF-8");
                }
                file_put_contents($file, $content);
            }
        }
    }
}

function zip_gpo($source, $destination)
{
    if (! extension_loaded('zip') || ! file_exists($source)) {
        return false;
    }

    $zip = new ZipArchive();
    if (! $zip->open($destination, ZIPARCHIVE::CREATE)) {
        return false;
    }

    if (is_dir($source) === true) {
        $files = new RecursiveIteratorIterator(new RecursiveDirectoryIterator($source), RecursiveIteratorIterator::SELF_FIRST);

        foreach ($files as $file) {

            // Ignore "." and ".." folders
            if (in_array(substr($file, strrpos($file, '/') + 1), array(
                '.',
                '..'
            )))
                continue;

            $file = realpath($file);

            if (is_dir($file) === true) {
                $zip->addEmptyDir(str_replace($source . '/', '', $file . '/'));
            } else if (is_file($file) === true) {
                $zip->addFromString(str_replace($source . '/', '', $file), file_get_contents($file));
            }
        }
    } else if (is_file($source) === true) {
        $zip->addFromString(basename($source), file_get_contents($source));
    }

    return $zip->close();
}

function recursive_copy($source, $dest)
{
    $sourceDir = opendir($source);
    @mkdir($dest);
    while (($file = readdir($sourceDir))) {
        if (($file != '.') && ($file != '..')) {
            if (is_dir($source . '/' . $file)) {
                recursive_copy($source . '/' . $file, $dest . '/' . $file);
            } else {
                copy($source . '/' . $file, $dest . '/' . $file);
            }
        }
    }
    closedir($sourceDir);
}

function unzip_gpo($gpo_zip, $name)
{
    if (preg_match("/\.zip$/", $gpo_zip)) {
        $gpo_zip = "/usr/share/sambaedu/gpo/" . $gpo_zip;
        if (! file_exists($gpo_zip)) {
            return false;
        }
        if (! extension_loaded('zip')) {
            return false;
        }
        $zip = new ZipArchive();
        if ($zip->open($gpo_zip) === TRUE) {
            $tmppath = sys_get_temp_dir() . "/" . $name;
            if (! is_dir($tmppath)) {
                mkdir($tmppath);
            }
            $zip->extractTo($tmppath);
        } else {
            return false;
        }
        return $zip->close();
    } else {
        $gpo_zip = "/usr/share/sambaedu/gpo/sambaedu-gpo/" . $gpo_zip;
        if (! file_exists($gpo_zip)) {
            return false;
        }
        $tmppath = sys_get_temp_dir() . "/" . $name;
        recursive_copy($gpo_zip, $tmppath);
        return true;
    }
}

/**
 * recupère les informations d'une gpo à importer
 *
 * la gpo peut être zippée ou pas
 *
 * @param string $dir
 *            dossier de la gpo
 * @return array|boolean
 */
function get_gpo_template_info(string $gpo)
{
    // $path = "/var/www/sambaedu/gpo/templates/";
    $path = "/usr/share/sambaedu/gpo/";

    $zip = new ZipArchive();
    if ($zip->open($path . $gpo) === TRUE) {
        $ini = $zip->getFromName('GPT.INI');
        $gptini = parse_ini_string($ini, true);
        $zip->close();
        if ($gptini && (isset($gptini['CSE']['gpcmachineextensionnames']) || isset($gptini['CSE']['gpcuserextensionnames'])))
            return $gptini;
        else
            error_log("gpo invalide : " . $gpo);
    } else {
        $path = "/usr/share/sambaedu/gpo/sambaedu-gpo/";
        $ini = file_get_contents($path . $gpo . "/GPT.INI");
        // on les met en clair
        $gptini = parse_ini_string($ini, true);
        if ($gptini && (isset($gptini['CSE']['gpcmachineextensionnames']) || isset($gptini['CSE']['gpcuserextensionnames'])))
            return $gptini;
        else
            error_log("gpo invalide : " . $gpo);
    }
    return false;
}

/**
 * liste les templates de GPO disponibles pour l'import
 * qui contiennent un prefixe se4_ (issues du paquet)
 *
 * @return array
 */
function list_gpo_templates()
{
    // $path = "/var/www/sambaedu/gpo/templates/";
    $path = "/usr/share/sambaedu/gpo/";

    // $dir = opendir($path);
    $gpos = scandir($path);
    $templates = array();
    foreach ($gpos as $gpo) {
        $match = array();
        if (preg_match("/^(se4_.*\.zip)$/", $gpo, $match)) {
            if ($info = get_gpo_template_info($match[1])) {
                $templates[] = array(
                    'displayname' => $info['General']['displayName'],
                    'version' => $info['General']['Version']
                );
            }
        }
    }
    usort($templates, 'compare_list_gpo_by_name');
    return $templates;
}

/**
 * liste les templates de GPO disponibles pour l'import
 * qui contiennent un prefixe se4_ (issues de gitlab)
 *
 * @return array
 */
function list_gpo_templates_git($config)
{
    set_git_proxy($config);
    require_once (dirname(__FILE__) . '/../vendor/autoload.php');
    $git = new CzProject\GitPhp\Git();
    if (! is_dir('/usr/share/sambaedu/gpo/sambaedu-gpo')) {
        $repo = $git->cloneRepository('https://gitlab.sambaedu.org/sambaedu/sambaedu-gpo.git', '/usr/share/sambaedu/gpo/sambaedu-gpo');
    } else {
        $repo = $git->open('/usr/share/sambaedu/gpo/sambaedu-gpo');
    }
    $repo->execute("reset", "--hard", "origin");
    $repo->pull();
    $path = "/usr/share/sambaedu/gpo/sambaedu-gpo";
    $gpos = scandir($path);
    $templates = array();
    foreach ($gpos as $gpo) {
        $match = array();
        if (preg_match("/^(se4_.*(?<!\.zip))$/", $gpo, $match)) {
            if ($info = get_gpo_template_info($match[1])) {
                $templates[] = array(
                    'displayname' => $info['General']['displayName'],
                    'version' => $info['General']['Version']
                );
            }
        }
    }
    usort($templates, 'compare_list_gpo_by_name');
    return $templates;
}

function check_gpo_templates($config)
{
    set_git_proxy($config);
    require_once (dirname(__FILE__) . '/../vendor/autoload.php');
    $git = new CzProject\GitPhp\Git();
    if (! is_dir('/usr/share/sambaedu/gpo/sambaedu-gpo')) {
        $repo = $git->cloneRepository('https://gitlab.sambaedu.org/sambaedu/sambaedu-gpo.git', '/usr/share/sambaedu/gpo/sambaedu-gpo');
    } else {
        $repo = $git->open('/usr/share/sambaedu/gpo/sambaedu-gpo');
    }
    if ($repo->hasChanges()) {
        set_config($config, "gpo_add", "wpkg,impression,redirections,applications,Wallpaper,optimisations,associations", "gpo");
        set_config($config, "gpo_delete", "mozilla,desktopinfo+,logs", "gpo");
        set_config($config, "gpo_once", "acces distant,activation smb3,Bureau,desactivation uac,lecteurs reseau,sysprep,veille_off,Windows update,wsusoffline,redirections,wpkg,WOL,proxy,Wallpaper", "gpo");
    }
}

/**
 * liste les templates de GPO disponibles pour l'import
 * qui contiennent un prefixe etab_ (issues d'un export ...)
 *
 * @return array
 */
function list_gpo_templates_etab()
{
    $path = "/usr/share/sambaedu/gpo/";

    // $dir = opendir($path);
    $gpos = scandir($path);
    $templates = array();
    foreach ($gpos as $gpo) {
        $match = array();
        if (preg_match("/^(etab_.*\.zip)$/", $gpo, $match)) {
            if ($info = get_gpo_template_info($match[1])) {
                $templates[] = array(
                    'displayname' => $info['General']['displayName'],
                    'version' => $info['General']['Version']
                );
            }
        }
    }
    usort($templates, 'compare_list_gpo_by_name');
    return $templates;
}

function gpo_version($version)
{
    // forme lisible par un humain
    $version = explode(".", $version);
    if (isset($version[1])) {
        $user = $version[0];
        $machine = $version[1];
    } else {
        $user = floor($version[0] / 0x10000);
        $machine = $version[0] - $user * 0x10000;
    }
    return [
        $user,
        $machine
    ];
}

/**
 * met à jour une gpo sur l'AD à partir de l'archive $gpo_zip.zip dans /var/www/sambaaedu/gpo/templates.
 * Si la GPO existe déjà sur le serveur AD, incrémente le numéro de version.
 * utilise smbclient pour copier les fichiers.
 *
 * @param array $config
 * @param string $displayname
 *            : nom de la gpo
 * @param string $gpo_zip
 *            : nom de l'archive ($gpo_zip.zip) normalement le même que $displayname
 * @return boolean
 */
function import_gpo(array $config, string $displayname, string $gpo_zip, bool $update = true, bool $force = false, bool $export = false)
{
    $message = [];
    $ret = 0;
    $old_version = 0;

    $gpo = search_ad($config, $displayname, "gpo");
    $local_gpos = read_gpo_json();
    if (isset($local_gpos[$displayname]) && ! is_array($local_gpos[$displayname]) && $local_gpos[$displayname] >= 0) {
        $old_version = $local_gpos[$displayname];
        unset($local_gpos[$displayname]);
    } else {
        $old_version = $local_gpos[$displayname]['version'] ?? 0;
    }
    if (count($gpo) > 0) {
        if ($update || $force) {
            $tmppath = sys_get_temp_dir() . "/" . $displayname;
            $info = get_gpo_template_info($gpo_zip);

            if (gpo_version($info['General']['Version'])[0] > gpo_version($old_version)[0] || gpo_version($info['General']['Version'])[1] > gpo_version($old_version)[1] || $force) {
                $increment = 0;
                if (isset($info['CSE']['gpcuserextensionnames'])) {
                    $increment = 0x10000;
                }
                if (isset($info['CSE']['gpcmachineextensionnames'])) {
                    $increment = $increment + 1;
                }
                $v_u = max([
                    gpo_version($info['General']['Version'])[0],
                    gpo_version($gpo[0]['versionnumber'])[0]
                ]);
                $v_m = max([
                    gpo_version($info['General']['Version'])[1],
                    gpo_version($gpo[0]['versionnumber'])[1]
                ]);
                $version = $v_u * 0x10000 + $v_m + $increment;
                $content = "[General]\r\nVersion=" . $version . "\r\ndisplayName=" . $displayname . "\r\n";
                if (unzip_gpo($gpo_zip, $displayname)) {
                    $handle = fopen($tmppath . "/GPT.INI", "w");
                    fwrite($handle, $content);
                    fclose($handle);
                    specialise_gpo($config, $tmppath, true);
                    if (isset($local_gpos[$displayname]['local'])) {
                        foreach ($local_gpos[$displayname]['local'] as $key => $patch) {
                            $data = read_gpo_sysvol($config, $gpo[0], $patch['type']);
                            if (is_array($data)) {
                                switch ($patch['type']['type']) {
                                    case "gpo":
                                        change_pol_key($data, $key, $patch['data']);
                                        break;
                                }
                                update_gpo_sysvol($config, $gpo[0], $patch['type'], $data);
                            }
                        }
                    }
                    if (sysvol_put($config, $gpo[0], $tmppath, $message)) {
                        $attrs = array();
                        $attrs['versionnumber'] = $version;
                        if (isset($info['CSE']['gpcuserextensionnames']))
                            $attrs['gpcuserextensionnames'] = $info['CSE']['gpcuserextensionnames'];
                        if (isset($info['CSE']['gpcmachineextensionnames']))
                            $attrs['gpcmachineextensionnames'] = $info['CSE']['gpcmachineextensionnames'];
                        if (isset($info['General']['CSE']['gpcfunctionalityversion']))
                            $attrs['CSE']['gpcfunctionalityversion'] = $info['CSE']['gpcfunctionalityversion'];
                        $local_gpos[$displayname]['version'] = $info['General']['Version'];
                        file_put_contents("/etc/sambaedu/applications/gpos.json", json_encode($local_gpos, JSON_PRETTY_PRINT));

                        modify_ad($config, $gpo[0]['cn'], "gpo", $attrs);

                        $ret = true;
                        // mise en place des liens des gpo dans l'arborescence (on ajoute mais on ne supprime pas)
                        $local_ous = gpolistcontainers($config, $gpo[0]['cn']);
                        if (empty($info['links'])) {
                            $info['links']['rdns'][0] = $config['ldap_base_dn'];
                        } else {
                            foreach ($info['links']['rdns'] as $k => $rdn) {
                                $info['links']['rdns'][$k] = $rdn . "," . $config['ldap_base_dn'];
                            }
                        }
                        foreach ($local_ous as $lou) {
                            foreach ($info['links']['rdns'] as $k => $ou) {
                                if (preg_match("/" . $ou . "/i", $lou)) {
                                    unset($info['links']['rdns'][$k]);
                                }
                            }
                        }
                        if (! empty($info['links']['rdns'])) {
                            foreach ($info['links']['rdns'] as $ou) {
                                $ret &= gposetlink($config, $ou, $gpo[0]['cn']);
                            }
                        }
                        // echo "copie des fichiers dans sysvol : " . implode("<br>", $message);
                        return $ret;
                    } else {
                        echo "pb de copie des fichiers dans sysvol : " . implode("<br>", $message);
                        return false;
                    }
                } else {
                    echo "pb : unzip " . $gpo_zip;
                    return false;
                }
            } else {
                return true;
            }
        } else {
            return true;
        }
    } else {
        $msg = "";
        $uuid = gpocreate($config, $displayname, $msg);
        if ($uuid) {
            return import_gpo($config, $displayname, $gpo_zip, true, false, $export);
        } else {
            echo "pb : creation gpo AD : " . $displayname . " " . $msg;
            return false;
        }
    }
}

/**
 * Exporte vers le se4fs une GPO présente dans L'AD sous la forme d'une archive zip
 * en la prefixant d'un superbe "etab_"
 *
 * @param array $config
 * @param string $displayname
 * @return boolean
 */
function export_gpo(array $config, string $displayname, bool $export = false)
{
    $gpo = search_ad($config, $displayname, "gpo");
    if (count($gpo) > 0) {
        $message = array();
        $res = 1;
        $tmppath = sys_get_temp_dir() . "/" . $displayname;
        exec("rm -fr " . escapeshellarg($tmppath), $message, $res);
        exec("mkdir -p " . escapeshellarg($tmppath), $message, $res);
        $command = "cd \"" . $config['domain'] . "/Policies/" . $gpo[0]['cn'] . "\";lcd \"" . $tmppath . "\";prompt OFF;recurse ON;mget *";
        exec('smbclient "//' . ad_url($config, "dns", true) . '/sysvol" --use-kerberos=required -c ' . escapeshellarg($command), $message, $res);
        // $content = "[General]\r\nVersion=" . $gpo[0]['versionnumber'] . "\r\ndisplayName=" . $displayname . "\r\n";
        $version = gpo_version($gpo[0]['versionnumber']);
        $content = "[General]\r\nVersion=" . $version[0] . "." . $version[1] . "\r\ndisplayName=" . $displayname . "\r\n";
        $content .= "[CSE]\r\n";
        if (isset($gpo[0]['gpcuserextensionnames']))
            $content .= "gpcuserextensionnames = \"" . $gpo[0]['gpcuserextensionnames'] . "\"\r\n";
        if (isset($gpo[0]['gpcmachineextensionnames']))
            $content .= "gpcmachineextensionnames = \"" . $gpo[0]['gpcmachineextensionnames'] . "\"\r\n";
        if (isset($gpo[0]['CSE']['gpcfunctionalityversion']))
            $content .= "gpcfunctionalityversion = \"" . $gpo[0]['CSE']['gpcfunctionalityversion'] . "\"\r\n";
        $ous = gpolistcontainers($config, $gpo[0]['cn']);
        $ou_rdn = [];
        foreach ($ous as $ou) {
            $rdn = preg_replace("/,?" . $config['ldap_base_dn'] . "$/", "", $ou);
            if (! empty($rdn)) {
                $ou_rdn[] = "rdns[] = \"" . $rdn . "\"";
            }
        }
        if (count($ou_rdn) > 0) {
            $content .= "[links]\r\n";
            $content .= implode("\r\n", $ou_rdn) . "\r\n";
        }
        $handle = fopen($tmppath . "/GPT.INI", "w");
        fwrite($handle, $content);
        fclose($handle);
        generalise_gpo($config, $tmppath, $export);
        // creation archive zip
        $zip = "/usr/share/sambaedu/gpo/etab_" . $displayname . ".zip";
        exec("rm -fr " . escapeshellarg($zip), $message, $res);
        if (zip_gpo($tmppath, $zip)) {
            exec("rm -fr " . escapeshellarg($tmppath), $message, $res);
            return "etab_" . $displayname . ".zip";
        }
    }
    return false;
}

function delete_gpo(array $config, string $gpo)
{
    $gpo = search_ad($config, $gpo, "gpo");
    if (! empty($gpo[0]['cn'])) {
        return gpodel($config, $gpo[0]['cn']);
    } else {
        return true;
    }
}

function increment_gpo(array $config, string $displayname)
{
    if (! empty($displayname)) {
        $gpo = search_ad($config, $displayname, "gpo");
        if (! empty($gpo[0]['displayname'])) {
            $zip = export_gpo($config, $gpo[0]['displayname']);
            $displayname = $gpo[0]['displayname'];
        } else {
            $zip = $displayname;
        }
        return import_gpo($config, $displayname, $zip);
    }
    return false;
}

function set_proxy_gpo(array $config)
{
    $html = "";
    if (increment_gpo($config, "proxy"))
        $html .= "gpo proxy modifiée<br>\n";
    // if (increment_gpo($config, "mozilla")) {
    // $html .= "gpo mozilla modifiée<br>\n";
    // }
    return $html;
}

// nouvelle gestion des gpo plus générique
/**
 * récupère un fichier dans une gpo
 *
 * @param array $config
 * @param array $gpo
 *            gpo récupéreé dans l'AD
 * @param array $file
 *            type de données à récupérer (voir les defines)
 * @return boolean | mixed $data données extraites du fichier sous forme array ou DOM
 */
function read_gpo_sysvol($config, $gpo, $file, $export = false)
{
    $tmppath = sys_get_temp_dir() . "/" . $gpo['cn'];
    if (! is_dir($tmppath)) {
        mkdir($tmppath);
    }
    $tmp_file = $tmppath . "/" . $file['file'];
    if (! file_exists($tmp_file)) {
        $command = "cd \"" . $config['domain'] . "/Policies/" . $gpo['cn'] . $file['path'] . ";lcd \"" . $tmppath . "\";get " . $file['file'];
        exec('smbclient "//' . ad_url($config, "dns", true) . '/sysvol" --use-kerberos=required -c ' . escapeshellarg($command), $message, $ret);
    }
    if (file_exists($tmp_file)) {
        switch ($file['type']) {
            case "gpo":
                $data = read_pol($tmp_file);
                break;
            case "gpp":
                if (! $export) {
                    $data = new DOMDocument();
                    $data->load($tmp_file);
                } else {
                    $data = file_get_contents($tmp_file);
                }
                break;
            case "ini":
                $data = parse_ini_file($tmp_file);
                break;
            default:
                $data = file_get_contents($tmp_file);
        }
        unlink($tmp_file);
        return $data;
    } else {
        echo "Erreur récupération.";
        return false;
    }
}

/*
 * Fonctions destinées à pousser des fichiers GPO dans sysvol.
 *
 * Par défaut l'appartenance et les acls ne sont pas bonnes suite au put, et donc il faut systématiquement les changer.
 */
function sysvol_acl_reset($config, $path, array &$message = NULL)
{
    return true;
    $ret = 0;
    exec('smbcacls "//' . ad_url($config, "dns", true) . '/sysvol/' . $config['domain'] . '/Policies/' . $path . '" --use-kerberos=required --sddl --set="' . GPO_SDDL . '"', $message, $ret);
    return ($ret == 0);
}

/**
 * pousse l'arborescence gpo dans sysvol
 *
 * @param unknown $config
 * @param unknown $gpo
 * @param unknown $source
 *            chemin local de l'arborescence commencant par uuid ou array si juste un fichier à changer
 * @param array $messages
 *            de sortie
 * @return boolean
 */
function sysvol_put($config, $gpo, $source, array &$message = NULL)
{
    // correction de l'arborescence de base si elle existe
    // sysvol_acl_reset($config, $gpo['cn']);
    // sysvol_acl_reset($config, $gpo['cn'] . "/User");
    // sysvol_acl_reset($config, $gpo['cn'] . "/Machine");
    // sysvol_acl_reset($config, $gpo['cn'] . "/GPT.INI");
    // on ne prend pas en compte ci dessus
    $res = true;
    $ret = 0;
    if (is_array($source) === true) {
        $command = "cd \"" . $config['domain'] . "/Policies/" . $gpo['cn'] . $source['path'] . ";lcd \"" . $source['tmppath'] . "\";mput " . $source['file'];
        $message[] = $command;
        // exec('smbclient "//' . ad_url($config, "dns") . '/sysvol" --use-kerberos=required -c ' . escapeshellarg($command), $message, $ret);
        exec('smbclient "//' . ad_url($config, "dns") . '/sysvol" -U"' . $config['ldap_admin_name'] . '%' . $config['ldap_admin_passwd'] . '" -c ' . escapeshellarg($command), $message, $ret);
        $res &= ($ret == 0);
    } else {
        $command = "cd \"" . $config['domain'] . "/Policies/" . $gpo['cn'] . "\";lcd \"" . $source . "\";prompt OFF;recurse ON;mput *";
        // exec('smbclient "//' . ad_url($config, "dns") . '/sysvol" --use-kerberos=required -c ' . escapeshellarg($command), $message, $ret);
        exec('smbclient "//' . ad_url($config, "dns") . '/sysvol" -U"' . $config['ldap_admin_name'] . '%' . $config['ldap_admin_passwd'] . '" -c ' . escapeshellarg($command), $message, $ret);
        $message[] = $command;
        $res &= ($ret == 0);
    }
    return $res;
}

function sysvol_put_old($config, $gpo, $source, array &$message = NULL)
{
    // correction de l'arborescence de base si elle existe
    // sysvol_acl_reset($config, $gpo['cn']);
    // sysvol_acl_reset($config, $gpo['cn'] . "/User");
    // sysvol_acl_reset($config, $gpo['cn'] . "/Machine");
    // sysvol_acl_reset($config, $gpo['cn'] . "/GPT.INI");
    // on ne prend pas en compte ci dessus
    $res = true;
    $ret = 0;
    if (is_array($source) === true) {
        $command = "cd \"/" . $config['domain'] . "/Policies/" . $gpo['cn'] . $source['path'] . "\";prompt OFF;lcd \"" . $source['tmppath'] . "\";mput \"" . $source['file'] . "\"";
        exec('smbclient "//' . ad_url($config, "dns", true) . '/sysvol" --use-kerberos=required -c ' . escapeshellarg($command), $message, $ret);
        $res &= ($ret == 0); // && sysvol_acl_reset($config, $gpo['cn'] . $source['path'] . "/" . $source['file'], $message);
    } elseif (is_dir($source) === true) {
        // on efface tout, c'est plus simple...
        $command = "cd \"/" . $config['domain'] . "/Policies/" . $gpo['cn'] . "\";prompt OFF;deltree *";
        exec('smbclient "//' . ad_url($config, "dns", true) . '/sysvol" --use-kerberos=required -c ' . escapeshellarg($command), $message, $ret);
        $res = ($ret == 0);

        $files = new RecursiveIteratorIterator(new RecursiveDirectoryIterator($source), RecursiveIteratorIterator::SELF_FIRST);

        foreach ($files as $file) {

            // Ignore "." and ".." folders
            if (in_array(substr($file, strrpos($file, '/') + 1), array(
                '.',
                '..'
            ))) {
                continue;
            }
            $ret = 0;
            $file = realpath($file);
            $relative_path = str_replace($source . '/', '', $file);
            $ldir = dirname($file);
            $dir = dirname($relative_path);
            if (is_dir($file) === true) {
                $command = "prompt OFF;mkdir \"/" . $config['domain'] . "/Policies/" . $gpo['cn'] . "/" . $relative_path . "\"";
                exec('smbclient "//' . ad_url($config, "dns", true) . '/sysvol" --use-kerberos=required -c ' . escapeshellarg($command), $message, $ret);
                $res &= ($ret == 0) && sysvol_acl_reset($config, $gpo['cn'] . "/" . $relative_path, $message);
            } elseif (is_file($file) === true) {
                $command = "lcd \"" . $ldir . "\";prompt OFF;cd  \"/" . $config['domain'] . "/Policies/" . $gpo['cn'] . "/" . $dir . "\";mput \"" . basename($file) . "\"";
                exec('smbclient "//' . ad_url($config, "dns", true) . '/sysvol" --use-kerberos=required -c ' . escapeshellarg($command), $message, $ret);
                $res &= ($ret == 0) && sysvol_acl_reset($config, $gpo['cn'] . "/" . $relative_path, $message);
            }
            // echo $command . " : ".implode("<br>", $message) . "<br>";
            // $message = [];
        }
    }
    return $res;
}

/**
 * met à jour un fichier dans une gpo, temoin pour incrementer si besoin
 *
 * @param array $config
 * @param array $gpo
 *            gpo récupéreé dans l'AD
 * @param array $file
 *            type de données à récupérer (voir les defines)
 * @return boolean | mixed $data données extraites du fichier sous forme array ou DOM
 */
function update_gpo_sysvol($config, &$gpo, $file, $data = NULL, $commit = false)
{
    $message = [];
    $tmppath = sys_get_temp_dir() . "/" . $gpo['cn'];
    if (! is_dir($tmppath)) {
        mkdir($tmppath);
    }
    $file['tmppath'] = $tmppath;
    $tmp_file = $tmppath . "/" . $file['file'];

    if (! empty($data)) {
        switch ($file['type']) {
            case "gpo":
                write_pol($tmp_file, $data);
                break;
            case "gpp":
                $data->save($tmp_file);
                break;
            case "ini":
                file_put_contents($tmp_file, $data);
                break;
            default:
                file_put_contents($tmp_file, $data);
        }
    }
    if ($commit) {
        if (sysvol_put($config, $gpo, $file, $message)) {
            switch ($file['target']) {
                case "user":
                    $gpo['increment_user'] = true;
                    break;
                case "machine":
                    $gpo['increment_machine'] = true;
            }
        } else {
            echo "Erreur écriture. " . implode("<br>", $message);
            return false;
        }
        if (file_exists($tmp_file)) {
            unlink($tmp_file);
            return true;
        } else {
            echo "Erreur mise a jour.";
            return false;
        }
    }
    return file_exists($tmp_file);
}

/**
 * incrémente une gpo locale si besoin
 *
 * @param unknown $config
 * @param array $gpo
 * @return boolean
 */
function increment_gpo_sysvol($config, $gpo, $file)
{
    if (update_gpo_sysvol($config, $gpo, $file, NULL, true)) {
        $version = $gpo['versionnumber'];
        if (@$gpo['increment_user']) {
            $version += 0x10000;
        }
        if (@$gpo['increment_machine']) {
            $version ++;
        }
        $attrs = array();
        $attrs['versionnumber'] = $version;
        $data = "[General]\r
Version=" . $version . "\r
displayName=" . $gpo['displayname'] . "\r
";
        if (update_gpo_sysvol($config, $gpo, GPT_INI, $data, true)) {
            return modify_ad($config, $gpo['cn'], "gpo", $attrs);
        } else {
            return false;
        }
    } else {
        return false;
    }
}
?>