<?php

/** fonctions pour verrouiller un poste lors d'un état transitoire, par exemple lors de la connexion rdp
 * entre le momentoù le poste est choisi et le moment où la session est effectivement ouverte, il ne faut pas
 * que d'autres utilisateurs puissent le choisir.
 *
 * @param unknown $computer
 * @param array $lock
 * @return unknown|boolean
 */
function computer_is_locked($computer, $lock = [])
{
    if (empty($lock)) {
        $lock = apcu_fetch('computer_lock');
    }
    if (isset($lock[$computer])) {
        return $lock[$computer];
    } else {
        return false;
    }
}

function computer_lock($computer, $user, &$lock = [], $store = true)
{
    if (empty($lock)) {
        $lock = apcu_fetch('computer_lock');
    }
    $lock[$computer] = $user;
    if ($store) {
        apcu_store('computer_lock', $lock, 600);
    }
}

function computer_unlock($computer, &$lock = [], $store = true)
{
    if (empty($lock)) {
        $lock = apcu_fetch('computer_lock');
    }
    if (! empty($lock[$computer])) {
        unset($lock[$computer]);
    }
    if ($store) {
        apcu_store('computer_lock', $lock, 600);
    }
}

/**
 * enregistrement des logs de demarrage ou de connexion
 *
 * @param unknown $config
 * @param unknown $user
 * @param unknown $machine
 * @param unknown $os
 * @param unknown $action
 * @param unknown $ip
 */
function log_connexion($config, $user, $machine, $os, $action, $ip = "", $error = 0, $clear = false, $etab = 'localhost')
{
    $db_link = db_connect($config, $etab);
    $machine = strtolower($machine);
    // statut de la machine dans la base sql.
    $status = get_machine_status($config, $machine);

    if ($clear) {
        $error = $status['error'] & ~ $error;
    } else {
        $error = $status['error'] | $error;
    }
    switch ($action) {
        case 'repair_startup':
        case 'startup':

            // on considère qu'au bout de 3 minutes, l'action aurait dû se terminer.
            // le score permet de classer les machines qui bootent vite !
            $score = 300 - (time() - strtotime($status['starttime']));
            if ($score <= 0) {
                $score = 0;
            }
            $error &= ~ SAMBAEDU_NO_WOL;
            switch ($status['state']) {
                case "off":
                    // nouveau boot
                    $query = mysqli_prepare($db_link, "insert into `machines` (name, os, starttime, error) values (?, ?, now(), 0)");
                    mysqli_stmt_bind_param($query, "ss", $machine, $os);
                    break;
                case "wol":
                    $query = mysqli_prepare($db_link, "update `machines` set wol=?, os=?, error=?, starttime=now()  where name=? and stoptime is null");
                    mysqli_stmt_bind_param($query, "isis", $score, $os, $error, $machine);
                    break;
                case "ipxe":
                    $query = mysqli_prepare($db_link, "update `machines` set ipxe=?, starttime=now(), os=?, error=? where name=? and stoptime is null");
                    mysqli_stmt_bind_param($query, "isis", $score, $os, $error, $machine);
                    break;
                default:
                    // if ($status['state'] != $os) {
                    if ($clear) {
                        $query = mysqli_prepare($db_link, "update `machines` set error=? where name=? and stoptime is null");
                        mysqli_stmt_bind_param($query, "is", $error, $machine);
                    } else {
                        $query = mysqli_prepare($db_link, "update `machines` set starttime=now(), os=?, error=? where name=? and stoptime is null");
                        mysqli_stmt_bind_param($query, "sis", $os, $error, $machine);
                    }
            }
            mysqli_stmt_execute($query);
            mysqli_stmt_close($query);

            $query = mysqli_prepare($db_link, "update `connexions` set logouttime=now() where netbios_name=? and logouttime is null and (DATE_SUB(now(),INTERVAL 5 MINUTE) >= logintime)");
            mysqli_stmt_bind_param($query, "s", $machine);
            mysqli_stmt_execute($query);
            mysqli_stmt_close($query);

            // computer_unlock($machine);
            break;
        case 'shutdown':
            if (($error & (SAMBAEDU_NO_SHUTDOWN | SAMBAEDU_NO_SHUTDOWN_LOG | SAMBAEDU_NO_WOL | SAMBAEDU_NO_IPXE)) || $clear) {
                $query = mysqli_prepare($db_link, "update `machines` set stoptime=now(), error=? where name=? and stoptime is null");
                mysqli_stmt_bind_param($query, "is", $error, $machine);
                mysqli_stmt_execute($query);
                mysqli_stmt_close($query);
                $query = mysqli_prepare($db_link, "update `connexions` set logouttime=now() where ( netbios_name='" . $machine . "' or netbios_name='l-" . $machine . "' ) and logouttime is null");
                mysqli_stmt_execute($query);
                mysqli_stmt_close($query);
            } else {
                $query = mysqli_prepare($db_link, "update `machines` set error=? where name=? and stoptime is null");
                mysqli_stmt_bind_param($query, "is", $error, $machine);
                mysqli_stmt_execute($query);
                mysqli_stmt_close($query);
            }
            break;
        case 'repair_logon':
        case 'logon':
            if (! empty($user) && $user != "root" && $user != "Debian-gdm" && $user != "lightdm") {
                $query = mysqli_prepare($db_link, "update `machines` set os=?, error=? where name=? and stoptime is null");
                mysqli_stmt_bind_param($query, "sis", $os, $error, $machine);
                mysqli_stmt_execute($query);
                mysqli_stmt_close($query);
                if (! $clear) {
                    $query = mysqli_prepare($db_link, "update `connexions` set logouttime=now() where username=? and netbios_name=? and logouttime is null");
                    mysqli_stmt_bind_param($query, "ss", $user, $machine);
                    mysqli_stmt_execute($query);
                    mysqli_stmt_close($query);

                    $query = mysqli_prepare($db_link, "insert into `connexions` (username, netbios_name, ip_address, logintime) values (?, ?, ?, now())");
                    mysqli_stmt_bind_param($query, "sss", $user, $machine, $ip);
                    mysqli_stmt_execute($query);
                    mysqli_stmt_close($query);
                }
            }
            computer_unlock($machine);
            break;
        case 'logoff':
            if (! empty($user) && $user != "root" && $user != "Debian-gdm" && $user != "lightdm") {
                $query = mysqli_prepare($db_link, "update `machines` set error=? where name=? and stoptime is null");
                mysqli_stmt_bind_param($query, "is", $error, $machine);
                mysqli_stmt_execute($query);
                mysqli_stmt_close($query);
                if (! $clear) {
                    $query = mysqli_prepare($db_link, "update `connexions` set logouttime=now() where username=? and netbios_name=? and logouttime is null");
                    mysqli_stmt_bind_param($query, "ss", $user, $machine);
                    mysqli_stmt_execute($query);
                    mysqli_stmt_close($query);
                }
            }
            break;
        case 'wpkg':
            if (($error & (SAMBAEDU_WPKG_ERROR | SAMBAEDU_WPKG_RUNNING)) || $clear) {
                $query = mysqli_prepare($db_link, "update `machines` set error=? where name=? and stoptime is null");
                mysqli_stmt_bind_param($query, "is", $error, $machine);
                mysqli_stmt_execute($query);
                mysqli_stmt_close($query);
            }
            break;
        case 'blacklist':
            $query = mysqli_prepare($db_link, "update `machines` set wol=0, ipxe=0, stoptime=now(), os=?, error=?  where name=? and stoptime is null");
            mysqli_stmt_bind_param($query, "ss", $os, $machine, $error);
            mysqli_stmt_execute($query);
            mysqli_stmt_close($query);
            break;
    }
    mysqli_close($db_link);
    get_machine_status($config, $machine, true);
}

function get_machine_status($config, $machine, $refresh = false, $etab = 'localhost')
{
    $db_link = db_connect($config, $etab);
    $machine = strtolower($machine);
    if ($refresh || ! $tab = apcu_fetch("machine_state_" . $machine)) {
        // $query = mysqli_prepare($db_link, "select starttime, os from `machines` where name = ? and stoptime=0 order by id desc limit 1");
        $query = mysqli_prepare($db_link, "select starttime, stoptime, wol, ipxe, os, error from `machines` where name = ? order by id desc limit 1");
        mysqli_stmt_bind_param($query, "s", $machine);
        mysqli_stmt_execute($query);
        mysqli_stmt_bind_result($query, $res_starttime, $res_stoptime, $res_wol, $res_ipxe, $res_os, $res_error);
        mysqli_stmt_store_result($query);
        $num_rows = mysqli_stmt_num_rows($query);
        $tab = array();
        $tab['state'] = "off";
        $tab['error'] = 0;
        $tab['starttime'] = 0;
        $tab['score'] = 0;

        if ($num_rows > 0) {
            while (mysqli_stmt_fetch($query)) {
                $tab['state'] = $res_os;
                $tab['error'] = $res_error;
                $tab['score'] = $res_wol + $res_ipxe;
                if ($res_stoptime == "0000-00-00 00:00:00") {
                    $tab['starttime'] = $res_starttime;
                    $query2 = mysqli_prepare($db_link, "select username, logintime from `connexions` where netbios_name = ? and logouttime is null order by logintime desc");
                    mysqli_stmt_bind_param($query2, "s", $machine);
                    mysqli_stmt_execute($query2);
                    mysqli_stmt_bind_result($query2, $res_username, $res_logintime);
                    mysqli_stmt_store_result($query2);
                    $num_rows2 = mysqli_stmt_num_rows($query2);
                    if ($num_rows2 > 0) {
                        while (mysqli_stmt_fetch($query2)) {
                            $tab['user'][] = array(
                                'name' => $res_username,
                                'logintime' => $res_logintime
                            );
                        }
                    }
                    mysqli_stmt_close($query2);
                } else {
                    $tab['state'] = "off";
                    $tab['starttime'] = 0;
                }
            }
        }
        mysqli_stmt_close($query);
        mysqli_close($db_link);
        $tab['locked'] = computer_is_locked($machine);
        apcu_store("machine_state_" . $machine, $tab);
    }

    return $tab;
}

function get_user_status($config, $user, $etab = 'localhost')
{
    $db_link = db_connect($config, $etab);
    $query = mysqli_prepare($db_link, "select netbios_name, logintime from `connexions` where username=? and logouttime is null order by logintime desc");
    mysqli_stmt_bind_param($query, "s", $user);
    mysqli_stmt_execute($query);
    mysqli_stmt_bind_result($query, $res_machine, $res_logintime);
    mysqli_stmt_store_result($query);
    $num_rows = mysqli_stmt_num_rows($query);
    $tab = array();
    if ($num_rows > 0) {
        while (mysqli_stmt_fetch($query)) {
            $tab[] = array(
                'machine' => strtolower($res_machine),
                'logintime' => $res_logintime
            );
        }
    }
    mysqli_stmt_close($query);
    mysqli_close($db_link);
    return $tab;
}

function list_user_connexions($config, $user, $start = 0, $nb = 10, $etab = 'localhost')
{
    $db_link = db_connect($config, $etab);
    $query = mysqli_prepare($db_link, "select netbios_name, logintime, logouttime from `connexions` where username=? order by id desc limit " . $start . "," . $nb);
    mysqli_stmt_bind_param($query, "s", $user);
    mysqli_stmt_execute($query);
    mysqli_stmt_bind_result($query, $res_machine, $res_logintime, $res_logouttime);
    mysqli_stmt_store_result($query);
    $num_rows = mysqli_stmt_num_rows($query);
    $tab = array();
    if ($num_rows > 0) {
        while (mysqli_stmt_fetch($query)) {
            $tab[] = array(
                'machine' => strtolower($res_machine),
                'logintime' => $res_logintime,
                'logouttime' => $res_logouttime
            );
        }
    }
    mysqli_stmt_close($query);
    mysqli_close($db_link);
    return $tab;
}

function list_machine_connexions($config, $machine, $start = 0, $nb = 10, $etab = 'localhost')
{
    $db_link = db_connect($config, $etab);
    $machine = strtolower($machine);

    $query = mysqli_prepare($db_link, "select username, logintime, logouttime from `connexions` where netbios_name=? order by id desc limit " . $start . "," . $nb);
    mysqli_stmt_bind_param($query, "s", $machine);
    mysqli_stmt_execute($query);
    mysqli_stmt_bind_result($query, $res_user, $res_logintime, $res_logouttime);
    mysqli_stmt_store_result($query);
    $num_rows = mysqli_stmt_num_rows($query);
    $tab = array();
    if ($num_rows > 0) {
        while (mysqli_stmt_fetch($query)) {
            $tab[] = array(
                'user' => $res_user,
                'logintime' => $res_logintime,
                'logouttime' => $res_logouttime
            );
        }
    }
    mysqli_stmt_close($query);
    mysqli_close($db_link);
    return $tab;
}

function list_machine_startups($config, $machine, $start = 0, $nb = 10, $etab = 'localhost')
{
    $db_link = db_connect($config, $etab);
    $machine = strtolower($machine);
    $query = mysqli_prepare($db_link, "select os, starttime, stoptime, error from `machines` where name=? order by starttime desc limit " . $start . "," . $nb);
    mysqli_stmt_bind_param($query, "s", $machine);
    mysqli_stmt_execute($query);
    mysqli_stmt_bind_result($query, $res_os, $res_starttime, $res_stoptime, $res_error);
    mysqli_stmt_store_result($query);
    $num_rows = mysqli_stmt_num_rows($query);
    $tab = array();
    if ($num_rows > 0) {
        while (mysqli_stmt_fetch($query)) {
            $tab[] = array(
                'os' => $res_os,
                'starttime' => $res_starttime,
                'stoptime' => $res_stoptime,
                'error' => $res_error
            );
        }
    }
    mysqli_stmt_close($query);
    mysqli_close($db_link);
    return $tab;
}

function diagnostic_machine($config, $machine, $etab = 'localhost')
{
    $db_link = db_connect($config, $etab);
    $machine = strtolower($machine);

     $query = mysqli_prepare($db_link, "select os, ipxe, wol, starttime, stoptime, error from `machines` where name=? and error>0 order by starttime desc limit 5");
    mysqli_stmt_bind_param($query, "s", $machine);
    mysqli_stmt_execute($query);
    mysqli_stmt_bind_result($query, $res_os, $res_starttime, $res_stoptime, $res_error);
    mysqli_stmt_store_result($query);
    $num_rows = mysqli_stmt_num_rows($query);
    $tab = array();
    if ($num_rows > 0) {
        while (mysqli_stmt_fetch($query)) {
            $tab[] = array(
                'os' => $res_os,
                'starttime' => $res_starttime,
                'stoptime' => $res_stoptime,
                'error' => $res_error
            );
        }
    }
    mysqli_stmt_close($query);
    mysqli_close($db_link);
    return $tab;
}

function open_sessions($config, $etab = 'localhost')
{
    $db_link = db_connect($config, $etab);
    $query = mysqli_prepare($db_link, "select username, netbios_name, logintime, ip_address from `connexions` where logouttime is null order by username");
    mysqli_stmt_execute($query);
    mysqli_stmt_bind_result($query, $res_username, $res_machine, $res_logintime, $res_ip);
    mysqli_stmt_store_result($query);
    $num_rows = mysqli_stmt_num_rows($query);
    $tab = array();
    if ($num_rows > 0) {
        while (mysqli_stmt_fetch($query)) {
            $tab[] = array(
                'machine' => strtolower($res_machine),
                'user' => $res_username,
                'logintime' => $res_logintime,
                'ip' => $res_ip
            );
        }
    }
    mysqli_stmt_close($query);
    mysqli_close($db_link);
    return $tab;
}

function clean_machines_status($config)
{
    $out = "";
    if (apcu_add("clean_status_lock", true, 1000)) {
        $time = time();
        // $machines = search_ad($config, "*", "computer", $config['dn']['computers']);
        $machines = search_ad($config, "*", "machine", $config['dn']['computers']);

        $keys = array_column($machines, "cn");
        // on cherche les machines demarrées win ou linux depuis plus de 9 minutes (temps entre 2 cron)
        $db_link = db_connect($config);
        // $query = mysqli_prepare($db_link, "select name, wol, ipxe, os, error from `machines` where stoptime=0 and
        // (DATE_SUB(now(),INTERVAL 9 MINUTE) >= starttime) order by id desc");
        $query = mysqli_prepare($db_link, "select name, wol, ipxe, os, error, starttime from `machines` where stoptime is null order by id desc");

        // (os='windows' or os='linux' or DATE_SUB(now(),INTERVAL 2 MINUTE) >= starttime) order by id desc");
        mysqli_stmt_execute($query);
        mysqli_stmt_bind_result($query, $res_name, $res_wol, $res_ipxe, $res_os, $res_error, $res_starttime);
        mysqli_stmt_store_result($query);
        $num_rows = mysqli_stmt_num_rows($query);
        $tab = array();
        // $out = count($machines) . " machines à tester, " . $num_rows . " déclarées allumées, ";
        if ($num_rows > 0) {
            // $out = "nettoyage des connexions des machines plantees :<br>\n";
            while (mysqli_stmt_fetch($query)) {
                $ip = "";
                if ($key = array_search($res_name, $keys)) {
                    $ip = $machines[$key]['iphostnumber'] ?? "";
                    unset($machines[$key]);
                }
                if (time() > strtotime($res_starttime) + 540) {
                    if (! $ping_os = fping($res_name, 0.05)) {
                        // pas de ping, on ferme les cnx
                        if ($res_os == "wol") {
                            $res_error |= SAMBAEDU_NO_WOL;
                            // $out .= strtolower($res_name) . " n'a pas été allumée, soit le WOL ne fonctionne pas, soit la machine est débranchée ou n'existe plus<br>\n";
                            $res_wol = 0;
                        } elseif ($res_os == "ipxe") {
                            $res_error |= SAMBAEDU_NO_IPXE;
                            // $out .= strtolower($res_name) . " a booté en ipxe, mais n'a pas démarré correctement de système d'exploitation ensuite<br>\n";
                            $res_ipxe = 0;
                        } else {
                            $res_error |= SAMBAEDU_NO_SHUTDOWN_LOG;
                            // $out .= "Détection de l'extinction de " . strtolower($res_name) . " os : " . $res_os . " : $res_error : Soit la gpo logs ne s'applique pas, soit la machine a été arrêtée brutalement.<br>\n";
                            $res_os = "off";
                        }
                        log_connexion($config, "", $res_name, "off", "shutdown", $ip, $res_error);
                        // $res_error ++;
                        // $query2 = mysqli_prepare($db_link, "update `machines` set wol=?, ipxe=?, os=?, error=?, stoptime=now() where name='" . $res_name . "' and stoptime=0");
                        // mysqli_stmt_bind_param($query2, "iisi", $res_wol, $res_ipxe, $res_os, $res_error);
                        // mysqli_stmt_execute($query2);
                        // mysqli_stmt_close($query2);
                        // $query2 = mysqli_prepare($db_link, "update `connexions` set logouttime=now() where ( netbios_name='" . $res_name . "' or netbios_name='l-" . $res_name . "' ) and logouttime=0");
                        // mysqli_stmt_execute($query2);
                        // mysqli_stmt_close($query2);
                        // $query2 = mysqli_prepare($db_link, "update `connexions` set logouttime=now() where netbios_name='l-" . $res_name . "' and logouttime=0");
                        // mysqli_stmt_execute($query2);
                        // mysqli_stmt_close($query2);
                    } else {
                        if ($res_os != $ping_os) {
                            $res_error |= SAMBAEDU_NO_WOL_LOG;
                            // $out .= "Etat d'allumage actualisé pour " . strtolower($res_name) . " : la gpo logs ou les scripts de démarrage Linux ne s'appliquent pas<br>\n";
                            // if ($ping_os == "windows" && $res_error > 4) {
                            // $out .= "tentative de réparation des gpo pour " . strtolower($res_name) . "...<br>";
                            // $out .= exec("smbclient -k //" . $res_name . "/c\$ -c " . escapeshellarg("deltree \\windows\\system32\\grouppolicy;deltree \\windows\\system32\\grouppolicyusers") . " 2>/dev/null");
                            // $out .= "<br>";
                            log_connexion($config, "", $res_name, $ping_os, "startup", $ip, $res_error);
                            // } else {
                            // log_connexion($config, "", $res_name, $ping_os, "repair_startup");
                            // }
                        }
                        $smb_status = machine_smbstatus($config, $res_name, $ping_os);
                        if (! empty($smb_status)) {
                            $sql_status = get_machine_status($config, $res_name);
                            if ($smb_status != "wpkg") {
                                if (! isset($sql_status['user']) || ! is_array($sql_status['user']) || ! in_array($smb_status, array_column($sql_status['user'], 'name'))) {
                                    $res_error |= SAMBAEDU_NO_LOGON_LOG;
                                    log_connexion($config, $smb_status, $res_name, $ping_os, "logon", $ip, $res_error);
                                    // $out .= "connexion actualisée pour " . strtolower($res_name) . " utilisateur " . $smb_status . "<br>\n";
                                }
                            } else {
                                log_connexion($config, "", $res_name, $ping_os, "wpkg", $ip, $res_error | SAMBAEDU_WPKG_RUNNING);
                            }
                        }
                    }
                }
            }
        }
        mysqli_stmt_close($query);
        mysqli_close($db_link);
        // on teste les machines considérées éteintes
        // $out .= count($machines) . " restantes à tester";
        foreach ($machines as $machine) {
            // $out .= $machine['cn'] . " : " . $machine['state'] . "\n";
            $ip = $machine['iphostnumber'] ?? "";
            $os = fping($machine['cn'], 0.05);
            /*
             * if (! $os && $os = fping($ip, 0.05) && gethostbyaddr($ip) == $machine['cn']) {
             * $out .= $machine['cn'] . " : Enregistrement DNS A " . $ip . " faux\n";
             * dns_delete($config, $machine, true);
             * dns_add($config, $machine);
             * }
             */
            if ($os) {
                // $out .= $machine['cn'] . " : ";
                $smb_status = machine_smbstatus($config, $machine['cn'], $os);
                if (! empty($smb_status) && $smb_status != "wpkg") {
                    // utilisateur connecté
                    log_connexion($config, $smb_status, $machine['cn'], $os, "logon", $ip, SAMBAEDU_NO_LOGON_LOG);
                    // $out .= "connexion actualisée pour l'utilisateur " . $smb_status . "<br>\n";
                } else {
                    // machine allumée sans connexion
                    log_connexion($config, "", $machine['cn'], $os, "startup", $ip, SAMBAEDU_NO_WOL_LOG);
                    // $out .= "déjà allumée<br>\n";
                }
            }
        }
        $temps = time() - $time;
        // $out .= "...test en " . $temps . " secondes<br>\n";
        unlock("clean_status");
    }
    return $out;
}

function error_message($error)
{
    $ret = [];
    $messages = [
        SAMBAEDU_NO_WOL => "La machine n&lsquo;a pas pu être allumée à distance, soit le WOL ne fonctionne pas, soit la machine est débranchée ou n&lsquo;existe plus.",
        SAMBAEDU_NO_IPXE => "La machine a booté en ipxe, mais n&lsquo;a pas démarré correctement de système d&lsquo;exploitation ensuite.",
        SAMBAEDU_NO_WOL_LOG => "Le système d&lsquo;exploitation n&lsquo;a pas signalé son démarrage, les GPO ou les scripts de démarrage ne se lancent pas",
        SAMBAEDU_NO_SHUTDOWN_LOG => "La machine n&lsquo;a pas signalé son arrêt, les GPO ou scripts d&lsquo;arrêt ne se lancent pas.",
        SAMBAEDU_SHUTDOWN_ERROR => "La machine ne répond pas à la demande d&lsquo;arrêt.",
        SAMBAEDU_NO_SHUTDOWN => "La machine s&lsquo;est arrêtée brutalement ou a perdu sa connexion réseau",
        SAMBAEDU_NO_LOGON_LOG => "La machine n&lsquo;a pas signalé l&lsquo;ouverture de session",
        SAMBAEDU_NO_LOGOFF_LOG => "La machine n&lsquo;a pas signalé la fermeture de session",
        SAMBAEDU_STARTUP_APP_ERROR => "Les scripts de démarrage ne se sont pas terminés correctemnt",
        SAMBAEDU_SHUTDOWN_APP_ERROR => "Les scripts d&lsquo;extinction ne se sont pas terminés correctement",
        SAMBAEDU_LOGON_APP_ERROR => "Les scripts d&lsquo;ouverture de session ne se sont pas terminés correctement",
        SAMBAEDU_LOGOFF_APP_ERROR => "Les scripts de fermeture de session ne se sont pas terminés correctement",
        SAMBAEDU_LOGON_SYS_APP_ERROR => "Les scripts système d&lsquo;ouverture de session ne se sont pas terminés correctement",
        SAMBAEDU_LOGOFF_SYS_APP_ERROR => "Les scripts système de fermeture de session ne se sont pas terminés correctement",
        SAMBAEDU_WPKG_RUNNING => "Wpkg est en cours d&lsquo;exécution",
        SAMBAEDU_WPKG_ERROR => "Au moins une erreur wpkg s&lsquo;est produite",
        SAMBAEDU_CLOUD_ERROR => "Au moins un dossier cloud n&lsquo;a pas été monté",
        SAMBAEDU_SLOW_NET_ERROR => "Connexion réseau lente"
    ];
    foreach ($messages as $key => $message) {
        if ($key & $error) {
            $ret[] = $message;
        }
    }
    return $ret;
}
?>