<?php
use GuzzleHttp\Exception\GuzzleException;

/**
 * statut complet des machines d'un parc
 * doit être appelé depuis le serveur local
 * le parc est a definir avec etab_suffix($uai)
 *
 * @param unknown $config
 * @param unknown $name
 * @return array
 */
function parc_local_status($config, $name)
{
    $parc = list_members_parc($config, $name, true);
    $wpkg = wpkg_parc_info($config, $name);
    foreach ($parc as $machine) {
        $machine['objectguid'] = to_guid($machine['objectguid']);
        // foreach ($machine['memberof'] as $kk => $g) {
        // $machine['memberof'][$kk] = ldap_dn2cn($g);
        // }
        $res[] = array_merge($machine, get_machine_status($config, $machine['cn']), [
            'wpkg' => $wpkg[$machine['cn']]
        ]);
    }
    return $res;
}

/**
 * statut complet des machines d'un parc
 * doit être appelé depuis le serveur central.
 * le parc est donc à définir dans le contexte local (avec etab_suffix($uai))
 *
 * @param unknown $config
 * @param unknown $name
 * @param string $etab
 * @return array
 */
function parc_status($config, $name, $etab = "")
{
    if (empty($etab)) {
        return parc_local_status($config, $name);
    } else {
        require_once (dirname(__FILE__) . '/../vendor/autoload.php');
        $opt = [
            'base_uri' => $config['se4_url'] . "/",
            'cookies' => true
        ];
        curl_proxy_options($config, $opt);
        $client = new GuzzleHttp\Client($opt);

        if (! empty($etab)) {
            $path = $etab . "/";
        } else {
            $path = "/";
        }
        try {
            $response = $client->get($path . 'api2/status/parc', [
                'query' => [
                    'name' => $name
                ]
            ]);
            $res = json_decode((string) $response->getBody(), true);
        } catch (GuzzleException $e) {
            // this is delivered each failed request
            echo "erreur : " . $e->getMessage();
            // echo "<pre>" . print_r($list, true) . "</pre>";
            $res = false;
        }
        return $res;
    }
}

function action_parc_html($config, &$html)
{
    $etab = $_SESSION['etab'] ?? $config['etab_ou'] ?? "";
    $html .= "<script type=\"text/javascript\" src=\"/elements/js/wz_tooltip_new.js\"></script>";
    // Verifie les droits
    if (have_right_or_delegation($config, SE_COMPUTER_VIEW)) {

        $html .= "<h1>Actions sur les stations</h1>";

        /* * *********************** Declaration des variables *********************************** */
        $action = $_POST['action'] ?? $_GET['action'] ?? "";
        $parc = isset($_POST['parc']) ? $_POST['parc'] : (isset($_GET['parc']) ? $_GET['parc'] : NULL);
        // $filtrecomp = $user['cn'] ?? $_GET['filtrecomp'] ?? "";
        $action_poste = $_POST['action_poste'] ?? $_GET['action_poste'] ?? "";
        if ($action_poste == "check") {
            $refresh = 1;
        } else {
            $refresh = 0;
        }
        $nomjour = date("l");
        if ($action == "") {
            $action = "detail";
        }
        if ($action == "choix_time") {
            $action = "detail";
        }

        switch ($action) {

            case "detail":

                $list_parcs = search_parcs($config, "*");
                $delegations = list_delegations($config);
                if (count($list_parcs) > 0) {
                    $html .= "<h3> Sélectionner un parc :</h3>";
                    $html .= "<CENTER>";
                    $html .= "<FORM id=\"select\" method=\"post\" action=\"action_parc.php\">\n";
                    $html .= "<SELECT NAME=\"parc\" SIZE=\"1\" onchange=submit()>";
                    $html .= "<option value=\"SELECTIONNER\">Sélectionner</option>";
                    $nb_etab = count(activated_etabs($config));
                    foreach ($list_parcs as $value_tmp) {
                        if (! empty($value_tmp["samaccountname"]) && (empty($delegations) || in_array($value_tmp["gdn"], array_column($delegations, "parcdn")))) {
                            $html .= "<option value=\"" . $value_tmp["samaccountname"] . "\"";
                            if ($parc == $value_tmp["samaccountname"]) {
                                $html .= " selected";
                                $etab = $value_tmp["etab"];
                            }
                            $nom_etab = "";
                            if ($nb_etab > 1) { // && $etab != $value_tmp["etab"]) {
                                $nom_etab = " " . etab_to_name($config, $value_tmp["etab"]);
                            }
                            $html .= ">" . $value_tmp["name"] . " " . " de type " . $value_tmp["type"] . $nom_etab . "\n";
                            $html .= "</option>";
                        }
                    }
                    // }

                    $html .= "</SELECT>&nbsp;&nbsp;\n";
                    $html .= "<img onmouseout=\"UnTip();\" onmouseover=\"Tip('Sélectionner un parc',WIDTH,250,SHADOW,true,DURATION,5000);\" name=\"action_image2\"  src=\"../elements/images/system-help.png\" alt=\"help\"> ";
                    // $html .= "<input type=\"submit\" value=\"Valider\">\n";
                    $html .= "</FORM>\n";
                    $html .= "</CENTER>\n";
                    $html .= "<br>";
                } else {
                    $html .= "<br><center>";
                    $html .= "Il n'existe pas encore de parc";
                    $html .= "</center>\n";
                }

                if (($parc != "") && ($parc != "SELECTIONNER")) {
                    // smbstatus();
                    $html .= "<CENTER><table align=center><tr>\n";
                    $html .= "<td colspan=3>";
                    $url = create_remote_token($config, search_machine($config, "w112" . etab_suffix($etab)), "master", $config['login']);
                    $html .= "<a href=\"" . $url . "\" onclick=\"openInNewWindow(event, this)\"><img src=\"../elements/images/rdp-small.png\" alt=\"Accès distant...\">Contrôle a distance du parc</a>";
                    $html .= "</td></tr><tr>\n";
                    
                    $html .= "<td>";
                    $html .= "<form id=\"stop\" action=\"wolstop_station.php\" method=\"post\">\n";
                    $html .= "<input type=\"hidden\" name=\"parc\" value=\"$parc\" />";
                    $html .= "<input type=\"hidden\" name=\"action\" value=\"stop\" />";
                    $html .= "<input type=\"submit\" value=\"Eteindre tous les postes\" onclick=\"if (window.confirm('Etes-vous sur de vouloir éteindre le parc ?')) {return true;} else {return false;}\" />";
                    $html .= "</form>\n";
                    $html .= "</td>";

                    $html .= "<td>";
                    $html .= "<form id=\"start\" action=\"wolstop_station.php\" method=\"post\">\n";
                    $html .= "<input type=\"hidden\" name=\"action_poste\" value=\"wol\" />";
                    $html .= "<input type=\"hidden\" name=\"parc\" value=\"$parc\" />";
                    $html .= "<input type=\"hidden\" name=\"action\" value=\"start\" />";
                    $html .= "<input type=\"submit\" value=\"Allumer tous les postes\" />";
                    $html .= "</form>\n";
                    $html .= "</td>";

                    // ===================================
                    $html .= "<td>";
                    $html .= "<form id=\"reboot\" action=\"wolstop_station.php\" method=\"post\">\n";
                    $html .= "<input type=\"hidden\" name=\"action_poste\" value=\"reboot\" />";
                    $html .= "<input type=\"hidden\" name=\"parc\" value=\"$parc\" />";
                    $html .= "<input type=\"hidden\" name=\"action\" value=\"reboot\" />";
                    $html .= "<input type=\"submit\" value=\"Rebooter tous les postes\" onclick=\"if (window.confirm('Etes-vous sur de vouloir rebooter le parc ?')) {return true;} else {return false;}\" />";
                    $html .= "</form>\n";
                    $html .= "</td>";
                    // ===================================
                    // Test le niveau de delegation
                    // manage ou view
                    // Si manage on peut aller sur programmer
                    $niveau_delegation = type_delegation($config, $parc);
                    if ($niveau_delegation & ~ SE_COMPUTER_VIEW) {
                        $html .= "<td>";
                        $html .= "<form id=\"program\" action=\"wolstop_station.php\" method=\"post\">\n";
                        $html .= "<input type=\"hidden\" name=\"programmation\" value=\"check\" />";
                        $html .= "<input type=\"hidden\" name=\"parc\" value=\"$parc\" />";
                        $html .= "<input type=\"hidden\" name=\"action\" value=\"timing\" />";
                        $html .= "<input type=\"submit\" value=\"Programmer l'allumage\" />";
                        $html .= "</form>\n";
                        $html .= "</td></tr><tr>\n";
                        $html .= "<td>";
                        $html .= "<form id=\"install_l\" action=\"lance_action.php\" method=\"post\">\n";
                        $html .= "<input type=\"hidden\" name=\"parc\" value=\"$parc\" />";
                        $html .= "<input type=\"hidden\" name=\"type\" value=\"install\" />";
                        $html .= "<input type=\"submit\" value=\"Installer Linux/Windows\" />";
                        $html .= "</form>\n";
                        $html .= "</td>\n";
                        $html .= "<td><form id=\"renomme\" action=\"lance_action.php\" method=\"post\">\n";
                        $html .= "<input type=\"hidden\" name=\"parc\" value=\"$parc\" />";
                        $html .= "<input type=\"hidden\" name=\"type\" value=\"renomme\" />";
                        $html .= "<input type=\"submit\" value=\"Renommer les postes\" />";
                        $html .= "</form>\n";
                        $html .= "</td>\n";
                        $html .= "<td>";
                        $html .= "<form id=\"wpkg\" action=\"lance_action.php\" method=\"post\">\n";
                        $html .= "<input type=\"hidden\" name=\"parc\" value=\"$parc\" />";
                        $html .= "<input type=\"hidden\" name=\"type\" value=\"wpkg\" />";
                        $html .= "<input type=\"submit\" value=\"Lancer Wpkg en mode interactif\" />";
                        $html .= "</form>\n";
                        $html .= "</td>\n";
                        $html .= "<td>";
                        $html .= "<form id=\"clone\" action=\"lance_action.php\" method=\"post\">\n";
                        $html .= "<input type=\"hidden\" name=\"parc\" value=\"$parc\" />";
                        $html .= "<input type=\"hidden\" name=\"type\" value=\"clonage\" />";
                        $html .= "<input type=\"hidden\" name=\"action\" value=\"new\" />";
                        $html .= "<input type=\"submit\" value=\"Cloner\" />";
                        $html .= "</form></td>\n";
                    }

                    if (count(list_members_parc($config, "serveurs_ltsp"), false) > 0) {
                        $html .= "<td>";
                        $html .= "<form id=\"ltsp\" action=\"lance_action.php\" method=\"post\">\n";
                        $html .= "<input type=\"hidden\" name=\"parc\" value=\"$parc\" />";
                        $html .= "<input type=\"hidden\" name=\"type\" value=\"ltsp\" />";
                        $html .= "<input type=\"submit\" value=\"Demarrer les postes en client LTSP\" />";
                        $html .= "</form>";
                        $html .= "</td></tr>\n";
                    }

                    $html .= "<tr><td colspan=\"2\" align=\"center\">";
                    $html .= "<form id=\"show\" action=\"show_parc.php\" method=\"post\">\n";
                    $html .= "<input type=\"hidden\" name=\"parc\" value=\"$parc\" />";
                    $html .= "<input type=\"submit\" value=\"Lister le parc\" />";
                    $html .= "</form>\n";
                    $html .= "</td>\n";
                    // $html .= "<td colspan=\"3\" align=\"center\"><form action=\"action_parc.php\" method=\"post\">\n";
                    $html .= "<td colspan=\"2\" align=\"center\"><form id=\"action\" action=\"action_parc.php\" method=\"post\">\n";
                    $html .= "<input type=\"hidden\" name=\"action_poste\" value=\"check\" />";
                    $html .= "<input type=\"hidden\" name=\"parc\" value=\"$parc\" />";
                    $html .= "<input type=\"hidden\" name=\"action\" value=\"detail\" />";
                    $html .= "<input type=\"submit\" value=\"Rafraîchir la page\" />";
                    $html .= "</form>\n";

                    $html .= "</td></tr></table>\n";

                    $html .= "</center>\n";

                    // require_once ("printers.inc.php");

                    $html .= "\n<br>\n<CENTER>\n";
                    $html .= "<table class='table-bordered' width=\"80%\">\n<tr class=menuheader style=\"height: 30\">\n";
                    $html .= "<td align=\"center\"></td>\n";
                    $html .= "<td align=\"center\">STATIONS CONCERNEES</td>\n";
                    $html .= "<td align=\"center\">ADRESSE IP</td>\n";
                    $html .= "<td align=\"center\">ETAT</td>\n";
                    $html .= "<td align=\"center\">CONNEXION</td>\n";
                    $html .= "<td align=\"center\">CONTROLE</td>\n";
                    $html .= "<td align=\"center\">Wpkg</td>\n";
                    $html .= "<td align=\"center\">Erreurs</td></tr>\n";
                    $html .= "</tr>\n";
                    $mp = parc_status($config, $parc, $etab);
                    // $html .= "<pre>" . print_r($mp, true) . "</pre>";
                    $html .= "<script>
                            function openInNewWindow(event, link) {
                                event.preventDefault(); // Empêche l'ouverture par défaut
                                window.open(link.href, '_blank', 'noopener,noreferrer,width=1920,height=1080');
                            }
                            </script>";

                    if (is_array($mp) && count($mp) > 0) {
                        foreach ($mp as $machine) {
                            $mpenc = ldap_hostname($config, $machine);
                            $mp_en_cours = urldecode($mpenc);
                            $resultat = get_os_icon($config, $machine);
                            // élimination des imprimantes de la liste
                            if ($resultat['os'] == "imprimante" || $resultat['os'] == "equipement") {
                                // unset ($mp[$loop]);
                                continue;
                            }
                            $from = urlencode("../parcs/action_parc.php?action=detail&parc=" . $parc);

                            // Affichage du tableau
                            $html .= "<tr>\n";
                            // Affichage de l'icone informatique
                            $html .= "<td align=\"center\">\n";
                            $machine['state'] = $machine['state'] ?? "";
                            $iphost = $machine['iphostnumber'] ?? "";
                            $machine['networkaddress'] = $machine['networkaddress'] ?? "";
                            $systemid = $iphost;
                            $ip_link = "";
                            $html .= "<img style=\"border: 0px solid ;\" src=\"../elements/images/" . $resultat['icone'] . "\" title=\"" . $resultat['os'] . " - " . $iphost . "\" alt=\"" . $resultat['os'] . "\" WIDTH=20 HEIGHT=20 onclick=\"popuprecherche('../ocsreports/machine.php?systemid=$systemid','popuprecherche','scrollbars=yes,width=500,height=550');\">";

                            // $html .= "<img style=\"border: 0px solid ;\" src=\"../elements/images/computer.png\" onclick=\"popuprecherche('../ocsreports/machine.php?systemid=$systemid','popuprecherche','scrollbars=yes,width=500,height=550');\" title=\"Station\" alt=\"Station\"></td>\n";
                            $html .= "<td align=center ><a href=\"cherche_machine.php?mpenc=" . $machine['cn'] . "&from=$from\">$mp_en_cours</a></td>\n";
                            if ($machine['state'] == "reservation") {
                                $ip_link = "ip réservée : " . $iphost;
                            } elseif ($machine['state'] == "active") {
                                $ip_link = "<a href=action_machine.php?retour=action_parc&action=reservation&machine=" . $machine['cn'] . "&mac=" . $machine['networkaddress'] . " onmouseout=\"UnTip();\" onmouseover=\"Tip('Réserver une ip pour ce poste.',WIDTH,250,SHADOW,true,DURATION,5000);\"> ip dynamique : $iphost</a>";
                            } elseif ($machine['state'] == "dns") {
                                $ip_link = "ip dns : " . $iphost;
                            } else {
                                $ip_link = "ip " . $iphost . " : " . $machine['state'];
                            }
                            $html .= "<td align=center>$ip_link</td>\n";
                            $html .= "<td align=center>\n";
                            // $etat
                            // $html .= "<div id='divip$loop'><img src=\"../elements/images/spinner.gif\"></img></div>\n";
                            switch (@$machine['state']) {
                                case "windows":
                                case "linux":
                                    if (isset($machine['cn'])) {
                                        $html .= "<a target=\"main\" href=\"action_machine.php?machine=" . $machine['cn'] . "&action=shutdown&parc=$parc&retour=action_parc\"" . "onmouseout=\"UnTip();\" onmouseover=\"Tip('La machine est actuellement allumée, cliquez pour l\'éteindre',WIDTH,250,SHADOW,true,DURATION,5000);\"" . "onclick=\"if (window.confirm('Etes-vous sur de vouloir éteindre la machine " . $machine['cn'] . " ?')) {return true;} else {return false;}\"/>" . "<img type=\"image\" border=\"0\" title=\"La machine est actuellement allumée, cliquez pour l\'éteindre\" src=\"../elements/images/enabled.png\"></a>\n";
                                    } else {
                                        $html .= "<img type=\"image\" src=\"../elements/images/enabled.png\" border=\"0\" title=\"" . $machine['cn'] . "\" title=\"" . $machine['cn'] . "\" />";
                                    }
                                    break;
                                case "wol":
                                case "ipxe":
                                    if (isset($machine['cn'])) {

                                        $html .= "<a target=\"main\" href=\"action_machine.php?machine=" . $machine['cn'] . "&action=wol&parc=$parc&retour=action_parc\" target='_blank' " . "onmouseout=\"UnTip();\" onmouseover=\"Tip('Etat la machine : " . $machine['state'] . "',WIDTH,250,SHADOW,true,DURATION,5000);\" >" . "<img type=\"image\" border=\"0\" title='Etat la machine : " . $machine['state'] . "' src=\"../elements/images/spinner.gif\">" . "</a>\n";
                                    } else {
                                        $html .= "<img type=\"image\" src=\"../elements/images/spinner.gif\" border=\"0\" title=\"" . $_POST['ip'] . "\" title=\"" . $_POST['ip'] . "\" />";
                                    }
                                    break;
                                default:

                                    if (isset($machine['cn'])) {

                                        $html .= "<a target=\"main\" href=\"action_machine.php?machine=" . $machine['cn'] . "&action=wol&parc=$parc&retour=action_parc\" target='_blank' " . "onmouseout=\"UnTip();\" onmouseover=\"Tip('La machine est actuellement éteinte, cliquez pour l\'allumer',WIDTH,250,SHADOW,true,DURATION,5000);\" >" . "<img type=\"image\" border=\"0\" title=\"La machine est actuellement éteinte, cliquez pour l\'allumer\" src=\"../elements/images/disabled.png\">" . "</a>\n";
                                    } else {
                                        $html .= "<img type=\"image\" src=\"../elements/images/disabled.png\" border=\"0\" title=\"" . $_POST['ip'] . "\" title=\"" . $_POST['ip'] . "\" />";
                                    }
                            }

                            /*
                             * $html .= "<script type='text/javascript'>
                             * // <![CDATA[
                             * new Ajax.Updater($('divip$loop'),'parcs_ajax_lib.php',{method: 'post', parameters: '?ip=$iphost&parc=$parc&nom_machine=" . $machine['cn'] . "&mode=ping_ip' });
                             * //]]>
                             * </script>\n";
                             */
                            $html .= "</td>\n";
                            $html .= "<td align=center>\n";
                            // $etat_session
                            // if (fping($iphost)){
                            // $html .= "<div id='divsession$loop'><img src=\"../elements/images/spinner.gif\"></img></div>\n";
                            $html .= "<a target=\"main\" href=\"show_histo.php?selectionne=2&amp;mpenc=" . $machine['cn'] . "&from=$from\">";
                            if (empty($machine['user'][0]['name'])) {
                                $html .= "<img type=\"image\" src=\"../elements/images/disabled.png\">\n";
                            } elseif ($machine['user'][0]['name'] == $config['se4install_name']) {
                                $texte = $machine['user'][0]['name'] . " est actuellement connecté sur ce poste";
                                $html .= "<img onmouseout=\"UnTip();\" onmouseover=\"Tip('" . $texte . "',WIDTH,250,SHADOW,true,DURATION,5000);\" src=\"../elements/images/travaux2.png\" border=\"0\" />";
                            } elseif ($machine['user'][0]['name'] == "wpkg") {
                                $texte = $machine['user'][0]['name'] . " est actuellement connecté sur ce poste";
                                $html .= "<img onmouseout=\"UnTip();\" onmouseover=\"Tip('" . $texte . "',WIDTH,250,SHADOW,true,DURATION,5000);\" src=\"../elements/images/travaux2.png\" border=\"0\" />";
                            } else {
                                $texte = $machine['user'][0]['name'] . " est actuellement connecté sur ce poste";
                                $html .= "<img onmouseout=\"UnTip();\" onmouseover=\"Tip('" . $texte . "',WIDTH,250,SHADOW,true,DURATION,5000);\" src=\"../elements/images/enabled.png\" border=\"0\" />";
                            }

                            /*
                             * $html .= "<script type='text/javascript'>aigle
                             * // <![CDATA[
                             * new Ajax.Updater($('divsession$loop'),'parcs_ajax_lib.php',{method: 'post', parameters: '?nom_machine=" . $machine['cn'] . "&mode=session'});
                             * //]]>
                             * </script>\n";
                             */
                            /*
                             * } else {
                             * $ret = "<span onmouseout=\"UnTip();\" onmouseover=\"Tip('Echec du ping',WIDTH,250,SHADOW,true,DURATION,5000);\">" . "<img type=\"image\" border=\"0\" title=\"Echec du ping\" src=\"../elements/images/disabled.png\">" . "</span>\n";
                             * $html .= ($ret);
                             * }
                             */
                            $html .= "</a></td>\n";
                            $html .= "<td align=\"center\">";
                            // if (fping($iphost)){
                            //$url = "rdp.php?machine=" . $machine['cn'] . "&mode=direct&type=rdp&from=" . $from;
                            $url = create_remote_token($config, $machine, "rdp", $config['login']);
                            $html .= "<a href=\"" . $url . "\" onclick=\"openInNewWindow(event, this)\"><img src=\"../elements/images/rdp-small.png\" alt=\"Accès distant...\"></a>";
                            // $html .= "<div id='divtsvnc$loop'><img src=\"../elements/images/spinner.gif\"></img></div>\n";
                            /*
                             * $html .= "<script type='text/javascript'>
                             * // <![CDATA[
                             * new Ajax.Updater($('divtsvnc$loop'),'parcs_ajax_lib.php',{method: 'post', parameters: '?ip=$iphost&mode=ts_vnc'});
                             * //]]>
                             * </script>\n";
                             */
                            /*
                             * } else {
                             * $ret = "<span onmouseout=\"UnTip();\" onmouseover=\"Tip('Echec du ping',WIDTH,250,SHADOW,true,DURATION,5000);\">" . "<img type=\"image\" border=\"0\" title=\"Echec du ping\" src=\"../elements/images/disabled.png\">" . "</span>\n";
                             * $html .= ($ret);
                             * }
                             */
                            $html .= "</td>";
                            $html .= "<td align=\"center\">";
                            $r = $machine['error'] & SAMBAEDU_WPKG_RUNNING;
                            $html .= "<a target=\"main\" href=\"../wpkg/poste_statuts.php?id_host=" . $machine['cn'] . "&from=$from\">";
                            $w = $machine['wpkg'] ?? "";
                            if ($r) {
                                $texte = "<li>" . implode("<li>", error_message($r));
                                $img = "../elements/images/spinner.gif";
                            } else {
                                switch ($w) {
                                    case 0:
                                        $texte = "Applications OK";
                                        $img = "../elements/images/enabled.png";
                                        break;
                                    case 1:
                                        $texte = "Au moins une mise à jour est en attente";
                                        $img = "../elements/images/warning.png";
                                        break;
                                    case 2:
                                        $texte = "Au moins une application n&lsquo;est pas installée";
                                        $img = "../elements/images/dialog-warning.png";
                                        break;
                                    default:
                                        $texte = "Pas de rapport wpkg";
                                        $img = "../elements/images/help.png";
                                        break;
                                }
                            }
                            $html .= "<img onmouseout=\"UnTip();\" onmouseover=\"Tip('" . $texte . "',WIDTH,250,SHADOW,true,DURATION,5000);\" type=\"image\" src=\"" . $img . "\" border=\"0\" title=\"" . $machine['cn'] . "\" title=\"" . $texte . "\" />";
                            $html .= "</a></td>\n";
                            $html .= "</td>\n";
                            $html .= "<td align=\"center\">";
                            $r = $machine['error'] & ~ SAMBAEDU_WPKG_RUNNING;
                            $html .= "<a target=\"main\" href=\"show_histo.php?selectionne=2&amp;mpenc=" . $machine['cn'] . "&from=$from\">";
                            if ($r > 0) {
                                $texte = "<li>" . implode("<li>", error_message($r));
                                $html .= "<img onmouseout=\"UnTip();\" onmouseover=\"Tip('" . $texte . "',WIDTH,250,SHADOW,true,DURATION,5000);\" src=\"../elements/images/dialog-warning.png\" border=\"0\" />";
                            } else {
                                $html .= "<img type=\"image\" src=\"../elements/images/enabled.png\" border=\"0\" title=\"" . $machine['cn'] . "\" title=\"" . $machine['cn'] . "\" />";
                            }
                            if ($machine['error'] & SAMBAEDU_SLOW_NET_ERROR){
                                $html .= "<img type=\"image\" src=\"../elements/images/snail.png\" border=\"0\" title=\"" . $machine['cn'] . "\" title=\"" . $machine['cn'] . "\" />";
                            }
                            $html .= "</a></td>\n";

                            $html .= "</tr>\n";
                        }
                    }
                    $html .= "</table>\n";
                    $html .= "</center>\n";
                    $html .= "<br>";
                    // detail_parc_printer($parc);
                    // $heure_act=date("H");
                    // $html .= $nomjour;
                    if ($refresh == 1)
                        $html .= refresh_page("action", 30);
                }

                switch ($nomjour) {

                    case "Monday":
                        $nomjour = "l";
                        break;

                    case "Tuesday":
                        $nomjour = "ma";
                        break;

                    case "Wednesday":
                        $nomjour = "me";
                        break;

                    case "Thursday":
                        $nomjour = "j";
                        break;

                    case "Friday":
                        $nomjour = "v";
                        break;

                    case "Saturday":
                        $nomjour = "s";
                        break;

                    case "Sunday":
                        $nomjour = "d";
                        break;
                }
                break;
            default:
                $html .= "coucou toi";
                break;
        }
    }
}

function rdp_html($config, &$html)
{
    require_once "remote.inc.php";
    // mode de connexion
    $mode = $_GET['mode'] ?? "list";
    $type = $_GET['type'] ?? "rdp";
    switch ($mode) {
        case "direct":
            // redirection vers l'url de guacamole avec un token
            $url = create_remote_token($config, $_GET['machine'], $type, $config['login']);
            header("Location: " . $url);
            exit();
            break;
        default:
            $login = $config['login'];
            // if (have_right($config, SE_COMPUTER_ADMIN)) {
            $html .= "<h1>accès distant</h1>\n";
            if (! file_exists("/etc/sambaedu/sambaedu.conf.d/guacamole.conf")) {
                if ($config['login'] == "admin") {
                    $html .= "Cet outil semble manquant sur le serveur !";
                    $html .= "<ul>\n";
                    $html .= "<li>Vous devez installer le module sambaedu-guacamole afin d'utiliser l'accès distant</li>\n";
                    $html .= "</ul>\n";
                } else {
                    $html .= "Fonctionalité non disponible";
                }
            } else {
                $html .= display_remote_list($config, $login);
            }
    }
}
?>