<?php

// recup des données RTE Ecowatt
// ...

/**
 * recupère les infos pour 4 jours.
 *
 * Cette fonction est utilisée par les serveurs d'établissements, qui vont directement récuperer les infos dont ils ont besoin sur
 * https://central.sambaedu.org qui relaie vers les serveurs maitres.
 *
 * L'info est mise en cache pour une durée aléatoire de 23 à 24H. Une seule requête est donc faite par jour.
 *
 * Si vous voulez récuperer en direct les infos chez RTE,
 *
 * Nécessite d'avoir créé un compte applications sur rte, et de mettre les identifiants récupéreé sur le site
 *
 * https://data.rte-france.com/catalog/-/api/consumption/Ecowatt/v4.0
 *
 * si vous voulez participer à la récupération des données,
 * mettre la clé en Base64 dans une variable rte_api_key = "xxxxxx==" dans sambaedu.conf
 * et signalez vous auprès de l'équipe sambaedu !
 *
 * @param unknown $config
 * @return $signal 1-3 : niveau d'alerte de la plage courante.
 */
function get_ecowatt($config)
{
    $signal = 1;
    $res = apcu_fetch('ecowatt');
    if (empty($res)) {
        // lock('ecowatt');
        require_once (dirname(__FILE__) . '/../vendor/autoload.php');
        if (! empty($config['rte_api_key'])) {
            if (apcu_add('ecowatt_lock', 900)) {
                try {
                    $opt = [
                        'base_uri' => 'https://digital.iservices.rte-france.com/',
                        'cookies' => true
                    ];
                    curl_proxy_options($config, $opt);
                    // create our http client (Guzzle)
                    $client = new GuzzleHttp\Client($opt);
                    // make a request to the token url,
                    $response = $client->request('POST', '/token/oauth/', [
                        'headers' => [
                            'Content-Type' => 'application/x-www-form-urlencoded',
                            'Authorization' => 'Basic ' . $config['rte_api_key']
                        ]
                    ]);

                    $responseBody = $response->getBody(true);
                    $responseArr = json_decode($responseBody, true);
                    $accessToken = $responseArr['access_token'];

                    // step2: use the token to make an API request
                    $response = $client->request('GET', '/open_api/ecowatt/v4/signals', [
                        'headers' => [
                            'Content-Type' => 'application/json; charset=UTF-8',
                            'Authorization' => 'Bearer ' . $accessToken
                        ]
                    ]);
                    if ($response) {
                        $res = json_decode((string) $response->getBody(), true)['signals'];
                    }
                } catch (\GuzzleHttp\Exception\TransferException $e) {}
            }
        } else {
            try {
                $opt = [
                    'base_uri' => 'https://central.sambaedu.org/'
                ];
                curl_proxy_options($config, $opt);
                // create our http client (Guzzle)
                $client = new GuzzleHttp\Client($opt);
                // make a request to the token url,
                $response = $client->request('GET', 'api/ecowatt.php', []);
                if ($response) {
                    $res = json_decode((string) $response->getBody(), true);
                }
            } catch (\GuzzleHttp\Exception\TransferException $e) {}
        }
        apcu_store('ecowatt', $res, (82800 + random_int(0, 3600)));
        // unlock('ecowatt');
    }
    if ($res) {
        $time0 = strtotime($res[0]['jour']);
        $time1 = strtotime($res[1]['jour']);
        if (time() < ($time0 + 86400)) {
            $index = (int) floor((time() - $time0) / 3600);
            $signal = $res[0]['values'][$index]['hvalue'];
        } elseif (time() < ($time1 + 86400)) {
            $index = (int) floor((time() - $time1) / 3600);
            $signal = $res[1]['values'][$index]['hvalue'];
        } else {
            $signal = 1;
        }
    } else {
        $signal = 1;
    }
    return $signal;
}

function relay_ecowatt($config)
{
    get_ecowatt($config);
    return apcu_fetch('ecowatt');
}
?>
