<?php

/**
 * Fonctions imprimantes

 * @Projet SambaEdu

 * @Auteurs Equipe Sambaedu, denis bonnenfant, Tom Ricci

 * 
 * @Note: Ce fichier de fonction doit etre appele par un include

 * @Licence Distribue sous la licence GPL
 */
function guid()
{
    $charid = strtoupper(md5(uniqid(rand(), true)));
    $hyphen = chr(45); // "-"
    $uuid = chr(123) . // "{"
    substr($charid, 0, 8) . $hyphen . substr($charid, 8, 4) . $hyphen . substr($charid, 12, 4) . $hyphen . substr($charid, 16, 4) . $hyphen . substr($charid, 20, 12) . chr(125); // "}"
    return $uuid;
}

function parse_printer_uri($uri, &$printer = array())
{
    if (preg_match("%^((.*):)*(.*)://((.*):(.*)@)*(.*)[:/](.*)$%", $uri, $u) == 1) {
        $printer['filtre'] = $u[2];
        $printer['protocole'] = $u[3];
        $printer['user'] = $u[4];
        $printer['password'] = $u[5];
        $printer['adresse'] = $u[7];
        $printer['port'] = $u[8];
        $printer['queue'] = $u[9] ?? "";
        return true;
    } else
        return false;
}

function get_printer_driver($config, $printer, $server = "")
{
    if (empty($server)) {
        $server = $config['se4fs_name'];
    }
    $driver = [];
    exec("rpcclient --user-kerberos=required -c 'getdriver \"" . $printer . "\"' " . $server . " 2>/dev/null", $out, $ret);
    if ($ret == 0) {
        foreach ($out as $line) {
            if (preg_match("/^\s*(.*): \[((.*\\\\3\\\\)?(.*))\]/", $line, $m) == 1) {
                if ($m[1] == "Dependentfiles") {
                    $driver['Dependentfiles'][] = $m[4];
                } else {
                    if ($m[4] == "(null)" || empty($m[4])) {
                        $driver[$m[1]] = "NULL";
                    } else {
                        $driver[$m[1]] = $m[4];
                    }
                }
            }
        }
        return $driver;
    } else
        return false;
}

// smbclient w10-> /var/lib/samba/printers/x64
function copy_driver_file($config, $server, $file)
{
    if ($file != "NULL") {
        exec("smbclient --use-kerberos=required //" . $server . "/print\$ -c " . escapeshellarg("cd x64\\3;ls " . $file . "*") . " 2>/dev/null", $out, $ret);
        if ($ret == 0) {
            foreach ($out as $line) {
                if (preg_match("/^\s*([a-zA-Z0-9_\-]+\.[a-zA-Z0-9_\-]+).*/", $line, $m) == 1) {
                    if ($m[1] == $file) {
                        $orig = $file;
                        break;
                    } elseif (strcasecmp($m[1], $file) == 0) {
                        $orig = $m[1];
                    }
                }
            }
            exec("smbclient --use-kerberos=required //" . $server . "/print\$ -c " . escapeshellarg("cd x64\\3;get " . $orig . " /var/lib/samba/printers/x64/" . $file) . " 2>/dev/null", $out, $ret);
        }
        return $out;
    } else {
        return true;
    }
}

/**
 * uploade les drivers sur le serveur.
 * Pour cela une imprimante locale partagée doit avoir été configurée sur un client windows 10
 *
 * @param unknown $config
 * @param unknown $printer
 *            nom de l'imprimante partagée sur le w10
 * @param unknown $server
 *            nom du w10
 */
function upload_printer_driver($config, $printer, $server)
{
    if ($driver = get_printer_driver($config, $printer, $server)) {

        copy_driver_file($config, $server, $driver['Driver Path']);
        copy_driver_file($config, $server, $driver['Datafile']);
        copy_driver_file($config, $server, $driver['Configfile']);
        copy_driver_file($config, $server, $driver['Helpfile']);
        foreach ($driver['Dependentfiles'] as $file) {
            copy_driver_file($config, $server, $file);
        }
        $command = "rpcclient --use-kerberos=required -c 'adddriver ";
        $command .= "\"" . $driver['Architecture'] . "\" \"" . $driver['Driver Name'] . ":" . $driver['Driver Path'] . ":" . $driver['Datafile'] . ":" . $driver['Configfile'] . ":";
        $command .= $driver['Helpfile'] . ":NULL:NULL:" . implode(",", $driver['Dependentfiles']) . "\" \"3\"' " . $config['se4fs_name'] . " 2>/dev/null";
        exec($command, $out, $ret);
        if ($ret == 0) {
            $files = [
                $driver['Driver Path'],
                $driver['Datafile'],
                $driver['Configfile'],
                $driver['Helpfile']
            ];
            foreach ($driver['Dependentfiles'] as $file) {
                $files[] = $file;
            }
            foreach ($files as $file) {
                unlink("/var/lib/samba/printers/x64/" . $file);
            }
            return $out[0];
        } else {
            return false;
        }
    }
}

function list_printers($config, $name = "", $fast = false)
{
    $printers = $p = $s = $res = $rep = $u = array();
    $printers_ad = array_column(search_ad($config, "*", "machine", $config['dn']['equipements']), 'cn');
    exec("lpstat -s", $res);
    $i = 0;
    foreach ($res as $ligne) {
        $p = $u = $rep = array();
        if (preg_match("/^(device for|matériel pour) (.*): (.*)$/", $ligne, $p) == 1) {
            if (empty($name) || (strcasecmp($name, $p[2]) == 0)) {
                $printers[$i]['name'] = trim($p[2]);
                $printers[$i]['url'] = $p[3];
                if (preg_match("|^.*://([0-9.]*):.*|", $p[3], $m)) {
                    $ip = $m[1];
                }
                if (! in_array($printers[$i]['name'], $printers_ad) && ! empty($ip)) {
                    add_printer_reservation($config, $printers[$i]['name'], "", $ip);
                }
                $i ++;
            }
        }
    }
    $res = [];
    exec("lpstat -l -p " . $name, $res);
    foreach ($res as $ligne) {
        $s = $t = array();
        $regex = "/^printer (\S+) ((is|now) (printing|idle)((.*)).  )*(.*) since (.*[0-9])( -)*/";
        if (preg_match($regex, $ligne, $s) == 1) {
            $i = array_search($s[1], array_column($printers, "name"));
            $printers[$i]['statut'] = $s[7];
            $printers[$i]['etat'] = $s[4];
            $printers[$i]['date'] = $s[8];
            if (! $fast) {
                $printers[$i]['smb_ready'] = get_smb_printer($config, $printers[$i]);
            }
        } elseif (preg_match("/^\s*(.*):[ ]?(.*)$/", $ligne, $t) == 1) {
            $printers[$i][strtolower($t[1])] = $t[2] ?? "";
        } else {
            $printers[$i]['message'] = trim($ligne);
        }
    }
    return $printers;
}

/**
 * crée une reservation ip pour l'imprimante
 * à faire avant de créer l'imprimante, on en aura besoin pour l'url de celle-ci.
 * Il faut connaitre l'adresse mac
 *
 * @param unknown $config
 * @param unknown $nom
 * @param unknown $mac
 * @param string $ip
 * @return boolean
 */
function add_printer_reservation($config, $nom, $mac = "", $ip = "")
{
    if (count(search_machine($config, $nom)) == 0 || count(search_machine($config, $ip)) == 0) {
        // on crée une machine avec un enregistrement mac, mais on ne définit pas d'ip réservée à ce stade.
        if (create_machine($config, $nom, $config['equipements_rdn'], "reservation imprimante")) {
            if (set_dhcp_reservation($config, $nom, $ip)) {
                $printer = search_machine($config, $nom);
                if (isset($printer['iphostnumber']))
                    return $printer['iphostnumber'];
            }
        }
    }
    return false;
}

/**
 * crée ou modifie une imprimante
 *
 * @param unknown $config
 * @param string $nom
 * @param array $params
 * @return boolean
 */
function add_printer($config, string $nom, string $url = "", string $description = "", string $lieu = "", string $driver = "")
{
    /*
     * if (! empty($params['filtre']))
     * $url = $params['filtre'] . ":";
     * $url .= $params['protocole'] . "://";
     * $url .= $params['addresse'];
     * if (! empty($params['port']) && ($params['protocole'] == "socket"))
     * $url .= ":" . $params['port'];
     * if (! empty($params['queue']))
     * $url .= ":" . $params['queue'];
     */
    $options = " -E ";
    if (! empty($url))
        $options .= " -v " . escapeshellarg($url);
    if (! empty($description))
        $options .= " -D " . escapeshellarg($description);
    if (! empty($lieu))
        $options .= " -L " . escapeshellarg($lieu);

    $message = array();
    $commande = "sudo lpadmin -p " . escapeshellarg($nom) . $options;
    $ret = 1;
    exec($commande, $message, $ret);

    if ($ret == 0) {
        exec("sudo smbcontrol smbd reload-printers");
        if (! empty($driver))
            set_smb_driver($config, $nom, $driver);
        return true;
    } else {
        return false;
    }
}

function cups_client_command($config, $printer, $action)
{
    switch ($action) {
        case "startup":
            // case "logon":
            // passage en ipp ?
            // $command = "/sbin/lpadmin -p " . $printer . " -v ipp://" . $config['se4fs_name'] . "." . $config['domain'] . "/printers/" . $printer . "\n";
            $command = "rm -f /etc/cups/client.conf\n";
            $command .= "systemctl restart cups\n";
            $command .= "/usr/sbin/cupsenable\n";
            $command .= "/sbin/lpadmin -p " . $printer . " -v smb://" . $config['se4fs_name'] . "." . $config['domain'] . "/" . $printer . "\n";
            $command .= "/sbin/lpadmin -p " . $printer . " -E -o auth-info-required=username,password\n";
            $command .= "/usr/sbin/cupsenable " . $printer . "\n";
            $command .= "/usr/bin/lpq enable -P " . $printer . "\n";
            break;
        case "shutdown":
            $command = "/sbin/lpadmin -x " . $printer . "\n";
    }
    return $command;
}

/**
 * récupère la gpo impression
 *
 * @param unknown $config
 * @return boolean | unknown $gpo
 */
function gpo_impr($config)
{
    $gpos = gpolist($config, 'Administrator');
    foreach ($gpos as $gpo) {
        if ($gpo['displayname'] == "imprimantes") {
            $gpo_impr = $gpo;
            break;
        }
    }
    if (! isset($gpo_impr)) {
        echo "Erreur : la gpo imprimantes n'existe pas, importer la depuis les gpo initiales.";
        return false;
    }
    return $gpo_impr;
}

/**
 * récupère le XML Printers dans la gpo imprimantes
 *
 * @param unknown $config
 * @param unknown $gpo
 * @return boolean | unknown $tmppath
 */
function get_Printers_XML($config, $gpo_impr)
{
    $tmppath = sys_get_temp_dir() . "/se4_imprimantes";
    if (! is_dir($tmppath)) {
        mkdir($tmppath);
    }

    $command = "cd \"" . $config['domain'] . "/Policies/" . $gpo_impr['uuid'] . "/User/Preferences/Printers\";lcd \"" . $tmppath . "\";get Printers.xml";
    exec('smbclient //' . ad_url($config, "dns") . '/sysvol --use-kerberos=required -c ' . escapeshellarg($command), $message, $ret);

    if (file_exists($tmppath . "/Printers.xml")) {
        // echo "Récupération Printers.xml OK.";
        return $tmppath;
    } else {
        echo "Erreur récupération.";
        return false;
    }
}

/**
 * récupère le XML Printers dans la gpo imprimantes
 *
 * @param unknown $config
 * @param unknown $gpo
 * @param unknown $tmppath
 * @return boolean
 */
function put_Printers_XML($config, $gpo_impr, $tmppath)
{
    $command = "cd \"" . $config['domain'] . "/Policies/" . $gpo_impr['uuid'] . "/User/Preferences/Printers\";lcd \"" . $tmppath . "\";put Printers.xml";
    exec('smbclient //' . ad_url($config, "dns") . '/sysvol --use-kerberos=required -c ' . escapeshellarg($command), $message, $ret);

    if ($ret == 0) {
        // echo "Écriture Printers.xml OK.";
    } else {
        echo "Erreur écriture.";
        return false;
    }
    rm_Printers_XML($config, $tmppath);
    increment_gpo($config, $gpo_impr['displayname']);
    return true;
}

/**
 * supprime le dossier $tmppath ainsi donc que le XML Printers
 *
 * @param unknown $config
 * @param unknown $tmppath
 * @return boolean
 */
function rm_Printers_XML($config, $tmppath)
{
    exec("rm -fr " . escapeshellarg($tmppath), $message, $ret);

    if ($ret == 0) {
        // echo "Suppression Printers.xml OK.";
        return true;
    } else {
        echo "Erreur suppression.";
        return false;
    }
}

//

/**
 * supprime une imprimante
 * fait le ménage dans les gpo si besoin
 *
 * @param unknown $printer
 *            nom
 * @return boolean
 */
function delete_printer($config, $printer)
{
    $groups = list_printer_groups($config, $printer)['groups'];
    foreach ($groups as $group) {
        remove_printer_group($config, $printer, $group['name']); // *********************************************
    }
    $machine = search_machine($config, $printer);
    if (isset($machine['cn'])) {
        delete_ad($config, $printer, "machine");
    }
    $message = array();
    $ret = 1;
    exec("sudo lpadmin -x " . escapeshellarg($printer), $message, $ret);
    if ($ret == 0) {
        return true;
    } else {
        return false;
    }
}

/**
 * assigne un driver à l'imprimante windows
 *
 * @param unknown $config
 * @return boolean
 */
function set_smb_driver($config, $printer, $driver)
{
    $message = array();
    $ret = 1;
    exec("rpcclient " . $config['se4fs_name'] . " --use-kerberos=required -c 'setdriver \"" . $printer . "\" \"" . $driver . "\"'", $message, $ret);
    if ($ret == 0) {

        return true;
    } else {
        return false;
    }
}

/**
 * assigne un driver à l'imprimante cups
 *
 * @param unknown $config
 * @return mixed[drivers]|boolean
 */
function set_cups_driver($config, $printer, $driver = "everywhere")
{
    $message = array();
    $ret = 1;
    exec("sudo lpadmin -p " . escapeshellarg($printer) . " -m " . escapeshellarg($driver), $message, $ret);
    if ($ret == 0) {
        return true;
    } else {
        return false;
    }
}

/**
 * liste les drivers windows disponibles sur le serveur
 *
 * @param unknown $config
 * @return mixed[drivers]|boolean
 */
function list_smb_drivers($config)
{
    $message = array();
    $ret = 1;
    exec("rpcclient " . $config['se4fs_name'] . " --use-kerberos=required -c enumdrivers", $message, $ret);
    if ($ret == 0) {
        $drivers = array();
        foreach ($message as $ligne) {
            $m = array();
            if (preg_match("/^.*Driver Name: \[(.*)\]$/", $ligne, $m) == 1) {
                $drivers[] = $m[1];
            }
        }
        $drivers = array_unique($drivers);
        return $drivers;
    } else {
        return false;
    }
}

/**
 * liste les drivers cups disponibles sur le serveur
 * obsolete, inutile depuis 2009...
 *
 * @param unknown $config
 * @return mixed[drivers]|boolean
 */
function list_cups_drivers($config)
{
    $message = array();
    $ret = 1;
    exec("lpinfo -m", $message, $ret);
    if ($ret == 0) {
        $drivers = $message;
        return $drivers;
    } else {
        return false;
    }
}

/**
 * liste les proprietés de l'imprimante windows sur le serveur
 *
 * @param unknown $config
 * @return mixed[drivers]|boolean
 */
function get_smb_printer($config, &$printer, $server = "")
{
    if (empty($server)) {
        $server = $config['se4fs_name'];
    }
    $message = array();
    $ret = 1;
    $command = "rpcclient --use-kerberos=required -c 'getprinter \"" . $printer['name'] . "\"' " . $server;
    exec($command, $message, $ret);
    if ($ret == 0) {
        foreach ($message as $ligne) {
            $m = array();
            if (preg_match("/^\s*description:\[(.*),(.*),(.*)\]$/", $ligne, $m) == 1) {
                $printer['smb_name'] = $m[1];
                $printer['smb_driver'] = $m[2];
                $printer['smb_comment'] = $m[3];
            }
        }
        return true;
    } else {
        return false;
    }
}

/**
 * liste les imprimantes windows disponibles sur le serveur
 *
 * @param unknown $config
 * @return mixed[drivers]|boolean
 */
function enum_smb_printers($config, $server = "")
{
    if (empty($server)) {
        $server = $config['se4fs_name'];
    }
    $message = array();
    $ret = 1;
    $command = "rpcclient --use-kerberos=required -c 'enumprinters' " . $server;
    exec($command, $message, $ret);
    if ($ret == 0) {
        $i = 0;
        $printer = [];
        foreach ($message as $ligne) {
            $m = array();
            if (preg_match("/^\s*description:\[.*\\\\(.+),(.+),(.+)\]$/", $ligne, $m) == 1) {
                $i ++;
                $printer[$i]['smb_name'] = $m[1];
                $printer[$i]['smb_driver'] = $m[2];
                $printer[$i]['smb_comment'] = $m[3];
            }
        }
        return $printer;
    } else {
        return [];
    }
}

/**
 * liste les groupes d'une imprimante, ou de toutes les imprimantes ************************************************************************************************
 *
 * @param unknown $config
 * @param string $printer
 * @return array[] (imprimante précisée dans les paramètres) | array[][] (groupes de toutes les imprimantes)
 */
function list_printer_groups($config, $printer = "")
{
    if (empty($printer)) {
        $all_printers = list_printers($config);
    } else {
        $all_printers = list_printers($config, $printer);
    }

    // if ($config['printers_migrated'] > 1){
    // foreach($all_printers as $printer){

    // }
    // }
    $mod = false;
    $log = "Log de l'ajout des imprimantes aux groupes\n";

    $gpo_impr = gpo_impr($config);

    $tmppath = get_Printers_XML($config, $gpo_impr);

    $printers = array();

    if ($tmppath) {
        // echo "Fonction get_Printers_XML dans list_printer_groups OK.";
    } else {
        echo "Erreur get_Printers_XML dans list_printer_groups.";
        $ret = rm_Printers_XML($config, $tmppath);
        return $printers;
    }

    $file = new DOMDocument();
    $file->load($tmppath . "/Printers.xml");

    $xpath = new DOMXPath($file);

    $nb_printers = count($all_printers);
    for ($i = 0; $i < $nb_printers; $i ++) {
        $groups = array();

        $FilterGroups = $xpath->query("/Printers/SharedPrinter[@name='" . $all_printers[$i]['name'] . "']/Filters/FilterGroup");

        foreach ($FilterGroups as $FilterGroup) {
            $groupname = $FilterGroup->getAttribute('name');
            $groupname = explode('\\', $groupname)[1];
            $grouptype = $FilterGroup->getAttribute('userContext');
            if ($grouptype == '0') {
                $grouptype = "machine";
                $searchtype = "parc";
            } else {
                $grouptype = "user";
                $searchtype = "group";
            }
            // correction pour avoir les imprimantes dans les groupes.
            // @TODO refaire la fonction complete une fois la migration faite.
            if (! in_array(strtolower($all_printers[$i]['name']), array_map("strtolower", search_printers($config, $groupname, $searchtype)))) {
                groupaddmember($config, $all_printers[$i]['name'] . "$", add_hostname_suffix($config, $groupname));
                $mod = true;
                $log .= "ajout de " . $all_printers[$i]['name'] . " à " . $groupname . "\n";
            }
            $groups[] = array(
                'name' => remove_hostname_suffix($config, $groupname),
                'type' => $grouptype
            );
        }

        $printers[] = array(
            'printer' => $all_printers[$i],
            'groups' => $groups
        );
    }

    if (! empty($printer)) {
        $printers = array(
            'printer' => $printers[0]['printer'],
            'groups' => $printers[0]['groups']
        );
    }
    if ($mod) {
        set_param($config, "printers_migrated", (@$config["printers_migrated"] + 1));
        file_put_contents("/tmp/printers.log", $log);
    }
    $ret = rm_Printers_XML($config, $tmppath);
    return $printers;
}

/**
 * liste toutes les imprimantes d'un groupe, ou de tous les groupes *********************************************************************************
 *
 * @param unknown $config
 * @param string $group
 * @return array[] (groupe précisé dans les paramètres) | array[][] (imprimantes des tous les groupes)
 */
function list_group_printers($config, $group = "")
{
    $gpo_impr = gpo_impr($config);

    $tmppath = get_Printers_XML($config, $gpo_impr);

    $groups = array();
    $all_printers = list_printers($config, "", false);
    if ($tmppath) {
        // echo "Fonction get_Printers_XML dans list_group_printers OK.";
    } else {
        echo "Erreur get_Printers_XML dans list_group_printers .";
        $ret = rm_Printers_XML($config, $tmppath);
        return $groups;
    }

    $file = new DOMDocument();
    $file->load($tmppath . "/Printers.xml");

    $xpath = new DOMXPath($file);

    if (empty($group)) {
        $all_groups = search_parcs($config, "*", "all");
        $all_groups = array_merge($all_groups, search_ad($config, "*", 'group', $config['dn']['groups']));
    } else {
        $all_groups = search_parcs($config, $group, "all");
        if (count($all_groups) == 0) {
            $all_groups = search_ad($config, $group, "ou");
        }
    }

    $nb_groups = count($all_groups);
    for ($i = 0; $i < $nb_groups; $i ++) {
        $group_name = $all_groups[$i]['name'] ?? "";
        $FilterGroups = $xpath->query("/Printers/SharedPrinter/Filters/FilterGroup[@name='" . strtoupper($config['samba_domain']) . "\\" . add_hostname_suffix($config, $group_name) . "']");

        $printers = array();

        foreach ($FilterGroups as $FilterGroup) {
            $SharedPrinter = $FilterGroup->parentNode->parentNode;
            // $printers[] = list_printers($config,$name = $SharedPrinter->getAttribute('name'), true)[0];
            $printers[] = $all_printers[array_search($name = $SharedPrinter->getAttribute('name'), array_column($all_printers, "name"))];
        }

        if (! empty($printers)) {
            $groups[] = array(
                'name' => remove_hostname_suffix($config, $group_name),
                'printers' => $printers
            );
        }
    }

    if (! empty($group)) {
        if (! empty($groups)) {
            $groups = array(
                'name' => $groups[0]['name'],
                'printers' => $groups[0]['printers']
            );
        }
    }

    $ret = rm_Printers_XML($config, $tmppath);
    return $groups;
}

function search_printers($config, $name, $type = "printer")
{
    // echo $name . "=>" .$type."<br>";
    $printers = $members = [];
    foreach (array_column(search_ad($config, $name, $type), "member") as $member) {
        $members = array_merge($members, $member);
    }

    foreach ($members as $member) {
        if (preg_match("/cn=(.*)," . $config['dn']['equipements'] . "/i", $member, $m)) {
            $printers[] = $m[1];
        }
    }
    return $printers;
}

/**
 * liste les imprimantes pour un poste et l'utilisateur connecté.
 *
 * @param unknown $config
 * @param unknown $machine
 * @param unknown $user
 * @return array : liste des imprimantes complète.
 */
function list_machine_printers($config, $machine = "", $user = "")
{
    $printers = [];
    if (! empty($machine)) {
        foreach (list_parcs_name($config, $machine) as $parc) {
            $printers = array_merge($printers, search_printers($config, $parc, "parc"));
        }
    }
    if (! empty($user)) {
        foreach (search_ad($config, $user, "user")[0]['memberof'] as $dn) {
            $printers = array_merge($printers, search_printers($config, ldap_dn2cn($dn), "group"));
        }
    }
    return array_unique($printers);
}

/**
 * ajoute/modifie l'imprimante dans la gpo se4_imprimantes **********************************************
 *
 * @param unknown $config
 * @param unknown $printer
 *            nom
 * @param unknown $group
 *            nom
 * @param unknown $default
 *            bool 0=>faux 1=>vrai
 * @return string|boolean
 */
function add_printer_in_gpo($config, $printer, $group, $default = "0")
{
    $gpo_impr = gpo_impr($config);

    // cherche le $type de groupe : parc/users
    $ou = search_ad($config, $group, 'group', $config['dn']['groups']);
    if (count($ou) > 0) {
        $type = "user";
    } else {
        $type = "machine";
    }

    $tmppath = get_Printers_XML($config, $gpo_impr);

    if ($tmppath) {
        // echo "Fonction get_Printers_XML dans add_printer_in_gpo OK.";
    } else {
        echo "Erreur get_Printers_XML dans add_printer_in_gpo.";
        $ret = rm_Printers_XML($config, $tmppath);
        return false;
    }

    $file = new DOMDocument();
    $file->formatOutput = true;
    $file->preserveWhiteSpace = false;
    $file->load($tmppath . "/Printers.xml");

    $xpath = new DOMXPath($file);
    $SharedPrinter = $xpath->query("/Printers/SharedPrinter[@name='" . $printer . "']"); // ***********************************************************

    if ($SharedPrinter->length < 1) { // l'imprimante n'existe pas encore, on l'ajoute
        $filePrinters = $file->getElementsByTagName('Printers');

        // <SharedPrinter clsid="{9A5E9697-9095-436d-A0EE-4D128FDFBCE5}" name="imprimante" status="imprimante" image="1" changed="2020-08-03 13:28:38" uid="{4994B896-1AB9-46B2-95DF-CC76A372882A}" removePolicy="1" userContext="1" bypassErrors="1">
        $fileSharedPrinter = $file->createElement("SharedPrinter");
        $fileSharedPrinter->setAttribute('clsid', "{9A5E9697-9095-436d-A0EE-4D128FDFBCE5}");
        $fileSharedPrinter->setAttribute('name', $printer);
        $fileSharedPrinter->setAttribute('status', $printer);
        $fileSharedPrinter->setAttribute('image', "1");
        $fileSharedPrinter->setAttribute('userContext', "1");
        $fileSharedPrinter->setAttribute('bypassErrors', "1");
        $fileSharedPrinter->setAttribute('removePolicy', "1");
        $fileSharedPrinter->setAttribute('changed', date("Y-m-d h:i:s"));
        $fileSharedPrinter->setAttribute('uid', guid());

        // <Properties action="R" comment="" path="\\###_SE4FS_NAME_###\imprimante" location="" default="0" skipLocal="0" deleteAll="0" persistent="0" deleteMaps="0" port=""/>
        $fileSharedPrinterProperties = $file->createElement("Properties");
        $fileSharedPrinterProperties->setAttribute('action', "R");
        $fileSharedPrinterProperties->setAttribute('comment', "");
        $fileSharedPrinterProperties->setAttribute('path', "\\\\" . $config['se4fs_name'] . "\\" . $printer);
        $fileSharedPrinterProperties->setAttribute('location', "");
        $fileSharedPrinterProperties->setAttribute('default', $default);
        $fileSharedPrinterProperties->setAttribute('skipLocal', "0");
        $fileSharedPrinterProperties->setAttribute('deleteAll', "0");
        $fileSharedPrinterProperties->setAttribute('persistent', "0");
        $fileSharedPrinterProperties->setAttribute('deleteMaps', "0");
        $fileSharedPrinterProperties->setAttribute('port', "");

        // <Filters> <FilterGroup bool="AND" not="0" name="###_SAMBA_DOMAIN_###\salle" sid="###_DOMAIN_SID_###-3250" userContext="0" primaryGroup="0" localGroup="0"/> <!-- ORDI DANS --> </Filters>
        // <Filters> <FilterGroup bool="OR" not="0" name="###_SAMBA_DOMAIN_###\Administratifs" sid="###_DOMAIN_SID_###-1117" userContext="1" primaryGroup="0" localGroup="0"/> <!-- UTILISATEUR DANS --> </Filters>
        $fileSharedPrinterFilters = $file->createElement("Filters");

        $fileSharedPrinterFilterGroup = $file->createElement("FilterGroup");
        $fileSharedPrinterFilterGroup->setAttribute('bool', "OR");
        $fileSharedPrinterFilterGroup->setAttribute('not', "0");
        $fileSharedPrinterFilterGroup->setAttribute('name', strtoupper($config['samba_domain']) . "\\" . add_hostname_suffix($config, $group));
        $sid = exec("wbinfo --name-to-sid " . add_hostname_suffix($config, $group));
        $sid = explode(" ", $sid)[0];
        $fileSharedPrinterFilterGroup->setAttribute('sid', $sid);
        if ($type == "machine") { // type SALLE
            $fileSharedPrinterFilterGroup->setAttribute('userContext', "0");
        } else { // type USER
            $fileSharedPrinterFilterGroup->setAttribute('userContext', "1");
        }
        $fileSharedPrinterFilterGroup->setAttribute('primaryGroup', "0");
        $fileSharedPrinterFilterGroup->setAttribute('localGroup', "0");

        $fileSharedPrinterFilters->appendChild($fileSharedPrinterFilterGroup);
        $fileSharedPrinter->appendChild($fileSharedPrinterProperties);
        $fileSharedPrinter->appendChild($fileSharedPrinterFilters);
        $filePrinters[0]->appendChild($fileSharedPrinter);
    } else { // l'imprimante existe déjà, on ajoute le groupe à l'existante
        $SharedPrinter[0]->setAttribute('changed', date("Y-m-d h:i:s"));
        $fileSharedPrinterFilters = $SharedPrinter[0]->getElementsByTagName('Filters'); // on cherche la node Filters à l'intérieur de l'imprimante existante

        $fileSharedPrinterFilterGroup = $file->createElement("FilterGroup"); // on créer une nouvelle node FilterGroup dans Filters
        $fileSharedPrinterFilterGroup->setAttribute('bool', "OR");
        $fileSharedPrinterFilterGroup->setAttribute('not', "0");
        $fileSharedPrinterFilterGroup->setAttribute('name', strtoupper($config['samba_domain']) . "\\" . add_hostname_suffix($config, $group));
        $sid = exec("wbinfo --name-to-sid " . add_hostname_suffix($config, $group));
        $sid = explode(" ", $sid)[0];
        $fileSharedPrinterFilterGroup->setAttribute('sid', $sid);
        if ($type == "machine") { // type SALLE
            $fileSharedPrinterFilterGroup->setAttribute('userContext', "0");
        } else { // type USER
            $fileSharedPrinterFilterGroup->setAttribute('userContext', "1");
        }
        $fileSharedPrinterFilterGroup->setAttribute('primaryGroup', "0");
        $fileSharedPrinterFilterGroup->setAttribute('localGroup', "0");

        $fileSharedPrinterFilters[0]->appendChild($fileSharedPrinterFilterGroup);
    }

    $file->save($tmppath . "/Printers.xml");

    $ret = put_Printers_XML($config, $gpo_impr, $tmppath);
    if ($ret) {
        // echo "Fonction put_Printers_XML & add_printer_in_gpo OK.";
        return true;
    } else {
        echo "Erreur put_Printers_XML & add_printer_in_gpo.";
        return false;
    }
}

/**
 * assigne une imprimante à un groupe (salle ou eleves|profs|administratifs) en l'ajoutant à la gpo
 *
 * @param unknown $config
 * @param unknown $printer
 * @param unknown $group
 * @return boolean|unknown
 */
function add_printer_group($config, $printer, $group)
{
    $attrs = array();

    // ad
    // on ajoute au groupe ad correspondant, potentiellement pour les droits et les vlans
    groupaddmember($config, $printer . "$", add_hostname_suffix($config, $group));

    // existe-t-il déjà l'imprimante dans la gpo ? si oui, on ajoute la salle dans celle-ci. *************************** un peu compliqué pour rien, on pourrait tout mettre dans add_printer_in_gpo() ou insérer cette fonction entière ici (et la suppimer)
    $groups = list_printer_groups($config, $printer)['groups'];
    if (count($groups) > 0) {
        foreach ($groups as $tmpgroup) {
            if ($tmpgroup['name'] == $group) {
                return true; // l'imprimante est déjà dans le groupe
            }
        }
        // l'imprimante existe mais n'est pas déployée dans le groupe
        return add_printer_in_gpo($config, $printer, $group);
    } else { // on crée l'imprimante & ajoute le groupe
        return add_printer_in_gpo($config, $printer, $group);
    }
    return false;
}

/**
 * supprime une imprimante d'un groupe (salle ou eleves|profs|administratifs) : **************************************
 * - l'imprimante existe et est déployée sur un seul groupe : on enlève l'imprimante de la gpo
 * - l'imprimante existe et est déployée sur plusieurs groupes : on supprime le groupe dans les filtres de l'imprimante
 *
 * @param unknown $config
 * @param unknown $printer
 * @param unknown $group
 * @return boolean|unknown
 */
function remove_printer_group($config, $printer, $group)
{
    groupdelmember($config, $printer . "$", add_hostname_suffix($config, $group));
    $gpo_impr = gpo_impr($config);

    $tmppath = get_Printers_XML($config, $gpo_impr);

    if ($tmppath) {
        // echo "Fonction get_Printers_XML dans remove_printer_group OK.";
    } else {
        echo "Erreur get_Printers_XML dans remove_printer_group.";
        $ret = rm_Printers_XML($config, $tmppath);
        return false;
    }

    $file = new DOMDocument();
    $file->formatOutput = true;
    $file->preserveWhiteSpace = false;
    $file->load($tmppath . "/Printers.xml");

    $xpath = new DOMXPath($file);
    $FilterGroup = $xpath->query("/Printers/SharedPrinter[@name='" . $printer . "']/Filters/FilterGroup[@name='" . strtoupper($config['samba_domain']) . "\\" . add_hostname_suffix($config, $group) . "']");

    if ($FilterGroup->length < 1) { // il n'y a pas cette imprimante dans ce groupe
        $ret = rm_Printers_XML($config, $tmppath);
        echo "Bizarre : il n'y a pas cette imprimante dans ce groupe.";
        return true;
    } else {
        if ($FilterGroup[0]->parentNode->childNodes->length == 1) { // [parentNode:Filters] il y a un seul groupe associé à cette imprimante, on supprime l'imprimante de la gpo
            $SharedPrinter = $FilterGroup[0]->parentNode->parentNode;
            $SharedPrinter->parentNode->removeChild($SharedPrinter);

            $file->save($tmppath . "/Printers.xml");

            $ret = put_Printers_XML($config, $gpo_impr, $tmppath);
            if ($ret) {
                // echo "Fonction put_Printers_XML & remove_printer_group OK.";
                return true;
            } else {
                echo "Erreur put_Printers_XML & remove_printer_group.";
                return false;
            }
            return true;
        } else { // il y a plusieurs groupes associés à cette imprimante, on ne supprime que le filtre pour ce groupe
            $FilterGroup[0]->parentNode->removeChild($FilterGroup[0]);

            $file->save($tmppath . "/Printers.xml");

            $ret = put_Printers_XML($config, $gpo_impr, $tmppath);
            if ($ret) {
                // echo "Fonction put_Printers_XML & remove_printer_group OK.";
                return true;
            } else {
                echo "Erreur put_Printers_XML & remove_printer_group.";
                return false;
            }
        }
    }
}

/**
 * correction de la gpo imprimantes : **************************************
 * - les utilisateurs & ordinateurs gardaient les imprimantes en mémoire même en cas de changement de contexte
 *
 * @param unknown $config
 * @return boolean
 */
function correct_gpo($config)
{
    if ($config['printers_migrated'] == 1) {
        $gpo_impr = gpo_impr($config);

        $tmppath = get_Printers_XML($config, $gpo_impr);

        if ($tmppath) {
            // echo "Fonction get_Printers_XML dans correct_gpo OK.";
        } else {
            echo "Erreur get_Printers_XML dans correct_gpo.";
            $ret = rm_Printers_XML($config, $tmppath);
            return false;
        }

        $file = new DOMDocument();
        $file->formatOutput = true;
        $file->preserveWhiteSpace = false;
        $file->load($tmppath . "/Printers.xml");

        $xpath = new DOMXPath($file);
        $SharedPrinters = $xpath->query("/Printers/SharedPrinter");
        foreach ($SharedPrinters as $SharedPrinter) {
            $SharedPrinter->setAttribute('image', "1");
            $SharedPrinter->setAttribute('removePolicy', "1");
            $SharedPrinter->setAttribute('changed', date("Y-m-d h:i:s"));
        }

        $Properties = $xpath->query("/Printers/SharedPrinter/Properties");
        foreach ($Properties as $Property) {
            $Property->setAttribute('action', "R");
        }

        $file->save($tmppath . "/Printers.xml");

        $ret = put_Printers_XML($config, $gpo_impr, $tmppath);
        if ($ret) {
            // echo "Fonction put_Printers_XML & correct_gpo OK.";
        } else {
            echo "Erreur put_Printers_XML & correct_gpo.";
            return false;
        }

        set_param($config, "printers_migrated", "2");

        echo "Correction terminée.<BR>\n";
        return true;
    }
    return true;
}

?>