<?php

/**
 * Librairie de fonctions utilisees dans l'interface d'administration

 * @Version $Id: samba-tool.inc.php 2018-28-09  jlcf $

 * @Projet SambaEdu

 * @Note: Ce fichier de fonction doit etre appele par un include

 * @Licence Distribue sous la licence GPL
 */

/**
 * file: samba-tool.inc.php
 *
 * @Repertoire: includes/
 */

// =============================================================
// Ensemble de fonctions destinées à remplacer les scripts sudo perl
// pour les opérations d'écritures dans l'AD SambaEdu

/*
 * ATTENTION
 *
 * le OU rdn utilisé par ces fonctions est relatif au nom de domaine AD.
 *
 *
 * function useradd ($prenom, $nom, $userpwd, $naissance, $sexe, $categorie, $employeeNumber) : Return $cn if succes.
 *
 * function userdel ($cn) : Return true if userdel succes false if userdel fail
 *
 * function groupadd ($cn, $inou, $description) : Return true if group is create false in other cases
 *
 * function groupdel ($cn) : Return true if group is delete false in other cases
 *
 * function groupaddmember ( $cn, $ingroup) : Return true if cn is add in ingroup false in other cases
 *
 * function groupdelmember ($cn, $ingroup) : Return true if cn is remove of ingroup false in other cases
 *
 * A faire si nécessaire :
 * function grouplist ($filter)
 * function groupaddlistmembers ( $cnlist, $ingroup)
 *
 */

/*
 * Fonctions de siecle.inc.php utilisées dans samba-tool.inc.php
 * useradd() -> creer_cn($nom,$prenom) //modifiée
 * -> verif_employeeNumber($employeeNumber) // modifiée
 */
function sambatool($config, $command, &$message, $error = true, $host = true)
{
    $RET = 0;
    if ($error) {
        $redir = "2>&1";
    } else {
        $redir = "2>/dev/null";
    }
    $kerb_option = " --use-kerberos=required";
    if ($host) {
        $host_option = " " . ad_url($config, "sambatool");
    } else {
        $host_option = "";
    }
    exec("/usr/bin/samba-tool " . $command . $kerb_option . $host_option . " " . $redir, $message, $RET);
    return $RET;
}

function userexist($config, $cn)
{
    /*
     * Return true if user exist false if not exist
     */
    return (count(search_user($config, $cn)) > 0);
    // $command = "user list";
    // $RES = sambatool($config, $command, $message);
    // if ($RES == 0) {
    // if (in_array($cn, $message))
    // return true;
    // else
    // return false;
    // } else {
    // return false;
    // }
}

/**
 * Ajout utilisateur
 *
 * @description : Ajoute un utlisateur dans l'AD
 * @paramètres config, $cn : nom court, $prenom, $nom, $userpwd, $naissance, $sexe  : M ou F, $categorie : Eleves ou Profs ou Administratifs, $employeeNumber
 * @Returns true ou false.
 */
function useradd($config, $cn, $prenom, $nom, $userpwd, $naissance, $sexe, $categorie, $employeeNumber)
{
    $office = password_hash($naissance, PASSWORD_DEFAULT);

    if (! isset($userpwd)) {
        $userpwd = $naissance;
    }
    $userpwd = escapeshellarg($userpwd);
    $prenom = escapeshellarg(ucfirst($prenom));
    $nom = escapeshellarg(ucfirst($nom));

    if (empty($employeeNumber)) {
        // Pas de champ job-title pour employeeNumber dans ce cas
        $command = "user create '$cn' '$userpwd' --use-username-as-cn --given-name=$prenom --surname=$nom --mail-address='$cn@" . $config['domain'] . "' --physical-delivery-office='$office'";
        if ($categorie != '')
            $command = $command . " --userou='ou=$categorie," . $config['people_rdn'] . "'";
    } else {
        $command = "user create '$cn' '$userpwd' --use-username-as-cn --given-name=$prenom --surname=$nom --mail-address='$cn@" . $config['domain'] . "' --job-title='$employeeNumber' --physical-delivery-office='$office'";
        if ($categorie != '')
            $command = $command . " --userou='ou=$categorie," . $config['people_rdn'] . "'";
    }
    $message = array();
    $RES = sambatool($config, $command, $message);
    if ($RES == 0) {
        $newcn = explode("'", $message[0]);
        // Ajout a un groupe principal
        $RES = groupaddmember($config, $newcn[1], $categorie);
        apcu_store("ldap_cache_invalid", true, 60);
        return true;
    } else
        return false;
}

/**
 * Suppression utilisateur
 *
 * @description : Supprime un utlisateur dans l'AD
 * @paramètres config, $cn
 * @Returns true ou false.
 */
function userdel($config, $cn)
{
    $user = search_ad($config, $cn, "user");
    if (isset($user[0]['dn'])) {
        $ret = ldap_delete($config['bind'], $user[0]['dn']);
        apcu_store("ldap_cache_invalid", true, 60);
        if (! $ret) {
            trigger_error("userdel ldap_error: " . ldap_error($config['bind']));
        }
        return $ret;
    } else {
        return true;
    }
}

/**
 * Passwd utilisateur
 *
 * @description : Met en place le mot de passe d'un utilisateur
 *
 * connexion a l'ad avec les droits admin.
 *
 * @paramètres $config, $dn, $password, $change = false
 * @Returns true ou false.
 */
function usersetpassword($config, $dn, $newpassword, $change = false, $oldpassword = "", $stack = false, &$html = "")
{
    if ($stack) {
        lock('passwords');
        $passwords = apcu_fetch("passwords");
        if (! is_array($passwords)) {
            $passwords = [];
        }
        if (isset($passwords[$dn])) {
            $passwords[$dn] = [
                'cn' => ldap_dn2cn($dn),
                'newpassword' => $newpassword,
                'change' => $change,
                'oldpassword' => $oldpassword
            ];
        } else {
            $passwords[$dn] = [
                'cn' => ldap_dn2cn($dn),
                'newpassword' => $newpassword,
                'change' => $change,
                'oldpassword' => $oldpassword
            ];
        }
        apcu_store('passwords', $passwords);
        unlock('passwords');
        return true;
    }
    if (! preg_match("/" . $config['dn']['people'] . "/", $dn)) {
        $user = search_user($config, $dn);
        if (count($user) > 1) {
            $dn = $user['dn'];
        }
    }
    $ret = false;
    if (! empty($dn)) {
        if ($config['bind'] && ! empty($newpassword)) {
            if (! empty($oldpassword)) {
                $attrs = [
                    [
                        "attrib" => "unicodepwd",
                        "modtype" => LDAP_MODIFY_BATCH_REMOVE,
                        "values" => [
                            iconv("UTF-8", "UTF-16LE", "\"" . $oldpassword . "\"")
                        ]
                    ],
                    [
                        "attrib" => "unicodepwd",
                        "modtype" => LDAP_MODIFY_BATCH_ADD,
                        "values" => [
                            iconv("UTF-8", "UTF-16LE", "\"" . $newpassword . "\"")
                        ]
                    ],
                    [
                        "attrib" => "pwdlastset",
                        "modtype" => LDAP_MODIFY_BATCH_REPLACE,
                        "values" => [
                            - 1
                        ]
                    ]
                ];
                $ret = ldap_modify_batch($config['bind'], $dn, $attrs);
            } else {
                $attrs['unicodepwd'] = iconv("UTF-8", "UTF-16LE", "\"" . $newpassword . "\"");
                if ($change) {
                    $attrs['pwdlastset'] = 0;
                }
                $ret = ldap_mod_replace($config['bind'], $dn, $attrs);
            }
            // ldap_close($config['bind']);
            file_put_contents("/tmp/ldap_err.log", $dn . ":   \n", FILE_APPEND);
            if ($change) {
                $reinit = "reinit";
            } else {
                $reinit = "";
            }

            if (! $ret) {
                file_put_contents("/tmp/ldap_err.log", "ERREUR $reinit : old : " . $oldpassword . " new : " . $newpassword . "\n", FILE_APPEND);

                if (ldap_errno($config['bind']) == 19) {
                    $html .= "le mot de passe est trop simple ou a déjà été utilisé.";
                } else {
                    $html .= ldap_error($config['bind']);
                }
                // file_put_contents("/tmp/ldap_err.log", "Erreur : " . $html . "\n", FILE_APPEND);
                // else
                // pas d'invalidation du cache
                // apcu_store("ldap_cache_invalid", true, 60);
            } else {
                $html .= "le mot de passe a été changé";
                file_put_contents("/tmp/ldap_err.log", "$reinit OK\n", FILE_APPEND);
            }
        }
    }
    return $ret;
}

function set_password_rule($config, $complexity = NULL, $length = NULL, $history = 1)
{
    $command = "domain passwordsettings set ";
    if (! empty($complexity)) {
        $command .= " --complexity=" . $complexity;
    }
    if (! empty($length)) {
        $command .= " --min-pwd-length=" . $length;
    }
    $command .= " --history-length=" . $history;
    $RES = sambatool($config, $command, $message);
    if ($RES == 0) {
        apcu_delete('password_rule_');
        return true;
    } else
        return false;
}

function get_password_rule($config, $user = "")
{
    $conf = apcu_fetch('password_rule_' . $user);
    if (! is_array($conf)) {
        $map = array(
            "Password complexity" => "complexity",
            "Password history length" => "history-length",
            "Minimum password length" => "min-pwd-length"
        );

        if (empty($user)) {
            $command = "domain passwordsettings show";
        } else {
            $command = "domain passwordsettings pso show-user " . $user;
        }
        $RES = sambatool($config, $command, $message, false);
        if ($RES == 0) {
            $conf = [];
            foreach ($message as $ligne) {
                if (preg_match("/^(.*)\s?:\s?(.*)$/", $ligne, $m)) {
                    if (isset($map[$m[1]])) {
                        $m[1] = $map[$m[1]];
                    }
                    $conf[$m[1]] = $m[2];
                }
            }
            apcu_store('password_rule_' . $user, $conf);
        } else {
            $conf = false;
        }
    }
    return $conf;
}

function set_password_activation_rule($config)
{
    if (! groupexist($config, "ent_activation")) {
        groupadd($config, "ent_activation", $config['groups_rdn'], "", $config['suffix']);
        $command = "domain passwordsettings pso create \"activation\" 1 --complexity=off --min-pwd-length=8";
        $RES = sambatool($config, $command, $message);
        $command = "domain passwordsettings pso apply \"activation\" \"ent_activation" . $config['suffix'] . "\"";
        $RES += sambatool($config, $command, $message);
        if ($RES == 0)
            return true;
        else
            return false;
    }
    return true;
}

function apply_user_pso($config, $user)
{
    $command = "domain passwordsettings pso apply \"activation\" " . $user;
    $RES = sambatool($config, $command, $message);
    if ($RES == 0)
        return true;
    else
        return false;
}

function unapply_user_pso($config, $user)
{
    $command = "domain passwordsettings pso unapply \"activation\" " . $user;
    $RES = sambatool($config, $command, $message);
    if ($RES == 0)
        return true;
    else
        return false;
}

/*
 * Gestion des OU
 */

/**
 * Test existence OU
 *
 * @description : vérifie l'existence de l'ou dans l'AD
 * @paramètres $config, $ou, $ouparent_rdn
 * @Returns true ou false.
 */
function ouexist($config, $dn)
{
    if ($config['bind']) {
        list ($ou, $parent) = explode(",", $dn, 2);
        $ret = @ldap_read($config['bind'], $dn, "(objectclass=organizationalunit)");
        if ($ret) {
            $info = ldap_get_entries($config['bind'], $ret);
        } else {
            // trigger_error(" " . $dn . " ");
        }
        if (isset($info["count"]) && $info["count"] > 0) {
            return true;
        } else {
            return false;
        }
    } else {
        echo "Echec du bind sasl";
        return false;
    }
}

/**
 * Ajout OU
 *
 * @description : Ajoute l'ou dans l'AD
 * @paramètres $config, $ou, $ouparent_rdn ou $config, $dn
 * @Returns true ou false.
 */
function ouadd($config, $ou, $ouparent_rdn = "", $prefix = "ou")
{

    /*
     * Return true if OU is create or if there already exists false in other cases
     */
    if (empty($ouparent_rdn)) {
        if (strcasecmp("ou", explode("=", $ou, 2)[0]) != 0) {
            trigger_error("#   " . debug_backtrace()[1]['function'] . " " . $ou . "   #");
            return true;
        }
        $dn = $ou;
        $ou_parent = ldap_dn2oudn($dn);
        $ou = ldap_dn2ou($dn);
        list ($prefix, $ou) = explode("=", $ou);
    } else {
        $ou_parent = $ouparent_rdn . "," . $config['ldap_base_dn'];
        $dn = $prefix . "=" . $ou . "," . $ou_parent;
    }

    // Ajoute le OU si il n'existe pas
    if (! ouexist($config, $dn)) {
        // Prépare les données
        $info = array();
        $info[$prefix] = $ou;
        // $info["name"] = "$ou";
        $info["objectClass"] = "organizationalUnit";
        // Ajout
        // $config['bind'] = bind_ad_gssapi($config);
        ldap_add($config['bind'], $dn, $info);
        $err = ldap_error($config['bind']);
        // ldap_close($config['bind']);
        if (ouexist($config, $dn)) {
            apcu_store("ldap_cache_invalid", true, 60);
            return true;
        } else {
            trigger_error("#   " . debug_backtrace()[1]['function'] . $err . " : " . $dn . ", " . $ou . "   #");
            return false;
        }
    } else {
        // le OU existe déja
        return true;
    }
}

/**
 * Suppression OU
 *
 * @description : supprime l'ou dans l'AD
 * @paramètres $config, $ou, $ouparent_rdn, $prefixe = ou, ou alors $ou=dn complet
 * @Returns true ou false.
 */
function oudel($config, $ou, $ouparent_rdn = "", $prefix = "ou")
{
    /*
     * Return true if OU is remove false in other cases
     */
    if (empty($ouparent_rdn)) {
        $dn = $ou;
        $ou_parent = ldap_dn2oudn($dn);
        $ou = ldap_dn2ou($dn);
        list ($prefix, $ou) = explode("=", $ou);
    } else {
        $ou_parent = $ouparent_rdn . "," . $config['ldap_base_dn'];
        $dn = $prefix . "=" . $ou . "," . $ou_parent;
    }

    if (ouexist($config, $dn)) {
        // $config['bind'] = bind_ad_gssapi();
        // Verifier si le OU est vide !
        if (! ouexist($config, $dn)) {
            // On efface le OU
            ldap_delete($config['bind'], $dn);
            // ldap_close($config['bind']);
        }
        if (! ouexist($ou, $dn)) {
            apcu_store("ldap_cache_invalid", true, 60);
            return true;
        } else {
            return false;
        }
    } else {
        return true;
    }
}

/*
 * Samba-Tool
 * Available subcommands:
 * add - Creates a new AD group.
 * addmembers - Add members to an AD group.
 * delete - Deletes an AD group.
 * list - List all groups.
 * listmembers - List all members of an AD group.
 * removemembers - Remove members from an AD group.
 */

/**
 * Listing d'un groupe
 *
 * @description : donne la liste des membres d'un groupe
 * ne fonctionne pas pour les machines !
 * @paramètres $config, $cn
 * @Returns array() ou false.
 */
function grouplistmembers($config, $cn)
{

    /*
     * Return array of member's cn
     */
    $message = array();
    $command = "group listmembers '$cn'";
    $res = sambatool($config, $command, $message);

    if ($res == 0)
        return $message;
    else
        return false;
}

/**
 * test existence groupe
 *
 * @description : vérifie si le groupe existe dans l'AD
 * @paramètres $config, $cn
 * @Returns true ou false.
 */
function groupexist($config, $cn)
{

    /*
     * Return true if cn group exist
     */
    $group = search_ad($config, $cn, "group_fast");
    return isset($group[0]['dn']);
}

/**
 * corrige les incohérences éventuelles au niveau du samaccountname
 * recherche tous les *_nom dans le contexte etab
 *
 * @param array $config
 * @param string $name
 *            nom du groupe péda
 * @return boolean
 */
function groupcheck(array $config, string $name)
{
    $groups = search_ad($config, $name, "group_fast");
    $res = true;
    foreach ($groups as $group) {
        if ($group['samaccountname'] != $group['cn'] . etab_suffix($group['etab'])) {
            $attrs = [
                'samaccountname' => $group['cn'] . etab_suffix($group['etab'])
            ];
            $res &= modify_ad($config, $group, "", $attrs);
        }
    }
    return $res;
}

/**
 * Ajout groupe
 *
 * @description : ajoute un groupe dans l'AD
 * @paramètres $config,
 * @paramètres $samaccountname : cn avec suffixe -XXXXY si multi étab
 * @paramètres $ou_rdn : branche du groupe (groups, parcs...)
 * @paramètres $description
 * @paramètres $suffix -XXXXY si multi-etab (si pas déjà dans le samaccountname)
 * @paramètres $id
 * @Returns true ou false.
 */
function groupadd($config, $name, $ou_rdn = "", $description = "", $suffix = "", $id = "")
{
    $cn = ldap_sam2cn($name);
    if (empty(ldap_sam2suffix($name)) && ! empty($suffix)) {
        $cn = $name;
        $name = $name . $suffix;
    }
    if (empty($ou_rdn)) {
        $ou_rdn = $config['groups_rdn'];
    }
    if (! empty($name)) {
        $groups = search_ad($config, $name, "group", $ou_rdn . "," . $config['ldap_base_dn']);
    } else {
        $groups = false;
    }
    if (! is_array($groups) || count($groups) == 0) {
        // Prépare les données
        $info = array();
        $info["cn"] = "$cn";
        $info["objectclass"] = array(
            "top",
            "group"
        );
        $info["samaccountname"] = $name;
        $info["grouptype"] = 0x80000002;
        if (! empty($description)) {
            $info["description"] = $description;
        }
        if (! empty($id)) {
            $info["info"] = $id;
        }
        // Ajout
        // $config['bind'] = bind_ad_gssapi($config);
        $ret = ldap_add($config['bind'], "cn=" . $cn . "," . $ou_rdn . "," . $config['ldap_base_dn'], $info);
        if ($ret) {
            if (preg_match("/rights/", $ou_rdn)) {
                apcu_store("ldap_rights_cache_invalid", true, 60);
            } elseif (preg_match("/delegations/", $ou_rdn)) {
                apcu_store("ldap_delegation_cache_invalid", true, 60);
            } else {
                apcu_store("ldap_cache_invalid", true, 60);
            }
        } else {
            $err = ldap_error($config['bind']);
            trigger_error("sam : " . $name . ", cn=" . $cn . "," . $ou_rdn . "," . $config['ldap_base_dn'] . " : " . $err);
        }
        return $ret;
    } else
        return true;
}

/**
 * Ajout groupe
 *
 * @description : ajoute un groupe dans l'AD
 * @paramètres $config,
 * @paramètres $samaccountname : cn avec suffixe -XXXXY si multi étab
 * @paramètres $ou_rdn : branche du groupe (groups, parcs...)
 * @paramètres $description
 * @paramètres $suffix -XXXXY si multi-etab (si pas déjà dans le samaccountname)
 * @paramètres $id
 * @Returns true ou false.
 */
function groupaddbydn($config, $groupdn, $description = "", $id = "")
{
    $info = array();
    $info["cn"] = ldap_dn2cn($groupdn);
    $info["objectclass"] = array(
        "top",
        "group"
    );
    $info["samaccountname"] = ldap_dn2sam($groupdn);
    $info["grouptype"] = 0x80000002;
    if (! empty($description)) {
        $info["description"] = $description;
    }
    if (! empty($id)) {
        $info["info"] = $id;
    }
    // Ajout
    // $config['bind'] = bind_ad_gssapi($config);
    $ret = ldap_add($config['bind'], $groupdn, $info);
    if ($ret) {
        if (preg_match("/rights/", $groupdn)) {
            apcu_store("ldap_rights_cache_invalid", true, 60);
        } elseif (preg_match("/delegations/", $groupdn)) {
            apcu_store("ldap_delegation_cache_invalid", true, 60);
        } else {
            apcu_store("ldap_cache_invalid", true, 60);
        }
    } else {
        $err = ldap_error($config['bind']);
        trigger_error("dn : " . $groupdn . " : " . $err);
    }
    return $ret;
}

/**
 * Suppression d'un groupe
 *
 * @description Principe : supprime un groupe dans l'AD
 * @exemple groupdel($config, $cn)
 * @parametres $cn : cn du groupe a supprimer
 * @returns true if group is delete false in other cases
 */
function groupdel($config, $groupsam)
{
    $group = search_ad($config, $groupsam, "groupsam");
    if (isset($group[0]['dn'])) {
        $ret = ldap_delete($config['bind'], $group[0]['dn']);
        if ($ret) {
            if (preg_match("/rights/", $group[0]['dn'])) {
                apcu_store("ldap_rights_cache_invalid", true, 60);
            } elseif (preg_match("/delegations/", $group[0]['dn'])) {
                apcu_store("ldap_delegation_cache_invalid", true, 60);
            } else {
                apcu_store("ldap_cache_invalid", true, 60);
            }
        } else {
            $err = ldap_error($config['bind']);
            trigger_error($err);
        }
        return $ret;
    } else {
        return true;
    }
}

/**
 * Suppression d'un groupe avec le dn
 *
 * @description Principe : supprime un groupe dans l'AD
 * @exemple groupdel($config, $dn)
 * @parametres $dn : dn du groupe a supprimer
 * @returns true if group is delete false in other cases
 */
function groupdelbydn($config, $dn)
{
    $group = search_ad($config, $dn, "dn");
    if (isset($group[0]['dn'])) {
        $ret = ldap_delete($config['bind'], $dn);
        if ($ret) {
            if (preg_match("/rights/", $group[0]['dn'])) {
                apcu_store("ldap_rights_cache_invalid", true, 60);
            } elseif (preg_match("/delegations/", $group[0]['dn'])) {
                apcu_store("ldap_delegation_cache_invalid", true, 60);
            } else {
                apcu_store("ldap_cache_invalid", true, 60);
            }
        } else {
            $err = ldap_error($config['bind']);
            trigger_error($err);
        }
        return $ret;
    } else {
        return true;
    }
}

/**
 * Ajout de membres dans un groupe
 *
 *
 * @description ajoute des membres à un groupe existant dans l'AD
 * @parametres $config, $member (cn ou array)
 * $groupsam : samaccountname
 * @returns true ou false
 */
function groupaddmember($config, $member, $groupsam, $invalidate_cache = true)
{
    if (empty($groupsam)) {
        return true;
    }
    if (! isset($member['dn']) && ! is_array($member)) {
        $m = search_user($config, $member, "all", true);
        if (! isset($m['dn'])) {
            $m = search_machine($config, $member);
            if (! isset($m['dn'])) {
                $m = search_group($config, $member);
                if (! isset($m['dn'])) {
                    return false;
                }
            }
        }
        $member = $m;
    }
    if (isset($groupsam['dn'])) {
        $group[0] = $groupsam;
    } else {
        // apcu_store("ldap_cache_invalid", true, 60);
        $group = search_ad($config, $groupsam, "group_fast", "all", [], "subtree", false, true);
    }
    if (! empty($group[0]['dn']) && ! empty($member['dn'])) {
        if (! isset($member['memberof']) || ! in_array($group[0]['dn'], $member['memberof'])) {
            // Prépare les données
            $info = array();
            $info["member"] = $member['dn'];
            // if (! isset($group[0]['member']) || (isset($group[0]['member']) && ! in_array($member['dn'], $group[0]['member']))) {
            $ret = ldap_mod_add($config['bind'], $group[0]['dn'], $info);
            if ($ret) {
                if ($invalidate_cache) {
                    if (preg_match("/rights/", $group[0]['dn'])) {
                        apcu_store("ldap_rights_cache_invalid", true, 60);
                    } elseif (preg_match("/delegations/", $group[0]['dn'])) {
                        apcu_store("ldap_delegation_cache_invalid", true, 60);
                    } else {
                        apcu_store("ldap_cache_invalid", true, 60);
                    }
                }
            } else {
                $err = ldap_error($config['bind']) . " : " . print_r($group[0]['dn'], true) . " : " . print_r($info, true);
                trigger_error($err);
            }
            return $ret;
        } else {
            return true;
        }
    } else
        return false;
}

/**
 * Suppression de membres dans un groupe
 * attention les machines ont un $ à la fin
 *
 * @description supprime des membres à un groupe existant dans l'AD
 * @parametres $config, $cn,
 * $ingroup samaccountname
 * @returns true ou false
 */
function groupdelmember($config, $cn, $groupsam, $invalidate_cache = true)
{
    if (! isset($cn['dn']) && ! is_array($cn)) {
        $m = search_user($config, $cn);
        if (! isset($m['dn'])) {
            $m = search_machine($config, $cn);
            if (! isset($m['dn'])) {
                $m = search_group($config, $cn);
                if (! isset($m['dn'])) {
                    trigger_error($cn . " n'existe pas");
                    return true;
                }
            }
        }
        $cn = $m;
    }
    // apcu_store("ldap_cache_invalid", true, 60);
    $groups = search_ad($config, $groupsam, "group", "all", [], "subtree", false, true);
    if (! empty($cn['dn']) && ! empty($groups[0]['dn']) && isset($cn['memberof']) && is_array($cn['memberof'])) {
        $ret = true;
        foreach ($groups as $group) {
            foreach ($cn['memberof'] as $gdn) {
                if (strcasecmp($group['dn'], $gdn) == 0) {
                    $info = array();
                    $info["member"] = $cn['dn'];
                    $ret &= ldap_mod_del($config['bind'], $gdn, $info);
                    $err = ldap_error($config['bind']);
                    if ($ret) {
                        if ($invalidate_cache) {
                            if (preg_match("/rights/", $group['dn'])) {
                                apcu_store("ldap_rights_cache_invalid", true, 60);
                            } elseif (preg_match("/delegations/", $group['dn'])) {
                                apcu_store("ldap_delegation_cache_invalid", true, 60);
                            } else {
                                apcu_store("ldap_cache_invalid", true, 60);
                            }
                        }
                    } else {
                        $err = ldap_error($config['bind']) . " : " . $gdn . " : " . print_r($info, true);
                        trigger_error($err);
                    }
                }
            }
        }
        return $ret;
    } else {
        // trigger_error($groupsam . " existe pas");
        return true;
    }
}

/**
 * Suppression de membres dans un groupe
 *
 * @description supprime des membres à un groupe existant dans l'AD methode rapide
 * @parametres $config
 * $memberdn,
 * $groupdn
 * @returns true ou false
 */
function groupdelmemberbydn($config, $memberdn, $groupdn, $invalidate_cache = true)
{
    $info = array();
    $info["member"] = $memberdn;
    $ret = ldap_mod_del($config['bind'], $groupdn, $info);
    $err = ldap_error($config['bind']);
    if (! $ret && preg_match("/unwilling/", $err)) {
        $ret = true;
    }
    if ($ret) {
        if ($invalidate_cache) {
            if (preg_match("/rights/", $groupdn)) {
                apcu_store("ldap_rights_cache_invalid", true, 60);
            } elseif (preg_match("/delegations/", $groupdn)) {
                apcu_store("ldap_delegation_cache_invalid", true, 60);
            } else {
                apcu_store("ldap_cache_invalid", true, 60);
            }
        }
    } else {
        $err = ldap_error($config['bind']) . " : " . $groupdn . " : " . print_r($info, true);
        trigger_error($err);
    }
    return $ret;
}

/**
 * Ajout d'un membre à un groupe methode rapide
 *
 *
 * @description ajoute un membre à un groupe, methode rapide !
 * @parametres $config
 * $memberdn,
 * $groupdn
 * @returns true ou false
 */
function groupaddmemberbydn($config, $memberdn, $groupdn, $invalidate_cache = true)
{
    $info = array();
    $info["member"] = $memberdn;
    $ret = ldap_mod_add($config['bind'], $groupdn, $info);
    $err = ldap_error($config['bind']);
    if (! $ret && preg_match("/Already exists/", $err)) {
        $ret = true;
    }
    if ($ret) {
        if ($invalidate_cache) {
            if (preg_match("/rights/", $groupdn)) {
                apcu_store("ldap_rights_cache_invalid", true, 60);
            } elseif (preg_match("/delegations/", $groupdn)) {
                apcu_store("ldap_delegation_cache_invalid", true, 60);
            } else {
                apcu_store("ldap_cache_invalid", true, 60);
            }
        }
    } else {
        $err = ldap_error($config['bind']) . " : " . $groupdn . " : " . print_r($info, true);
        trigger_error($err);
    }
    return $ret;
}

/**
 *
 * @description retourne la liste des gpo active pour l'utilisateur ou la machine
 *
 * @paramètres array $config, string $cn : utilisateur, machine...
 * @returns array liste des gpo [ uuid, displayname ]
 */
function gpolist(array $config, string $cn)
{
    $message = array();
    $command = "gpo list " . escapeshellarg($cn);
    $RES = sambatool($config, $command, $message);
    $gpo = array();
    $match = array();
    if ($RES == 0) {
        foreach ($message as $key => $ligne) {
            preg_match("/^\s*(.*)\s({.*})$/", $ligne, $match);
            if (isset($match[1]) && isset($match[2])) {
                $gpo[$key]['uuid'] = $match[2];
                $gpo[$key]['displayname'] = $match[1];
            }
        }
    }
    return $gpo;
}

function gposetlink(array $config, string $container, string $gpo, bool $enforce = false, bool $disable = false)
{
    $message = array();
    $command = "gpo getlink " . escapeshellarg($container);
    $RES = sambatool($config, $command, $message);
    if ($RES != 0) {
        $attrs = array(
            'gplink' => array()
        );
        // $config['bind'] = bind_ad_gssapi($config);
        $res = ldap_mod_del($config['bind'], $container, $attrs);
        // ldap_close($config['bind']);
    }

    $command = "gpo setlink " . escapeshellarg($container) . " " . escapeshellarg($gpo);
    if ($enforce)
        $command .= " --enforce";
    if ($disable)
        $command .= " --disable";
    $RES = sambatool($config, $command, $message);
    if ($RES == 0) {
        apcu_store("ldap_cache_invalid", true, 60);
        return true;
    }
    return false;
}

function gpolistcontainers(array $config, string $gpo)
{
    $message = array();
    $command = "gpo listcontainers " . escapeshellarg($gpo);
    $RES = sambatool($config, $command, $message);
    $ou = array();
    if ($RES == 0) {
        foreach ($message as $ligne) {
            $match = array();
            if (preg_match("/^\s*dn: (.*)$/", strtolower($ligne), $match)) {
                $ou[] = $match[1];
            }
        }
    }
    return $ou;
}

function gpogetlink(array $config, string $container)
{
    $message = array();
    $command = "gpo getlink " . escapeshellarg($container);
    $RES = sambatool($config, $command, $message);
    $gpo = array();
    if ($RES == 0) {
        $key = 0;
        foreach ($message as $ligne) {
            $match = array();
            if (preg_match("/^\s*(.*)\s*\:\s*(.*)$/", $ligne, $match)) {
                if (trim($match[1]) == "GPO") {
                    $gpo[$key]['uuid'] = $match[2];
                } elseif (trim($match[1]) == "Name") {
                    $gpo[$key]['displayname'] = $match[2];
                } elseif (trim($match[1]) == "Options") {
                    $gpo[$key]['options'] = $match[2];
                    $key ++;
                }
            }
        }
    }
    return $gpo;
}

function gpodellink(array $config, string $container, string $gpo)
{
    $message = array();
    $command = "gpo dellink " . escapeshellarg($container) . " " . escapeshellarg($gpo);
    $RES = sambatool($config, $command, $message);
    if (($RES == 0) || ($RES == 255)) {
        apcu_store("ldap_cache_invalid", true, 60);
        return true;
    }
    return false;
}

function gposetinheritance(array $config, string $container, bool $flag = true)
{
    $message = array();
    $command = "gpo setinheritance " . escapeshellarg($container);
    if ($flag)
        $message .= " inherit";
    else
        $message .= " block";
    $RES = sambatool($config, $command, $message);
    if ($RES == 0) {
        apcu_store("ldap_cache_invalid", true, 60);
        return true;
    }
    return false;
}

function gpogetinheritance(array $config, string $container)
{
    $message = array();
    $command = "gpo getinheritance " . escapeshellarg($container);
    $RES = sambatool($config, $command, $message);
    if ($RES == 0) {
        if (preg_match("/GPO_INHERIT/", $message[0]))
            return "inherit";
        else
            return "block";
    }
    return false;
}

function gpofetch(array $config, string $gpo, string $dir = "/var/www/sambaedu/temp/policies")
{
    $message = array();
    $command = "gpo fetch " . escapeshellarg($gpo) . " --tmpdir=" . escapeshellarg($dir);
    $RES = sambatool($config, $command, $message);
    if ($RES == 0) {
        return true;
    }
    return false;
}

function gpocreate(array $config, string $displayname, &$msg = "")
{
    $message = $match = array();
    $command = "gpo create " . escapeshellarg($displayname);
    $RES = sambatool($config, $command, $message);
    if ($RES == 0 && preg_match("/({.*})/i", implode("\n", $message), $match)) {
        apcu_store("ldap_cache_invalid", true, 60);
        return $match[1];
    } else {
        $msg .= implode("<br>", $message);
        return false;
    }
}

function gpodel(array $config, string $gpo)
{
    $message = array();
    $command = "gpo del " . escapeshellarg($gpo);
    $RES = sambatool($config, $command, $message);
    if (($RES == 0) || ($RES == 255)) {
        apcu_store("ldap_cache_invalid", true, 60);
        return true;
    }
    return false;
}

function dsacl_set(array $config, string $dn, string $sddl)
{
    $message = array();
    $command = "dsacl set --objectdn=" . escapeshellarg($dn) . " --sddl=" . escapeshellarg($sddl);
    $RES = sambatool($config, $command, $message);
    if (($RES == 0) || ($RES == 255)) {
        apcu_store("ldap_cache_invalid", true, 60);
        return true;
    }
    return false;
}

function dsacl_get(array $config, string $dn)
{
    $message = array();
    $command = "dsacl get --objectdn=" . escapeshellarg($dn);
    $RES = sambatool($config, $command, $message, false);
    if (($RES == 0) || ($RES == 255)) {
        $sddl = $message[1];
        return $sddl;
    }
    return false;
}

function dns_delete($config, $machine, $ip = "")
{
    $RET = 0;
    $message = array();
    if (isset($machine['cn'])) {
        $name = $machine['cn'];
    } else {
        $name = $machine;
    }
    if (empty($name)) {
        $host = gethostbyaddr($ip);
        if (empty($host) || $host == $ip) {
            return true;
        } else {
            $name = explode(".", $host)[0];
        }
    }
    putenv('RES_OPTIONS=retrans:1 retry:1 timeout:1 attempts:1');
    if (isset($name)) {
        if ($ips = gethostbynamel($name)) {
            foreach ($ips as $i) {
                $message = array();
                $command = "dns delete " . ad_url($config, "dns") . " " . $config['domain'] . " " . $name . " A " . $i;
                exec("/usr/bin/samba-tool " . $command . " --use-kerberos=required", $message, $RET);
            }
        }
    }
    echo "Suppression de l'enregistrement DNS " . $name . " : " . implode(", \n", $message) . "<br>\n";
    return ($RET == 0);
}

function dns_add($config, $machine, $ip = "")
{
    $RET = 0;
    $message = array();
    if (isset($machine['cn'])) {
        $name = strtolower($machine['cn']);
    } elseif (! preg_match("/(l-|dhcp-).*/", $machine) && ! empty($m = search_machine($config, $machine))) {
        $name = strtolower($m['cn']);
    }
    if (empty($name)) {
        return true;
    }
    if (empty($ip)) {
        if (empty($machine['iphostnumber'])) {
            return false;
        } else {
            $ip = $machine['iphostnumber'];
        }
    }
    putenv('RES_OPTIONS=retrans:1 retry:1 timeout:1 attempts:1');
    // enregistrements A
    $ips = gethostbynamel($name);
    if ($ips) {
        foreach ($ips as $i) {
            if ($i != $ip) {
                $message = array();
                $command = "dns delete " . ad_url($config, "dns") . " " . $config['domain'] . " " . $name . " A " . $i;
                exec("/usr/bin/samba-tool " . $command . " --use-kerberos=required", $message, $RET);
            }
        }
    } else {
        $command = "dns add " . ad_url($config, "dns") . " " . $config['domain'] . " " . $name . " A " . $ip;
        exec("/usr/bin/samba-tool " . $command . " --use-kerberos=required", $message, $RET);
    }
    echo $name . " : " . implode("\n", $message) . "\n";
    return ($RET == 0);
}

/**
 * crée l'enregistrement reverse dns si besoin ainsi que la zone.
 * utile pour les clients linux pour permettre le bon fonctionnement de nfsv4 avec secu kerberos.
 *
 * supprime l'enregistrement ne correspondant pas au fqdn si il existe.
 *
 * @param array $config
 * @param string $name
 * @param string $ip
 * @return boolean true si l'enregistrement existe.
 */
function dns_add_ptr(array $config, string $name, string $ip)
{
    // on ne fait rien, c'est trop lourd comme charge
    // return true;
    $RET = 0;
    $message = array();
    if (empty($name)) {
        return true;
    }
    $blocs = explode('.', $ip);
    $zone = $blocs[2] . "." . $blocs[1] . "." . $blocs[0] . ".in-addr.arpa";
    $host = gethostbyaddr($ip);
    if (empty($host) || $host == $ip) {
        $command = "dns zoneinfo " . ad_url($config, "dns") . " " . $zone;
        exec("/usr/bin/samba-tool " . $command . " --use-kerberos=required", $message, $RET);
        if ($RET != 0) {
            $RET = 0;
            $command = "dns zonecreate " . ad_url($config, "dns") . " " . $zone;
            exec("/usr/bin/samba-tool " . $command . " --use-kerberos=required", $message, $RET);
            if ($RET != 0) {
                // trigger_error($command . " : " . implode(" ", $message));
                return false;
            }
        } else {
            $message = [];
        }
    } elseif (strtolower($host) != $name . "." . $config['domain']) {
        $command = "dns delete " . ad_url($config, "dns") . " " . $zone . " " . $blocs[3] . " PTR " . strtolower($host);
        exec("/usr/bin/samba-tool " . $command . " --use-kerberos=required", $message, $RET);
        if ($RET != 0) {
            trigger_error($command . " : " . implode(" ", $message));
            return false;
        }
    } else {
        return true;
    }
    $RET = 0;
    $command = "dns add " . ad_url($config, "dns") . " " . $zone . " " . $blocs[3] . " PTR " . $name . "." . $config['domain'];
    exec("/usr/bin/samba-tool " . $command . " --use-kerberos=required", $message, $RET);
    echo $name . " : " . implode("\n", $message) . "\n";
    if ($RET != 0) {
        trigger_error($command . " : " . implode(" ", $message));
        return false;
    } else {
        return true;
    }
}

function dns_delete_ptr(array $config, string $ip)
{
    $RET = 0;
    $message = array();

    $blocs = explode('.', $ip);
    $zone = $blocs[2] . "." . $blocs[1] . "." . $blocs[0] . ".in-addr.arpa";
    $host = gethostbyaddr($ip);
    if (empty($host) || $host == $ip) {
        return true;
    } else {
        $command = "dns delete " . ad_url($config, "dns") . " " . $zone . " " . $blocs[3] . " PTR " . $host;
        exec("/usr/bin/samba-tool " . $command . " --use-kerberos=required", $message, $RET);
        echo $ip . " : " . implode("\n", $message) . "\n";
        return ($RET == 0);
    }
}

function dns_wpad($config)
{
    $RET = 0;
    putenv('RES_OPTIONS=retrans:1 retry:1 timeout:1 attempts:1');
    $message = array();
    $command = "dns query " . ad_url($config, "dns") . " " . $config['domain'] . " " . "wpad A";
    $wpad = explode(" ", trim(exec("/usr/bin/samba-tool " . $command . " --use-kerberos=required")))[1];
    $command = "dns delete " . $config['se4ad_name'] . " " . $config['domain'] . " " . "wpad" . " A " . $wpad;

    if (! empty($config['proxy_url']) && ($config['proxy_type'] != "aucun")) {
        $command = "dns add " . ad_url($config, "dns") . " " . $config['domain'] . " " . "wpad CNAME " . explode("/", $config['proxy_url'])[2] . ".";
    } else {
        $command = "dns query " . ad_url($config, "dns") . " " . $config['domain'] . " " . "wpad CNAME";
        $wpad = explode(" ", trim(exec("/usr/bin/samba-tool " . $command . " --use-kerberos=required")))[1];
        $command = "dns delete " . ad_url($config, "dns") . " " . $config['domain'] . " " . "wpad" . " CNAME " . $wpad;
    }
    exec("/usr/bin/samba-tool " . $command . " --use-kerberos=required", $message, $RET);
    if ($RET == 0) {
        return true;
    }
    return false;
}

function decode_dnsrecord($dnsrecord)
{
    $blob = base64_decode($dnsrecord);
    $record = unpack('Slength/Stype/Cversion/Crank/Sflags/Lserial/Nttl/Lreserved/Ltimestamp/a*data', $blob);
    $soa = unpack('Lserial/Lrefresh/Lretry/Lexpire/Lttl/a*data', $record['data']);
    $prim = unpack('Clength/a*data', $soa['data']);
    return $prim;
}

/** declenche la réplication de la partition domaine entre 2 AD ou vers un AD distant
 * 
 * ne fait rien dans le cas où il n'y a pas de réplication
 * 
 * quand une modification est faite via l'interface sur un AD, il est nécessaire de la propager sur l'ad de l'étab
 * sur lequel l'utilisateur se connecte.
 * 
 * @param unknown $config
 * @param string $dest_uai : uai de destination
 * @param string $source_uai : uai source, par défaut l'AD central
 * @return boolean
 */
function replicate_ad($config, $dest_uai, $source_uai = 0)
{
    if ($dest_uai == $source_uai) {
        return true;
    }
    if (empty($source_uai)) {
        $source_ad = $config['se4ad_name'];
    }
    $dest_ad = "se4ad-" . $dest_uai;
    $message = array();
    $command = "drs replicate $dest_ad $source_ad " . $config['ldap_base_dn'];
    $RES = sambatool($config, $command, $message, false, false);
    return ($RES == 0);
}
?>