<?php
use Favicon\FaviconDLType;

function read_shortcut($config, $path)
{
    $shortcut = file($path);
    if (is_array($shortcut)) {
        foreach ($shortcut as $ligne) {
            $l = explode("=", $ligne, 2);
            if (count($l) == 2) {
                $s[$l[0]] = trim($l[1]);
            }
        }
        return $s;
    } else {
        return false;
    }
}

function read_shortcut_json()
{
    $filename = "/etc/sambaedu/applications/shortcuts/shortcuts.json";
    if (file_exists($filename)) {
        $shortcuts = json_decode(file_get_contents($filename), true) ?? [];
    } else {
        $shortcuts = [];
    }

    $path = "/usr/share/sambaedu/applications/shortcuts/";
    $files = scandir($path);
    foreach ($files as $file) {
        if (preg_match("/.*\.inc\.json/", $file)) {
            $shortcuts = array_merge_recursive(json_decode(file_get_contents($path . $file), true), $shortcuts);
        }
    }
    $path = "/etc/sambaedu/applications/shortcuts/";
    $files = scandir($path);
    foreach ($files as $file) {
        if (preg_match("/.*\.inc\.json/", $file)) {
            $shortcuts = array_merge_recursive(json_decode(file_get_contents($path . $file), true), $shortcuts);
        }
    }
    foreach ($shortcuts as $key => $value) {
        if (! isset($value["name"]))
            unset($shortcuts[$key]);
    }
    return $shortcuts;
}

function make_url_shortcut($config, $shortcut)
{
    $text = "[InternetShortcut]
    URL=
    HotKey=
    IconFile=
    IconIndex=
    ShowCommand=
    Modified=
    Roamed=
    IDList=
    Author=
    WhatsNew=
    Comment=
    Desc=
    ";
    return $text;
}

function make_shortcut($config, $name, $os, $user = "", $userprofile = "")
{
    // $shortcuts = json_decode(file_get_contents("/etc/sambaedu/applications/shortcuts/shortcuts.json"), true);
    $shortcuts = read_shortcut_json();
    $key = array_search($name, array_combine(array_keys($shortcuts), array_column($shortcuts, 'name')));
    $shortcut = $shortcuts[$key];

    if (! empty($shortcut['windows']) && $os == "windows") {
        if (empty($shortcut['windows']['icon'])) {
            $shortcut['windows']['icon'] = "%temp%\\" . $shortcut['name'] . ".ico";
        }
        $tmp_path = sys_get_temp_dir() . "/" . $shortcut['name'] . "-" . microtime();
        $shortcut['windows']['link'] = preg_replace("/\\\$userprofile/", $userprofile, $shortcut['windows']['link']);
        $shortcut['windows']['args'] = preg_replace("/\\\$userprofile/", $userprofile, $shortcut['windows']['args']);
        $shortcut['windows']['path'] = preg_replace("/\\\$userprofile/", $userprofile, $shortcut['windows']['path']);
        $shortcut['windows']['link'] = preg_replace("/\\\$user/", $user, $shortcut['windows']['link']);
        $shortcut['windows']['args'] = preg_replace("/\\\$user/", $user, $shortcut['windows']['args']);
        $shortcut['windows']['path'] = preg_replace("/\\\$user/", $user, $shortcut['windows']['path']);

        $cmd = "/usr/bin/mslink -l '" . $shortcut['windows']['link'] . "' -n '" . $shortcut['name'] . "' -a '" . $shortcut['windows']['args'] . "' -w '" . $shortcut['windows']['path'] . "' -i '" . $shortcut['windows']['icon'] . "'";
        // file_put_contents(sys_get_temp_dir() . "/mslink", iconv("UTF-8", "ISO-8859-15", preg_replace("/\\\\/", "\\\\\\\\", $cmd)));
        // file_put_contents(sys_get_temp_dir() . "/mslink", preg_replace("/\\\\/", "\\\\\\\\", $cmd));
        // exec(iconv("UTF-8", "ISO-8859-15", preg_replace("/\\\\/", "\\\\\\\\", $cmd)));
        $out = " -o \"" . $tmp_path . "\"";
        exec(iconv("UTF-8", "ISO-8859-15", $cmd) . $out);
        $ret = file_get_contents($tmp_path);
        unlink($tmp_path);
        return $ret;
    }

    if (! empty($shortcut['linux']) && $os == "linux") {
        $shortcut['linux']['link'] = preg_replace("/\\\$user/", $user, $shortcut['linux']['link']);
        $shortcut['linux']['args'] = preg_replace("/\\\$user/", $user, $shortcut['linux']['args']);
        $shortcut['linux']['path'] = preg_replace("/\\\$user/", $user, $shortcut['linux']['path']);

        $out = "#!/usr/bin/env xdg-open
[Desktop Entry]
Encoding=UTF-8
Type=Application
Terminal=false
StartupNotify=true
Categories=Application
Exec=" . $shortcut['linux']['link'] . " " . $shortcut['linux']['args'] . "
Hidden=false
Name=" . $shortcut['name'] . "
Comment=Raccourci ajouté par Sambaedu\n";
        if (! empty($shortcut['linux']['startupwmclass'])) {
            $out .= "StartupWMClass=" . $shortcut['linux']['startupwmclass'] . "\n";
        }
        switch ($shortcut['place']) {
            case "startup":
                // \${HOME}/.config/autostart/$name.desktop
                $out .= "X-GNOME-Autostart-enabled=true\n";
                $out .= "Icon=/home/" . $user . "/.local/share/icons/" . $shortcut['name'] . ".png\n";
                break;
            case "desktop":
            // ne fonctionne pas bien pour Gnome, le raccourci doit être activé manuellement à chaque fois !
            case "taskbar":
                // "\${HOME}/.local/share/applications/"
                $out .= "Icon=/home/" . $user . "/.local/share/icons/" . $shortcut['name'] . ".png\n";
                break;
        }
        if (! empty($shortcut['linux']['path'])) {
            $out .= "Path=" . preg_replace("/$\/home\/\$HOME/", "/home/" . $user, $shortcut['linux']['path']) . "\n";
        }

        return $out;
    }
}

function make_shortcut_icon($shortcut, $os = "linux")
{
    if (empty($shortcut)) {
        return (file_get_contents("../elements/images/system-run.png"));
    }
    if (file_exists("/usr/share/sambaedu/applications/shortcuts/" . $shortcut . ".png") && ! file_exists("/etc/sambaedu/applications/shortcuts/" . $shortcut . ".png")) {
        copy("/usr/share/sambaedu/applications/shortcuts/" . $shortcut . ".png", "/etc/sambaedu/applications/shortcuts/" . $shortcut . ".png");
    }
    if ($os == "windows") {
        if (file_exists("/etc/sambaedu/applications/shortcuts/" . $shortcut . ".ico")) {
            return (file_get_contents("/etc/sambaedu/applications/shortcuts/" . $shortcut . ".ico"));
        } else {
            if (file_exists("/etc/sambaedu/applications/shortcuts/" . $shortcut . ".png")) {
                try {
                    $imagick = new Imagick("/etc/sambaedu/applications/shortcuts/" . $shortcut . ".png");
                    $imagick->resizeimage(128, 128, Imagick::FILTER_QUADRATIC, 1);
                    $imagick->setImageFormat("png");
                    $imagick->setImageFormat("ico");
                    $imagick->writeImage("/etc/sambaedu/applications/shortcuts/" . $shortcut . ".ico");
                    return (file_get_contents("/etc/sambaedu/applications/shortcuts/" . $shortcut . ".ico"));
                } catch (ImagickException $e) {
                    return (file_get_contents("../elements/images/system-run.ico"));
                }
            } else {
                return (file_get_contents("../elements/images/system-run.ico"));
            }
        }
    } else {
        if (file_exists("/etc/sambaedu/applications/shortcuts/" . $shortcut . ".png")) {
            return (file_get_contents("/etc/sambaedu/applications/shortcuts/" . $shortcut . ".png"));
        } else {
            return (file_get_contents("../elements/images/system-run.png"));
        }
    }
}

function shortcut_create_script($config, $id, $os, $action)
{
    if ($info = apcu_fetch("apps." . $id)) {
        $user = $info['user'];
        $list_u = $info['list_u'];
        $list_m = $info['list_m'];
        $list_app = $info['liste_applications'];
        $script = "";
        if ($os == "windows" && $action == "logon") {
            $script .= "chcp 28605\r\n";
            $script .= "if exist \"%userprofile%\\AppData\\Roaming\\shortcuts.txt\" (for /F \"usebackq delims=\"  %%A in  (\"%userprofile%\\AppData\\Roaming\\shortcuts.txt\") do (del /f /q \"%%A\"))\r\n";
            $script .= "if exist \"%userprofile%\\AppData\\Roaming\\shortcuts.txt\" (del /f /q \"%userprofile%\\AppData\\Roaming\\shortcuts.txt\")\r\n";
        }
        // $shortcuts = json_decode(file_get_contents("/etc/sambaedu/applications/shortcuts/shortcuts.json"), true);
        $shortcuts = read_shortcut_json();
        if (count($list_u) > 0) {
            foreach ($shortcuts as $shortcut) {
                $shortcut_owner = preg_split('/(\s*,*\s*)*,+(\s*,*\s*)*/', $shortcut['owner']);
                // recherche de raccourcis liés à une application déployée (app.truc)
                $apps = [];
                foreach ($shortcut_owner as $s) {
                    $m = [];
                    if (preg_match("/^app\.(.*)$/", $s, $m)) {
                        $apps[] = $m[1];
                    }
                }
                if ((empty($apps) || count(array_intersect($apps, $list_app)) > 0) && count(array_intersect($shortcut_owner, $list_u)) > 0) {
                    if (! empty($shortcut['windows']['link']) && $os == "windows") {
                        if ($action == "logon") {
                            switch ($shortcut['place']) {
                                case "startup":
                                    $path = "%userprofile%\\AppData\\Roaming\\Microsoft\\Windows\\Start Menu\\Programs\\Startup\\";
                                    break;
                                case "desktop":
                                    $path = "\\\\%se4fs%\\users\\%username%\\Bureau\\";
                                    break;
                                case "taskbar":
                                    // @TODO non fonctionnel avec 10/11 il faut le refaire avec une GPO et un json généré.
                                    $path = "%userprofile%\\AppData\\Roaming\\Microsoft\\Internet Explorer\\Quick Launch\\User Pinned\\TaskBar\\";
                                    break;
                            }
                            $script .= "curl.exe --output \"" . iconv("UTF-8", "ISO8859-15", $path . $shortcut['name']) . ".lnk\" -F \"action=file\" -F \"shortcut=" . iconv("UTF-8", "ISO8859-15", $shortcut['name']) . "\" -F \"os=windows\" -F \"machine=%computername%\" -F \"user=%username%\" -F \"userprofile=%userprofile%\" \"http://" . $config['se4fs_name'] . "/gpo/shortcuts_out.php\"\r\n";
                            $script .= "echo " . iconv("UTF-8", "ISO8859-15", $path . $shortcut['name']) . ".lnk>>\"%userprofile%\\AppData\\Roaming\\shortcuts.txt\"\r\n";
                            $script .= "curl.exe --output \"" . iconv("UTF-8", "ISO8859-15", "%temp%\\" . $shortcut['name']) . ".ico\" -F \"action=icon\" -F \"shortcut=" . iconv("UTF-8", "ISO8859-15", $shortcut['name']) . "\" -F \"os=windows\" -F \"machine=%computername%\" -F \"user=%username%\"   -F \"userprofile=%userprofile%\" \"http://" . $config['se4fs_name'] . "/gpo/shortcuts_out.php\"\r\n";
                        } else {
                            switch ($shortcut['place']) {
                                case "startup":
                                    $path = "%userprofile%\\AppData\\Roaming\\Microsoft\\Windows\\Start Menu\\Programs\\Startup\\";
                                    break;
                                case "desktop":
                                    $path = "\\\\%se4fs%\\users\\%username%\\Bureau\\";
                                    break;
                                case "taskbar":
                                    $path = "%userprofile%\\AppData\\Roaming\\Microsoft\\Internet Explorer\\Quick Launch\\User Pinned\\TaskBar\\";
                                    break;
                            }
                            // $script .= "echo " . $path . $shortcut['name'] . ".lnk>>\"%userprofile%\\AppData\\Roaming\\shortcuts.txt\"\r\n";
                        }
                    } elseif (! empty($shortcut['linux']['link']) && $os == "linux") {
                        if (file_exists("/etc/sambaedu/applications/shortcuts/" . $shortcut['name'] . ".png")) {
                            $script .= "mkdir -p \${HOME}/.local/share/icons\n";
                            $icon = "\${HOME}/.local/share/icons/" . $shortcut['name'] . ".png";
                            $script .= "[ -f \"$icon\" ] || curl -s -o \"$icon\" -F \"shortcut=" . $shortcut['name'] . "\" -F \"action=icon\" -F \"os=linux\" http://" . $config['se4fs_name'] . "/gpo/shortcuts_out.php\n";
                        }
                        $script .= "# $action pour " . implode(" ", $list_u) . "->" . $shortcut['owner'] . "\n";
                        if ($action == "logon") {
                            switch ($shortcut['place']) {
                                case "startup":
                                    $path = "\${HOME}/.config/autostart/";
                                    break;
                                case "desktop":
                                    $path = "\${HOME}/Bureau/";
                                    break;
                                case "taskbar":
                                    $path = "\${HOME}/.local/share/applications/";
                                    $script .= "dconf read /org/gnome/shell/favorite-apps | grep -q \"" . $shortcut['name'] . ".desktop\"\n";
                                    $script .= "if [ \$? -eq 1 ]; then\n";
                                    $script .= "list=\$(dconf read /org/gnome/shell/favorite-apps|sed \"s/\[\(.*\)\]/[\\1, '" . $shortcut['name'] . ".desktop']/\")\n";
                                    $script .= "dconf write /org/gnome/shell/favorite-apps \"\$list\"\n";
                                    $script .= "fi\n";
                                    break;
                            }
                            $script .= "mkdir -p \"$path\"\n";
                            $script .= "/usr/bin/curl -s -o \"" . $path . $shortcut['name'] . ".desktop\" -F \"action=file\" -F \"os=linux\" -F \"user=" . $user['cn'] . "\" -F \"shortcut=" . $shortcut['name'] . "\"  http://" . $config['se4fs_name'] . "/gpo/shortcuts_out.php || true\n";
                            $script .= "chmod 755 \"" . $path . $shortcut['name'] . ".desktop\"\n";
                            $script .= "dbus-launch gio set \"" . $path . $shortcut['name'] . ".desktop\" \"metadata::trusted\" true\n";
                        } else {
                            switch ($shortcut['place']) {
                                case "startup":
                                    $path = "\${HOME}/.config/autostart/";
                                    break;
                                case "desktop":
                                    $path = "\${HOME}/Bureau/";
                                    // $script .= "rm -f \"" . $path . $shortcut['name'] . ".desktop\"\n";

                                    break;
                                case "taskbar":
                                    $path = "\${HOME}/.local/share/applications";
                                    $script .= "list=\$(dconf read /org/gnome/shell/favorite-apps|sed \"s/, \\\"\(" . $shortcut['name'] . "\.desktop\)\\\"//\")\n";
                                    $script .= "dconf write /org/gnome/shell/favorite-apps \"\$list\"\n";
                                    break;
                            }
                        }
                    }
                }
            }
        }
        // gestion des raccourcis machine
        // Windoze
        if ($os == "windows" && $action == "startup") {
            $script .= "chcp 28605\r\n";
            $script .= "if exist \"%TEMP%\\shortcuts.txt\" (for /F \"usebackq delims=\"  %%A in  (\"%TEMP%\\shortcuts.txt\") do (del /f /q \"%%A\"))\r\n";
            $script .= "if exist \"%TEMP%\\shortcuts.txt\" (del /f /q \"%TEMP%\\shortcuts.txt\")\r\n";
        }

        if (count($list_m) > 0) {
            foreach ($shortcuts as $shortcut) {
                $shortcut_owner = preg_split('/(\s*,*\s*)*,+(\s*,*\s*)*/', $shortcut['owner']);
                if (count(array_intersect($shortcut_owner, $list_m)) > 0) {
                    if (! empty($shortcut['windows']['link']) && $os == "windows") {
                        $script .= "curl.exe --output \"" . iconv("UTF-8", "ISO8859-15", "%temp%\\" . $shortcut['name']) . ".ico\" -F \"action=icon\" -F \"shortcut=" . iconv("UTF-8", "ISO8859-15", $shortcut['name']) . "\" -F \"os=windows\" -F \"machine=%computername%\" -F \"user=%username%\" \"http://" . $config['se4fs_name'] . "/gpo/shortcuts_out.php\"\r\n";
                        switch ($action) {
                            case "startup":
                                switch ($shortcut['place']) {
                                    case "startup":
                                        $path = "C:\\ProgramData\\Microsoft\\Windows\\Start Menu\\Programs\\StartUp\\";
                                        $script .= "curl.exe --output \"" . iconv("UTF-8", "ISO8859-15", $path . $shortcut['name']) . ".lnk\" -F \"action=file\" -F \"shortcut=" . $shortcut['name'] . "\" -F \"os=windows\" -F \"machine=%computername%\" -F \"user=%username%\" \"http://" . $config['se4fs_name'] . "/gpo/shortcuts_out.php\"\r\n";
                                        break;
                                    case "desktop":
                                        $path = "%public%\\Desktop\\";
                                        $script .= "curl.exe --output \"" . iconv("UTF-8", "ISO8859-15", $path . $shortcut['name']) . ".lnk\" -F \"action=file\" -F \"shortcut=" . $shortcut['name'] . "\" -F \"os=windows\" -F \"machine=%computername%\" -F \"user=%username%\" \"http://" . $config['se4fs_name'] . "/gpo/shortcuts_out.php\"\r\n";
                                        break;
                                }
                                break;
                            case "logon":
                                switch ($shortcut['place']) {
                                    case "taskbar":
                                        $path = "%AppData%\\Microsoft\\Internet Explorer\\Quick Launch\\User Pinned\\TaskBar\\";
                                        $script .= "curl.exe --output \"" . iconv("UTF-8", "ISO8859-15", $path . $shortcut['name']) . ".lnk\" -F \"action=file\" -F \"shortcut=" . $shortcut['name'] . "\" -F \"os=windows\" -F \"machine=%computername%\" -F \"user=%username%\" \"http://" . $config['se4fs_name'] . "/gpo/shortcuts_out.php\"\r\n";
                                        break;
                                }
                                break;
                            default:
                            // $script .= "del /f /q \"" . $path . $shortcut['name'] . ".lnk\"\r\n";
                        }
                        if (! empty($path)) {
                            $script .= "echo " . iconv("UTF-8", "ISO8859-15", $path . $shortcut['name']) . ".lnk>>\"%TEMP%\\shortcuts.txt\"\r\n";
                        }
                        // Linux
                    } elseif (! empty($shortcut['linux']['link']) && $os == "linux") {
                        $script .= "# $action pour " . implode(" ", $list_m) . " -> " . $shortcut['owner'] . "\n";
                        switch ($action) {
                            case "startup":
                                // pas d'icone dans ce cas !
                                switch ($shortcut['place']) {
                                    case "startup":
                                        $path = "/etc/xdg/autostart/";
                                        $script .= "/usr/bin/curl -s -o " . $path . $shortcut['name'] . ".desktop -F \"action=file\" -F \"os=linux\" -F \"shortcut=" . $shortcut['name'] . "\" http://" . $config['se4fs_name'] . "/gpo/shortcuts_out.php || true\n";
                                        $script .= "chmod 755 \"" . $path . $shortcut['name'] . ".desktop\"\n";
                                        break;
                                }
                                break;
                            case "logon":
                                if (file_exists("/etc/sambaedu/applications/shortcuts/" . $shortcut['name'] . ".png")) {
                                    $script .= "mkdir -p \${HOME}/.local/share/icons\n";
                                    $icon = "\${HOME}/.local/share/icons/" . $shortcut['name'] . ".png";
                                    $script .= "[ -f \"$icon\" ] || curl -s -o $icon -F \"shortcut=" . $shortcut['name'] . "\" -F \"action=icon\" -F \"os=linux\" http://" . $config['se4fs_name'] . "/gpo/shortcuts_out.php\n";
                                }
                                switch ($shortcut['place']) {
                                    case "desktop":
                                        $path = "\${HOME}/Bureau/";
                                        $script .= "/usr/bin/curl -s -o \"" . $path . $shortcut['name'] . ".desktop\" -F \"action=file\" -F \"os=linux\" -F \"user=" . $user['cn'] . "\" -F \"shortcut=" . $shortcut['name'] . "\"  http://" . $config['se4fs_name'] . "/gpo/shortcuts_out.php || true\n";
                                        $script .= "chmod 755 \"" . $path . $shortcut['name'] . ".desktop\"\n";
                                        $script .= "dbus-launch gio set \"" . $path . $shortcut['name'] . ".desktop\" \"metadata::trusted\" true\n";
                                        break;
                                    case "taskbar":
                                        $path = "\${HOME}/.local/share/applications";
                                        $script .= "dconf read /org/gnome/shell/favorite-apps | grep -q \"" . $shortcut['name'] . ".desktop\"\n";
                                        $script .= "if [ \$? -eq 1 ]; then\n";
                                        $script .= "list=\$(dconf read /org/gnome/shell/favorite-apps|sed \"s/\[\(.*\)\]/[\\1, '" . $shortcut['name'] . ".desktop']/\")\n";
                                        $script .= "dconf write /org/gnome/shell/favorite-apps \"\$list\"\n";
                                        $script .= "fi\n";
                                        $script .= "/usr/bin/curl -s -o \"" . $path . $shortcut['name'] . ".desktop\" -F \"action=file\" -F \"os=linux\" -F \"user=" . $user['cn'] . "\" -F \"shortcut=" . $shortcut['name'] . "\" http://" . $config['se4fs_name'] . "/gpo/shortcuts_out.php || true\n";
                                        $script .= "chmod 755 \"" . $path . $shortcut['name'] . ".desktop\"\n";
                                        $script .= "dbus-launch gio set \"" . $path . $shortcut['name'] . ".desktop\" \"metadata::trusted\" true\n";
                                        break;
                                }
                                break;
                            default:
                                switch ($shortcut['place']) {
                                    case "startup":
                                        $path = "\${HOME}/.config/autostart/";
                                        break;
                                    case "desktop":
                                        $path = "\${HOME}/Bureau/";
                                        $script .= "rm -f \"" . $path . $shortcut['name'] . ".desktop\"\n";
                                        break;
                                    case "taskbar":
                                        $path = "\${HOME}/.local/share/applications";
                                        $script .= "list=\$(dconf read /org/gnome/shell/favorite-apps|sed \"s/, '\(" . $shortcut['name'] . "\.desktop\)'//\")\n";
                                        $script .= "dconf write /org/gnome/shell/favorite-apps \$list\n";
                                        break;
                                }
                                break;
                        }
                    }
                }
            }
        }
        return $script;
    }
}

function export_shortcuts($config, $shortcuts)
{
    return file_put_contents("/etc/sambaedu/applications/shortcuts/shortcuts.json", json_encode($shortcuts, JSON_PRETTY_PRINT));
}

function get_wine_shortcuts($config)
{
    $path = "/home/" . $config['se4install_name'] . "/Bureau/";
    $w = [];
    $dir = scandir($path);
    if (is_array($dir)) {
        foreach ($dir as $file) {
            if (pathinfo($file, PATHINFO_EXTENSION) == "desktop") {
                if ($link = read_shortcut($config, $path . $file)) {
                    $i = 0;
                    if (preg_match("/^(.*) (wine .*)$/", $link['Exec'], $m)) {
                        $w[$i]['name'] = $link['Name'];
                        $w[$i]['linux']['link'] = $m[2];
                        if (isset($link['Path'])) {
                            $w[$i]['linux']['path'] = preg_replace("#/home/" . $config['se4install_name'] . "#", "\$HOME", $link['Path']);
                        }
                        $w[$i]['owner'] = "";
                        $w[$i]['place'] = "";
                        if (isset($link['StartupWMClass'])) {
                            $w[$i]['startupwmclass'] = $link['StartupWMClass'];
                        }
                        // on cherche l'icone
                        $icon = $link['Icon'];
                        $it = new RecursiveDirectoryIterator("/home/" . $config['se4install_name'] . "/.local/share/icons");
                        foreach (new RecursiveIteratorIterator($it) as $f) {
                            $name = pathinfo($f, PATHINFO_FILENAME);
                            if ($name == $icon) {
                                copy($f->getPathname(), "/etc/sambaedu/applications/shortcuts/" . $w[$i]['name'] . ".png");
                                break;
                            }
                        }
                        $i ++;
                    }
                }
            }
        }
    }
    return $w;
}

function select_expandable($key, $name, $choices, $values, $expand)
{
    $html = "";
    if ($expand) {
        $html = "<select name=\"$name[$key]\" onclick=\"submit();\">";
        foreach ($choices as $choice) {
            $html .= "<option value=\"$choice\" ";
            if (in_array($choice, $values)) {
                $html .= "selected";
            }
            $html .= ">$choice</option>";
        }
        $html .= "</select>";
    } else {
        $html = "<input name=\"expand[$key]\" type=\"submit\" value=\"";

        $html .= implode(" ", $values);
        $html .= "\>";
    }
    return $html;
}

/**
 * génère un formulaire pour un raccourci.
 * remplit le tableau $shortcuts[$key] si le POST[shortcut] contient des infos et que POST[valider][$key] est défini
 *
 * @param unknown $config
 * @param unknown $shortcuts
 * @param unknown $key
 * @return string
 */
function form_shortcut($config, &$shortcuts, $key, $type = "all")
{
    $shortcut = $_POST['shortcut'] ?? [];
    $html = "";
    $navs = [
        'default' => [
            'name' => "Navigateur par défaut",
            'windows' => "",
            'linux' => "firefox"
        ],
        "firefox" => [
            'name' => "Mozilla Firefox",
            'windows' => "Firefox.exe",
            'linux' => "firefox"
        ],
        "chrome" => [
            'name' => "Google Chrome",
            'windows' => "Chrome.exe",
            'linux' => "chromium"
        ],
        "edge" => [
            'name' => "Microsoft Edge",
            'windows' => "%windir%\explorer.exe",
            'command' => "microsoft-edge:",
            'linux' => "firefox"
        ]
    ];

    if (! empty($_POST['valider'][$key]) && ! empty($shortcut['name'])) {
        // on sauvegarde les valeurs en POST
        $shortcuts[$key] = $shortcut;
        if (! empty($_FILES['icon']['name'])) {
            $dest_file = sys_get_temp_dir() . "/" . $_FILES['icon']['name'];
            if (file_exists($dest_file)) {
                unlink($dest_file);
            }
            if (! is_uploaded_file($_FILES['icon']['tmp_name'])) {
                // return "<p style='color:red'>Impossible d'importer le fichier.</p><br>\n";
            }
            if (! move_uploaded_file($_FILES['icon']['tmp_name'], $dest_file)) {
                // return "<p style='color:red'>Impossible de copier le fichier.</p>\n";
            }
            if (! $imagick = new Imagick($dest_file)) {
                unset($_FILES);
                unset($_POST);
                return "<p style='color:red'>Format d'image invalide.</p><br>\n";
            }
            $imagick->resizeimage(128, 128, Imagick::FILTER_QUADRATIC, 1);
            $imagick->setImageFormat("png");
            $imagick->writeImage("/etc/sambaedu/applications/shortcuts/" . $shortcut['name'] . ".png");
            $imagick->setImageFormat("ico");
            $imagick->writeImage("/etc/sambaedu/applications/shortcuts/" . $shortcut['name'] . ".ico");
            unset($_FILES);
            unset($_POST);
        } elseif (preg_match("/^http/", $shortcut['windows']['args']) && ! file_exists("/etc/sambaedu/applications/shortcuts/" . $shortcut['name'] . ".png")) {
            // raccourci web récupéré automatiquement
            require_once (dirname(__FILE__) . '/../vendor/autoload.php');
            $favicon = new Favicon\Favicon();
            $settings = array(
                // Cache directory
                'dir' => sys_get_temp_dir() . "/",
                // Cache timeout in seconds
                'timeout' => 86400
            );
            $favicon->cache($settings);
            if ($f = $favicon->get($shortcut['windows']['args'], FaviconDLType::DL_FILE_PATH)) {
                $tmpfile = sys_get_temp_dir() . "/" . $f;
            } else {
                $tmpfile = dirname(__FILE__) . '/../elements/images/web.png';
            }
            if (! $imagick = new Imagick($tmpfile)) {
                return "<p style='color:red'>Format d'image invalide.</p><br>\n";
            }
            $imagick->resizeimage(128, 128, Imagick::FILTER_QUADRATIC, 1);
            $imagick->setImageFormat("png");
            $imagick->writeImage("/etc/sambaedu/applications/shortcuts/" . $shortcut['name'] . ".png");
            $imagick->setImageFormat("ico");
            $imagick->writeImage("/etc/sambaedu/applications/shortcuts/" . $shortcut['name'] . ".ico");
        }
    } else {
        // on remplit le formulaire à partir du fichier
        $shortcut = $shortcuts[$key];
    }

    // $html .= "<pre>" . print_r($shortcuts, true) . "</pre>";
    if (! empty($_POST['del'][$key])) {
        unset($shortcuts[$key]);
        unset($shortcut);
    } elseif (! empty($_POST['edit'][$key])) {

        $html .= '<h2><img src="../gpo/shortcuts_out.php?action=icon&shortcut=' . urlencode(@$shortcut['name']) . '&os=linux" height=50 width=50/>';
        $html .= 'Nom : ';
        $html .= '<input type="text" name="shortcut[name]" value="' . @$shortcut['name'] . '" size="20"  />';

        $html .= ' Icone  :  <input type="file" name="icon" value=""/><br>';
        $html .= 'Entrez les valeurs à modifier. Il est possible d\'utiliser "$user" pour définir le nom de l\'utilisateur dans les chemins du raccourci, et "$userprofile" pour l\'emplaçement réél du profil local Windows<br>';
        $html .= '<br>Emplacement ';
        $places = [
            "desktop" => "bureau",
            "startup" => "demarrage auto",
            "taskbar" => "barre des tâches (ne fonctionne pas avec Windows)"
            // "bookmark" => "signet dans le navigaateur web"
        ];
        $html .= '<select name="shortcut[place]"/>';
        foreach ($places as $k => $val) {
            $html .= "<option value=$k";
            if ($k == @$shortcut['place']) {
                $html .= " selected>";
            } else {
                $html .= ">";
            }
            $html .= $val . "</option>";
        }
        $html .= '</select><br>';
        $html .= 'Propriétaire (Utilisateur, Groupe/Machine, Salle, Parc) séparé par des , : ';
        $html .= '<input type="text" name="shortcut[owner]" id="name" value="' . @$shortcut['owner'] . '" size="20"  />';
        $html .= '<br />';
        if ($type == "url" || (isset($shortcut['windows']) && preg_match("/^http/", $shortcut['windows']['args']))) {
            $html .= 'Navigateur à utiliser :';
            $html .= '<select name="shortcut[windows][link]">';
            $l = "firefox";
            foreach ($navs as $k => $nav) {
                $html .= "<option value=" . $nav["windows"];
                if ($nav['windows'] == @$shortcut['windows']['link']) {
                    $l = $nav['linux'];
                    $html .= " selected>";
                } else {
                    $html .= ">";
                }
                $html .= $nav['name'] . "</option>";
            }
            $html .= '</select><br>';
            $html .= 'Url du site : ';
            $html .= '<input type="text" name="shortcut[windows][args]" value="' . htmlspecialchars(@$shortcut['windows']['args']) . '" size="50"  />';
            $html .= '<input type="hidden" name="shortcut[linux][args]" value="' . htmlspecialchars(@$shortcut['windows']['args']) . '"/>';
            $html .= '<input type="hidden" name="shortcut[linux][link]" value="' . htmlspecialchars($l) . '"/>';
            $html .= '<br>';
        } else {
            $html .= 'chemin de la cible windows (UNC, pas de lecteur réseau) : ';
            $html .= '<input type="text" name="shortcut[windows][link]" value="' . @$shortcut['windows']['link'] . '" size="50"  />';
            $html .= '<br>';
            $html .= 'Arguments windows : ';
            $html .= '<input type="text" name="shortcut[windows][args]" value="' . htmlspecialchars(@$shortcut['windows']['args']) . '" size="50"  />';
            $html .= '<br>';
            $html .= 'Dossier de base Windows (si besoin) : ';
            $html .= '<input type="text" name="shortcut[windows][path]" value="' . @$shortcut['windows']['path'] . '" size="50"  />';
            $html .= '<br>';
            $html .= 'Icone Windows (si différente de l\'exe) : ';
            $html .= '<input type="text" name="shortcut[windows][icon]" value="' . @$shortcut['windows']['icon'] . '" size="50"  />';
            $html .= '<br>';

            $html .= 'chemin de la cible linux : ';
            $html .= '<input type="text" name="shortcut[linux][link]" value="' . @$shortcut['linux']['link'] . '" size="50"  />';
            $html .= '<br>';
            $html .= 'Arguments linux : ';
            $html .= '<input type="text" name="shortcut[linux][args]" value="' . htmlspecialchars(@$shortcut['linux']['args']) . '" size="50"  />';
            $html .= '<br>';
            $html .= 'Dossier de base linux (si besoin) : ';
            $html .= '<input type="text" name="shortcut[linux][path]" value="' . @$shortcut['linux']['path'] . '" size="50"  />';
            $html .= '<br>';
            $html .= 'Nom de la fenêtre linux (facultatif) : ';
            $html .= '<input type="text" name="shortcut[linux][startupwmclass]" value="' . @$shortcut['linux']['startupwmclass'] . '" size="30"  />';
            $html .= '<br>';
        }
        $html .= "<input type=submit name=valider[$key] value=\"Valider\"></h2>";
    } elseif (! empty($shortcut['name'])) {
        $html .= '<img src="../gpo/shortcuts_out.php?action=icon&shortcut=' . urlencode($shortcut['name']) . '&os=linux" height=50 width=50/>';
        $html .= "<input type=submit name=edit[$key] value=\"modifier " . @$shortcut['name'] . "\"> (" . @$shortcut['place'] . " / " . @$shortcut['owner'] . ") ";
        $html .= "<input type=submit name=del[$key] value=\"Supprimer\">";
    } else {
        $html .= "<input type=submit name=edit[$key] value=\"Nouveau...\">";
    }
    $html .= "<p>";
    return $html;
}

function list_shortcuts($config, $owner = "", $type = "all")
{
    $html = "";
    $filename = "/etc/sambaedu/applications/shortcuts/shortcuts.json";
    if (file_exists($filename)) {
        $shortcuts = json_decode(file_get_contents($filename), true) ?? [];
    } else {
        $shortcuts = [];
    }
    // $shortcuts = array_merge($shortcuts, get_wine_shortcuts($config));
    if (count($shortcuts) > 0) {
        foreach ($shortcuts as $key => $shortcut) {
            $shortcut_owner = preg_split('/(\s*,*\s*)*,+(\s*,*\s*)*/', $shortcut['owner']);
            if (empty($owner) || in_array($owner, $shortcut_owner)) {
                $html .= form_shortcut($config, $shortcuts, $key, $type);
            }
        }
        if (! empty($_POST['valider'][$key + 1])) {
            $key ++;
            $html .= form_shortcut($config, $shortcuts, $key, $type);
        }
        file_put_contents($filename, json_encode($shortcuts, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES));
        $shortcuts[$key + 1]['owner'] = $owner;
        $html .= form_shortcut($config, $shortcuts, $key + 1, $type);
    } else {
        $shortcuts[] = [];
        if (! empty($owner)) {
            $shortcuts[0]['owner'] = $owner;
        }
        $html .= form_shortcut($config, $shortcuts, 0, $type);
        if (count($shortcuts[0]) > 1) {
            file_put_contents($filename, json_encode($shortcuts, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES));
        }
    }
    return $html;
}